Lua_AvatarItem = class("Lua_AvatarItem", Lua_DisplayObject)
function Lua_AvatarItem:ctor(p)	
	self:superCall("ctor", p)
end

function Lua_AvatarItem:buildShowItem()
	self:superCall("buildShowItem")
	self.avatarIcon = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0)
end

function Lua_AvatarItem:dispose()
	self._avatarBg = nil;
	self.avatarIcon = nil;
	self.roleFrame = nil;
	self.roleTitle = nil;
	self.roleName = nil;
	if self.roleVip then
		self.roleVip:dispose();
		self.roleVip = nil;
	end
	self.roleLevel = nil;
	self.owner = nil;
	self.callBack = nil;
	self:superCall("dispose");
end

function Lua_AvatarItem:checkInBound(touchX, touchY)
	local posX = self.posX or 0
	local posY = self.posY or 0
	local scaleX = self.scaleX or 1
	local scaleY = self.scaleY or 1
	local width = self.avatarIcon.width * self.avatarIcon.scaleX
	local height = self.avatarIcon.height * self.avatarIcon.scaleY
	local minX = posX - width / 2 * scaleX
	local maxX = posX + width / 2 * scaleX
	local minY = posY - height / 2 * scaleY
	local maxY = posY + height / 2 * scaleY
	if touchX >= minX and touchX <= maxX and touchY >= minY and touchY <= maxY then
		return true
	end
	return false
end

function Lua_AvatarItem:setCanClick(bValue)
	if bValue then
		self.avatarIcon.mouseEnable = true;
		self.avatarIcon:addClickFuntion(self, self.onAvatarClick);
	else
		self.avatarIcon.mouseEnable = false;
		self.avatarIcon:removeClickFuntion(self, self.onAvatarClick);
	end
end

function Lua_AvatarItem:setAvatarClick(owner, callBack)
	self.owner = owner;
	self.callBack = callBack;
	if self.owner == nil or self.callBack == nil then
		self:setCanClick(false);
	else
		self:setCanClick(true);
	end
end

function Lua_AvatarItem:onAvatarClick(go)
	if self.isRobot and self.isRobot > 0 then
		OperateAlert.showLangValue(123657);
		return;
	end

	if self.roleId and tonumber(self.roleId) > 0 then
		if DataCenter.roleData:isMyself(self.roleId) then
			OperateAlert.showLangValue(123656);
		else
			if self.serverFlag == nil then
				Lua_WindowUtil.openWin("Lua_PlayerInfoWindow", Lua_WinParam.ACTION_OPEN, self.roleId);
			else
				Lua_WindowUtil.openWin("Lua_PlayerInfoWindow", Lua_WinParam.ACTION_OPEN, { id = self.roleId, server_flag = self.serverFlag });
			end
		end
	end

	if self.leagueId and tonumber(self.leagueId) > 0 then
		if DataCenter.roleData:isSameAlliance(self.leagueId) then
			OperateAlert.showLangValue(123659)
		else
			Lua_WindowUtil.openWin("Lua_LeagueDetailWindow", Lua_WinParam.ACTION_OPEN, self.leagueId)
		end
	end

	if self.owner and self.callBack then
		self.callBack(self.owner);
	end
end

function Lua_AvatarItem:init()
	self.roleId = nil;
	self.isRobot = nil;
	self.leagueId = nil;
	self:setAvatarBg(self.bgAtlas, self.bgSprite)
	self.avatarIcon:setEnable(false)
	if self.roleName then self.roleName:setEnable(false) end
	if self.roleVip then self.roleVip:setEnable(false) end
	if self.roleFrame then self.roleFrame:setEnable(false) end
	if self.roleTitle then self.roleTitle:setEnable(false) end
	if self.roleLevelBg then self.roleLevelBg:setEnable(false) end
	if self.roleLevel then self.roleLevel:setEnable(false) end
	if self.leagueLevelTxt then self.leagueLevelTxt:setEnable(false) end
end

function Lua_AvatarItem:setAvatarBg(bgAtlas, bgSprite, bgScale)
	self.bgAtlas = bgAtlas;
	self.bgSprite = bgSprite;
	bgScale = bgScale or 1;
	if strIsNullOrEmpty(bgAtlas) or strIsNullOrEmpty(bgSprite) then
		if self.avatarBg then self.avatarBg:setEnable(false); end
	else
		if self.avatarBg == nil then
			self.avatarBg = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0)
			self.avatarBg.siblingIndex = 0
		end
		self.avatarBg:setEnable(true);
		self.avatarBg:setAltas(bgAtlas, bgSprite);
		self.avatarBg:resetSize();
		self.avatarBg:setScale(bgScale, bgScale, bgScale);
	end
end

function Lua_AvatarItem:setAvatar(atlas, sprite, scale)
	scale = scale or 1;
	self.avatarIcon:setEnable(true);
	self.avatarIcon:setAltas(atlas, sprite);
	self.avatarIcon:resetSize();
	self.avatarIcon:setScale(scale, scale, 1);
end

------------------------------ 玩家头像 ------------------------------

function Lua_AvatarItem:setRoleId(roleId, serverFlag, isRobot)
	self.roleId = roleId; -- 玩家ID
	self.serverFlag = serverFlag; -- 服务器标识
	self.isRobot = isRobot; -- 是否机器人 大于0是
	if roleId == nil or tonumber(roleId) <= 0 then
		self:setCanClick(false);
	else
		self:setCanClick(true);
	end
end

function Lua_AvatarItem:setRoleDressUp(dress_info, showFrame, showTitle, avatarScale)
	local id = DataCenter.roleData:getDressUpId(Lua_DressUpDefine.MAINTYPE_HEAD, dress_info);
	self:setRoleAvatar(id, avatarScale);

	if showFrame ~= false then
		id = DataCenter.roleData:getDressUpId(Lua_DressUpDefine.MAINTYPE_FRAME, dress_info);
		self:setRoleFrame(id);
	end

	if showTitle ~= false then
	-- 	id = DataCenter.roleData:getDressUpId(Lua_DressUpDefine.MAINTYPE_TITLE, dress_info);
	-- 	self:setRoleTitle(id);
	end
end

function Lua_AvatarItem:setRoleAvatar(avatarId, scale, atlas, sprite)
	if strIsNullOrEmpty(self.bgAtlas) or strIsNullOrEmpty(self.bgSprite) then
		self.bgAtlas = "common_altas_3" -- 默认，也是整个游戏的设置
		self.bgSprite = "xui_common_widget_019"
	end
	self:setAvatarBg(self.bgAtlas, self.bgSprite)

	scale = scale or 0.9
	local vo = DataCenter.roleUtil:getDressUpVoById(avatarId)
	if vo == nil then
		if strIsNullOrEmpty(atlas) or strIsNullOrEmpty(sprite) then
			if DataCenter.roleData.sex == 1 then
				self:setAvatar("role_avatar_1", "nanzhu", scale)
			else
				self:setAvatar("role_avatar_1", "nvzhu", scale)
			end
		else
			self:setAvatar(atlas, sprite, scale)
		end
	else
		self:setAvatar(vo.atlas, vo.sprite, scale)
	end
end

function Lua_AvatarItem:setRoleFrame(frameId, scale)
	local vo = DataCenter.roleUtil:getDressUpVoById(frameId);
	if vo == nil or vo.default == 1 then
		if self.roleFrame then
			self.roleFrame:setEnable(false);
		end
	else
		if self.roleFrame == nil then
			self.roleFrame = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
			self:updateSiblingIndex()
		end
		self.roleFrame:setEnable(true);
		self.roleFrame:setAltas(vo.atlas, vo.sprite);
		self.roleFrame:resetSize();
		scale = scale or 1
		self.roleFrame:setScale(scale, scale, 1);
	end
end

-- function Lua_AvatarItem:setRoleTitle(titleId)
-- 	local vo = DataCenter.roleUtil:getDressUpVoById(titleId);
-- 	if vo == nil then
-- 		if self.roleTitle ~= nil then
-- 			self.roleTitle:setEnable(false);
-- 		end
-- 	else
-- 		if self.roleTitle == nil then
-- 			self.roleTitle = UICreate.createUIAtlasImage(self.content, nil, nil, 0, -30, 0, 0);
-- 		end
-- 		self.roleTitle:setEnable(true);
-- 		self.roleTitle:setAltas(vo.atlas, vo.sprite);
-- 		self.roleTitle:resetSize();
-- 	end
-- end

function Lua_AvatarItem:setRoleName(name)
	if self.roleName == nil then
		self.roleName = UICreate.createLabel(self.content, nil, 0, -65, 100, 31, 22);
		self.roleName.pivotValue = 4
		self.roleName.alignmentValue = 4;
		self.roleName.horizontalOverflowValue = 1;
		self.roleName.colorStr = Lua_ColorDefine.transColor("846651");
	end
	self.roleName:setEnable(true)
	self.roleName.text = name;
end

function Lua_AvatarItem:setRoleVip(vipNum)
	if self.roleVip == nil then
		self.roleVip = Lua_NumberImage.new(self.content);
		self.roleVip:setLocation(-40, 32, 0);
		self.roleVip.numOffsetX = -4;
		self.roleVip.numAtlas = "number";
		self.roleVip.firstStr = "xui_vip_number_v";
		self.roleVip.preStr = "xui_vip_number_";
		self:updateSiblingIndex()
	end
	self.roleVip:setEnable(true)
	self.roleVip.num = vipNum;
end

function Lua_AvatarItem:setRoleLevel(level, bgPosY, txtPosY)
	bgPosY = bgPosY or -42
	txtPosY = txtPosY or -42
	if self.roleLevelBg == nil then
		self.roleLevelBg = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);

		self.roleLevel = UICreate.createLabel(self.content, nil, 0, 0, 100, 31, 22);
		self.roleLevel.pivotValue = 4
		self.roleLevel.alignmentValue = 4;
		self.roleLevel.horizontalOverflowValue = 1;
		self.roleLevel:setEffectDistance(2, 2);

		self:updateSiblingIndex()
	end

	self.roleLevelBg:setEnable(true)
	self.roleLevelBg:setLocation(0, bgPosY, 0)
	self:setRoleLevelBg("common_altas_3", "xui_common_widget_035") -- 默认，也是整个游戏的设置

	self.roleLevel:setEnable(true)
	self.roleLevel:setLocation(0, txtPosY, 0)
	self:setRoleLevelStyle("huakang_font", 22, "FFFFFF", 0) -- 默认，也是整个游戏的设置
	self.roleLevel.text = level;
end

function Lua_AvatarItem:setRoleLevelBg(atlas, sprite, width, height)
	self.roleLevelBg:setAltas(atlas , sprite)
	if width == nil or height == nil then
		self.roleLevelBg:resetSize()
	else
		self.roleLevelBg:SetDimensions(width, height)
	end
end

function Lua_AvatarItem:setRoleLevelStyle(fontName, fontSize, colorStr, effectStyle, effectColorStr)
	self.roleLevel.fontName = fontName or "huakang_font";
	self.roleLevel.fontSize = fontSize or 22;
	self.roleLevel.colorStr = Lua_ColorDefine.transColor(colorStr or "FFFFFF");
	self.roleLevel.effectStyle = effectStyle or 2;
	effectColorStr = effectColorStr or "000000";
	self.roleLevel.effectColorStr = Lua_ColorDefine.transColor(effectColorStr);
end

function Lua_AvatarItem:updateSiblingIndex()
	local index = self.avatarIcon.siblingIndex;

	if self.roleLevelBg then
		index = index + 1;
		self.roleLevelBg.siblingIndex = index;
	end

	if self.roleLevel then
		index = index + 1;
		self.roleLevel.siblingIndex = index;
	end

	if self.roleVip then
		index = index + 1;
		self.roleVip.siblingIndex = index;
	end

	if self.roleFrame then
		index = index + 1;
		self.roleFrame.siblingIndex = index;
	end

	if self.roleVip then
		index = index + 1;
		self.roleVip:setSiblingIndex(index);
	end
end

------------------------------ 联盟头像 ------------------------------

function Lua_AvatarItem:setLeagueId(leagueId)
	self.leagueId = leagueId;
	if leagueId == nil or tonumber(leagueId) <= 0 then
	self:setCanClick(false);
	else
		self:setCanClick(true);
	end
end

function Lua_AvatarItem:setLeagueAvatarId(id, t, scale)
	local sprite = DataCenter.allianceUtil:getAllianceHeadByIndex(id, t)
	self:setLeagueAvatar(sprite, scale)
end

function Lua_AvatarItem:setLeagueAvatar(sprite, scale)
	if self.avatarBg then self.avatarBg:setEnable(false) end
	self:setAvatar("allianceHead_atlas", sprite, scale)
end

-- 设置联盟头像等级
function Lua_AvatarItem:setLeagueLevel(level)
	if self.leagueLevelTxt == nil then
		self.leagueLevelTxt = UICreate.createLabel(self.content, nil, 0, -36, 100, 35, 26);
		self.leagueLevelTxt.alignmentValue = 4;
		self.leagueLevelTxt.pivotValue = 4
		self.leagueLevelTxt.horizontalOverflowValue = 1;
		self.leagueLevelTxt.fontName = "huakang_font";
		self.leagueLevelTxt.colorStr = Lua_ColorDefine.transColor("FFFFFF");
		self.leagueLevelTxt.effectStyle = 2;
		self.leagueLevelTxt.effectColorStr = Lua_ColorDefine.transColor("272728");
		self.leagueLevelTxt:setEffectDistance(2, 2);
	end
	self.leagueLevelTxt:setEnable(true);
	self.leagueLevelTxt.text = level;
end

------------------------------ 英雄头像 ------------------------------

function Lua_AvatarItem:setHeroAvatar(id, scale)
	scale = scale or 0.9
	local config = Lua_HeroConfigData.getConfig(id)
	self:setAvatar(config.avatar_atlas_1, config.avatar_sprite_1, scale);
end