--[[
    local data = Array()
    data:add({ textStr = "1", redPointId = Lua_RedPointList.MailTab });
    data:add({ textStr = "2" });
    data:add({ textStr = "3" });
    data:add({ textStr = "4" });

    self.buttonList = Lua_ButtonList.new(self.content, Lua_AxisDefine.Horizontal);
    self.buttonList:setLocation(0, 0, 0);
    self.buttonList:setButtonStyle("common_altas_1", "ui_common_tab_02_s", "ui_common_tab_02", 166, 69, 1, 1);
    self.buttonList:setButtonText(1, 6, "5C504B", "8B5E34", 30, Lua_MyUISettings.huakangFontName, UnityEngine.FontStyle.Normal);
    self.buttonList:setButtonLine("common_altas_1", "ui_common_line_013");
    self.buttonList:setRedPoint(0, 0);
    self.buttonList:setButtonItem(166, 69);
    self.buttonList:setClickCallBack(self, self.onBtnClick);
    self.buttonList:setData(data);
    self.buttonList:setSelected(1);
--]]

Lua_ButtonList = class("Lua_ButtonList", Lua_DisplayObject);
function Lua_ButtonList:ctor(p, showMode, needDrag, width, height)
	self.showMode = showMode;
	self.needDrag = (needDrag ~= nil) and needDrag or false;
	self.rectWidth = (width == nil) and 0 or width;
	self.rectHeight = (height == nil) and 0 or height;
	self._canDrag = true;
	self.moveTime = 2;
	self._clickMoveOffset = nil; -- 点击后移动偏移值
	
	self.itemWidth = 0;
	self.itemHeight = 0;
	self.minX = 0;
	self.maxX = 0;
	self.minY = 0;
	self.maxY = 0;

	self:superCall("ctor", p);
end

function Lua_ButtonList:buildShowItem()
	self:superCall("buildShowItem");
end

function Lua_ButtonList:dispose()
	self:removeMoveTimer();
	if self.lineList ~= nil then
		self.lineList:dispose();
		self.lineList:clear();
		self.lineList = nil;
	end
	if self.showList ~= nil then
		self.showList:dispose();
		self.showList:clear();
		self.showList = nil;
	end
	if self.hideList ~= nil then
		self.hideList:dispose();
		self.hideList:clear();
		self.hideList = nil;
	end
	self.sub = nil;
	self.subRect = nil;
	if self.scrollContainer then
		self.scrollContainer:dispose();
		self.scrollContainer = nil;
	end
	self.cellClass = nil;
	self.owner = nil;
	self.callBack = nil;
	self.data = nil;
	self:superCall("dispose");
end

-- needDrag = true 设置无效 默认0
-- Horizontal 1:从左往右 0:居中 -1:从右往左
-- Vertical 1:从上往下 0:居中 -1:从下往上
function Lua_ButtonList.Set:pivot(value)
    self._pivot = value;
end

-- function Lua_ButtonList.Get:pivot()
--     return self._pivot or 0;
-- end

function Lua_ButtonList.Set:clickMoveOffset(value)
    self._clickMoveOffset = value;
end

function Lua_ButtonList:setContentRect(width, height)
	if self.rectWidth == width and self.rectHeight == height then
		return;
	end

	self.rectWidth = width or 0;
	self.rectHeight = height or 0;
	if self.data then
		self:updateShowSize();
		self:setData(self.data);
	end
end

-- 显示背景
function Lua_ButtonList:setButtonBg(bgAtlas, bgSprite, bgWidth, bgHeight)
	self.bgAtlas = bgAtlas;
	self.bgSprite = bgSprite;
	self.bgWidth = bgWidth;
	self.bgHeight = bgHeight;
end

-- 按钮样式
function Lua_ButtonList:setButtonStyle(btnAtlas, btnNormalSprite, btnSelectedSprite, btnWidth, btnHeight, btnNormalAlpha, btnSelectedAlpha, btnSelectedOffset)
	self.btnAtlas = btnAtlas;
	self.btnNormalSprite = btnNormalSprite;
	self.btnSelectedSprite = btnSelectedSprite;
	self.btnWidth = btnWidth;
	self.btnHeight = btnHeight;
	self.btnNormalAlpha = btnNormalAlpha;
	self.btnSelectedAlpha = btnSelectedAlpha;
	self.btnSelectedOffset = btnSelectedOffset;
end

-- 文本样式（当显示的是美术字时 该设置仅posX, posY有效）
function Lua_ButtonList:setButtonText(posX, posY, normalColor, selectedColor, normalSize, selectedSize, fontName, fontStyle)
	self.textPosX = posX;
	self.textPosY = posY;
	self.textNormalColor = normalColor;
	self.textSelectedColor = selectedColor;
	self.normalSize = normalSize;
	self.selectedSize = selectedSize;
	self.textName = fontName;
	self.textStyle = fontStyle;
end

-- 分割线 lineStyle = 1 显示在按钮前 默认0显示在按钮后
function Lua_ButtonList:setButtonLine(lineAtlas, lineSprite, lineWidth, lineHeight, linePosX, linePosY, lineStyle, linePosOffset)
	if self.lineList == nil then
		self.lineList = Array();
	else
		self.lineList:clear();
	end
	self.lineAtlas = lineAtlas;
	self.lineSprite = lineSprite;
	self.lineWidth = (lineWidth == nil) and 0 or lineWidth;
	self.lineHeight = (lineHeight == nil) and 0 or lineHeight;
	self.linePosX = (linePosX == nil) and 0 or linePosX;
	self.linePosY = (linePosY == nil) and 0 or linePosY;
	self.lineStyle = (lineStyle == nil) and 0 or lineStyle;
	self.linePosOffset = (linePosOffset == nil) and 0 or linePosOffset;
end

function Lua_ButtonList:setButtonCell(cellClass)
	self.cellClass = cellClass
end

function Lua_ButtonList:setRedPoint(posX, posY)
	self.redPointPosX = posX;
	self.redPointPosY = posY;
end

function Lua_ButtonList:setButtonItem(itemWidth, itemHeight)
	self.itemWidth = itemWidth;
	self.itemHeight = itemHeight;
end

function Lua_ButtonList:setClickCallBack(owner, callBack)
	self.owner = owner;
	self.callBack = callBack;
end

function Lua_ButtonList:updateShowSize()
	if self.showMode == Lua_AxisDefine.Horizontal then
		self.showWidth = self.showCount * self.itemWidth;
		self.showHeight = self.itemHeight;
	else
		self.showWidth = self.itemWidth;
		self.showHeight = self.showCount * self.itemHeight;
	end

	self.rectWidth = self.rectWidth <= 0 and self.showWidth or self.rectWidth;
	self.rectHeight = self.rectHeight <= 0 and self.showHeight or self.rectHeight;
	self.showWidth = self.showWidth < self.rectWidth and self.rectWidth or self.showWidth;
	self.showHeight = self.showHeight < self.rectHeight and self.rectHeight or self.showHeight;

	self.maxX = (self.showWidth - self.rectWidth) / 2;
	self.minX = -self.maxX;
	self.maxY = (self.showHeight - self.rectHeight) / 2;
	self.minY = -self.maxY;

	if self.showMode == Lua_AxisDefine.Horizontal then
		self.subMinPos = -(self.showWidth - self.rectWidth) / 2;
		self.subMaxPox = -self.subMinPos;
	else
		self.subMinPos = -(self.showHeight - self.rectHeight) / 2;
		self.subMaxPox = -self.subMinPos;
	end

	self:setRectSize(self.rectWidth, self.rectHeight);
end

function Lua_ButtonList:getShowParent()
	if self.needDrag == true then
		if self.scrollContainer == nil then
			self.scrollContainer = UICreate.createScrollContainer(self.content, 0, 0, self.rectWidth, self.rectHeight);
			self.scrollContainer.horizontal = self.showMode == Lua_AxisDefine.Horizontal;
			self.scrollContainer.vertical = self.showMode == Lua_AxisDefine.Vertical;
			self.scrollContainer.movementTypeValue = 1;
			self.scrollContainer.elasticity = 0.3;
			self.scrollContainer.inertia = true;
			self.scrollContainer:setScrollPosition(0, 0, 0);
			self.scrollContainer:setScrollSize(0, 0);
			self.scrollContainer:setScrollAnchor(0.5, 0.5, 0.5, 0.5);
			self.scrollContainer:setScrollPivot(4);
			self.scrollContainer:setNormalizedPosition(0, 1);
			self.scrollContainer:addMousDownFuntion(self, self.onMouseDownFun);
			self.scrollContainer:addMouseUpFuntion(self, self.onMouseUpFun);
			self.scrollContainer:addRectMask2D();

			self.sub = self.scrollContainer.scrollContent;
			self.subRect = self.scrollContainer.scrollRect;
		end
		local horizontalNormalizedPosition = self.scrollContainer.horizontalNormalizedPosition;
		local verticalNormalizedPosition = self.scrollContainer.verticalNormalizedPosition;
		if self.isFirstEnter ~= false then
		self.isFirstEnter = false;
			horizontalNormalizedPosition = 0;
			verticalNormalizedPosition = 1;
		end
		LuaUtil.setRectSize(self.subRect, self.showWidth, self.showHeight);
		self.scrollContainer:setNormalizedPosition(horizontalNormalizedPosition, verticalNormalizedPosition);
		return self.sub;
	end
	return self.content;
end

function Lua_ButtonList:onMouseDownFun()
	self:removeMoveTimer();
	self.startMousePos = self.mousePosition;
	self.startClickTime = getTime();
	self.pressItem = self:checkClick();
end

function Lua_ButtonList:onMouseUpFun()
	local winPos = LuaUtil.getTouchPosInverseObj(self.sub)
	local value = self.mousePosition - self.startMousePos
	if self.pressItem and self.pressItem:checkInBound(winPos.x, winPos.y) and (not self._canDrag or Mathf.Abs(value) < 2) then
		if self._clickMoveOffset ~= nil then
			self:moveToIndex(self.pressItem.index - self._clickMoveOffset, false)
		end
		self.pressItem:onPressClick()
	end
	self.pressItem = nil
end

function Lua_ButtonList.Get:mousePosition()
	if self.showMode == Lua_AxisDefine.Horizontal then
		return CommonUtil.mousePositionX;
	end
	return CommonUtil.mousePositionY;
end

function Lua_ButtonList:checkClick()
	local winPos = LuaUtil.getTouchPosInverseObj(self.sub);
	for i = 1, self.showList:getSize() do
		if self.showList[i]:checkInBound(winPos.x, winPos.y) then
			return self.showList[i];
		end
	end
	return nil;
end

function Lua_ButtonList:moveToIndex(idx, immediately)
	idx = idx <= 0 and 1 or idx
	idx = idx > self.showCount and self.showCount or idx

	local targetPos
	if self.showMode == Lua_AxisDefine.Horizontal then
		targetPos = self.subMaxPox;
		targetPos = targetPos - (idx - 1) * self.itemWidth
	else
		targetPos = self.subMinPos;
		targetPos = targetPos + (idx - 1) * self.itemHeight
	end

	if targetPos > self.subMaxPox then
		targetPos = self.subMaxPox
	elseif targetPos < self.subMinPos then
		targetPos = self.subMinPos
	end

	if immediately == true then
		MsgUtil.callFunc("FPS_INTERMISSION_HIGHT");
		self:setSubPosition(targetPos)
	else
		self.targetPos = targetPos;
		self:removeMoveTimer();
		self:startMoveTimer();
	end
end

function Lua_ButtonList:setSubPosition(value)
	if self.showMode == Lua_AxisDefine.Horizontal then
		self.scrollContainer:setScrollPosition(value, 0, 0);
	else
		self.scrollContainer:setScrollPosition(0, value, 0);
	end
end

function Lua_ButtonList:startMoveTimer()
	if self.bMoveTimer ~= true then
		self.bMoveTimer = true;
		self.startMoveTime = getTime()
		Lua_CommonUtil.addTimer("Lua_ButtonList_Move", 0.01, self, self.onMoveTimer);
	end
end

function Lua_ButtonList:removeMoveTimer()
	if self.bMoveTimer == true then
		self.bMoveTimer = false;
		Lua_CommonUtil.removeTimer(self, self.onMoveTimer);
	end
end

function Lua_ButtonList:onMoveTimer()
	local dt = getTime() - self.startMoveTime
	if dt >= self.moveTime then
		self:removeMoveTimer()
		self:setSubPosition(self.targetPos)
	else
		self:setSubPosition(Mathf.Lerp(self.curSubPos, self.targetPos, dt / self.moveTime))
	end
end

function Lua_ButtonList.Get:curSubPos()
	if self.showMode == Lua_AxisDefine.Horizontal then
		return self.subRect.anchoredPosition.x;
	else
		return self.subRect.anchoredPosition.y;
	end
end

function Lua_ButtonList.Set:canDrag(value)
	self._canDrag = value;
	self.scrollContainer.canDrag = value;
end

function Lua_ButtonList.Get:bDragEnd()
	if self.showMode == Lua_AxisDefine.Horizontal then
		if self.curSubPos <= self.subMinPos then
			return true;
		end
	else
		if self.curSubPos >= self.subMaxPox then
			return true;
		end
	end
	return false;
end

-- 是否到底
function Lua_ButtonList.Get:isTheEnd()
    if self.showMode == Lua_AxisDefine.Horizontal then
        if self.showWidth <= self.rectWidth then
            return true;
        end
    else
        if self.showHeight <= self.rectHeight then
            return true;
        end
    end
    return false;
end

function Lua_ButtonList:setData(data)
	self:returnItem();
	self.selectedIdx = nil;

	self.data = data;
	if data == nil then
		return;
	end

	if data.typeof ~= nil and data:typeof() == "Array" then
		self.showCount = data:getSize();
	else
		self.showCount = #data;
	end

	self:updateShowSize();
	local p = self:getShowParent();
	local w;
	local posX;
	local posY;

	if self.bgAtlas ~= nil and self.bgSprite ~= nil then
		if self.btnBg == nil then
			self.bgWidth = self.bgWidth == nil and self.showWidth or self.bgWidth;
			self.bgHeight = self.bgHeight == nil and self.showHeight or self.bgHeight;
			self.btnBg = UICreate.createUIAtlasImage(p, self.bgAtlas, self.bgSprite, 0, 0, self.bgWidth, self.bgHeight);
		end
	end

	if self.showMode == Lua_AxisDefine.Horizontal then
		if self._pivot == 1 then
			w = 0 - self.itemWidth;
		elseif self._pivot == -1 then
			w = self.itemWidth * self.showCount;
			w = -w / 2 + self.itemWidth / 2;
		else
			w = self.itemWidth * self.showCount;
			w = -w / 2 + self.itemWidth / 2;
		end
	else
		if self._pivot == 1 then
			w = 0 + self.itemHeight;
		elseif self._pivot == -1 then
			w = -self.itemHeight * (self.showCount - 1);
			w = -w - self.itemHeight;
		else
			w = self.itemHeight * self.showCount;
			w = w / 2 - self.itemHeight / 2;
		end
	end

	local item;
	for i = 1, self.showCount do
		item = self:createItem(p, i);
		if self.showMode == Lua_AxisDefine.Horizontal then
			posX = self.itemWidth * ((i - 1) % self.showCount) + w;
			posY = 0;
		else
			posX = 0;
			posY = -self.itemHeight * ((i - 1) % self.showCount) + w;
		end
		item:setLocation(posX, posY, 0);
		item:setData(data[i]);
		item:updateTxtSizeByContent();
	end

	if self.lineAtlas ~= nil and self.lineSprite ~= nil then
		local showCount = self.showCount;
		if self.lineStyle == 1 then
			if self.showMode == Lua_AxisDefine.Horizontal then
				w = w - self.itemWidth / 2;
			else
				w = w + self.itemHeight / 2;
			end
		else
			showCount = self.showCount - 1;
			if self.showMode == Lua_AxisDefine.Horizontal then
				w = w + self.itemWidth / 2;
			else
				w = w - self.itemHeight / 2;
			end
		end
		local line;
		for i = 1, showCount do
			line = self.lineList[i];
			if i > showCount then
				if line ~= nil then
					line:setEnable(false);
				end
			else
				if line == nil then
					line = UICreate.createUIAtlasImage(p, self.lineAtlas, self.lineSprite, 0, 0, self.lineWidth, self.lineHeight);
					self.lineList:add(line);
				else
					line:setEnable(true);
				end
				if self.showMode == Lua_AxisDefine.Horizontal then
					posX = self.itemWidth * ((i - 1) % showCount) + w + self.linePosOffset;
					posY = self.linePosY;
				else
					posX = self.linePosX;
					posY = -self.itemHeight * ((i - 1) % showCount) + w + self.linePosOffset;
				end
				line:setLocation(posX, posY, 0);
			end
		end
	end
end

function Lua_ButtonList:createItem(p, index)
	if self.showList == nil then
		self.showList = Array();
	end
	if self.hideList == nil then
		self.hideList = Array();
	end

	local len = self.hideList:getSize();
	local item = nil;
	if len > 0 then
		item = self.hideList:remove();
		item:setEnable(true);
	end

	if self.cellClass == nil then
		if item == nil then
			item = Lua_ButtonItem.new(p);
		end
		item:setButtonStyle(self.btnAtlas, self.btnNormalSprite, self.btnSelectedSprite, self.btnWidth, self.btnHeight, self.btnNormalAlpha, self.btnSelectedAlpha, self.btnSelectedOffset)
		item:setButtonText(self.textPosX, self.textPosY, self.textNormalColor, self.textSelectedColor, self.normalSize, self.selectedSize, self.textName, self.textStyle);
		item:setRedPoint(self.redPointPosX, self.redPointPosY);
	else
		if item == nil then
			item = self.cellClass.new(p);
		end

		if item.setButtonStyle ~= nil then
			item:setButtonStyle(self.btnAtlas, self.btnNormalSprite, self.btnSelectedSprite, self.btnWidth, self.btnHeight, self.btnNormalAlpha, self.btnSelectedAlpha, self.btnSelectedOffset);
		end
		if item.setButtonText ~= nil then
			item:setButtonText(self.textPosX, self.textPosY, self.textNormalColor, self.textSelectedColor, self.normalSize, self.selectedSize, self.textName, self.textStyle);
		end
		if item.setRedPoint ~= nil then
			item:setRedPoint(self.redPointPosX, self.redPointPosY);
		end

	end
	item:setRectSize(self.itemWidth, self.itemHeight);
	item:setButtonInfo(self.showMode, index)
	item:setClick(self, self.onButtonClick)
	item:setCanClick(not self.needDrag)

	self.showList:add(item);
	return item;
end

function Lua_ButtonList:returnItem()
	if self.showList == nil or self.hideList == nil then
		return;
	end

	local len = self.showList:getSize();
	local item;
	while len > 0 do
		item = self.showList:remove(len);
		item:setEnable(false);
		self.hideList:add(item);
		len = len - 1;
	end
end

function Lua_ButtonList:onButtonClick(index)
	if self.data == nil or self.data[index] == nil then
		return;
	end

	if type(self.data[index]) == type_str_table and not DataCenter.funcOpenData:isOpened(self.data[index].openId, true) then
		return;
	end

	self:setSelected(index);
end

function Lua_ButtonList:setSelected(index)
	index = index == nil and 1 or index;
	if self.data == nil or self.data[index] == nil or self.selectedIdx == index then
		return;
	end

	if type(self.data[index]) == type_str_table then
		if self.data[index].openState ~= false then
			self:updateSelected(index);
		end
	else
		self:updateSelected(index);
	end

	if self.owner ~= nil and self.callBack ~= nil then
		self.callBack(self.owner, index, self.data[index]);
	end
end

function Lua_ButtonList:updateSelected(index)
	for i = 1, self.showCount do
		self.showList[i]:setSelected(index);
	end
	self.selectedIdx = index;
end

function Lua_ButtonList:setOpenState(index, state)
	if self.data == nil or self.data[index] == nil then
		return;
	end
	self.data[index].openState = state;
end




















Lua_ButtonCell = class("Lua_ButtonCell", Lua_DisplayObject)
function Lua_ButtonCell:ctor(p, skin, frameBuild)
	self.minX = 0
	self.maxX = 0
	self.minY = 0
	self.maxY = 0
	self:superCall("ctor", p, skin, frameBuild)
end

function Lua_ButtonCell:dispose()
	self.owner = nil
	self.callBack = nil
	self.data = nil
	self:superCall("dispose")
end

function Lua_ButtonCell:checkInBound(touchX, touchY)
	local posX = self.posX or 0
	local posY = self.posY or 0
	local scaleX = self.scaleX or 1
	local scaleY = self.scaleY or 1
	local minX = posX - self.w / 2 * scaleX
	local maxX = posX + self.w / 2 * scaleX
	local minY = posY - self.h / 2 * scaleY
	local maxY = posY + self.h / 2 * scaleY
	if touchX >= minX and touchX <= maxX and touchY >= minY and touchY <= maxY then
		return true
	end
	return false
end

function Lua_ButtonCell:setButtonInfo(showMode, index)
	self.showMode = showMode
	self.index = index
	self.content.name = self.classname..self.index
end

function Lua_ButtonCell:setData(data)

end

function Lua_ButtonCell:setSelected(index)

end

function Lua_ButtonCell:setClick(owner, callBack)
	self.owner = owner
	self.callBack = callBack
end

function Lua_ButtonCell:setCanClick(bCan)

end

function Lua_ButtonCell:onPressClick(go)
	if self.owner and self.callBack then
		self.callBack(self.owner, self.index)
	end
end

-- 根据内容设置字号
function Lua_ButtonCell:updateTxtSizeByContent()
end




















Lua_ButtonItem = class("Lua_ButtonItem", Lua_ButtonCell)

function Lua_ButtonItem:dispose()
	if self.redPointIcon ~= nil then
		self.redPointIcon:dispose();
		self.redPointIcon = nil;
	end
	self:superCall("dispose");
end

function Lua_ButtonItem:setButtonStyle(btnAtlas, btnNormalSprite, btnSelectedSprite, btnWidth, btnHeight, btnNormalAlpha, btnSelectedAlpha, btnSelectedOffset)
	self.btnAtlas = (btnAtlas == nil) and "common_altas_1" or btnAtlas;
	self.btnNormalSprite = (btnNormalSprite == nil) and "ui_common_tab_06" or btnNormalSprite;
	self.btnSelectedSprite = (btnSelectedSprite == nil) and "ui_common_tab_05" or btnSelectedSprite;
	btnWidth = (btnWidth == nil) and 0 or btnWidth;
	btnHeight = (btnHeight == nil) and 0 or btnHeight;
	self.btnNormalAlpha = (btnNormalAlpha == nil) and 1 or btnNormalAlpha;
	self.btnSelectedAlpha = (btnSelectedAlpha == nil) and 1 or btnSelectedAlpha;
	self.btnSelectedOffset = (btnSelectedOffset == nil) and 0 or btnSelectedOffset;

	if self.btnImg == nil then
		self.btnImg = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
	end
	self.btnImg:SetDimensions(btnWidth, btnHeight);
	self.btnImg:setAltas(self.btnAtlas, self.btnNormalSprite);
	self.btnImg.alpha = self.btnNormalAlpha;
end

function Lua_ButtonItem:setButtonText(posX, posY, normalColor, selectedColor, normalSize, selectedSize, fontName, fontStyle)
	self.textPosX = (posX == nil) and 0 or posX;
	self.textPosY = (posY == nil) and 0 or posY;
	self.textNormalColor = (normalColor == nil) and "FFFFFF" or normalColor;
	self.textSelectedColor = (selectedColor == nil) and "8B5E34" or selectedColor;
	self.normalSize = (normalSize == nil) and 30 or normalSize;
	self.selectedSize = (selectedSize == nil) and 30 or selectedSize;
	self.textName = (fontName == nil) and Lua_MyUISettings.huakangFontName or fontName;
	self.textStyle = (fontStyle == nil) and UnityEngine.FontStyle.Normal or fontStyle;
end

function Lua_ButtonItem:setRedPoint(posX, posY)
	self.redPointPosX = (posX == nil) and 0 or posX;
	self.redPointPosY = (posY == nil) and 0 or posY;
end

function Lua_ButtonItem:setData(data)
	self.data = data;
	if data == nil then
		return;
	end

	if data.textAtlas == nil or data.textNormalSprite == nil then
		if self.btnText == nil then
			self.btnText = UICreate.createLabel(self.content, nil, 0, 0, 100, 31, 22);
		end
		self.btnText:setLocation(self.textPosX, self.textPosY);
		self.btnText.pivot = UIUtils.Pivot.Center;
		self.btnText.fontName = self.textName;
		self.btnText.fontStyle = self.textStyle;
		self.btnText.fontSize = self.normalSize;
		self.btnText.alignmentValue = 4;
		self.btnText.horizontalOverflowValue = 1;
		self.btnText.colorStr = Lua_ColorDefine.transColor(self.textNormalColor);
		if type(data.textStr) == type_str_number then
			self.btnText:setLangValue(data.textStr);
		else
			self.btnText.text = data.textStr;
		end
	else
		if self.textIcon == nil then
			self.textIcon = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
		end
		self.textIcon:setLocation(self.textPosX, self.textPosY);
		self.textIcon:setAltas(data.textAtlas, data.textNormalSprite);
		self.textIcon:resetSize();
	end

	-- 红点
	local redPointId = data.redPointId;
	if redPointId ~= nil and DataCenter.redPointData:hasRedPointVoById(redPointId) then
		self.redPointIcon = Lua_RedPointIcon.new(self.content, redPointId);
		self.redPointIcon:setLocation(self.redPointPosX, self.redPointPosY);
	end
end

function Lua_ButtonItem:setSelected(index)
	if index == self.index then -- 选中
		self.btnImg:setAltas(self.btnAtlas, self.btnSelectedSprite);
		self.btnImg.alpha = self.btnSelectedAlpha;
		if self.showMode == Lua_AxisDefine.Horizontal then
			self.btnImg:setLocation(0, self.btnSelectedOffset, 0);
		else
			self.btnImg:setLocation(self.btnSelectedOffset, 0, 0);
		end

		if self.btnText ~= nil then
			self.btnText.fontSize = self.selectedSize;
			self.btnText.colorStr = Lua_ColorDefine.transColor(self.textSelectedColor);
		end

		if self.textIcon ~= nil and self.data.textSelectedSprite ~= nil then
			self.textIcon:setAltas(self.data.textAtlas, self.data.textSelectedSprite);
		end

		if self.redPointIcon ~= nil then
			self.redPointIcon:clickHandler();
		end
	else
		self.btnImg:setAltas(self.btnAtlas, self.btnNormalSprite);
		self.btnImg.alpha = self.btnNormalAlpha;
		self.btnImg:setLocation(0, 0, 0);

		if self.btnText ~= nil then
			self.btnText.fontSize = self.normalSize;
			self.btnText.colorStr = Lua_ColorDefine.transColor(self.textNormalColor);
		end

		if self.textIcon ~= nil then
			if DataCenter.funcOpenData:isOpened(self.data.openId) then
				self.textIcon:setAltas(self.data.textAtlas, self.data.textNormalSprite);
			else
				local textSprite = self.data.textUnOpenSprite == nil and self.data.textNormalSprite or self.data.textUnOpenSprite;
				self.textIcon:setAltas(self.data.textAtlas, textSprite);
			end
		end
	end
end

function Lua_ButtonItem:setCanClick(bCan)
	if bCan == true then
		self.btnImg.mouseEnable = true;
		self.btnImg:addClickFuntion(self, self.onPressClick);
	else
		self.btnImg.mouseEnable = false;
		self.btnImg:removeClickFuntion(self, self.onPressClick);
	end
end










Lua_CommonButtonItem1 = class("Lua_CommonButtonItem1", Lua_ButtonCell)
function Lua_CommonButtonItem1:ctor(p)
    self:superCall("ctor", p)
end

function Lua_CommonButtonItem1:buildShowItem()
    self:superCall("buildShowItem")

    self.nameBg = UICreate.createUIAtlasImage(self.content, "common_altas_3", "xui_common_toggle_003_b", 0, 0, 0, 0)
    self.nameBg:addClickFuntion(self, self.onPressClick)

    self.nameText = UICreate.createLabel(self.content, nil, 0, -5, 100, 31, 30)
    self.nameText.pivotValue = 4
    self.nameText.alignmentValue = 4
    self.nameText.horizontalOverflowValue = 1
    self.nameText.fontName = Lua_MyUISettings.huakangFontName
end

function Lua_CommonButtonItem1:dispose()
    self.data = nil
    self:superCall("dispose")
end

function Lua_CommonButtonItem1:setData(data)
	self.data = data
	if type(data) == type_str_number then
		self.nameText:setLangValue(data)
	else
		self.nameText.text = data
	end
end

function Lua_CommonButtonItem1:setSelected(index)
    if index == self.index then -- 选中
        self.nameBg:setAltas("common_altas_3", "xui_common_toggle_003_f")
        self.nameText.fontSize = 30
        self.nameText.colorStr = Lua_ColorDefine.transColor("894E23")
    else
        self.nameBg:setAltas("common_altas_3", "xui_common_toggle_003_b")
        self.nameText.fontSize = 28
        self.nameText.colorStr = Lua_ColorDefine.transColor("B2B2A6")
    end
end










Lua_CommonButtonItem2 = class("Lua_CommonButtonItem2", Lua_ButtonCell)
function Lua_CommonButtonItem2:ctor(p)
	self:superCall("ctor", p)
end

function Lua_CommonButtonItem2:buildShowItem()
	self:superCall("buildShowItem")

	self.nameBg = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_common_button_006_n", 0, 0, 216, 67)
	self.nameBg:setScale(0.86, 0.86, 1)
	self.nameBg:addClickFuntion(self, self.onPressClick)

	self.nameText = UICreate.createLabel(self.content, nil, 0, 3, 100, 31, 28)
	self.nameText.pivotValue = 4
	self.nameText.alignmentValue = 4
	self.nameText.horizontalOverflowValue = 1
	self.nameText.verticalOverflowValue = 1
	self.nameText.fontName = Lua_MyUISettings.huakangFontName
end

function Lua_CommonButtonItem2:dispose()
	self:superCall("dispose")
end

function Lua_CommonButtonItem2:setData(data)
	if type(data) == type_str_number then
		self.nameText:setLangValue(data)
	else
		self.nameText.text = data
	end
end

function Lua_CommonButtonItem2:setSelected(index)
	if index == self.index then -- 选中
		self.nameBg.alpha = 1
		self.nameText.colorStr = Lua_ColorDefine.transColor("8F5123")
	else
		self.nameBg.alpha = 0.01
		self.nameText.colorStr = Lua_ColorDefine.transColor("594D4D")
	end
end