checkRequire("Lua_ChatDragContainerItem");
Lua_CommonChatPanel = class("Lua_CommonChatPanel", Lua_DisplayObject);
function Lua_CommonChatPanel:ctor(p)
	self:superCall("ctor", p, "CommonChatPanel_skin");
end

function Lua_CommonChatPanel:buildShowItem()
	self:superCall("buildShowItem");
	
	self.defaultText = LanUtil.getValue(171006);
	self.chatInput:setDefaultLangValue(171006)
	self.chatInput.text = "";

	self.showChat = true;
	self.chatObject:SetActive(self.showChat);
	self.voiceObject:SetActive(not self.showChat);

	self.chatContainer = Lua_DragContainer.new(self.content, 720, 350, Lua_AxisDefine.Vertical, true);
	self.chatContainer:setLocation(0, 220, 0);

	self.btnState:addClickFuntion(self, self.onBtnStateClick);
	self.btnEmoji:addClickFuntion(self, self.onBtnEmojiClick);
	self.btnSend:addClickFuntion(self, self.onBtnSendClick);
	self.btnVoice.clickScale = false;
	self.btnVoice:addMousDownFuntion(self, self.onBtnVoiceClick);
	self.btnVoice:addMouseUpFuntion(self, self.onBtnVoiceUpClick);

	self:addLuaEvent("ADD_CHAT_MSG", self, self.onChatUpdate);
	self:addLuaEvent("CHAT_ADD_FACE", self, self.onFaceSelected);
	self:addCSharpEvent("VOICE_DONE", self, self.onVoiceDone);
end

function Lua_CommonChatPanel:dispose()
	Lua_CommonUtil.removeTimer(self, self.checkVoiceTime);
	if self.chatContainer ~= nil then
		self.chatContainer:dispose();
		self.chatContainer = nil;
	end
	self:superCall("dispose");
end

function Lua_CommonChatPanel:setChatChannel(chatChannel)
	self.chatChannel = chatChannel;

	local chatList = DataCenter.chatData:getChannelMsgList(chatChannel);
	local len = chatList:getSize();
	DataCenter.chatData:setChannelNewMsgNum(self.chatChannel, 0);
	local chatLen = self.chatContainer.showDragDataList:getSize();
	local tempLen = len - chatLen;
	if tempLen >= 0 then
		 while tempLen > 0 do
		 	self.chatContainer:addSubContainer(Lua_ChatDragContainerItem, nil, false);
		 	tempLen = tempLen - 1;
		 end
	else
		local item;
		while tempLen < 0 do
			item = self.chatContainer.showDragDataList:remove(1);
			item:returnCellToPool();
			item:dispose();
			tempLen = tempLen + 1;
		end
	end
	self.chatContainer:setChildData(chatList, false);
	if len > 4 then
		self.chatContainer:moveToIndex(len, true);
	end
end

function Lua_CommonChatPanel:onChatUpdate(chatVo)
	if chatVo == nil then
		return;
	end

	if chatVo.channel == self.chatChannel then
		local msg_limit = Lua_ChatChannelData.getConfigDataById(self.chatChannel).msg_limit;
		local chatLen = self.chatContainer.showDragDataList:getSize();
		if chatLen >= msg_limit then
			local list = DataCenter.chatData:getChannelMsgList(self.chatChannel);
			self.chatContainer:setChildData(list, false);
		else
			self.chatContainer:addSubContainer(Lua_ChatDragContainerItem, chatVo, true);
		end
		if chatLen > 4 then
			self.chatContainer:moveToIndex(chatLen, true);
		end
	end
end

function Lua_CommonChatPanel:onFaceSelected(val)
	local str = self.chatInput.text;
	self.chatInput.text = str.."/"..val;
end

function Lua_CommonChatPanel:onVoiceDone(obj)
	Lua_ChatUtil.sendVoiceMsg(self.chatChannel);
	ChatRecordSound.getInstance():clear();
end

function Lua_CommonChatPanel:onBtnStateClick(go)
	self.showChat = not self.showChat;
	self.chatObject:SetActive(self.showChat);
	self.voiceObject:SetActive(not self.showChat);
	if self.showChat then
		self.btnState:setAltas("chat_atlas", "ui_liaotian_button_001_n");
	else
		self.btnState:setAltas("chat_atlas", "ui_liaotian_button_003_n");
	end
	self.btnState:resetSize();
end

function Lua_CommonChatPanel:onBtnEmojiClick(go)
	local data = { offsetX = 0, offsetY = 0, isAdapter = false };
	Lua_WindowUtil.openWin("Lua_ChatAddItemWindow", Lua_WinParam.ACTION_OPEN, data);
end

function Lua_CommonChatPanel:onBtnSendClick(go)
	local str = self.chatInput.text;
	if str == self.defaultText or utf8len(str) <= 0 then
		OperateAlert.showLangValue(171023);
		return;
	end

	str = Lua_ChatUtil.replaceChatMsg(str);
	local sendState = Lua_ChatUtil.checkSendMsg(str);
	if sendState == 1 or sendState == 2 then
	    OperateAlert.showLangValue(171020);
	else
	    Lua_ChatUtil.sendMsg(self.chatChannel, str, true, true);
	end
	self.chatInput.text = "";
end

function Lua_CommonChatPanel:onBtnVoiceClick(go)
	Lua_WindowUtil.openWin("Lua_ChatSoundWindow", Lua_WinParam.ACTION_OPEN, 10);

	if ChatRecordSound.getInstance():startRecord() == true then
		self.voice_startTime = getTime();
		self.voice_limit = Lua_ChatChannelData.getConfigDataById(self.chatChannel).voice_limit;
		Lua_CommonUtil.addTimer("checkVoiceTime", 0.01, self, self.checkVoiceTime);
	else
		self:onBtnVoiceUpClick(nil);
	end
end

function Lua_CommonChatPanel:onBtnVoiceUpClick(go)
	Lua_WindowUtil.openWin("Lua_ChatSoundWindow", Lua_WinParam.ACTION_CLOSE);
	ChatRecordSound.getInstance():stopRecord();
	Lua_CommonUtil.removeTimer(self, self.checkVoiceTime);
end

function Lua_CommonChatPanel:checkVoiceTime()
	if (getTime() - self.voice_startTime) >= self.voice_limit then
		self:onBtnVoiceUpClick(nil);
	end
end