checkRequire("Lua_DragListContainerData")
Lua_DragListContainer = class("Lua_DragListContainer", Lua_DragContainer)
function Lua_DragListContainer:ctor(p, rw, rh, dt, isMask)
	self:superCall("ctor", p, rw, rh, dt, isMask)
end

-- data 数据集合（Array）或者是单个数据
-- cellClass 显示item
-- cellWidth 单个cell宽度
-- cellHeight 单个cell高度
-- perLine 列数（单个数据的话，该选项默认为1，设置无效）
-- offsetVal 对应方向的偏移值
-- bCheckReBound 是否立即检测区域（默认是）
function Lua_DragListContainer:addChildData(data, cellClass, cellWidth, cellHeight, perLine, offsetVal, bCheckReBound)
	if data == nil or cellClass == nil then
		return
	end

	local vo = Lua_DragListContainerData.new(self._dragType)
	vo:setCellSpacing(self.cellSpacingX, self.cellSpacingY)
	vo:setData(data, cellClass, cellWidth, cellHeight, perLine, offsetVal)
	self.showDragDataList:add(vo)

	if self._dragType == Lua_AxisDefine.Horizontal then
		self.subRectW = self.subRectW + vo.showWidth
		self.subRectH = vo.showHight > self.subRectH and vo.showHight or self.subRectH
	else
		self.subRectW = vo.showWidth > self.subRectW and vo.showWidth or self.subRectW
		self.subRectH = self.subRectH + vo.showHight
	end
	
	if bCheckReBound == true then
		self:updateChildPos()
	end
end

function Lua_DragListContainer:updateChildPos()
	-- horizontal 0-1 vertical 1-0
	local horizontalNormalizedPosition = 0
	local verticalNormalizedPosition = 1
	if self.isFirstEnter == false then
		horizontalNormalizedPosition = self.horizontalNormalizedPosition
		verticalNormalizedPosition = self.verticalNormalizedPosition
	end
	self.scrollContainer:setNormalizedPosition(horizontalNormalizedPosition, verticalNormalizedPosition)
	self.isFirstEnter = false

	self:updateSubSize()

	self.cellsCount = 0
	local len = self.showDragDataList:getSize()
	local vo
	local startPosX
	local startPosY
	local value = 0
	local posX
	local posY
	for i = 1, len do
		vo = self.showDragDataList[i]
		self.cellsCount = self.cellsCount + vo.dataLen

		startPosX = -self.subRectW / 2 + self.cellPivotX * vo.showWidth
		startPosY = self.subRectH / 2 - (1 - self.cellPivotY) * vo.showHight
		if self._dragType == Lua_AxisDefine.Horizontal then
			posX = startPosX + value
			posY = startPosY
			value = value + vo.showWidth
		else
			posX = startPosX
			posY = startPosY - value
			value = value + vo.showHight
		end
		vo:setInfo(self, i, posX, posY)
	end

	self:updateDragItem()
	if not self.delayTween then
		self:playTween()
	end
end

function Lua_DragListContainer:updateSubSize()
	self.subRectW = self.subRectW < self._rectWidth and self._rectWidth or self.subRectW
	self.subRectH = self.subRectH < self._rectHeight and self._rectHeight or self.subRectH
	LuaUtil.setRectSize(self.subRect, self.subRectW, self.subRectH)

	if self._dragType == Lua_AxisDefine.Horizontal then
		self.subMinPos = -(self.subRectW - self._rectWidth) / 2
		self.subMaxPox = -self.subMinPos
		if self.bUpdateSubSize ~= true then
			self.bUpdateSubSize = true
			self:setSubPosition(self.subMaxPox)
		end
	else
		self.subMinPos = -(self.subRectH - self._rectHeight) / 2
		self.subMaxPox = -self.subMinPos
		if self.bUpdateSubSize ~= true then
			self.bUpdateSubSize = true
			self:setSubPosition(self.subMinPos)
		end
	end
end

function Lua_DragListContainer:updateDragItem()
	if self.bTweenTimer == true then
		return
	end

	local subRectX = self.subRect.anchoredPosition.x
	local subRectY = self.subRect.anchoredPosition.y
	local centerOffX -- cell当前轴心的x坐标转换为居中x坐标偏移值
	local centerOffY -- cell当前轴心的x坐标转换为居中x坐标偏移值
	local posX -- 把对象X转化为父对象sub X坐标
	local posY -- 把对象Y转化为父对象sub Y坐标
	local vo
	local index = 1
	local siblingIndex = 0
	local len = self.showDragDataList:getSize()
    for i = 1, len do
        vo = self.showDragDataList[i]

        centerOffX = (self.cellPivotX - 0.5) * vo.showWidth
        centerOffY = (self.cellPivotY - 0.5) * vo.showHight
        posX = vo.posX - centerOffX + subRectX
        posY = vo.posY - centerOffY + subRectY

        if self:checkItemInBound(posX, posY, vo.showWidth, vo.showHight) then
            index, siblingIndex = vo:getCellFromPool(index, siblingIndex, subRectX, subRectY)
            siblingIndex = siblingIndex + 1
        else
            vo:returnCellToPool()
        end

        -- index, siblingIndex = vo:getCellFromPool(index, siblingIndex, subRectX, subRectY)
        -- siblingIndex = siblingIndex + 1        
    end
end

function Lua_DragListContainer:playTween()
	self:removeTweenTimer()
	if self.showTween == true then
		self.showTween = false

		local tweenData = Lua_UITweenConfigData.getConfigData(self.tweenId)
		if tweenData == nil then
			return
		end

		local len = self.showList:getSize()
		local item
		for i = 1, len do
			item = self.showList[i]
			TweenUtil.getInstance():stop(item.content)

			if tweenData.startRot then
				LuaUtil.setObjRot(item.content, tweenData.startRot.x, tweenData.startRot.y, tweenData.startRot.z)
			end

			local startPosX
			local startPosY
			if tweenData.startPos then
				if tweenData.startPos.x ~= 0 then
					startPosX = tweenData.startPos.x * self.subRectW
					startPosY = item.y
				elseif tweenData.startPos.y ~= 0 then
					startPosX = item.x
					startPosY = tweenData.startPos.y * self.subRectH
				end
			end
			startPosX = startPosX == nil and item.x or startPosX
			startPosY = startPosY == nil and item.y or startPosY
			if tweenData.offsetPosTab then
				startPosX = startPosX + tweenData.offsetPosTab[1]
				startPosY = startPosY + tweenData.offsetPosTab[2]
			end
			LuaUtil.setObjPos(item.content, startPosX, startPosY, 0)

			if tweenData.startScale then
				LuaUtil.setObjScale(item.content, tweenData.startScale.x, tweenData.startScale.y, tweenData.startScale.z)
			end

			local delay = (i - 1) * tweenData.delayTime + tweenData.firstDelayTime
			local offect = tweenData.offect
			local targetPosX = item.x
			local targetPosY = item.y
			local targetRot = tweenData.targetRotTab
			local targetScale = tweenData.targetScaleTab
			item:setEnable(not tweenData.delayShow)

			if tweenData.bFade then
				TweenUtil.getInstance():addTweenTargetAlpha(item.content, tweenData.keepTime, delay, tweenData.fadeValue, tweenData.isFadeIn)
			end

			TweenUtil.getInstance():addTween(item.content, tweenData.keepTime, delay, 
				targetPosX, targetPosY, 0, 
				targetRot[1], targetRot[2], targetRot[3], 
				targetScale[1], targetScale[2], targetScale[3], 
				nil, nil, offect, tweenData.delayShow)

			if self.tweenTime < (tweenData.keepTime + delay) then
				self.tweenTime = tweenData.keepTime + delay
			end
		end

		self.bTweenTimer = true
		Lua_CommonUtil.delayCall(self.tweenTime, self, self.onTweenCompleted)
	end
end

function Lua_DragListContainer:moveToIndex(idx, immediately, isBottom)
	if self.cellsCount <= 0 then
		return
	end

	if not self.delayTween and self.bTweenTimer then
		self.moveIndex = idx
		self.immediately = immediately
		self.isBottom = isBottom
		return
	end

	immediately = immediately ~= nil and immediately or false
	isBottom = isBottom ~= nil and isBottom or false
	idx = idx <= 0 and 1 or idx
	idx = idx > self.cellsCount and self.cellsCount or idx

	local targetPos
	local vo
	local len = self.showDragDataList:getSize()
	local dataLen = 0
	if self._dragType == Lua_AxisDefine.Horizontal then
		targetPos = self.subMaxPox
		for i = 1, len do
			vo = self.showDragDataList[i]
			dataLen = dataLen + vo.dataLen
			if dataLen >= idx then
				if i > 1 then
					dataLen = dataLen - vo.dataLen
					idx = idx - dataLen
				end
				targetPos = targetPos - vo:getPosByIdx(idx)
				break
			else
				targetPos = targetPos - vo.showWidth
				if isBottom then
					targetPos = targetPos - vo.blockSizeX + self._rectWidth
				end
			end
		end
	else
		targetPos = self.subMinPos
		for i = 1, len do
			vo = self.showDragDataList[i]
			dataLen = dataLen + vo.dataLen
			if dataLen >= idx then
				if i > 1 then
					dataLen = dataLen - vo.dataLen
					idx = idx - dataLen
				end
				targetPos = targetPos + vo:getPosByIdx(idx)
				break
			else
				targetPos = targetPos + vo.showHight
				if isBottom then
					targetPos = targetPos + vo.blockSizeY - self._rectHeight
				end
			end
		end
	end

	if targetPos > self.subMaxPox then
		targetPos = self.subMaxPox
	elseif targetPos < self.subMinPos then
		targetPos = self.subMinPos
	end

	if immediately then
		self:setSubPosition(targetPos)
		self:updateDragItem()
		if self.delayTween then
			self:playTween()
		end
	else
		self:moveToTargetPos(targetPos)
	end
end

function Lua_DragListContainer:clearList()
	self:superCall("clearList")
	self.subRectW = 0
	self.subRectH = 0
	self.bUpdateSubSize = false
end