-- 界面多个模型场景展示
Lua_MultiModelScenePanel = class("Lua_MultiModelScenePanel");
function Lua_MultiModelScenePanel:ctor()
	self:superCall("ctor");
	self.bSceneShow = false;
	self.bSceneLoaded = false;
end

function Lua_MultiModelScenePanel:dispose()
	if self.showModelList ~= nil then
		for k, model in pairs (self.showModelList) do
			model:dispose();
			self.showModelList[k] = nil;
		end
		self.showModelList = nil;
	end

    if self.oldShowSceneId ~= nil and self.oldShowSceneId > 0 then
    	Scene3DManager.instance:destroyScene(self.oldShowSceneId, 0);
    end

    if self.curShowSceneId ~= nil and self.curShowSceneId > 0 then
    	Scene3DManager.instance:destroyScene(self.curShowSceneId, 0);
    end

    ModelCameraManager.instance:changeAniController(nil);
    self.camControllerName = nil;
	self.oldShowSceneId = nil;
	self.curShowSceneId = 0;
	self.sceneTab = nil;
	self.sceneShowFun = nil;
	self.sceneLoadedFun = nil;
	self:superCall("dispose");
end

-- sceneId 场景id
-- camControllerName 模型相机节点名
function Lua_MultiModelScenePanel:showModelScene(sceneId, camControllerName)
	if sceneId == nil or sceneId == 0 then
        return;
    end

    -- 场景
    self.camControllerName = camControllerName;
    if self.curShowSceneId ~= sceneId or not Scene3DManager.instance:isShowingScene(sceneId, 0) then
        self.bSceneShow = false;
        self.bSceneLoaded = false;
        local lastSceneId = self.curShowSceneId;
        self.curShowSceneId = sceneId;
        Scene3DManager.instance:OpenSceneBySceneId(sceneId, self, self.onShowScene, self.onSceneLoaded);
        if lastSceneId ~= nil and lastSceneId ~= sceneId then
	        self:destroyShowScene(lastSceneId);
	        if self.showModelList ~= nil then
				for k, model in pairs (self.showModelList) do
					model:setSceneInfo(sceneId, 0);
					model:setEnable(false);
				end
			end
	    end
    elseif self.bSceneShow then -- 已经显示完成
    	ModelCameraManager.instance:changeAniController(self.camControllerName); 
    end
end


-- index 模型索引位
-- modelUrl 模型url
-- avatarName 模型资源名
-- uiModelDir 模型朝向
-- uiModelScale 模型缩放值
-- pos 模型坐标
function Lua_MultiModelScenePanel:addSingleModel(index, modelUrl, avatarName, effects, dir, scale, pos, bPlanarShadow)
	if self.curShowSceneId == nil or self.curShowSceneId <= 0 then
		return;
	end
    
    if self.showModelList == nil then
    	self.showModelList = {};
    end
    local model = self.showModelList[index];
    if model == nil then
    	model = Lua_SingleModellItem.new(self.curShowSceneId, 0);
    	self.showModelList[index] = model;
    end
    model:setData(modelUrl, avatarName, effects, dir, scale, pos, bPlanarShadow);
    model:setEnable(self.bSceneShow);
end

function Lua_MultiModelScenePanel:onShowScene(sceneId)
    if sceneId == self.curShowSceneId then
    	if not strIsNullOrEmpty(self.camControllerName) then
	        local cfg = Lua_SceneConfigData.getSceneConfig(sceneId);
			if cfg ~= nil then
				local uiSceneCam = cfg.uiSceneCam;
				local px = (uiSceneCam ~= nil and #uiSceneCam > 0) and uiSceneCam[1] or 0;
				local py = (uiSceneCam ~= nil and #uiSceneCam > 1) and uiSceneCam[2] or 0;
				local pz = (uiSceneCam ~= nil and #uiSceneCam > 2) and uiSceneCam[3] or 0;
				local rx = (uiSceneCam ~= nil and #uiSceneCam > 3) and uiSceneCam[4] or 0;
				local ry = (uiSceneCam ~= nil and #uiSceneCam > 4) and uiSceneCam[5] or 180;
				local rz = (uiSceneCam ~= nil and #uiSceneCam > 5) and uiSceneCam[6] or 0;
				ModelCameraManager.instance:setCamInfo(px, py, pz, rx, ry, rz);
			end
		end
        self.bSceneShow = true;
        self:setModelEnable(true);
        self:destroyShowScene(self.oldShowSceneId);
        self.oldShowSceneId = self.curShowSceneId;
        ModelCameraManager.instance:changeAniController(self.camControllerName); 
        if self.sceneTab ~= nil and self.sceneShowFun ~= nil then
			self.sceneShowFun(self.sceneTab, sceneId);
		end
    end
end

function Lua_MultiModelScenePanel:onSceneLoaded(sceneId)
	if sceneId == self.curShowSceneId then
        self.bSceneLoaded = true;
        if self.sceneTab ~= nil and self.sceneLoadedFun ~= nil then
			self.sceneLoadedFun(self.sceneTab, sceneId);
		end
    end
end

function Lua_MultiModelScenePanel:addSceneCallBack(tab, showFun, loadedFun)
	if tab == nil then
		return;
	end

	if showFun == nil and loadedFun == nil then
		return;
	end

	self.sceneTab = tab;
	self.sceneShowFun = showFun;
	self.sceneLoadedFun = loadedFun;
	if self.bSceneShow and self.sceneShowFun then
		self.sceneShowFun(self.sceneTab, sceneId);
	end
	if self.bSceneLoaded and self.sceneLoadedFun then
		self.sceneLoadedFun(self.sceneTab, sceneId);
	end
end

function Lua_MultiModelScenePanel:destroyShowScene(sceneId)
	if sceneId ~= nil and sceneId > 0 then
		local cfg = Lua_SceneConfigData.getSceneConfig(sceneId);
		if cfg ~= nil and cfg.bControllDispose then
			Scene3DManager.instance:destroyScene(sceneId, 0);
		end
	end
end

function Lua_MultiModelScenePanel:setModelEnable(value)
	if self.showModelList ~= nil then
	    for _, model in pairs (self.showModelList) do
			model:setEnable(value);
		end
	end
end





















Lua_SingleModellItem = class("Lua_SingleModellItem")
function Lua_SingleModellItem:ctor(sceneId, fightId)
	self.showModel = ModelEffComponent(sceneId, fightId)
	self.showModel:setLuaCallBack(self, self.onLoadCompleted, self.onAniCompleted)
end

function Lua_SingleModellItem:dispose()
	if self.showModel then
		self.showModel:clearLoad()
		self.showModel:dispose()
		self.showModel = nil
	end
	self.effects = nil
end

function Lua_SingleModellItem:setSceneInfo(sceneId, fightId)
	self.showModel:setSceneInfo(sceneId, fightId)
end

function Lua_SingleModellItem:setData(modelUrl, avatarName, effects, dir, scale, pos, bPlanarShadow)
	if strIsNullOrEmpty(modelUrl) or strIsNullOrEmpty(avatarName) then
		return
	end

	self.bLoaded = false;
	self.effects = effects;
	if bPlanarShadow == nil then bPlanarShadow = false; end
	self.showModel.bPlanarShadow = bPlanarShadow;
	self.showModel:setUrl(modelUrl, avatarName)
    self.showModel:setRot(dir or 0)
    self.showModel:setScale(scale or 1)
    local px = (pos and #pos > 0) and pos[1] or 0
    local py = (pos and #pos > 1) and pos[2] or 0
    local pz = (pos and #pos > 2) and pos[3] or 0
    self.showModel:setLocation(px, py, pz)
end

function Lua_SingleModellItem:setEnable(bValue)
	self.showModel:setEnable(bValue)
end

function Lua_SingleModellItem:onLoadCompleted()
	self.bLoaded = true
	-- self.showModel:setAction(Lua_ThingDefine.UIWAIT)
	if self.effects then
		if self.effects.typeof ~= nil and self.effects:typeof() == "Array" then
			for i = 1, self.effects:getSize() do
				self.showModel:addHangerEff(self.effects[i])
			end
		else
			for i = 1, #self.effects do
				self.showModel:addHangerEff(self.effects[i])
			end
		end
	end
end

function Lua_SingleModellItem:onAniCompleted()

end