Lua_NumberImage = class("Lua_NumberImage", Lua_DisplayObject);
function Lua_NumberImage:ctor(p)
	self:superCall("ctor", p);
	self.bPreLoad = false;
end

--[[
	self.numberImage = Lua_NumberImage.new(self.content);
	self.numberImage:setLocation(0, 0, 0);
	self.numberImage.pivot = 0;
	self.numberImage.numAtlas = "number"
	self.numberImage.firstStr = "Architecture_fonts_bei";
	self.numberImage.lastStr = "Architecture_fonts_bei";
	self.numberImage.preStr = "Architecture_fonts_";
	self.numberImage.num = "x1423";
]]

function Lua_NumberImage:buildShowItem()
	self:superCall("buildShowItem");

	self.pivot = 0;
	self.scaleVal = 1;
	self.numAtlas = "number";
	self.firstStr = "";
	self.lastStr = "";
	self.preStr = "";
	self.numOffsetX = 0;
	self.numOffsetY = 0;
	self.totalWidth = 0;
	self.totalHeight = 0;
	self.rowLimit = nil;
	self.showArr = Array();
	self.hideArr = Array();
end

function Lua_NumberImage:dispose()
	if self.bPreLoad == true then
		self.bPreLoad = false;
		DataCenter.preLoadData:removeBySign("Lua_NumberImage", self, self.onLoaded);
	end

	if self.showArr ~= nil then
		self.showArr:dispose();
		self.showArr:clear();
		self.showArr = nil;
	end

	if self.hideArr ~= nil then
		self.hideArr:dispose();
		self.hideArr:clear();
		self.hideArr = nil;
	end

	self:superCall("dispose");
end

-- 1:从左往右 0:居中 -1:从右往左
function Lua_NumberImage.Set:pivot(val)
	self._pivot = val;
end

-- function Lua_NumberImage.Get:pivot()
-- 	return self._pivot;
-- end

function Lua_NumberImage.Set:numAtlas(param)
	if self._numAtlas ~= param then
		self._numAtlas = param;
		if self.bPreLoad then -- 预加载中
			self.bPreLoad = false;
			DataCenter.preLoadData:removeBySign("Lua_NumberImage", self, self.onLoaded);
		end
	end
end

-- function Lua_NumberImage.Get:numAtlas()
-- 	return self._numAtlas;
-- end

-- param:整型或字符串
function Lua_NumberImage.Set:num(param)
	self.totalWidth = 0;
	self.totalHeight = 0;
	self:returnIcon();
	if param == nil or strIsNullOrEmpty(self._numAtlas) or strIsNullOrEmpty(self.preStr) then
		return;
	end

	self._num = tostring(param);

	if DataCenter.recordDic == nil then
		DataCenter.recordDic = {};
	end
	local url = DataCenter.recordDic[self._numAtlas];
	if url == nil then
		url = LanUtil.StringProfile("/uiatlas/", UrlManager.fixFileSuffix(self._numAtlas));
		DataCenter.recordDic[self._numAtlas] = url;
	end

	if self.bPreLoad ~= true and not LoadManager.getInstance():hasResource(url) then
		self.bPreLoad = true;
		local vo = Lua_PreLoadResInfo.new();
		vo.resType = Lua_LoadFileType.ATLAS;
		vo.resName = self._numAtlas;

		local loadList = Array();
		loadList:add(vo);

		DataCenter.preLoadData:preLoad("Lua_NumberImage", Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onLoaded, nil, loadList, nil);
	elseif not self.bPreLoad then
		self:updateShow();
	end
end

function Lua_NumberImage.Get:num()
	return self._num;
end

function Lua_NumberImage:onLoaded()
	self.bPreLoad = false;
	self:updateShow();
end

function Lua_NumberImage:updateShow()
	local len = string.len(self.num);
	local icon;
	local str;

	if not strIsNullOrEmpty(self.firstStr) then
		icon = self:addIcon();
		icon:setAltas(self._numAtlas, self.firstStr);
		icon:resetSize();
	end

	for i = 1, len do
		icon = self:addIcon();
		str = string.sub(self.num, i, i);
		if str == "." then -- .
			str = "10";
		elseif str == "-" then -- -
			str = "11";
		elseif str == "+" then -- +
			str = "12";
		elseif str == "w" then -- 万
			str = "13";
		elseif str == "y" then -- 亿
			str = "14";
		elseif str == "z" then -- 兆
			str = "15";
		elseif str == "j" then -- 京
			str = "16";
		end
		icon:setAltas(self._numAtlas, self.preStr .. str);
		icon:resetSize();
	end

	if not strIsNullOrEmpty(self.lastStr) then
		icon = self:addIcon();
		icon:setAltas(self._numAtlas, self.lastStr);
		icon:resetSize();
	end
	
	local row = 1;
	local rowLimit = self:getRowLimit(row);
	len = self.showArr:getSize();
	local width = self:getWidth(1, rowLimit, len);
	local height = self:getHeight(len);
	if self.totalWidth < width then self.totalWidth = width; end
	self.totalHeight = height;

	local idx = 1;
	local posX = self:getStartX(width);
	local posY = self:getStartY(height);
	local offectX = 0;
	if len > 1 then
		offectX = self.numOffsetX;
	end
	for i = 1, len do
		icon = self.showArr[i];
		if idx > rowLimit then
			idx = 1;

			row = row + 1;
			rowLimit = self:getRowLimit(row);

			width = self:getWidth(i, rowLimit, len);
			if self.totalWidth < width then self.totalWidth = width; end

			posX = self:getStartX(width);
			posY = posY - icon.height * self.scaleVal + self.numOffsetY;
		end
		icon:setLocation(posX, posY, 0);
		posX = posX + icon.width * self.scaleVal + offectX;
		idx = idx + 1;
	end
end

-- 默认9999
function Lua_NumberImage:getRowLimit(row)
	if self.rowLimit == nil or self.rowLimit[row] == nil then
		return 9999;
	end
	return self.rowLimit[row];
end

function Lua_NumberImage:getWidth(startIdx, rowLimit, maxLen)
	local len = startIdx + rowLimit - 1;
	len = len > maxLen and maxLen or len;
	local icon;
	local offect = 0;
	if len > 1 then
		if self._pivot == -1 then
			offect = self.numOffsetX;
		else
			offect = self.numOffsetX / 2;
		end
	end
	local width = 0;
	for i = startIdx, len do
		icon = self.showArr[i];
		width = width + icon.width * self.scaleVal + offect;
	end
	return width;
end

function Lua_NumberImage:getHeight(maxLen)
	local idx = 1;
	local row = 1;
	local rowLimit = self:getRowLimit(row);
	local height = 0;
	for i = 1, maxLen do
		icon = self.showArr[i];
		if idx > rowLimit then
			idx = 1;
			row = row + 1;
			rowLimit = self:getRowLimit(row);
			height = height + icon.height * self.scaleVal - self.numOffsetY;
		end
		idx = idx + 1;
	end
	return height;
end

function Lua_NumberImage:getStartX(width)
	if self._pivot == 1 then
		return 0;
	elseif self._pivot == -1 then
		return -width;
	else
		return -width / 2;
	end
end

function Lua_NumberImage:getStartY(height)
	if self._pivot == 1 then
		return 0;
	elseif self._pivot == -1 then
		return -height;
	else
		return height / 2;
	end
end

function Lua_NumberImage:setAlpha(alpha)
	local len = self.showArr:getSize();
	for i = 1, len do
		self.showArr[i].alpha = alpha;
	end
end

function Lua_NumberImage:setGray(bGray)
    local len = self.showArr:getSize();
    for i = 1, len do
        self.showArr[i]:setGray(bGray);
    end
end

function Lua_NumberImage:setEnable(bool)
    local len = self.showArr:getSize();
    for i = 1, len do
        self.showArr[i]:setEnable(bool);
    end
end

function Lua_NumberImage:getRenderObj()
	if self.showArr:getSize() > 0 then
		return self.showArr[1];
	end
	return nil;
end

function Lua_NumberImage:addIcon()
	if self.content == nil or self.isDispose_3 == true then
		return;
	end

	if self.showArr == nil then
		self.showArr = Array();
	end
	if self.hideArr == nil then
		self.hideArr = Array();
	end

    local len = self.hideArr:getSize();
    local icon = nil;
    if len > 0 then
        icon = self.hideArr:remove(1);
        icon:setEnable(true);
    end

	if icon == nil then
		icon = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
	end
	icon.pivotValue = 3;
    icon:setScale(self.scaleVal, self.scaleVal, 1);
    self.showArr:add(icon);
    return icon;
end

function Lua_NumberImage:returnIcon()
	if self.showArr == nil or self.hideArr == nil then
		return;
	end

    local len = self.showArr:getSize();
    local icon;
    while len > 0 do
        icon = self.showArr:remove(len);
        icon:setEnable(false);
        self.hideArr:add(icon)
        len = len - 1;
    end
end