-- 数字键盘输入
Lua_NumericKeyboardInputItem = class("Lua_NumericKeyboardInputItem", Lua_DisplayObject)
function Lua_NumericKeyboardInputItem:ctor(p)
	self:superCall("ctor", p, "NumericKeyboardInputItem_skin");
end
function Lua_NumericKeyboardInputItem:buildShowItem()
	self:superCall("buildShowItem");

	self:setValue(0, 0, 100);
	self.btnLess:addMousDownFuntion(self, self.onLessBtnClick);
	self.btnLess:addMouseUpFuntion(self, self.onBtnUpClick);
	self.btnPlus:addMousDownFuntion(self, self.onPlusBtnClick);
	self.btnPlus:addMouseUpFuntion(self, self.onBtnUpClick);
	self.btnMin:addClickFuntion(self, self.onMinBtnClick);
	self.btnMax:addClickFuntion(self, self.onMaxBtnClick);
end

function Lua_NumericKeyboardInputItem:dispose()
	if self.bTimer == true then
		self.bTimer = false;
		self:removeTimer(self, self.onLongPress);
	end
	self.minTips = nil;
	self.maxTips = nil;
	self.owner = nil;
	self.callBack = nil;
	self:superCall("dispose");
end

function Lua_NumericKeyboardInputItem:setNumInput(offsetX, offsetY)
	self.offsetX = offsetX == nil and 0 or offsetX;
	self.offsetY = offsetY == nil and 0 or offsetY;
	self.countImg.clickScale = false;
	self.countImg:addClickFuntion(self, self.onShowInputWindow);
end

function Lua_NumericKeyboardInputItem:setTips(minTips, maxTips)
	self.minTips = minTips;
	self.maxTips = maxTips;
end

function Lua_NumericKeyboardInputItem:setCallBack(owner, callBack)
	self.owner = owner;
	self.callBack = callBack;
end

function Lua_NumericKeyboardInputItem:setMinMaxEnable(minShow, maxShow)
	self.btnMin:setEnable(minShow);
	self.btnMax:setEnable(maxShow);
end

function Lua_NumericKeyboardInputItem:setValue(curNum, minNum, maxNum)
	self.curNum = curNum;
	self.minNum = minNum;
	self.maxNum = maxNum;
	self:updateNum();
end

function Lua_NumericKeyboardInputItem:onLessBtnClick(go)
	self.bLess = true;
	self:startTimer();
end

function Lua_NumericKeyboardInputItem:onPlusBtnClick(go)
	self.bLess = false;
	self:startTimer();
end

function Lua_NumericKeyboardInputItem:onBtnUpClick(go)
	self:cleanTimer();
end

function Lua_NumericKeyboardInputItem:startTimer()
	if self.bTimer ~= true then
		self.bTimer = true;
		self.clickTime = getTime();
		self:addTimer("Lua_NumericKeyboardInputItem", 0.01, self, self.onLongPress, nil, -1);
	end
end

function Lua_NumericKeyboardInputItem:cleanTimer()
	if self.bTimer == true then
		self.bTimer = false;
		self:removeTimer(self, self.onLongPress);
		if not self.bLongPress then
			self:onLessPlusNum();
		end
	end
end

function Lua_NumericKeyboardInputItem:onLongPress()
	if getTime() - self.clickTime > 0.3 then
		self.bLongPress = true;
		self:onLessPlusNum();
	else
		self.bLongPress = false;
	end
end

function Lua_NumericKeyboardInputItem:onLessPlusNum()
	if self.bLess == true then
		self.curNum = self.curNum - 1;
	else
		self.curNum = self.curNum + 1;
	end
	if self.curNum < self.minNum or self.curNum > self.maxNum then
		self:cleanTimer();
	end
	self:updateNum();
end

function Lua_NumericKeyboardInputItem:onMinBtnClick(go)
	self.curNum = self.minNum
	self:updateNum();
end

function Lua_NumericKeyboardInputItem:onMaxBtnClick(go)
	self.curNum = self.maxNum;
	self:updateNum();
end

function Lua_NumericKeyboardInputItem:updateNum()
	if self.curNum < self.minNum then
		self.curNum = self.minNum;
		if self.minTips ~= nil and self.minTips ~= "" then
			OperateAlert.show(self.minTips);
		end
	elseif self.curNum > self.maxNum then
		self.curNum = self.maxNum;
		if self.maxTips ~= nil and self.maxTips ~= "" then
			OperateAlert.show(self.maxTips);
		end
	end
	self.countText.text = tostring(self.curNum);

	if self.owner ~= nil and self.callBack ~= nil then
		self.callBack(self.owner, self.curNum);
	end
end

function Lua_NumericKeyboardInputItem:onShowInputWindow(go)
	local data = { posX = self.offsetX, posY = self.offsetY,
		curNum = self.curNum, minNum = self.minNum, maxNum = self.maxNum,
		owner = self, callBack = self.onInputClick
	 };
	Lua_WindowUtil.openWin("Lua_NumericKeyboardWindow", Lua_WinParam.ACTION_AUTO, data);
end

function Lua_NumericKeyboardInputItem:onInputClick(num)
	self.curNum = num;
	self.countText.text = tostring(self.curNum);
	if self.owner ~= nil and self.callBack ~= nil then
		self.callBack(self.owner, self.curNum);
	end
end