-- 红点显示图标
Lua_RedPointIcon = class("Lua_RedPointIcon", Lua_DisplayObject);
function Lua_RedPointIcon:ctor(p, id1, id2)
	self.idList = Array();
	self.idList:add(id1);
	self.idList:add(id2);
	self:superCall("ctor", p);
end

function Lua_RedPointIcon:buildShowItem()
	self:superCall("buildShowItem");
	self.showObject = UICreate.createGameObject(self.content, 0, 0, 0);
	self:initData();
end

function Lua_RedPointIcon:dispose()
	if self.eff ~= nil then
		self.eff:dispose();
		self.eff = nil;
	end
	if self.redImg ~= nil then
		self.redImg:dispose();
		self.redImg = nil;
	end
	if self.numTxt ~= nil then
		self.numTxt:dispose();
		self.numTxt = nil;
	end
	if self.showObject ~= nil and IsNil(self.showObject) == false then
		UnityEngine.GameObject.Destroy(self.showObject);
		self.showObject = nil;
	end
	if self.idList ~= nil then
		self.idList:clear();
		self.idList = nil;
	end
	self:superCall("dispose");
end

function Lua_RedPointIcon:initData()
	local len = self.idList:getSize();
	if len > 0 then
		self.content.name = "redPoint_"..self.idList[1];
		self:addLuaEvent("RedPoint_Update_State", self, self.onUpdateRedPointState);
		self:onUpdateRedPointState(self.idList[1]);
	else
		self.content.name = self.classname;
		self:removeLuaEvent("RedPoint_Update_State", self, self.onUpdateRedPointState);
		self.showObject:SetActive(false);
	end
end

function Lua_RedPointIcon:onUpdateRedPointState(id)
	if self.idList:indexOf(id) ~= -1 then
		local isActive = false;
		local showNum = 0;
		local vo;
		for i = 1, self.idList:getSize() do
			vo = DataCenter.redPointData:getRedPointVoById(self.idList[i]);
			if vo ~= nil and vo.showState then
				isActive = true;
				showNum = showNum + vo.showNum;
			end
		end
		self.showObject:SetActive(isActive);
		if isActive then
			self:initUI(vo);
		end
		if self.numTxt ~= nil then
			self.numTxt.text = tostring(showNum);
		end
	end
end

function Lua_RedPointIcon:initUI(vo)
	if self.bInitUI or vo == nil then return end
	self.bInitUI = true;

	local atlas = vo.atlas;
	local spriteName = vo.spriteName;
	local effName = vo.effName;
	local bShowNum = vo.bShowNum;

	if not strIsNullOrEmpty(atlas) and not strIsNullOrEmpty(spriteName) then
		if self.redImg == nil then
			self.redImg = UICreate.createUIAtlasImage(self.showObject, nil, nil, 0, 0, 0, 0);
		end
		self.redImg:setEnable(true);
		self.redImg:setAltas(atlas, spriteName);
	else
		if self.redImg ~= nil then
			self.redImg:setEnable(false);
		end
	end

	if not strIsNullOrEmpty(effName) then
		if self.eff == nil then
			self.eff = UICreate.createModelEffImg(self.showObject, effName, 0, 0, 0, 0);
		end
		self.eff:setEnable(true);
	else
		if self.eff ~= nil then
			self.eff:setEnable(false);
		end
	end

	if bShowNum and self.numTxt == nil then
		self.numTxt = UICreate.createLabel(self.showObject, nil, 0, 0, 100, 31, 22);
		self.numTxt.pivot = Lua_WindowUtil.getPivot(4);
		self.numTxt.alignmentValue = 4;
		self.numTxt.horizontalOverflowValue = 1;
	end

	if self.numTxt ~= nil then
		self.numTxt.text = "";
	end
end

-- 点击处理
function Lua_RedPointIcon:clickHandler()
	local len = self.idList:getSize();
	if len > 0 then
		local vo;
		for i = 1, len do
			vo = DataCenter.redPointData:getRedPointVoById(self.idList[i]);
			if vo.clickCancle then
				DataCenter.redPointData:updateRedPointState(vo.id, false, true);
			end
		end
	end
end

function Lua_RedPointIcon:changeRedPoint(id1, id2)
	if (id1 == nil and id2 == nil) or (id1 ~= nil and self.idList:indexOf(id1) == -1) or (id2 ~= nil and self.idList:indexOf(id2) == -1) then
		self.idList:clear();
		self.idList:add(id1);
		self.idList:add(id2);
		self.bInitUI = false;
		self:initData();
	end
end


------------------------------------------无配置红点------------------------------------------

--atlas或者spriteName传值为空就使用默认红点图片
function Lua_RedPointIcon:createRedPointImg(atlas, spriteName)-- 图片
	atlas = strIsNullOrEmpty(atlas) and "common_altas_1" or atlas;
	spriteName = strIsNullOrEmpty(spriteName) and "xui_common_label_01" or spriteName;
	if self.redImg == nil then
		self.redImg = UICreate.createUIAtlasImage(self.showObject, atlas, spriteName, 0, 0, 0, 0);
	else
		self.redImg:setAltas(atlas, spriteName);
	end
end

--effUrl传值为空就使用默认红点特效
function Lua_RedPointIcon:createRedPointEff(effName) -- 特效
	self:createRedPointImg();
	-- if strIsNullOrEmpty(effName) == nil then
	-- 	return
	-- end
	-- if self.eff == nil then
	-- 	self.eff = UICreate.createModelEffImg(self.showObject, effName, 0, 0, 0, 0);
	-- else
	-- 	self.eff:setUrl("/effects/uiEffects/"..effName, effName);
	-- end
end

--更新无配置红点显示
function Lua_RedPointIcon:updateRedPointShow(state, showCount)
	if state == true and showCount ~= nil then
		if self.numTxt == nil then
			self.numTxt = UICreate.createLabel(self.showObject, nil, 0, 0, 100, 31, 22);
			self.numTxt.pivot = Lua_WindowUtil.getPivot(4);
			self.numTxt.alignmentValue = 4;
			self.numTxt.horizontalOverflowValue = 1;
		end
		self.numTxt:setEnable(true);
		self.numTxt.text = showCount;
	else
		if self.numTxt ~= nil then
			self.numTxt:setEnable(false);
		end
	end
	self.showObject:SetActive(state);
end