--[[
self.scrollBoard = Lua_ScrollBoard.new(self.content, Lua_AxisDefine.Horizontal, 500, 200);
self.scrollBoard:setListCell(Lua_ScrollBoardItem, 100, 100); 
self.scrollBoard:setLocation(0, -200, 0);
self.scrollBoard:setChangeScale(1, 0.1, 0.001);

local arr = Array();
for i=1,10 do
    arr:add(i)
end
self.scrollBoard:setData(arr);





Lua_ScrollBoardItem = class("Lua_ScrollBoardItem", Lua_ScrollBoardCell);
function Lua_ScrollBoardItem:ctor(p)
    self:superCall("ctor", p);
end

function Lua_ScrollBoardItem:buildShowItem()
    self:superCall("buildShowItem");

    self.banner = UICreate.createUIAtlasImage(self.content, "iconantlas1", "tips_bg", 0, 0, 100, 100);

    self.text = UICreate.createLabel(self.content, "", 0, 0, 100, 30, 22);
    self.text.pivot = Lua_WindowUtil.getPivot(4);
    self.text.alignmentValue = 4;
end

function Lua_ScrollBoardItem:dispose()
    self:superCall("dispose");
end

function Lua_ScrollBoardItem:setData(data)
    self.text.text = data;
end
--]]

checkRequire("Lua_ScrollBoardCell");
Lua_ScrollBoard = class("Lua_ScrollBoard", Lua_DisplayObject)
function Lua_ScrollBoard:ctor(p, ltype, rectW, rectH)
    self.ltype = ltype;
    self.vsize = Vector4(0, 0, rectW, rectH);
    self.viewRect = Lua_Rectangle.new(-rectW / 2, -rectH / 2, rectW / 2, rectH / 2);
    self.startMouse = -1;
    self.selectIdx = 1;
    self.showList = Array();
    self.hideList = Array();
    self.canDrag = true;
    self.subPos = 0;
    self.cellHeight = 0;
    self.cellWidth = 0;
    self.bRefresh = false;
    self.backSpeed = 3;
    self.moveSpd = 1300;
    self._isClick = false;
    self.cellChangePos = 0;
    self.cellStartScale = 1;
    self.cellChangeScale = 0;
    self.cellScaleSpd = 0;
    self:superCall("ctor", p);
end

function Lua_ScrollBoard:buildShowItem()
    self:superCall("buildShowItem");
    self:setRectSize(self.vsize.z, self.vsize.w);

    self.mask = UICreate.createUIMaskContainer(self.content, nil, nil, 0, 0, 0, 0);
    self.mask.pivot = Lua_WindowUtil.getPivot(4);
    self.mask:setAnchorMin(0, 0);
    self.mask:setAnchorMax(1, 1);
    self.mask:SetDimensions(0, 0);
    self.mask:setSubRectPivot(0.5, 0.5);
    self.mask:setColor(255, 255, 255, 1);
    self.mask.clickScale = false;
    self.mask:addMousDownFuntion(self, self.onMouseDownHandle);
    self.mask:addMouseUpFuntion(self, self.onMouseUpHandle);
    self.mask:addMask();
end

function Lua_ScrollBoard:setRect(rectW, rectH)
    self.vsize = Vector4(0, 0, rectW, rectH);
    self:setRectSize(self.vsize.z, self.vsize.w);
end

function Lua_ScrollBoard:setListCell(cell, cellWidth, cellHeight)
    self.cell = cell;
    self.cellWidth = cellWidth;
    self.cellHeight = cellHeight;
end

function Lua_ScrollBoard:setChangePos(changePos)
    self.cellChangePos = changePos;
end

function Lua_ScrollBoard:setChangeScale(startScale, changeScale, changeSpd)
    self.cellStartScale = startScale;
    self.cellChangeScale = changeScale;
    self.cellScaleSpd = Mathf.Abs(changeSpd);
end

function Lua_ScrollBoard:setLuaArrayData(data)
    if data == nil then
        return;
    end
    self.data = data;
    
    self:clearScrollBoardTimer();
    self:initItemShowPos();
end

function Lua_ScrollBoard:moveToIndex(val, immediately, isMoveRight)
    if self.data == nil or self.data:getSize() == 0 then
        return;
    end

    if val <= 0 then
        val = 1;
    end

    if val > self.data:getSize() then
        val = self.data:getSize();
    end
    
    if immediately == true then
        self.selectIdx = val;
        self:clearScrollBoardTimer();
        self:initItemShowPos();
    else
        local item = self:getMidItem();
        if item ~= nil then
            local idx = item.showIndex;
            self.moveDistance = 0;
            while (idx ~= val) do
                if isMoveRight then
                    idx = idx - 1;
                else
                    idx = idx + 1;
                end

                if self.ltype == Lua_AxisDefine.Horizontal then
                    if self.moveDistance == 0 then
                        self.moveDistance = isMoveRight and (self.cellWidth - item.itemX) or (self.cellWidth + item.itemX);
                    else
                        self.moveDistance = self.moveDistance + self.cellWidth;
                    end
                else
                    if self.moveDistance == 0 then
                        self.moveDistance = isMoveRight and (self.cellHeight + item.itemY) or (self.cellHeight - item.itemY);
                    else
                        self.moveDistance = self.moveDistance + self.cellHeight;
                    end
                end

                if idx < 1 then
                    idx = self.data:getSize();
                elseif idx > self.data:getSize() then
                    idx = 1;
                end
            end
            self:updateItemPos(val, isMoveRight);
        end
    end
end

function Lua_ScrollBoard:onMouseDownHandle(go, state)
    self:onPress(go, true);
end

function Lua_ScrollBoard:onMouseUpHandle(go, state)
    self:onPress(go, false);
end

function Lua_ScrollBoard:onPress(go, state)
    self:clearScrollBoardTimer();
    if self.data == nil or self.data:getSize() <= 0 or self.canDrag == false then
        return;
    end

    if state == true then       
        if self.ltype == Lua_AxisDefine.Horizontal then
            self.startMouse = CommonUtil.mousePositionX;
        else
            self.startMouse = CommonUtil.mousePositionY;
        end
    else
    	local isAdd = true;
    	local moveIdx = 0;
        if self.ltype == Lua_AxisDefine.Horizontal then
            local moveDis = CommonUtil.mousePositionX - self.startMouse;
            if Mathf.Abs(moveDis) < 100 then -- 判断未移动，检测是否点击非选中按钮
                local winPos = LuaUtil.getTouchPosInverseObj(self.content);
            	local item = self:checkClick(winPos);
                if item ~= nil then
                    moveIdx = item.showIndex;
                	isAdd = winPos.x < self.cellWidth / 2;
                    self.moveDistance = Mathf.Abs(item.itemX);
                end
            else
                local item = self:getMidItem();
                if item ~= nil then
                    if moveDis > 0 then
                        isAdd = true;
                        moveIdx = item.showIndex - 1;
                        self.moveDistance = self.cellWidth - item.itemX;
                    else
                        isAdd = false;
                        moveIdx = item.showIndex + 1;
                        self.moveDistance = self.cellWidth + item.itemX;
                    end
                	
                	if moveIdx < 1 then
                		moveIdx = self.data:getSize();
                	elseif moveIdx > self.data:getSize() then
                		moveIdx = 1;
                	end
                end
            end
        else
            local moveDis = CommonUtil.mousePositionY - self.startMouse;
            if Mathf.Abs(moveDis) < 100 then -- 判断未移动，检测是否点击非选中按钮
                local winPos = LuaUtil.getTouchPosInverseObj(self.content);
                local item = self:checkClick(winPos);
                if item ~= nil then
                    moveIdx = item.showIndex;
                    isAdd = winPos.y < -self.cellHeight / 2;
                    self.moveDistance = Mathf.Abs(item.itemY);
                end
            else
                local item = self:getMidItem();
                if item ~= nil then
                    if moveDis > 0 then
                        isAdd = true;
                        moveIdx = item.showIndex - 1;
                        self.moveDistance = self.cellHeight - item.itemY;
                    else
                        isAdd = false;
                        moveIdx = item.showIndex + 1;
                        self.moveDistance = self.cellHeight + item.itemY;
                    end
                    
                    if moveIdx < 1 then
                        moveIdx = self.data:getSize();
                    elseif moveIdx > self.data:getSize() then
                        moveIdx = 1;
                    end
                end
            end
        end

        if moveIdx > 0 then -- 移动
            self:updateItemPos(moveIdx, isAdd);
        end
    end
end

function Lua_ScrollBoard:getMidItem()
    local len = self.showList:getSize();
    for i = 1, len do
         if self.showList[i]:isBounds(0, 0) == true then
            return self.showList[i];
        end
    end
    return nil;
end

function Lua_ScrollBoard:checkClick(winPos)
    local len = self.showList:getSize();
    for i = 1, len do
         if self.showList[i]:isBounds(winPos.x, winPos.y) == true then
           	return self.showList[i];
        end
    end
    return nil;
end

function Lua_ScrollBoard:updateItemPos(val, isMoveRight)
    if self.data == nil or self.data:getSize() == 0 then
        return;
    end

    self.selectIdx = val;
    self.isMoveRight = isMoveRight;
    self.startMoveTime = getTime();
    self:addScrollBoardTimer();
end

function Lua_ScrollBoard:addScrollBoardTimer()
	if self._isClick == false then
        self._isClick = true;
        Lua_CommonUtil.addTimer("onScrollEnterFrame", 0.01, self, self.onScrollEnterFrame);
    end
end

function Lua_ScrollBoard:clearScrollBoardTimer()
	if self._isClick == true then
        self._isClick = false;
        Lua_CommonUtil.removeTimer(self,self.onScrollEnterFrame);
    end
end

function Lua_ScrollBoard:onScrollEnterFrame()
	local t = getTime();
	local dt = t - self.startMoveTime;
	self.startMoveTime = t;
    local moveDis = self.moveSpd * dt;
    self.moveDistance = self.moveDistance - moveDis;
    if self.moveDistance < 0 then
        self:clearScrollBoardTimer();
        moveDis = moveDis + self.moveDistance;
        self.moveDistance = 0;
    end
	moveDis = moveDis * (self.isMoveRight and 1 or -1);

    local len = self.showList:getSize();
	local item;
    local step = 1;
    while step <= len do
        item = self.showList[step];
        if self.ltype == Lua_AxisDefine.Horizontal then
            item:setLocationOffset(moveDis, 0, 0);
        else
            item:setLocationOffset(0, moveDis, 0);
        end

        -- 检测是否超出界限
        if self:checkInBound(item) ~= true then
            self:returnCellToPool(item);
            self.showList:remove(step);
            len = len - 1;
        else
            step = step + 1;
        end
    end

	len = self.showList:getSize();
    if len <= 0 then
        return;
    end

    local halfNum = Mathf.Ceil(len / 2);
    local num;
    for i = 1, len do
        item = self.showList[i];
        num = Mathf.Abs(i - halfNum);
        self:checkScale(item, num);
        self:checkPosition(item, num);
    end

	if self.isMoveRight then
		item = self.showList[1];
		if self.ltype == Lua_AxisDefine.Horizontal then
			local itemX = item.itemX;
			while (itemX - self.cellWidth / 2) > (-self.vsize.z / 2) do
                local val = item.showIndex - 1;
                if val < 1 then
                    val = self.data:getSize();
                end
                itemX = itemX - self.cellWidth;
                item = self:getCellFromPool(item.showIndex);
                item:setLocation(itemX, 0, 0);
                item:setData(self.data[val]);
                item.showIndex = val;
                self:checkPosition(item, 0);
                self:checkScale(item, 0);
                self.showList:add(item, 1);
            end
		else
			local itemY = item.itemY;
			while (itemY - self.cellHeight / 2) > (-self.vsize.w / 2) do
                local val = item.showIndex - 1;
                if val < 1 then
                    val = self.data:getSize();
                end
                itemY = itemY - self.cellHeight;
                item = self:getCellFromPool();
                item:setLocation(0, itemY, 0);
                item:setData(self.data[val]);
                item.showIndex = val;
                self:checkPosition(item, 0);
                self:checkScale(item, 0);
                self.showList:add(item, 1);
            end
		end
	else
		item = self.showList[len];
		if self.ltype == Lua_AxisDefine.Horizontal then
			local itemX = item.itemX;
			while (itemX + self.cellWidth / 2) < (self.vsize.z / 2) do
                local val = item.showIndex + 1;
                if val > self.data:getSize() then
                    val = 1;
                end
                itemX = itemX + self.cellWidth;
                item = self:getCellFromPool(item.showIndex);
                item:setLocation(itemX, 0, 0);
                item:setData(self.data[val]);
                item.showIndex = val;
                self:checkPosition(item, 0);
                self:checkScale(item, 0);
                self.showList:add(item);
            end
		else
			local itemY = item.itemY;
			while (itemY + self.cellHeight / 2) < (self.vsize.w / 2) do
                local val = item.showIndex + 1;
                if val > self.data:getSize() then
                    val = 1;
                end
                itemY = itemY + self.cellHeight;
                item = self:getCellFromPool();
                item:setLocation(0, itemY, 0);
                item:setData(self.data[val]);
                item.showIndex = val;
                self:checkPosition(item, 0);
                self:checkScale(item, 0);
                self.showList:add(item);
            end
		end
	end

    -- 检测完成动画
    if self.moveDistance == 0 then
        self:checkCompleteSelect();
    end
end

function Lua_ScrollBoard:checkCompleteSelect()
    local len = self.showList:getSize();
    for i = 1, len do
         if self.showList[i].showIndex == self.selectIdx then
            self.showList[i]:onPressClick(self.selectIdx);
            break;
        end
    end
end

function Lua_ScrollBoard:checkInBound(item)
	if item ~= nil and self.viewRect ~= nil then
        if self.ltype == Lua_AxisDefine.Horizontal then
            if (item.minX >= self.viewRect.width) or (item.maxX <= self.viewRect.x) then
                return false;
            end
        else
            if (item.maxY <= self.viewRect.y) or (item.minY >= self.viewRect.height) then
                return false;
            end
        end
	end
	return true
end

function Lua_ScrollBoard:getCellFromPool()
	local item;
	if self.hideList:getSize() > 0 then
		item = self.hideList:remove(1);
		if item ~= nil then
			item:setEnable(true);
		end
	end

	if item == nil then
		item = self.cell.new(self.mask.sub);
		item:setBoxSize(self.cellWidth, self.cellHeight);
	end
	return item;
end

function Lua_ScrollBoard:returnCellToPool(item)
	if item == nil then
		return;
	end

    TweenUtil.getInstance():stop(item.content);
	item:setEnable(false);
	self.hideList:add(item);
end

function Lua_ScrollBoard:initItemShowPos()
    if self.data == nil or self.data:getSize() == 0 then
        return;
    end

    local item;
    while self.showList:getSize() > 0 do
        item = self.showList:remove();
        self:returnCellToPool(item);
    end

    local val = self.selectIdx;
    if val < 1 then
        val = 1;
    elseif val > self.data:getSize() then
        val = self.data:getSize();
    end

    self.selectIdx = val;
    item = self:getCellFromPool();
    item:setLocation(0, 0, 0);
    item:setData(self.data[val]);
    item.showIndex = val;
    self:checkPosition(item, 0);
    self:checkScale(item, 0);
    self.showList:add(item);

    local num;
    if self.ltype == Lua_AxisDefine.Horizontal then
        num = 0;
        local itemX = item.itemX;
        while (itemX + self.cellWidth / 2) < (self.vsize.z / 2) do
            num = num + 1;
            val = item.showIndex + 1;
            if val > self.data:getSize() then
                val = 1;
            end
            itemX = itemX + self.cellWidth;
            item = self:getCellFromPool(item.showIndex);
            item:setLocation(itemX, 0, 0);
            item:setData(self.data[val]);
            item.showIndex = val;
            self:checkPosition(item, num);
            self:checkScale(item, num);
            self.showList:add(item);
        end

        num = 0;
        item = self.showList[1];
        itemX = item.itemX;
        while (itemX - self.cellWidth / 2) > (-self.vsize.z / 2) do
            num = num + 1;
            val = item.showIndex - 1;
            if val < 1 then
                val = self.data:getSize();
            end
            itemX = itemX - self.cellWidth;
            item = self:getCellFromPool(item.showIndex);
            item:setLocation(itemX, 0, 0);
            item:setData(self.data[val]);
            item.showIndex = val;
            self:checkPosition(item, num);
            self:checkScale(item, num);
            self.showList:add(item, 1);
        end
    else
        num = 0;
        local itemY = item.itemY;
        while (itemY + self.cellHeight / 2) < (self.vsize.w / 2) do
            num = num + 1;
            val = item.showIndex + 1;
            if val > self.data:getSize() then
                val = 1;
            end
            itemY = itemY + self.cellHeight;
            item = self:getCellFromPool();
            item:setLocation(0, itemY, 0);
            item:setData(self.data[val]);
            item.showIndex = val;
            self:checkPosition(item, num);
            self:checkScale(item, num);
            self.showList:add(item);
        end

        num = 0;
        item = self.showList[1];
        itemY = item.itemY;
        while (itemY - self.cellHeight / 2) > (-self.vsize.w / 2) do
            num = num + 1;
            val = item.showIndex - 1;
            if val < 1 then
                val = self.data:getSize();
            end
            itemY = itemY - self.cellHeight;
            item = self:getCellFromPool();
            item:setLocation(0, itemY, 0);
            item:setData(self.data[val]);
            item.showIndex = val;
            self:checkPosition(item, num);
            self:checkScale(item, num);
            self.showList:add(item, 1);
        end
    end

    -- 触发选中
    self:checkCompleteSelect();
end

function Lua_ScrollBoard:checkPosition(item, num)
    if item == nil or self.cellChangePos == 0 or self.cellPosSpd == 0 then
        return;
    end

    local cellEndPos = num * self.cellChangePos;
    if self.ltype == Lua_AxisDefine.Horizontal then
        item:setLocation(item.itemX, cellEndPos, 0);
    end
end

function Lua_ScrollBoard:checkScale(item, num)
    if item == nil or self.cellChangeScale == 0 or self.cellScaleSpd == 0 then
        return;
    end

    local cellEndScale = Mathf.Abs(self.cellStartScale - (num * self.cellChangeScale));
    if self.cellStartScale == cellEndScale then
        item:setScale(cellEndScale, cellEndScale);
        return;
    end

    local s;
    local dis;
    if self.ltype == Lua_AxisDefine.Horizontal then
        dis = Mathf.Abs(item.itemX);
    else
        dis = Mathf.Abs(item.itemY);
    end
    
    if self.cellStartScale > cellEndScale then
        s = self.cellStartScale - dis * self.cellScaleSpd;
        if s < cellEndScale then
            s = cellEndScale;
        end
    elseif self.cellStartScale < cellEndScale then
        s = self.cellStartScale + dis * self.cellScaleSpd;
        if s > cellEndScale then
            s = cellEndScale;
        end
    else
        s = cellEndScale;
    end

    item:setScale(s, s);
end

function Lua_ScrollBoard:dispose()
    self:clearScrollBoardTimer();
    local item;
    if self.showList ~= nil then
        while self.showList:getSize() > 0 do
            item = self.showList:remove();
            TweenUtil.getInstance():stop(item.content);
            item:dispose();
            item = nil;
        end
        self.showList:dispose();
        self.showList:clear();
        self.showList = nil;
    end

    if self.hideList ~= nil then
        while self.hideList:getSize() > 0 do
            item = self.hideList:remove();
            TweenUtil.getInstance():stop(item.content);
            item:dispose();
            item = nil;
        end
        self.hideList:dispose();
        self.hideList:clear();
        self.hideList = nil;
    end

    self.data = nil;
    self.vsize = nil;
    self.viewRect = nil;
    self.cellStartScale = nil;
    self.cellEndScale = nil;
    self.cellScaleSpd = nil;
    self.mask = nil;
    self:superCall("dispose");
end