--[[
    self.scrollText = Lua_ScrollText.new(self.content, Lua_AxisDefine.Horizontal, 124, 25);
    self.scrollText:setLocation(0, -180, 0);
    self.scrollText:setShowTxt(100, 24, "", 24);

    self.scrollText = Lua_ScrollText.new(self.content, Lua_AxisDefine.Vertical, 25, 124);
    self.scrollText:setLocation(0, -180, 0);
    self.scrollText:setShowTxt(24, 100, "", 24);
--]]
Lua_ScrollText = class("Lua_ScrollText", Lua_DisplayObject)
function Lua_ScrollText:ctor(p, dt, rw, rh)
    self.scrollType = dt;
    self.rectWidth = rw;
    self.rectHeight = rh;
    self.moveTime = 1;
    self.waitTime = 0;
    self:superCall("ctor", p);
end

function Lua_ScrollText:buildShowItem()
    self:superCall("buildShowItem");
    self:setRectSize(self.rectWidth, self.rectHeight);

    self.mask = UICreate.createUIMaskContainer(self.content, nil, nil, 0, 0, 0, 0);
    self.mask.pivot = Lua_WindowUtil.getPivot(4);
    self.mask:setAnchorMin(0, 0);
    self.mask:setAnchorMax(1, 1);
    self.mask:SetDimensions(0, 0);
    self.mask:setSubRectPivot(0.5, 0.5);
    self.mask:setColor(255, 255, 255, 1);
    self.mask.clickScale = false;
    self.mask:addRectMask2D();

    LuaUtil.setRectSize(self.mask.subRect, self.rectWidth, self.rectHeight);

    self.showText = UICreate.createLabel(self.mask.sub, nil, 0, 0, 100, 22, 22);
end

function Lua_ScrollText:dispose()
    self:cleanTimer();
    self.showText = nil;
    self.mask = nil;
    self:superCall("dispose")
end

function Lua_ScrollText:setTextStyle(width, height, fontName, fontSize, colorStr, effectStyle, effectColorStr)
    self.showText:SetDimensions(width, height);
    self.showText.fontName = fontName or "huakang_font";
    self.showText.fontSize = fontSize or 22;
    self.showText.colorStr = Lua_ColorDefine.transColor(colorStr or "FFFFFF");
    self.showText.effectStyle = effectStyle or 2;
    effectColorStr = effectColorStr or "000000";
    self.showText.effectColorStr = Lua_ColorDefine.transColor(effectColorStr);
    self.showText:setEffectDistance(2, 2);
end

function Lua_ScrollText:setShowTxt(str)
    self:cleanTimer();
    self.showText.text = str;
    
    if self.scrollType == Lua_AxisDefine.Horizontal then
        self.showText.horizontalOverflowValue = 1;
        self.showText.verticalOverflowValue = 0;
        self.showText:resetSize();
        local value = self.showText.width - self.rectWidth;
        if value > 0 then
            self.showText.pivotValue = 3;
            self.showText.alignmentValue = 3;

            self.startPosX = -self.rectWidth / 2 + 5;
            self.endPosX = self.startPosX - value - 8;
            self.showText:setLocation(self.startPosX, 0, 0);

            self.moveType = 1;
            self.startPos = self.startPosX;
            self.endPos = self.endPosX;
            self:startTimer();
        else
            self.showText.pivotValue = 4;
            self.showText.alignmentValue = 4;
            self.showText:setLocation(0, 0, 0);
        end
    else
        self.showText.horizontalOverflowValue = 0;
        self.showText.verticalOverflowValue = 1;
        self.showText:resetSize();
        local value = self.showText.height - self.rectHeight;
        if value > 0 then
            self.showText.pivotValue = 1;
            self.showText.alignmentValue = 1;

            self.startPosY = self.rectHeight / 2 - 8;
            self.endPosY = self.showText.height + self.startPosY + 5;
            self.showText:setLocation(0, self.startPosY, 0);
            self:startTimer();
        else
            self.showText.pivotValue = 4;
            self.showText.alignmentValue = 4;
            self.showText:setLocation(0, 0, 0);
        end
    end
end

function Lua_ScrollText:startTimer()
    if self.bTimer ~= true then
        self.bTimer = true;
        self.startTime = getTime();
        self:addTimer("Lua_ScrollText", 0.01, self, self.onTextMove, nil, -1);
    end
end

function Lua_ScrollText:cleanTimer()
    if self.bTimer == true then
        self.bTimer = false;
        self:removeTimer(self, self.onTextMove);
    end
end

function Lua_ScrollText:onTextMove()
    local dt = getTime() - self.startTime
    if self.waitTime > 0 then
        self.waitTime = self.waitTime - dt;
        if self.waitTime <= 0 then
            self.startTime = getTime();
        end
        return;
    end    

    if self.scrollType == Lua_AxisDefine.Horizontal then
        if dt >= self.moveTime then
            self.waitTime = 100;
            if self.moveType == 1 then
                self.moveType = 2;
                self.startPos = self.endPosX;
                self.endPos = self.startPosX;
            else
                self.moveType = 1;
                self.startPos = self.startPosX;
                self.endPos = self.endPosX;
            end
        else
            local value = Mathf.Lerp(self.startPos, self.endPos, dt / self.moveTime)
            self.showText:setLocation(value, 0, 0);
        end
    else
        if dt >= self.moveTime then
            self.startTime = getTime();
        else
            local value = Mathf.Lerp(self.startPosY, self.endPosY, dt / self.moveTime)
            self.showText:setLocation(0, value, 0);
        end
    end
end