-- 兑换活动配置表Vo
Lua_CashRewardVo = class("Lua_CashRewardVo")
function Lua_CashRewardVo:ctor(id, actIdx, config)
	self.id = id;
	self.actIdx = actIdx;
	self.config = config;
	if self.config == nil then
		self.config = getConfig({"cash_reward", id });
	end
end

function Lua_CashRewardVo:dispose()
	if self.itemList ~= nil then
		self.rewardList = nil; 
		self.itemList = nil; 
		self.rewardArr:dispose();
		self.rewardArr:clear();
		self.rewardArr = nil;
	end
end

function Lua_CashRewardVo:updateServerInfo(serverInfo)
	self.actEndTick = serverInfo.end_tick;
	self.residueTime = serverInfo.end_tick - DataCenter.roleData.serverTime; -- 活动结束剩余时间
	self.closeResidueTime = serverInfo.close_tick - DataCenter.roleData.serverTime; --活动商店关闭剩余时间
	self.roundId = serverInfo.round_id; -- 轮次
	self.closeTick = serverInfo.close_tick; -- 关闭时间戳
	self.worldLevel = serverInfo.world_level; -- 世界等级
	self.giftState = serverInfo.gift_state; -- 礼包状态
	self.showRedPoint = serverInfo.first_open; -- 是否当天第一次点击   0：红点消失  1： 红点显示

	self:updateRoundItemList();
end

function Lua_CashRewardVo:updateRoundItemList()
	if self.roundId == nil or self.worldLevel == nil or self._itemList == nil then
		return nil;
	end

	if self.itemList ~= nil then
		self.rewardList = nil; -- 活动界面奖励预览中道具列表
		self.itemList = nil; -- 存储该轮次所有道具信息
		self.rewardArr:dispose();
		self.rewardArr:clear();
		self.rewardArr = nil;
	end
	self.itemList = {};
	self.rewardList = {};
	self.rewardArr = Array();

	local obj;
	local vo;
	-- 读取符合世界区间的道具信息
	for i = 1, #self._itemList do
		obj = self._itemList[i];
		vo = self.itemList[obj];
		vo = vo == nil and Lua_CashRewardItemVo.new(obj) or vo;
		if self:judgeWorldLevel(vo.worldLevel,self.worldLevel) then 
			local count = #self.rewardList;
			self.itemList[vo.id] = vo;
			self.rewardArr:add(vo.id);
			self.rewardList[count + 1] = vo.rewardItem;
		end
	end

	--道具限购次数
	if self.giftState ~= nil then
		for i = 1, #self.giftState do
			obj = self.giftState[i];
			vo = self:getRoundItemById(obj.key);
			if vo ~= nil then
				vo.limitTimes = obj.value;
			end
		end
	end
	self:hasReceiveReward();
end

function Lua_CashRewardVo:hasReceiveReward()
	local mIconId = self.itemCost.id;
	local moneyTypeCount = DataCenter.moneyData:getMoney(mIconId) ~= 0 and DataCenter.moneyData:getMoney(mIconId) or DataCenter.packData:getItemCountById(mIconId);
	self.hasReward = false;
	
	for k, v in pairs(self.itemList) do
		if v.recommend == 1 and v.limitTimes > 0 and moneyTypeCount >= v.recharge then
			self.hasReward = true;
			break;
		end
	end
end

-- 根据roundID获取对应详细信息
function Lua_CashRewardVo:getRoundItemById(id)
	if id == nil or self.itemList == nil or self.itemList[id] == nil then 
		return nil;
	end
	return self.itemList[id];
end

-- 判断世界等级是否满足
function Lua_CashRewardVo:judgeWorldLevel(worldLevels,level)
	if worldLevels == nil or level == nil then
		return ;
	end

	if worldLevels[1] <= level and level <= worldLevels[2] then
		return true;
	else
		return false;
	end
end

----------cash_reward配置表信息------
-- 商店兑换道具
function Lua_CashRewardVo.Get:_itemList()
	return self.config.item_list;
end

-- 活动兑换货币
function Lua_CashRewardVo.Get:itemCost()
	if self._itemCost == nil then
		local obj = self.config.item_cost;
		self._itemCost = Lua_ToolVo.createToolVo(obj,1);
	end
	return self._itemCost;
end

-- 活动描述
function Lua_CashRewardVo.Get:des()
	return self.config.des;
end

-- 图标所在的图集
function Lua_CashRewardVo.Get:iconAtlas()
	return self.config.icon_atlas;
end

-- 图标名
function Lua_CashRewardVo.Get:iconSprite()
	return self.config.icon_sprite;
end

-- 主题顺序
function Lua_CashRewardVo.Get:roundSort()
	return self.config.round_sort;
end




-----------------------------------------------------------------------------------
-- 兑换商店道具Vo
Lua_CashRewardItemVo = class("Lua_CashRewardItemVo")
function Lua_CashRewardItemVo:ctor(id, config)
	self.id = id;
	self.config = config;
	if self.config == nil then
		self.config = getConfig({"cash_reward_item", id });
	end
end

function Lua_CashRewardItemVo:dispose()
end

-- 商店道具
function Lua_CashRewardItemVo.Get:rewardItem()
	return self.config.reward_item;
end

-- 限购次数
function Lua_CashRewardItemVo.Get:_limitTimes()
	return self.config.limit_times;
end

-- 获取当前可购买次数
function Lua_CashRewardItemVo.Get:limitTimes()
	return  self._culLimitTimes == nil and self._limitTimes or self._culLimitTimes;
end

-- 设置当前可购买次数
function Lua_CashRewardItemVo.Set:limitTimes(value)
	self._culLimitTimes =  value ;
end

-- 价格
function Lua_CashRewardItemVo.Get:recharge()
	return self.config.recharge;
end

-- 世界等级
function Lua_CashRewardItemVo.Get:worldLevel()
	return self.config.world_level;
end

-- 推荐 （0：不推荐，1：推荐）
function Lua_CashRewardItemVo.Get:recommend()
	return self.config.recommend;
end

