Lua_ChatHeadIconItem = class("Lua_ChatHeadIconItem", Lua_DisplayObject);
function Lua_ChatHeadIconItem:ctor(p, needClick)
    self.needClick = needClick ~= nil and needClick or false;
    self:superCall("ctor", p);
end

function Lua_ChatHeadIconItem:buildShowItem()
    self:superCall("buildShowItem");

    self.headItem = Lua_AvatarItem.new(self.content);

    if self.needClick then
        self.headItem.avatarIcon.clickScale = false;
        self.headItem.avatarIcon:addMousDownFuntion(self, self.onHeadClick);
        self.headItem.avatarIcon:addMouseUpFuntion(self, self.onHeadUpClick);
    end
end

function Lua_ChatHeadIconItem:dispose()
    Lua_CommonUtil.removeTimer(self, self.onRightCheckPoint);

    if self.headItem ~= nil then
        self.headItem:dispose();
        self.headItem = nil;
    end
    self:superCall("dispose");
end

function Lua_ChatHeadIconItem:setData(data)
    self.data = data;
    self.player_id = data.id;

    local roleData = DataCenter.roleData;
    local avatarId = roleData:getDressUpId(Lua_DressUpDefine.MAINTYPE_HEAD, data.dress_info);
    local frameId = roleData:getDressUpId(Lua_DressUpDefine.MAINTYPE_FRAME, data.dress_info);
    self.headItem:setAvatarBg("common_altas_1", "xui_common_widget_003");
    self.headItem:setRoleAvatar(avatarId, 0.7);
    self.headItem:setRoleFrame(frameId, 0.7);

    self.headItem:setRoleLevel(data.level, -33, -31);
    self.headItem:setRoleLevelBg("common_altas_1", "xui_common_widget_016");
    self.headItem:setRoleLevelStyle(Lua_MyUISettings.defaultFontName, 16, "FEFEFE", 2, "000000");
end

function Lua_ChatHeadIconItem:onHeadClick()
    self.isPoint = false;
    self.startTime = getTime();
    if tonumber(self.player_id) == tonumber(DataCenter.roleData.roleId) then
        OperateAlert.showLangValue(171025);
        return;
    end

    if DataCenter.chatData.openChannelTag == Lua_ChatData.CHANNEL_FRIEND then
        Lua_WindowUtil.openWin("Lua_PlayerInfoWindow", Lua_WinParam.ACTION_OPEN, {id = tonumber(self.player_id), server_flag = self.data.server_flag});
        return;
    end
    
    Lua_CommonUtil.addTimer("onRightCheckPoint", 0.01, self, self.onRightCheckPoint);
end

function Lua_ChatHeadIconItem:onHeadUpClick()
    Lua_CommonUtil.removeTimer(self, self.onRightCheckPoint);
    if self.isPoint == true then
        return;
    end

    if DataCenter.chatData.openChannelTag == Lua_ChatData.CHANNEL_FRIEND then
        return;
    end

    if tonumber(self.player_id) == tonumber(DataCenter.roleData.roleId) then
        return;
    end

    Lua_WindowUtil.openWin("Lua_PlayerInfoWindow", Lua_WinParam.ACTION_OPEN, {id = tonumber(self.player_id), server_flag = self.data.server_flag});
end

function Lua_ChatHeadIconItem:onRightCheckPoint()
    if (getTime() - self.startTime) > 2 then
        self.isPoint = true;
        Lua_CommonUtil.removeTimer(self, self.onRightCheckPoint);
        DataCenter.chatData:setChatShowItem(Lua_ChatData.CONTENT_POINT, self.data);
        Lua_MsgUtil.callFunc("CHAT_SHOW_TYPE_UPDATE");
    end
end