Lua_ExpeditionData = class("Lua_ExpeditionData")
function Lua_ExpeditionData:ctor()
	self:superCall("ctor");
	self:dispose();
end

function Lua_ExpeditionData:dispose()
	self.cur_layer = nil;
	self.cur_stage = nil;
	self.cur_location = nil;
	self.cur_difficulty = 0;
	self.cur_state = 0;
	self.buff_state = 0;

	self.left_sec = 0;
	self.expedition_amount = 0;
	self.coin_amount = 0;
	self.role_level = DataCenter.roleData.level;
	self.one_key_skip = 0;
	self.star_num = 0;
	self.all_pass = 0;

	self.stageDic = nil;
	self.passDic = nil;
	self.starDic = nil;
	self.winHeroDic = nil;

	self.haveHeroDeath = false;
	self.havaHeroFullBlood = true;
end

function Lua_ExpeditionData.Set:cur_layer(value)
	if self._cur_layer and self._cur_layer ~= value then
		self.bLayerChange = true
	end
	self._cur_layer = value
end

function Lua_ExpeditionData.Get:cur_layer()
	return self._cur_layer or 1
end

function Lua_ExpeditionData.Set:cur_stage(value)
	if self._cur_stage and self._cur_stage ~= value then
		self.bStageChange = true
	end
	self._cur_stage = value
end

function Lua_ExpeditionData.Get:cur_stage()
	return self._cur_stage or 0
end

function Lua_ExpeditionData.Set:cur_location(value)
	if self._cur_location and self._cur_location ~= value then
		self.bLocationChange = true
	end
	self._cur_location = value
end

function Lua_ExpeditionData.Get:cur_location()
	return self._cur_location or 1
end

function Lua_ExpeditionData:fromServer50149001(msg)
	if msg == nil then
		return;
	end
	
	self.cur_layer = msg.sel_station; -- 当前选择第几层
	self.cur_stage = msg.top_stage; -- 当前挑战第几关
	self.cur_location = msg.top_sel_location <= 0 and 1 or msg.top_sel_location; -- 已选择路线, 0 未选择, 有效位置从1开始
	self.cur_difficulty = msg.top_difficulty; -- 当前最高选择难度， 0 没有难度选择
	self.cur_state = msg.cur_state; -- 操作状态 0：无操作 1：操作中
	self.buff_state = msg.buff_state; -- 0 未通关，1 已通关未选择增益，2 已选增益

	self.left_sec = tonumber(msg.left_sec); --剩余结束时间，单位秒
	self.expedition_amount = msg.expedition_amount; -- 本轮获得的远征币数
	self.coin_amount = msg.coin_amount; -- 本轮获得的金币数
	self.role_level = msg.level; -- 本轮挑战时的等级
	self.one_key_skip = msg.one_key_skip; -- 跳过战斗（0：否 1：是）
	self.star_num = msg.star_num; -- 当前星数
	self.all_pass = msg.all_pass; -- 0：未全部通关，1：全部通关
	self.force = msg.force; -- 本轮战力

	self.stageDic = nil;
	local list = msg.map_list;
	local obj;
	if list ~= nil then
		for i = 1, #list do
			obj = list[i];
			for j = 1, #obj.affair do
				self:setLatticeEvent(obj.key, obj.affair[j].key, obj.affair[j].value);
			end
		end
	end

	self.passDic = nil;
	list = msg.pass_list;
	if list ~= nil then
		for i = 1, #list do
			obj = list[i];
			for j = 1, #obj.affair do
				-- print(obj.key, obj.affair[j].key)
				self:setLatticeState(obj.key, obj.affair[j].key, obj.affair[j].value);
			end
		end
	end

	self.starDic = nil;
	self:setStarState(msg.star_reward);
	if self.isWait ~= true then
		self:checkInfoUpdate();
	else
		Lua_MsgUtil.callFunc("UPDATE_EXPEDITION_INFO_TEMP");
	end
	self:checkClearanceRedPoint();
end

function Lua_ExpeditionData:checkInfoUpdate()
	if self.bLayerChange ~= true and (self.bStageChange == true or self.bLocationChange == true) then
		Lua_MsgUtil.callFunc("UPDATE_EXPEDITION_INFO", true);
	else
		Lua_MsgUtil.callFunc("UPDATE_EXPEDITION_INFO");
	end

	self.bLayerChange = false
	self.bStageChange = false
	self.bLocationChange = false
	self.isWait = false
end

function Lua_ExpeditionData:fromServer50149002(msg)
	if msg == nil then
		return;
	end

	self.haveHeroDeath = false;
	self.havaHeroFullBlood = true;
	self.havaHeroDic = {}
	if self.havaHeroArr == nil then
		self.havaHeroArr = Array();
	end
	local arr = Array();
	local obj;
	local vo;
	for i = 1, #msg.hero_list do
		obj = msg.hero_list[i];
		if obj.blood <= 0 then
			self.haveHeroDeath = true;
		end
		if obj.blood > 0 and obj.blood < obj.max_blood then
			self.havaHeroFullBlood = false;
		end

		vo = self.havaHeroArr:remove();
		if vo == nil then
			vo = Lua_HeroVo.new(obj.hero_id);
		else
			vo:setData(obj.hero_id);
		end
		vo:fromServer(obj);
		arr:add(vo);
		self.havaHeroDic[vo.unique_id] = i
	end
	self.havaHeroArr:clear();
	self.havaHeroArr = arr;
end

function Lua_ExpeditionData:fromServer50149013(msg)
	if msg == nil then
		return;
	end

	Lua_WindowUtil.openWin("Lua_ExpeditionRewardDetailsWindow", Lua_WinParam.ACTION_OPEN, msg.item_list);
end

function Lua_ExpeditionData:fromServer80149022(msg)
	if msg == nil then
		return;
	end
	
	self.winHeroDic = {}
	local info
	for i = 1, #msg.win_hero_id_list do
		info = msg.win_hero_id_list[i]
		self.winHeroDic[info] = 1
	end
end

function Lua_ExpeditionData:checkTips()
	if not Lua_WindowUtil.isWindowShowing("Lua_ExpeditionWindow") then
		return;
	end
	if self.buff_state == 1 then
		Lua_WindowUtil.openWin("Lua_ExpeditionBuffChooseWindow", Lua_WinParam.ACTION_OPEN);
	else
		self:checkInfoUpdate();
	end
end

-- 玩家模型位置
function Lua_ExpeditionData:getModelPosInfo()
	local cur_id = self.cur_layer * 1000 + self.cur_difficulty * 100 + self.cur_stage;
	local config = getConfig("expedition_map", cur_id);
	local cur_index = self.cur_location;
	if self.cur_state == 1 or self.buff_state == 1 then
		-- 已经进入下一关卡
		if config.prev_id > 0 then
			cur_id = config.prev_id;
			if self.passDic ~= nil and self.passDic[cur_id] ~= nil then
				for key, value in pairs(self.passDic[cur_id]) do
					cur_index = key; -- 上一通关关卡选择位置
				end
			end
		end
	end
	return cur_id, cur_index;
end

-- 设置每个格子事件
function Lua_ExpeditionData:setLatticeEvent(stageId, index, eventId)
	if self.stageDic == nil then
		self.stageDic = {};
	end
	if self.stageDic[stageId] == nil then
		self.stageDic[stageId] = {};
	end
	self.stageDic[stageId][index] = eventId;
end

function Lua_ExpeditionData:getLatticeEvent(stageId, index)
	if self.stageDic == nil or self.stageDic[stageId] == nil then
		return nil;
	end
	return self.stageDic[stageId][index];
end

-- 设置每个格子状态
function Lua_ExpeditionData:setLatticeState(stageId, index, eventId)
	if self.passDic == nil then
		self.passDic = {};
	end
	if self.passDic[stageId] == nil then
		self.passDic[stageId] = {};
	end
	self.passDic[stageId][index] = eventId;
end

function Lua_ExpeditionData:getLatticeState(stageId, index)
	if self.passDic == nil or self.passDic[stageId] == nil then
		return false, nil;
	end
	return true, self.passDic[stageId][index];
end

-- 设置星数奖励领取状态
function Lua_ExpeditionData:setStarState(list, arr)
	if self.starDic == nil then
		self.starDic = {};
	end
	local state = false;
	if list ~= nil then
		local obj;
		for i = 1, #list do
			obj = list[i];
			if obj.value == 1 then
				state = true;
			end
			if arr and obj.value == 2 and obj.value ~= self.starDic[obj.key] then
				arr:add(obj.key);
			end
			self.starDic[obj.key] = obj.value;
		end
	end
	self:checkBoxRedPoint(state);
end

function Lua_ExpeditionData:getStarState(id)
	if self.starDic == nil or self.starDic[id] == nil then
		return 0;
	end
	return self.starDic[id];
end

function Lua_ExpeditionData:getStageInfo(id)
	local config = getConfig("expedition_map", id);
	for i = 1, config.map do
		local bPass, passEventId = self:getLatticeState(id, i);
		if bPass and passEventId ~= nil then
			return config.map, i;
		end
	end
	return config.map, 0;
end

function Lua_ExpeditionData:checkHeroIsWin(id)
	if self.winHeroDic == nil then
		return false
	end
	id = tonumber(tostring(id))
	return self.winHeroDic[id] == 1
end










------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
function Lua_ExpeditionData:getLineupHeroVo(uniqueId)
	uniqueId = tonumber(uniqueId)
	if self.havaHeroDic == nil or self.havaHeroDic[uniqueId] == nil then
		return nil
	end
	local index = self.havaHeroDic[uniqueId]
	return self.havaHeroArr[index]
end

function Lua_ExpeditionData:getHeroList(campType, careerType)
	local arr = Array();
	if self.havaHeroArr ~= nil then
		local heroVo;
		for i = 1, self.havaHeroArr:getSize() do
			heroVo = self.havaHeroArr[i];
			if (campType == 0 or heroVo.campType == campType) and (careerType == 0 or heroVo.career == careerType) then
				arr:add(heroVo);
			end
		end
	end
	arr:sortf(self, self.heroSortFun);
	return arr;
end

function Lua_ExpeditionData:heroSortFun(a, b)
	local _a = a.blood / a.max_blood
	local _b = b.blood / b.max_blood
	if _a > 0 and _b > 0 then
		if a.lineup_id < b.lineup_id then
			return false
		elseif a.lineup_id == b.lineup_id then
			if a.heroForce < b.heroForce then
				return false
			end
		end
	elseif _a <= 0 and _b > 0 then
		return false
	elseif _a <= 0 and _b <= 0 then
		if a.heroForce < b.heroForce then
			return false
		end
	end
	return true
end

function Lua_ExpeditionData:checkPackHaveHero(teamIdxDic)
	if self.havaHeroArr == nil then
		return false
	end

	local existDic = {} --已上阵英雄
	local heroVo
	for i = 1, #teamIdxDic do
		for j = 1, teamIdxDic[i]:getSize() do
			heroVo = self:getLineupHeroVo(teamIdxDic[i][j])
			if heroVo then
				existDic[heroVo.id] = j
			end
		end
	end

	for i = 1, self.havaHeroArr:getSize() do
		heroVo = self.havaHeroArr[i]
		if heroVo.blood > 0 and existDic[heroVo.id] == nil then
			return true
		end
	end
	return false
end










--------------------------------------------------
function Lua_ExpeditionData:checkClearanceRedPoint()
	local sub_times = DataCenter.expeditionUtil:getConstConfig("sub_times", 1, 0) * 60;
	if self.left_sec - sub_times > 0 then
		local id = self.cur_layer * 1000 + self.cur_difficulty * 100 + self.cur_stage;
		local config = getConfig("expedition_map", id);
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Expedition_Clearance, config.last_stage ~= 1);
	else
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Expedition_Clearance, false);
	end

	local state = (self.cur_stage == 8 and self.cur_state == 0 and self.buff_state == 2);
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Expedition_Reward, state);
end

function Lua_ExpeditionData:checkBoxRedPoint(state)
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Expedition_Box, state);
end