-- 战斗记录数据VO
checkRequire("Lua_FightCampPosConfig");
checkRequire("Lua_FightTemplateConfigData");
Lua_FightRecordVo = class("Lua_FightRecordVo");
function Lua_FightRecordVo:ctor(rId)
    self.recordId = rId;
    self._curRotation = 1;
    self.isComplete = false;
    self.isReplay = false;
    self.bPlayedStartEff = false;
    self.recordInfoDic = {};
    self.attackerConnNum = 0
    self.defenderConnNum = 0
end

function Lua_FightRecordVo:init()
	self._curRotation = 1;
	self.deathThingNum = nil;
	self.totalThingNum = nil;
	self.totalHp = nil;
	self.curHp = nil;
	self.attackerMaxHp = nil;
	self.attackerHp = nil;
	self.defenderMaxHp = nil;
	self.defenderHp = nil;
	self._bossHurt = nil;
	self.plotId = 0; -- 剧情id
end

function Lua_FightRecordVo:clear()
	self._speedIdx = nil;
	if self.recordInfoDic ~= nil then
		for k, arr in ipairs(self.recordInfoDic) do
			arr:clear();
			self.recordInfoDic[k] = nil;
		end
		self.recordInfoDic = {};
	end
end

function Lua_FightRecordVo:dispose()
	self.player = nil;
	self.plotId = 0;
	self:clear();
	self.recordInfoDic = nil;
	self:clearConnect();
end

function Lua_FightRecordVo.Get:templateConfig()
	return Lua_FightTemplateConfigData.getConfigData(self.templateId);
end

-- 战斗结果
function Lua_FightRecordVo.Get:result()
	return self._result or 2;
end

-- 播放速度索引
function Lua_FightRecordVo.Set:speedIdx(value)
	if DataCenter.vipData == nil then
		value = 1;
	else
		local vipVo = DataCenter.vipData.curVipLevelVo;
		if vipVo ~= nil and vipVo.fightPlaySpeed ~= nil then
			if value > #vipVo.fightPlaySpeed then
				value = #vipVo.fightPlaySpeed;
			end
		else
			value = 1;
		end
	end

	if self._speedIdx ~= value then
		self._speedIdx = value;

		if self.isReplay ~= true then
			Lua_ProxyManager.getProxy("Lua_RoleProxy"):sOperate(self.templateConfig.speedKey, value);
		end

		local player = DataCenter.fightData:getFightPlayerById(self.recordId);
		if player ~= nil then
			player:setPlaySpdAndSpdRate(self.speed, self.speedRate);
		end
		Lua_MsgUtil.callFunc("fightPlay_speed_change");
	end
end

-- 播放速度索引
function Lua_FightRecordVo.Get:speedIdx()
	if self.isReplay == true then -- 回放默认2倍速
		if self._speedIdx then
			return self._speedIdx;
		end
		return 2;
	end

	if self._speedIdx == nil then
		self._speedIdx = DataCenter.fightData:getFightSpeedIdx(self.templateId)
	end
	return self._speedIdx;
end

-- 显示播放速度
function Lua_FightRecordVo.Get:showSpeed()
	local vipVo = DataCenter.vipData.curVipLevelVo;
	if vipVo == nil then
		return 1;
	end
	return vipVo:getFightPlayShowSpeedByIdx(self.speedIdx);
end

-- 实际播放速度
function Lua_FightRecordVo.Get:speed()
	local vipVo = DataCenter.vipData.curVipLevelVo;
	if vipVo == nil then
		return 1;
	end
	return vipVo:getFightPlaySpeedByIdx(self.speedIdx);
end

-- 实际播放速度缩放比
function Lua_FightRecordVo.Get:speedRate()
	local vipVo = DataCenter.vipData.curVipLevelVo;
	if vipVo == nil then
		return 1;
	end
	local normalSpd = vipVo:getFightPlaySpeedByIdx(1);
	local curSpd = vipVo:getFightPlaySpeedByIdx(self.speedIdx);
	return curSpd / normalSpd;
end

function Lua_FightRecordVo.Get:sceneId()
	if self.templateConfig == nil then
		return 0;
	end
	return self.templateConfig.scene_id;
end

function Lua_FightRecordVo.Get:maxRotation()
	if self.templateId == 1005 or self.templateId == 1006 then
		return DataCenter.infinityHallUtil:getConstConfig("max_round", 1, 0);
	end
	return 0;
end

function Lua_FightRecordVo.Get:maxRound()
	if self.templateConfig == nil then
		return 0;
	end
	return self.templateConfig.maxRound;
end

function Lua_FightRecordVo.Get:ownType()
    if self.attacker ~= nil and DataCenter.roleData:isMyself(self.attacker.id) then
        return Lua_FightDefine.FIGHT_ATTACKER;
    elseif self.defender ~= nil and DataCenter.roleData:isMyself(self.defender.id) then
        return Lua_FightDefine.FIGHT_DEFENDER;
    end
    return -1;
end

function Lua_FightRecordVo.Get:isIgnoreResult()
	if self.isReplay == true or self.templateConfig.ignore_result == 1 then
		return true;
	end
	return false;
end

function Lua_FightRecordVo:getCampPosByType(formationType)
	if formationType == Lua_FightDefine.FIGHT_ATTACKER then
		return self.attacker ~= nil and self.attacker.camp_pos_id or 0;
	else
		return self.defender ~= nil and self.defender.camp_pos_id or 0;
	end
end

function Lua_FightRecordVo:getDirByType(formationType)
	local camp_pos_id;
	if formationType == Lua_FightDefine.FIGHT_ATTACKER then
		camp_pos_id = self.attacker.camp_pos_id;
	else
		camp_pos_id = self.defender.camp_pos_id;
	end
	return Lua_FightCampPosConfig.getConfigData(camp_pos_id):getDirByType(formationType);
end

function Lua_FightRecordVo:getPosxByIdx(formationType, idx)
	local camp_pos_id;
	if formationType == Lua_FightDefine.FIGHT_ATTACKER then
		camp_pos_id = self.attacker.camp_pos_id;
	else
		camp_pos_id = self.defender.camp_pos_id;
	end
	return Lua_FightCampPosConfig.getConfigData(camp_pos_id):getPosxByIdx(formationType, idx);
end

function Lua_FightRecordVo:getPosyByIdx(formationType, idx)
	return 0;
end

function Lua_FightRecordVo:getPoszByIdx(formationType, idx)
	local camp_pos_id;
	if formationType == Lua_FightDefine.FIGHT_ATTACKER then
		camp_pos_id = self.attacker.camp_pos_id;
	else
		camp_pos_id = self.defender.camp_pos_id;
	end
	return Lua_FightCampPosConfig.getConfigData(camp_pos_id):getPoszByIdx(formationType, idx);
end

function Lua_FightRecordVo:getCenterPosByType(formationType)
	if formationType == Lua_FightDefine.FIGHT_ATTACKER then
		return Lua_FightCampPosConfig.getConfigData(self.attacker.camp_pos_id).attackCenter;
	else
		return Lua_FightCampPosConfig.getConfigData(self.defender.camp_pos_id).defendCenter;
	end
end

function Lua_FightRecordVo:getKey(rotation, session, round)
	return rotation * 100000  + session * 1000 + round; -- 轮次 > 场次 > 回合
end

-- 战斗回合数据、战斗回合数据
function Lua_FightRecordVo:updateFightInfo(msg)
    if msg == nil then
        return;
    end

    self.player = msg.rotation_player; -- 每轮战斗双方信息
    self.templateId = msg.template_id; -- 战斗模板配置id
    self.isComplete = msg.is_complete == 1; -- 是否已经完整数据：0否，1是。后续由80102002分发
    self.isReplay = msg.is_replay == 1; -- 1：为回放的战斗记录
    self.stageId = msg.stage_id; -- 关卡id
    self.flag = msg.flag; -- 客户端用标识
    self._teamId = tonumber(msg.team_id); -- 队伍id
    self._index = msg.index; -- 战斗顺位索引 1-N
    self.plotId = msg.plot_id; -- 剧情id
	self.fightTick = tonumber(msg.fight_tick); -- 战斗开始时间 0：从头开始播 其他：计算时间播
	if self.fightTick > 0 then
		self._skipTime = DataCenter.roleData.serverTime - self.fightTick;
	end
    self:updateRecordInfo(msg.record_list); -- 战斗记录数据
end

-- 战斗分段数据
function Lua_FightRecordVo:updateFightSegmentInfo(msg)
    if msg == nil and self.recordId ~= tonumber(tostring(msg.record_id)) then
        return;
    end

    self.isComplete = msg.is_complete == 1; --是否分发完毕：0否，1是
    self:updateRecordInfo(msg.record_list); -- 战斗记录数据
    self.player = msg.rotation_player; -- 每轮战斗双方信息
end

function Lua_FightRecordVo.Get:attacker()
	if self.player and #self.player > 0 then
		local len = #self.player
		if len >= self._curRotation then
			return self.player[self._curRotation].attacker;
		else
			return self.player[len].attacker;
		end
	end
	return nil;
end

function Lua_FightRecordVo.Get:attackerLineup()
	if self.attacker then
		local len = #self.attacker.lineup
		if len >= self._curRotation then
			return self.attacker.lineup[self._curRotation];
		else
			return self.attacker.lineup[len];
		end		
	end
	return nil
end

function Lua_FightRecordVo.Get:defender()
	if self.player and #self.player > 0 then
		local len = #self.player
		if len >= self._curRotation then
			return self.player[self._curRotation].defender;
		else
			return self.player[len].defender;
		end
	end
	return nil;
end

function Lua_FightRecordVo.Get:defenderLineup()
	if self.defender then
		local len = #self.defender.lineup
		if len >= self._curRotation then
			return self.defender.lineup[self._curRotation];
		else
			return self.defender.lineup[len];
		end		
	end
	return nil
end

-- 队伍ID
function Lua_FightRecordVo.Get:teamId()
	return self._teamId or 0;
end

-- 战斗顺位索引 1-N
function Lua_FightRecordVo.Get:index()
	return self._index or 0;
end

function Lua_FightRecordVo.Get:skipTime()
	if self._skipTime and self._skipTime > 0 then
		return self._skipTime;
	end
	return 0;
end

-- 当前播放轮次
function Lua_FightRecordVo.Set:curRotation(value)
	self._curRotation = value;
end

function Lua_FightRecordVo:updateRecordInfo(list)
	-- rotation 当前轮次，1开始
	-- round 当前回合数，1开始
	-- type 类型 
		---- 1：单位进场
		---- 2：释放技能（对目标或友军造成的伤害或回血），伤害值和技能目标在hurt_list字段
		---- 3：buff变化（新增或移除），新增或者移除的buff在buff_list字段
		---- 4：buff伤害（buff扣血或回血），伤害值和目标在hurt_list字段
		---- 5：不可行动（受buff控制效果导致的不可行动，buff_id为对应的控制效果）
	-- fighter_id 行动单位id
	-- fighter_list 进场战斗单位
	-- skill_id 释放的技能id
	-- buff_id 生效buff的id（buff的配置id）
	-- hurt_list 技能目标、技能伤害、buff伤害列表
	-- buff_list 新增或者移除的buff信息
	if list == nil or #list <= 0 then
		return;
	end

	for i = 1, #list do
		local info = list[i];
		local rotation = info.rotation;
		local session = info.session;
		local round = info.round;
		local key = self:getKey(rotation, session, round);
		local arr = self.recordInfoDic[key];
		if arr == nil then
			arr = Array();
			self.recordInfoDic[key] = arr;
		end
		arr:add(info);
		self:setFighterInfo(info.fighter_list);
	end
end

function Lua_FightRecordVo:getMaxActionByRound(rotation, session, round)
	if self.recordInfoDic == nil then
		return 0;
	end

	local key = self:getKey(rotation, session, round);
	local arr = self.recordInfoDic[key];
	if arr == nil then
		return 0;
	end
	return arr:getSize();
end

function Lua_FightRecordVo:getActionInfo(rotation, session, round, idx)
	if idx <= 0 or self.recordInfoDic == nil then
		return nil;
	end

	local key = self:getKey(rotation, session, round);
	local arr = self.recordInfoDic[key];
	if arr == nil or arr:getSize() < idx then
		return nil;
	end
	return arr[idx];
end

function Lua_FightRecordVo:getNextActionInfo(rotation, session, round, idx)
	if idx <= 0 or self.recordInfoDic == nil then
		return nil, idx;
	end

	local key = self:getKey(rotation, session, round);
	local arr = self.recordInfoDic[key];
	if arr == nil then -- 违规
		return nil, idx;
	end

	local len = arr:getSize();
	if idx > len then -- 违规
		return nil, idx;
	elseif idx < len then
		return arr[idx + 1], idx + 1;
	else 
		key = self:getKey(rotation, session, round + 1);
		arr = self.recordInfoDic[key];
		if arr ~= nil then
			return arr[1], 1;
		else
			key = self:getKey(rotation, session + 1, 1);
			arr = self.recordInfoDic[key];
			if arr ~= nil then
				return arr[1], 1;
			else
				key = self:getKey(rotation + 1, 1, 1);
				arr = self.recordInfoDic[key];
				if arr ~= nil then
					return arr[1], 1;
				end
			end
		end
	end

	return nil, idx;
end

function Lua_FightRecordVo:setFighterInfo(list)
	if list == nil then
		return;
	end

	if self.fightStandDic == nil then
		self.fightStandDic = {};
	end
	if self.fighterDic == nil then
		self.fighterDic = {};
	end
	if self.fighterIdDic == nil then
		self.fighterIdDic = {};
	end
	local obj;
	local id;
	for i = 1, #list do
		obj = list[i];
		id = tonumber(obj.id);
		if self.fighterDic[obj.aod_type] == nil then
			self.fighterDic[obj.aod_type] = {};
		end
		if self.fighterDic[obj.aod_type][obj.type] == nil then
			self.fighterDic[obj.aod_type][obj.type] = Array();
		end
		if self.fighterDic[obj.aod_type][obj.type]:indexOf(id) == -1 then
			self.fighterDic[obj.aod_type][obj.type]:add(id);
		end

		if self.fightStandDic[obj.aod_type] == nil then
			self.fightStandDic[obj.aod_type] = {};
		end
		self.fightStandDic[obj.aod_type][obj.pos] = id;

		if self.fighterIdDic[id] == nil then
			self.fighterIdDic[id] = {};
		end
		self.fighterIdDic[id] = tonumber(obj.sort_id);
	end
end

function Lua_FightRecordVo:getFighterByPos(side, pos)
	if self.fightStandDic == nil or self.fightStandDic[side] == nil or self.fightStandDic[side][pos] == nil then
		return 0;
	end
	return self.fightStandDic[side][pos];
end

function Lua_FightRecordVo:getFighterByHero(side, heroId)
	if self.fightStandDic == nil or self.fighterIdDic == nil then
		return -1, 0;
	end

	if side == 0 then
		if self.fightStandDic[Lua_FightDefine.FIGHT_ATTACKER] ~= nil then
			for pos, id in pairs(self.fightStandDic[Lua_FightDefine.FIGHT_ATTACKER]) do
		        if self.fighterIdDic[id] ~= nil and (heroId <= 0 or self.fighterIdDic[id] == heroId) then
		        	return pos, id;
		        end
		    end
		end

		if self.fightStandDic[Lua_FightDefine.FIGHT_DEFENDER] ~= nil then
			for pos, id in pairs(self.fightStandDic[Lua_FightDefine.FIGHT_DEFENDER]) do
		        if self.fighterIdDic[id] ~= nil and (heroId <= 0 or self.fighterIdDic[id] == heroId) then
		        	return pos, id;
		        end
		    end
		end
	elseif self.fightStandDic[side] ~= nil then
		print("getFighterByHero", side, heroId);
		for pos, id in pairs(self.fightStandDic[side]) do
	        if self.fighterIdDic[id] ~= nil and (heroId <= 0 or self.fighterIdDic[id] == heroId) then
	        	return pos, id;
	        end
	    end
	end
	return -1, 0;
end

function Lua_FightRecordVo:getFighterArr(aod_type, t)
	if self.fighterDic == nil or self.fighterDic[aod_type] == nil then
		return nil;
	end
	return self.fighterDic[aod_type][t];
end

function Lua_FightRecordVo:getFighterInfo(id)
	if self.fighterIdDic == nil then
		return nil;
	end
	return self.fighterIdDic[id];
end





---------------------------------------------------------------
---------------------------------------------------------------
---------------------------------------------------------------
function Lua_FightRecordVo:setResult(msg)
	if msg == nil then
		return
	end
	self._result = msg.result
end





---------------------------------------------------------------
---------------------------------------------------------------
---------------------------------------------------------------
function Lua_FightRecordVo.Get:bossHurt()
    return self._bossHurt or 0;
end

function Lua_FightRecordVo.Set:bossHurt(value)
	value = Mathf.Abs(value)
	if self._bossHurt == nil then
		self._bossHurt = value;
	else
	    self._bossHurt = self._bossHurt + value;
	end
	Lua_MsgUtil.callFunc("FIGHT_BOSSHURT_UPDATE", value);
end





---------------------------------------------------------------
---------------------------------------------------------------
---------------------------------------------------------------
function Lua_FightRecordVo.Get:starCondArr()
	if self._starCondArr == nil then
		if self.templateId == Lua_FightDefine.TEMPLATE_ID_STARTRIAL then
			self._starCondArr = DataCenter.starTrialUitl:getStageVo(self.stageId).starCondArr
		elseif self.templateId == Lua_FightDefine.TEMPLATE_ID_KUNXU then
			self._starCondArr = DataCenter.kunXuHolyLandData:getLevelInfoById(self.stageId).starCondArr
		end
	end
	return self._starCondArr
end

-- 战斗星级
-- 检查战斗结果 condType = 1
function Lua_FightRecordVo:checkResult()
	if self.starCondArr then
		local result = self.result
		if self.templateId == Lua_FightDefine.TEMPLATE_ID_KUNXU then
			result = 1
			local arr = DataCenter.fightData:getTeamArrById(self.teamId)
			local vo
			local player
			for i = 1, arr:getSize() do
				vo = DataCenter.fightData:getRecordVoById(arr[i])
				player = DataCenter.fightData:getFightPlayerById(arr[i])
				if vo.result == 2 or (player and player.isPlayEnd ~= true) then
					result = 2
				end
			end
		end
		for i = 1, self.starCondArr:getSize() do
			self.starCondArr[i]:checkResult(result)
		end
		Lua_MsgUtil.callFunc("FIGHTPLAY_CHECK_UPDATE", self.recordId)
	end
end

-- 检查阵亡人数 condType = 2
function Lua_FightRecordVo:checkDeath()
	if self.starCondArr then
		local num = ThingUtil.instance:getSurviveNum(self.sceneId, self.recordId, self.ownType);
		if self.totalNum == nil then
			self.totalNum = num;
		end
		self.deathNum = self.totalNum - num;

		local deathNum = self.deathNum
		if self.templateId == Lua_FightDefine.TEMPLATE_ID_KUNXU then
			deathNum = 0
			local arr = DataCenter.fightData:getTeamArrById(self.teamId)
			local vo
			for i = 1, arr:getSize() do
				vo = DataCenter.fightData:getRecordVoById(arr[i])
				deathNum = deathNum + (vo.deathNum or 0)
			end
		end
		for i = 1, self.starCondArr:getSize() do
			self.starCondArr[i]:checkDeath(deathNum)
		end
		Lua_MsgUtil.callFunc("FIGHTPLAY_CHECK_UPDATE", self.recordId);
	end
end

-- 检查剩余血量 condType = 3
function Lua_FightRecordVo:checkHealth()
	-- if self.starCondArr then
	-- 	local num = tonumber(tostring(ThingUtil.instance:getSurviveHp(self.sceneId, self.recordId, self.ownType)));
	-- 	if self.totalHp == nil then
	-- 		self.totalHp = num;
	-- 	end
	-- 	self.curHp = num;
	-- 	num = self.curHp / self.totalHp;
	-- 	num = tonumber(string.format("%.3f", num))

	-- 	for i = 1, self.starCondArr:getSize() do
	-- 		self.starCondArr[i]:checkHealth(num);
	-- 	end
	-- 	Lua_MsgUtil.callFunc("FIGHTPLAY_CHECK_UPDATE", self.recordId);
	-- end

	if self.templateId == Lua_FightDefine.TEMPLATE_ID_COPY_MORE
	or self.templateId == Lua_FightDefine.TEMPLATE_ID_KUNXU then
		local num = tonumber(tostring(ThingUtil.instance:getSurviveHp(self.sceneId, self.recordId, Lua_FightDefine.FIGHT_ATTACKER)));
		if self.attackerMaxHp == nil then
			self.attackerMaxHp = num;
		end
		self.attackerHp = num;

		num = tonumber(tostring(ThingUtil.instance:getSurviveHp(self.sceneId, self.recordId, Lua_FightDefine.FIGHT_DEFENDER)));
		if self.defenderMaxHp == nil then
			self.defenderMaxHp = num;
		end
		self.defenderHp = num;

		Lua_MsgUtil.callFunc("FIGHTPLAY_CHECK_UPDATE");
	end
end

-- 检查英雄阵营和职业
-- function Lua_FightRecordVo:checkCampAndCareer(campCareerDic, campDic, careerDic, playerNum)
-- 	if self.starCondArr then
-- 		for i = 1, self.starCondArr:getSize() do
-- 			self.starCondArr[i]:checkCampAndCareerMax(campCareerDic, campDic, careerDic, playerNum)
-- 			self.starCondArr[i]:checkCampAndCareerMin(campCareerDic, campDic, careerDic, playerNum)
-- 		end
-- 		Lua_MsgUtil.callFunc("FIGHTPLAY_CHECK_UPDATE", self.recordId);
-- 	end
-- end

-- 检查回合数 condType = 6
function Lua_FightRecordVo:checkRound(round)
	if self.starCondArr then
		for i = 1, self.starCondArr:getSize() do
			self.starCondArr[i]:checkRound(round);
		end
		Lua_MsgUtil.callFunc("FIGHTPLAY_CHECK_UPDATE", self.recordId);
	end

	-- 寂灭岛评分
	if self.templateId == Lua_FightDefine.TEMPLATE_ID_COPY_SINGLE or self.templateId == Lua_FightDefine.TEMPLATE_ID_COPY_MORE then
		Lua_MsgUtil.callFunc("FIGHTPLAY_CHECK_UPDATE");
	end
end










---------------------------------------------------------------
---------------------------------------------------------------
---------------------------------------------------------------
-- 英雄连携技能
function Lua_FightRecordVo:checkConnect()
	local player = DataCenter.fightData:getFightPlayerById(self.recordId)
	local bUpdateAttacker = self.bCheckConn ~= false or (self.templateId ~= 1005 and self.templateId ~= 1006)
	local list = player.fighter_list
	local id
	local aHeroDic = {}
	local aCampDic = {}
	local aCareerDic = {}
	local dHeroDic = {}
	local dCampDic = {}
	local dCareerDic = {}
	local config
	for i = 1, #list do
		if list[i].type ~= 3 then
			id = tonumber(tostring(list[i].sort_id))
			config = Lua_HeroConfigData.getConfig(id)
			if config == nil then
				config = Lua_MonsterConfigData.getMonsterConfig(id)
				config = Lua_HeroConfigData.getConfig(config.heroId)
			end
			if list[i].aod_type == Lua_FightDefine.FIGHT_ATTACKER then
				aHeroDic[config.id] = config.id
				if aCampDic[config.camp] == nil then
					aCampDic[config.camp] = Array()
				end
					aCampDic[config.camp]:add(config.id)

				if aCareerDic[config.career] == nil then
					aCareerDic[config.career] = Array()
				end
				aCareerDic[config.career]:add(config.id)
			else
				dHeroDic[config.id] = config.id
				if dCampDic[config.camp] == nil then
					dCampDic[config.camp] = Array()
				end
				dCampDic[config.camp]:add(config.id)

				if dCareerDic[config.career] == nil then
					dCareerDic[config.career] = Array()
				end
				dCareerDic[config.career]:add(config.id)
			end
		end
	end

	local vo
	if bUpdateAttacker then
		self.attackerConnNum = 0
		if self.attackerConnArr == nil then
			self.attackerConnArr = Array()
		else
			local len = self.attackerConnArr:getSize()
			while len > 0 do
				vo = self.attackerConnArr:remove(len)
				vo:remove()
				vo = nil
				len = len - 1
			end
		end
	end
	self.defenderConnNum = 0
	if self.defenderConnArr == nil then
		self.defenderConnArr = Array()
	else
		local len = self.defenderConnArr:getSize()
		while len > 0 do
			vo = self.defenderConnArr:remove(len)
			vo:remove()
			vo = nil
			len = len - 1
		end
	end
	local dConnDic = {}
	for i = 1, #list do
		if list[i].type ~= 3 then
			id = tonumber(tostring(list[i].sort_id))
			config = Lua_HeroConfigData.getConfig(id)
			if config == nil then
				config = Lua_MonsterConfigData.getMonsterConfig(id)
				config = Lua_HeroConfigData.getConfig(config.heroId)
			end
			if list[i].aod_type == Lua_FightDefine.FIGHT_ATTACKER then
				if bUpdateAttacker and config.connect_id > 0 then
					vo = Lua_HeroConnectVo.new(config.connect_id)
					vo:setData(aHeroDic, aCampDic, aCareerDic)
					if vo.isActivate then
						self.attackerConnNum = self.attackerConnNum + 1
					end
					self.attackerConnArr:add(vo)
				end
			else
				if config.connect_id > 0 and dConnDic[config.connect_id] == nil then
					vo = Lua_HeroConnectVo.new(config.connect_id)
					vo:setData(dHeroDic, dCampDic, dCareerDic)
					if vo.isActivate then
						self.defenderConnNum = self.defenderConnNum + 1
					end
					dConnDic[config.connect_id] = 1
					self.defenderConnArr:add(vo)
				end
			end
		end
	end
	self.attackerConnArr:sort("sortIdx", false)
	self.defenderConnArr:sort("sortIdx", false)

	aHeroDic = nil
	aCampDic = nil
	aCareerDic = nil
	dHeroDic = nil
	dCampDic = nil
	dCareerDic = nil
	dConnDic = nil
	self.bCheckConn = false
end

function Lua_FightRecordVo:clearConnect()
	if self.attackerConnArr then
		local len = self.attackerConnArr:getSize()
		local vo
		while len > 0 do
			vo = self.attackerConnArr:remove(len)
			vo:remove()
			vo = nil
			len = len - 1
		end
		self.attackerConnArr = nil
	end
	if self.defenderConnArr then
		local len = self.defenderConnArr:getSize()
		local vo
		while len > 0 do
			vo = self.defenderConnArr:remove(len)
			vo:remove()
			vo = nil
			len = len - 1
		end
		self.defenderConnArr = nil
	end
end










---------------------------------------------------------------
---------------------------------------------------------------
---------------------------------------------------------------
-- 战斗记录
function Lua_FightRecordVo:writeFightRecord()
	FightPlayManager.instance:addFightRecordWriteStr(self.recordId, "战斗记录："..self.recordId, true);
	FightPlayManager.instance:addFightRecordWriteStr(self.recordId, "战斗模板："..self.templateConfig.title, false);
	FightPlayManager.instance:addFightRecordWriteStr(self.recordId, "攻击方："..self.attacker.name, false);
	FightPlayManager.instance:addFightRecordWriteStr(self.recordId, "防守方："..self.defender.name, false);

	local arr = Array();
	for k, _ in pairs(self.recordInfoDic) do
		arr:add(k);
	end
	arr:sortf(self, self.recordSort);

	local key;
	local recordFighterDic = {};
	for i = 1, arr:getSize() do
		key = arr[i];
		self:writeSingleRecord(self.recordInfoDic[key], recordFighterDic);
	end

	FightPlayManager.instance:writeFightRecord(self.recordId);
end

function Lua_FightRecordVo:recordSort(a, b)
	if a > b then
		return false;
	end
	return true;
end

function Lua_FightRecordVo:writeSingleRecord(arr, recordFighterDic)
	if arr == nil or arr:getSize() <= 0 then
		return;
	end
	
	local info;
	local str;
	local obj;
	local rn = -1;
	for i = 1, arr:getSize() do
		info = arr[i];
		str = "";

		if i == 1 then
			FightPlayManager.instance:addFightRecordWriteStr(self.recordId, "", false);
			FightPlayManager.instance:addFightRecordWriteStr(self.recordId, "当前轮次："..info.rotation.."，当前场次："..info.session, false);
		end
		
		if info.round ~= rn then
			rn = info.round;
			FightPlayManager.instance:addFightRecordWriteStr(self.recordId, "  当前回合："..info.round, false);
		end

		str = "";
		if info.type == 1 or info.type == 11 then
			if info.type == 1 then
				str = "    单位进场：\n";
			else
				str = "    召唤单位进场：\n";
			end

			for j = 1, #info.fighter_list do
				local v = info.fighter_list[j];
				if v.aod_type == 1 then
					if v.type == 0 then
						obj = Lua_HeroConfigData.getConfig(v.sort_id);
						recordFighterDic[v.id] = { info = v, name = "攻方"..obj.name, pos = v.pos };
						str = str.."      攻方英雄：";
					elseif v.type == 1 or v.type == 4 then
						obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
						recordFighterDic[v.id] = { info = v, name = "攻方"..obj.name, pos = v.pos };
						str = str.."      攻方怪物：";
					elseif v.type == 2 then
						obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
						recordFighterDic[v.id] = { info = v, name = "攻方"..obj.name, pos = v.pos };
						str = str.."      攻方BOSS：";
					elseif v.type == 3 then
						recordFighterDic[v.id] = { info = v, name = "攻方主角", pos = v.pos };
						str = str.."      攻方主角：";
					elseif v.type == 5 then
						obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
						recordFighterDic[v.id] = { info = v, name = "攻方"..obj.name, pos = v.pos };
						str = str.."      攻方木桩BOSS：";
					end
				elseif v.aod_type == 2 then
					if v.type == 0 then
						obj = Lua_HeroConfigData.getConfig(v.sort_id);
						recordFighterDic[v.id] = { info = v, name = "守方"..obj.name, pos = v.pos };
						str = str.."      守方英雄：";
					elseif v.type == 1 or v.type == 4 then
						obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
						recordFighterDic[v.id] = { info = v, name = "守方"..obj.name, pos = v.pos };
						str = str.."      守方怪物：";
					elseif v.type == 2 then
						obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
						recordFighterDic[v.id] = { info = v, name = "守方"..obj.name, pos = v.pos };
						str = str.."      守方BOSS：";
					elseif v.type == 3 then
						recordFighterDic[v.id] = { info = v, name = "守方主角", pos = v.pos };
						str = str.."      守方主角：";
					elseif v.type == 5 then
						obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
						recordFighterDic[v.id] = { info = v, name = "守方"..obj.name, pos = v.pos };
						str = str.."      守方木桩BOSS：";
					end
				end

				str = str..v.id..recordFighterDic[v.id].name.."，位置："..v.pos.."，最大生命值："..v.max_health.."，当前生命值："..v.health;
				if v.buff_list ~= nil and #v.buff_list > 0 then
					str = str.."，拥有BUFF："
					for k = 1, #v.buff_list do
						str = str..v.buff_list[k].id;
						obj = Lua_BuffConfigData.getConfigData(v.buff_list[k].sort_id);
						if obj ~= nil and obj.name ~= nil then
							str = str..v.buff_list[k].sort_id..obj.name.."，";
						else
							str = str..v.buff_list[k].sort_id.."，";
						end
						if k < #v.buff_list then
							str = str.."护盾值上限："..v.buff_list[k].shield_upper.."，";
						else
							str = str.."护盾值上限："..v.buff_list[k].shield_upper;
						end
					end
				end
				str = str.."\n";
			end

			FightPlayManager.instance:addFightRecordWriteStr(self.recordId, str, false);
			str = self:recordPropInfo("", info.fight_prop, recordFighterDic);
			FightPlayManager.instance:addFightRecordWriteStr(self.recordId, str, false);
		elseif info.type == 12 then
			str = "    播放剧情："..info.event_id.."\n";
			FightPlayManager.instance:addFightRecordWriteStr(self.recordId, str, false);
		else
			if info.type == 2 then -- 技能释放
				str = "    释放技能：\n";
				str = str .. self:recordPropInfo("行动前", info.fight_prop, recordFighterDic);
				if tonumber(info.fighter_id) > 0 then
					str = str.."      释放单位："..info.fighter_id..recordFighterDic[info.fighter_id].name.." 位置"..recordFighterDic[info.fighter_id].pos.."\n"
				end
				obj = Lua_SkillConfigData.getConfigData(info.skill_id);
				str = str .. "      技能ID："..info.skill_id.."\n";
				str = str .. "      技能名："..obj.name.."\n";
				str = str .. self:getHurtStrInfo("        ", info, recordFighterDic);
			elseif info.type == 3 then -- buff变化
				str = "    buff变化：\n";
				str = str .. self:recordPropInfo("行动前", info.fight_prop, recordFighterDic);
				str = str .. self:getHurtStrInfo("        ", info, recordFighterDic);
			elseif info.type == 4 then -- buff伤害
				str = "    buff伤害：\n";
				str = str .. self:recordPropInfo("行动前", info.fight_prop, recordFighterDic);
				str = str .. self:getHurtStrInfo("        ", info, recordFighterDic);
			elseif info.type == 5 then -- 不可行动
				if tonumber(info.fighter_id) > 0 then
					str = "      单位："..info.fighter_id..recordFighterDic[info.fighter_id].name.." 位置"..recordFighterDic[info.fighter_id].pos.."\n"
				end
				str = str.."    不可行动\n"
				-- obj = Lua_BuffConfigData.getConfigData(info.buff_id);
				-- str = "    不可行动：\n      受BUFF："..info.buff_id.."("..obj.name..")控制后不可行动\n";
				-- str = str .. self:recordPropInfo("行动前", info.fight_prop, recordFighterDic);
				-- str = str .. self:getHurtStrInfo("      ", info, recordFighterDic);
			elseif info.type == 6 then -- 攻方退场
				str = "    攻方退场\n";
				str = str .. self:recordPropInfo("行动前", info.fight_prop, recordFighterDic);
			elseif info.type == 7 then -- 守方退场
				str = "    守方退场\n";
				str = str .. self:recordPropInfo("行动前", info.fight_prop, recordFighterDic);
			elseif info.type == 8 then -- 更新攻击方主角技能冷却
				str = "    更新浣熊技能冷却：\n";
				if tonumber(info.fighter_id) > 0 then
					str = str.."      释放单位："..info.fighter_id..recordFighterDic[info.fighter_id].name.." 位置"..recordFighterDic[info.fighter_id].pos.."\n"
				end
				str = str .. self:recordPropInfo("行动前", info.fight_prop, recordFighterDic);
				for j = 1, #info.skill_list do
					obj = info.skill_list[j];
					local cfg = Lua_SkillConfigData.getConfigData(obj.skill_id);
					str = str.."      "..obj.index..cfg.name.."("..obj.skill_id..")技能释放回合 "..obj.active_round;
					if obj.is_cool == 1 then
						str = str.."  技能冷却中\n"
					else
						str = str.."  技能不在冷却中\n"
					end
				end
				str = str.."    当前能量值："..info.energy.."\n";
			elseif info.type == 9 then -- 站位变化
				str = "    站位变化：\n";
				str = str .. self:recordPropInfo("行动前", info.fight_prop, recordFighterDic);
				for j = 1, #info.fighter_list do
					obj = info.fighter_list[j];
					if recordFighterDic[obj.id] ~= nil then
						str = str.."      "..obj.id.."("..recordFighterDic[obj.id].name..")从位置 "..recordFighterDic[obj.id].pos.." 移到 "..obj.pos.."\n";
						recordFighterDic[obj.id].pos = obj.pos;
						recordFighterDic[obj.id].info = obj;
					else
						str = str.."      匪夷所思的移动"..obj.id.." 移到 "..obj.pos.."\n";
					end
				end
			elseif info.type == 10 then -- 销毁场地效果
				str = "    销毁场地效果\n";
				str = str .. self:recordPropInfo("行动前", info.fight_prop, recordFighterDic);
				local site_effect = info.site_effect;
				if site_effect ~= nil then
					for j = 1, #site_effect do
						obj = site_effect[j];
						local cfg = Lua_BuffConfigData.getConfigData(obj.buff_id);
						if obj.aod_pos == Lua_FightDefine.FIGHT_ATTACKER then
							str = str.."      销毁攻方场地效果 "..obj.buff_id.."("..cfg.name..")\n";
						elseif obj.aod_pos == Lua_FightDefine.FIGHT_DEFENDER then
							str = str.."      销毁守方场地效果 "..obj.buff_id.."("..cfg.name..")\n";
						else
							str = str.."      销毁中间场地效果 "..obj.buff_id.."("..cfg.name..")\n";
						end
					end
				end

				-- 刷新buff
				str = str .. self:recordBuff("        ", info, recordFighterDic);
			end

			FightPlayManager.instance:addFightRecordWriteStr(self.recordId, str, false);
		end
	end
	FightPlayManager.instance:addFightRecordWriteStr(self.recordId, "当前轮次："..info.rotation.."当前场次："..info.session.."，回合数："..info.round.." 结束", false);
end

-- 属性
function Lua_FightRecordVo:recordPropInfo(preStr, list, recordFighterDic)
	if list ~= nil and #list > 0 then
		local bHasProp = false;
		local str = ""
		for j = 1, #list do
			local fight_prop = list[j];
			if fight_prop.prop ~= nil and #fight_prop.prop > 0 and recordFighterDic[fight_prop.id] ~= nil then
				local attrStr = DataCenter.attributesUtil:getListAttrStr(fight_prop.prop, "、");
				if not strIsNullOrEmpty(attrStr) then
					bHasProp = true;
					str = str.."        "..fight_prop.id.."("..recordFighterDic[fight_prop.id].name.." 位置"..recordFighterDic[fight_prop.id].pos.."):";
					str = str..attrStr.."\n";
				end
			end
		end

		if bHasProp == true then
			str = "      "..preStr.."属性：\n" .. str;
			return str;
		end
	end
	return "";
end

function Lua_FightRecordVo:getHurtStrInfo(preStr, info, recordFighterDic)
	local str = "";
	if info == nil then
		return str;
	end

	-- 复活
	if info.fighter_list ~= nil and #info.fighter_list > 0 then
		for i = 1, #info.fighter_list do
			local v = info.fighter_list[i];
			if v.aod_type == 1 then
				if v.type == 0 then
					obj = Lua_HeroConfigData.getConfig(v.sort_id);
					recordFighterDic[v.id] = { info = v, name = "攻方"..obj.name, pos = v.pos };
					str = str..preStr.."      复活攻方英雄：";
				elseif v.type == 1 or v.type == 4 then
					obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
					recordFighterDic[v.id] = { info = v, name = "攻方"..obj.name, pos = v.pos };
					str = str..preStr.."      复活攻方怪物：";
				elseif v.type == 2 then
					obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
					recordFighterDic[v.id] = { info = v, name = "攻方"..obj.name, pos = v.pos };
					str = str..preStr.."      复活攻方BOSS：";
				elseif v.type == 3 then
					recordFighterDic[v.id] = { info = v, name = "攻方主角", pos = v.pos };
					str = str..preStr.."      复活攻方主角：";
				elseif v.type == 5 then
					obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
					recordFighterDic[v.id] = { info = v, name = "攻方"..obj.name, pos = v.pos };
					str = str..preStr.."      复活攻方木桩BOSS：";
				end
			elseif v.aod_type == 2 then
				if v.type == 0 then
					obj = Lua_HeroConfigData.getConfig(v.sort_id);
					recordFighterDic[v.id] = { info = v, name = "守方"..obj.name, pos = v.pos };
					str = str..preStr.."      复活守方英雄：";
				elseif v.type == 1 or v.type == 4 then
					obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
					recordFighterDic[v.id] = { info = v, name = "守方"..obj.name, pos = v.pos };
					str = str..preStr.."      复活守方怪物：";
				elseif v.type == 2 then
					obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
					recordFighterDic[v.id] = { info = v, name = "守方"..obj.name, pos = v.pos };
					str = str..preStr.."      复活守方BOSS：";
				elseif v.type == 3 then
					recordFighterDic[v.id] = { info = v, name = "守方主角", pos = v.pos };
					str = str..preStr.."      复活守方主角：";
				elseif v.type == 5 then
					obj = Lua_MonsterConfigData.getMonsterConfig(v.sort_id);
					recordFighterDic[v.id] = { info = v, name = "守方"..obj.name, pos = v.pos };
					str = str..preStr.."      复活守方木桩BOSS：";
				end
			end

			str = str..v.id..recordFighterDic[v.id].name.."，位置："..v.pos.."，最大生命值："..v.max_health.."，当前生命值："..v.health;
			if v.buff_list ~= nil and #v.buff_list > 0 then
				str = str.."，拥有BUFF："
				for k = 1, #v.buff_list do
					str = str..v.buff_list[k].id;
					obj = Lua_BuffConfigData.getConfigData(v.buff_list[k].sort_id);
					if obj ~= nil and obj.name ~= nil then
						str = str..v.buff_list[k].sort_id..obj.name.."，";
					else
						str = str..v.buff_list[k].sort_id.."，";
					end
					if k < #v.buff_list then
						str = str.."护盾值上限："..v.buff_list[k].shield_upper.."，";
					else
						str = str.."护盾值上限："..v.buff_list[k].shield_upper;
					end
				end
			end
			str = str.."\n";
		end
	end

	-- 伤害
	if info.multi_attack ~= nil and #info.multi_attack > 0 then
		for i = 1, #info.multi_attack do
			local obj = info.multi_attack[i];
			-- 伤害列表
			local list = obj.attack_list;
			if list ~= nil and #list > 0 then
				str = str..preStr.."第"..i.."段伤害\n";
				for j = 1, #list do
					local hurtObj = list[j];
					if hurtObj.is_target == 0 then
						str = str..preStr.."非技能目标："..hurtObj.id.."("..recordFighterDic[hurtObj.id].name.." 位置"..recordFighterDic[hurtObj.id].pos..")";
					elseif hurtObj.is_target == 1 then
						str = str..preStr.."是技能目标："..hurtObj.id.."("..recordFighterDic[hurtObj.id].name.." 位置"..recordFighterDic[hurtObj.id].pos..")";
					else
						str = str..preStr..hurtObj.id.."("..recordFighterDic[hurtObj.id].name.." "..recordFighterDic[hurtObj.id].pos..")";
					end
					str = str.."，伤害前的护盾值是："..hurtObj.shield_value;
					local hurt_list = hurtObj.hurt_list;
					local haveBuff = false;
					if hurt_list ~= nil and #hurt_list > 0 then
						str = str.."，";
						for n = 1, #hurt_list do
							local hurtListObj = hurt_list[n];
							if hurtListObj.hurt ~= nil and #hurtListObj.hurt > 0 then
								for k = 1, #hurtListObj.hurt do
									local hurt = hurtListObj.hurt[k];
									if hurt.type == 0 then
										str = str.."造成普通伤害"..hurt.value;
									elseif hurt.type == 1 then
										str = str.."造成暴击伤害"..hurt.value;
									elseif hurt.type == 2 then
										str = str.."造成护盾吸收伤害"..hurt.value;
									elseif hurt.type == 3 then
										str = str.."造成buff伤害"..hurt.value;
									elseif hurt.type == 4 then
										str = str.."造成克制伤害"..hurt.value;
									elseif hurt.type == 5 then
										str = str.."造成克制抵挡伤害"..hurt.value;
									elseif hurt.type == 6 then
										str = str.."造成回血"..hurt.value;
									elseif hurt.type == 7 then
										str = str.."造成闪避";
									elseif hurt.type == 8 then
										str = str.."造成格挡伤害";
									elseif hurt.type == 9 then
										str = str.."免疫";
									else
										str = str.."伤害类型 "..hurt.type.." 伤害值 "..hurt.value;
									end

									if k < #hurtListObj.hurt then
										str = str.."，";
									end
								end
							end

							local buffStr = self:recordBuff(preStr, hurtListObj, recordFighterDic, "");
							if not strIsNullOrEmpty(buffStr) then
								haveBuff = true;
								str = str .."\n".. buffStr;
							end
						end
					end
					
					if haveBuff then
						str = str.."\n"..preStr.."变化后的血量："..hurtObj.health.."\n";
					else
						str = str.."，变化后的血量："..hurtObj.health.."\n";
					end
				end
			end

			-- 溅射列表
			list = obj.sputter_list;
			if list ~= nil and #list > 0 then
				str = str..preStr.."第"..i.."段溅射\n";
				for j = 1, #list do
					local sputterObj = list[j];
					local from_id = sputterObj.from_id;
					local sputter_hurt = sputterObj.sputter_hurt;
					if recordFighterDic[from_id] ~= nil then
						str = str..preStr.."  溅射来源："..from_id.."("..recordFighterDic[from_id].name.." 位置"..recordFighterDic[from_id].pos..")\n";
					else
						str = str..preStr.."  溅射来源："..from_id.."\n";
					end
					if sputter_hurt ~= nil and #sputter_hurt > 0 then
						for m = 1, #sputter_hurt do
							local hurtObj = sputter_hurt[m];
							if hurtObj.is_target == 0 or hurtObj.is_target == 1 then
								str = str..preStr.."    溅射目标："..hurtObj.id.."("..recordFighterDic[hurtObj.id].name.." 位置"..recordFighterDic[hurtObj.id].pos..")";
							else
								str = str..preStr.."    溅射"..hurtObj.id.."("..recordFighterDic[hurtObj.id].name.." "..recordFighterDic[hurtObj.id].pos..")";
							end
							str = str.."，伤害前的护盾值是："..hurtObj.shield_value;
							local hurt_list = hurtObj.hurt_list;
							if hurt_list ~= nil and #hurt_list > 0 then
								str = str.."，";
								for n = 1, #hurt_list do
									local hurtListObj = hurt_list[n];
									if hurtListObj.hurt ~= nil and #hurtListObj.hurt > 0 then
										for k = 1, #hurtListObj.hurt do
											local hurt = hurtListObj.hurt[k];
											if hurt.type == 0 then
												str = str.."造成普通伤害"..hurt.value;
											elseif hurt.type == 1 then
												str = str.."造成暴击伤害"..hurt.value;
											elseif hurt.type == 2 then
												str = str.."造成护盾吸收伤害"..hurt.value;
											elseif hurt.type == 3 then
												str = str.."造成buff伤害"..hurt.value;
											elseif hurt.type == 4 then
												str = str.."造成克制伤害"..hurt.value;
											elseif hurt.type == 5 then
												str = str.."造成克制抵挡伤害"..hurt.value;
											elseif hurt.type == 6 then
												str = str.."造成回血"..hurt.value;
											elseif hurt.type == 7 then
												str = str.."造成闪避";
											elseif hurt.type == 8 then
												str = str.."造成格挡伤害";
											elseif hurt.type == 9 then
												str = str.."免疫";
											else
												str = str.."伤害类型 "..hurt.type.." 伤害值 "..hurt.value;
											end

											if k < #hurtListObj.hurt then
												str = str.."，";
											end
										end
									end 
								end
							end
									
							str = str.."，变化后的血量："..hurtObj.health.."\n";
						end
					end
				end
			end
		end
	end

	-- buff
	str = str .. self:recordBuff(preStr, info, recordFighterDic);
	
	-- 场地特效移除
	if info.site_effect ~= nil and #info.site_effect > 0 then
		for j = 1, #info.site_effect do
			local obj = info.site_effect[j];
			local cfg = Lua_BuffConfigData.getConfigData(obj.buff_id);
			if obj.aod_pos == Lua_FightDefine.FIGHT_ATTACKER then
				str = str..preStr.."      增加攻方场地效果 "..obj.buff_id.."("..cfg.name..")\n";
			elseif obj.aod_pos == Lua_FightDefine.FIGHT_DEFENDER then
				str = str..preStr.."      增加守方场地效果 "..obj.buff_id.."("..cfg.name..")\n";
			else
				str = str..preStr.."      增加中间场地效果 "..obj.buff_id.."("..cfg.name..")\n";
			end
		end
	end

	-- 位移还原
	if info.shift_pos ~= nil and #info.shift_pos > 0 then
		for i = 1, #info.shift_pos do
			local obj = info.shift_pos[i];
			local pos = tonumber(tostring(obj.value));
			local rid = obj.key;
			str = str..preStr..rid.."("..recordFighterDic[rid].name..")从位置 "..recordFighterDic[rid].pos.." 移到 "..pos.."\n";
			recordFighterDic[rid].pos = pos;
		end
	end

	return str;
end

function Lua_FightRecordVo:recordBuff(preStr, info, recordFighterDic, lastStr)
	local str = "";
	if info ~= nil and info.buff_list ~= nil and #info.buff_list > 0 then
		lastStr = lastStr == nil and "\n" or lastStr;
		for j = 1, #info.buff_list do
			obj = info.buff_list[j];
			str = str..preStr..obj.id.."("..recordFighterDic[obj.id].name.." 位置"..recordFighterDic[obj.id].pos..")";
			if obj.pre_buff ~= nil and #obj.pre_buff > 0 then
				str = str.."战前新增BUFF：";
				for k = 1, #obj.pre_buff do
					local cfg = Lua_BuffConfigData.getConfigData(obj.pre_buff[k]);
					if cfg ~= nil and cfg.name ~= nil then
						str = str..cfg.name.."("..obj.pre_buff[k]..")";
					else
						str = str.."("..obj.pre_buff[k]..")";
					end

					if k == #obj.pre_buff then
						str = str.."，";
					else
						str = str.."、";
					end
				end
			end

			if obj.add_buff ~= nil and #obj.add_buff > 0 then
				str = str.."新增BUFF：";
				for k = 1, #obj.add_buff do
					local cfg = Lua_BuffConfigData.getConfigData(obj.add_buff[k]);
					if cfg ~= nil and cfg.name ~= nil then
						str = str..cfg.name.."("..obj.add_buff[k]..")";
					else
						str = str.."("..obj.add_buff[k]..")";
					end

					if k == #obj.add_buff then
						str = str.."，";
					else
						str = str.."、";
					end
				end
			end

			str = str .. "拥有BUFF：";
			local buff_list = obj.buff_list;
			if buff_list ~= nil and #buff_list > 0 then
				for k = 1, #buff_list do
					local buffObj = buff_list[k];
					str = str..buffObj.id;
					local cfg = Lua_BuffConfigData.getConfigData(buffObj.sort_id);
					if k < #buff_list then
						str = str..cfg.name.."("..buffObj.sort_id..")、";
					else
						str = str..cfg.name.."("..buffObj.sort_id..")";
					end
				end
				str = str..lastStr;
			else
				str = str.."身上暂无buff"..lastStr;
			end
		end
	end

	return str;
end