Lua_StarCondVo = class("Lua_StarCondVo")
function Lua_StarCondVo:ctor(templateId)
    self.templateId = templateId
    self.condArr = Array()
end

function Lua_StarCondVo:parseCond(cond)
    if cond == nil then
        return
    end

    local vo
    if type(cond[1]) == "number" then -- 一个条件
        vo = Lua_SingleStarCondVo.new(self.templateId, cond[1])
        vo:parseCond(cond[2])
        self.condArr:add(vo)
        self._desc = vo.desc
    else
        for i = 1, #cond do
            vo = Lua_SingleStarCondVo.new(self.templateId, cond[i][1])
            vo:parseCond(cond[i][2])
            self.condArr:add(vo)
            if self._desc == nil then
                self._desc = vo.desc
            else
                self._desc = LanUtil.formateStr(10046, self.desc, vo.desc)
            end
        end
    end
end

function Lua_StarCondVo:init()
    for i = 1, self.condArr:getSize() do
        self.condArr[i]:init()
    end
end

function Lua_StarCondVo:getUnMeetVo()
    for i = 1, self.condArr:getSize() do
        if not self.condArr[i].bMeet then
            return self.condArr[i]
        end
    end
    return nil
end

function Lua_StarCondVo.Get:desc()
    return self._desc
end

function Lua_StarCondVo.Get:isMeet()
    for i = 1, self.condArr:getSize() do
        if not self.condArr[i].bMeet then
            return false
        end
    end
    return true
end

-- 战斗结果
function Lua_StarCondVo:checkResult(value)
    for i = 1, self.condArr:getSize() do
        if self.condArr[i].condType == 1 then
            self.condArr[i]:checkResult(value)
        end
    end
end

-- 阵亡人数
function Lua_StarCondVo:checkDeath(value)
    for i = 1, self.condArr:getSize() do
        if self.condArr[i].condType == 2 then
            self.condArr[i]:checkResult(value)
        end
    end
end

-- 剩余血量
function Lua_StarCondVo:checkHealth(value)
    for i = 1, self.condArr:getSize() do
        if self.condArr[i].condType == 3 then
            self.condArr[i]:checkHealth(value)
        end
    end
end

function Lua_StarCondVo:checkCampAndCareerMax(campCareerDic, campDic, careerDic, playerNum)
    for i = 1, self.condArr:getSize() do
        if self.condArr[i].condType == 4 then -- 阵营和职业不少于
            self.condArr[i]:checkCampAndCareerMax(campCareerDic, campDic, careerDic, playerNum)
        end
        if self.condArr[i].condType == 7 then -- 两个阵营中一个不少于
            self.condArr[i]:checkCampAndCareerMax2(campCareerDic, campDic, careerDic, playerNum)
        end
    end
end

-- 阵营和职业不多于
function Lua_StarCondVo:checkCampAndCareerMin(campCareerDic, campDic, careerDic, playerNum)
    for i = 1, self.condArr:getSize() do
        if self.condArr[i].condType == 5 then
            self.condArr[i]:checkCampAndCareerMin(campCareerDic, campDic, careerDic, playerNum)
        end
    end
end

-- 回合数
function Lua_StarCondVo:checkRound(value)
    for i = 1, self.condArr:getSize() do
        if self.condArr[i].condType == 6 then
            self.condArr[i]:checkRound(value)
        end
    end
end




















Lua_SingleStarCondVo = class("Lua_SingleStarCondVo")
function Lua_SingleStarCondVo:ctor(templateId, condType)
    self.templateId = templateId
    self.condType = condType
    self:init()
end

function Lua_SingleStarCondVo:parseCond(cond)
    if cond == nil then
        return
    end

    for i = 1, #cond do
        self["value"..i] = tonumber(cond[i])
    end
end

function Lua_SingleStarCondVo.Get:config()
    if self.templateId == Lua_FightDefine.TEMPLATE_ID_STARTRIAL then
        return getConfig("star_trial_star_cond", self.condType)
    elseif self.templateId == Lua_FightDefine.TEMPLATE_ID_KUNXU then
        return getConfig("holy_land_cond", self.condType)
    end
    return nil   
end

function Lua_SingleStarCondVo.Get:desc()
    if self._desc == nil then
        local str = self.config.desc
        if not strIsNullOrEmpty(str) then
            if self.condType == 1 then -- 战斗结果
                self._desc = str
            elseif self.condType == 2 then -- 阵亡人数
                self._desc = LanUtil.formateStr(str, tostring(self.value1))
            elseif self.condType == 3 then -- 剩余血量
                self._desc = LanUtil.formateStr(str, tostring(self.value1 * 100))
            elseif self.condType == 4 or self.condType == 5 then -- 英雄阵营和职业
                self._desc = LanUtil.formateStr(str, DataCenter.heroUtil:getCampName(self.value1), DataCenter.heroUtil:getHeroCareer(self.value2), tostring(self.value3))
            elseif self.condType == 6 then -- 回合数
                self._desc = LanUtil.formateStr(str, tostring(self.value1))
            elseif self.condType == 7 then
                self._desc = LanUtil.formateStr(str, DataCenter.heroUtil:getCampName(self.value1), DataCenter.heroUtil:getCampName(self.value2), DataCenter.heroUtil:getHeroCareer(self.value3), tostring(self.value4))
            else
                self._desc = str
            end
        end
    end
    return self._desc
end

function Lua_SingleStarCondVo:init()
    self._bMeet = (self.config.init_state == 1)
end

function Lua_SingleStarCondVo.Get:bMeet()
    if self._bMeet == nil then
        return true
    end
    return self._bMeet
end

function Lua_SingleStarCondVo:checkResult(value)
    self._bMeet = value == 1
end

function Lua_SingleStarCondVo:checkDeath(value)
    self._bMeet = value <= self.value1
end

function Lua_SingleStarCondVo:checkHealth(value)
    self._bMeet = value >= self.value1
end

function Lua_SingleStarCondVo:checkCampAndCareerMax(campCareerDic, campDic, careerDic, playerNum)
    if self.value1 == nil or self.value2 == nil or self.value3 == nil then
        return
    end

    if self.value1 == 0 then -- 无阵营要求
        if self.value2 == 0 then -- 无职业要求
            self._bMeet = playerNum >= self.value3
        else
            local num = careerDic[self.value2] or 0
            self._bMeet = num >= self.value3
        end
    else
        if self.value2 == 0 then -- 无职业要求
            local num = campDic[self.value1] or 0
            self._bMeet = num >= self.value3
        else
            local tab = campCareerDic[self.value1]
            if tab == nil then
                self._bMeet = false
            else
                local num = tab[self.value2] or 0
                self._bMeet = num >= self.value3
            end
        end
    end
end

function Lua_SingleStarCondVo:checkCampAndCareerMax2(campCareerDic, campDic, careerDic, playerNum)
    if self.value1 == nil or self.value2 == nil or self.value3 == nil or self.value4 == nil then
        return
    end

    if self.value1 == 0 and self.value2 == 0 then -- 无阵营要求
        if self.value3 == 0 then -- 无职业要求
            self._bMeet = playerNum >= self.value4
        else
            local num = careerDic[self.value3] or 0
            self._bMeet = num >= self.value4
        end
    else
        if self.value3 == 0 then -- 无职业要求
            local num = campDic[self.value1] or 0
            self._bMeet = num >= self.value4
            if not self._bMeet then
                num = campDic[self.value2] or 0
                self._bMeet = num >= self.value4
            end
        else
            local tab = campCareerDic[self.value1]
            if tab == nil then
                tab = campCareerDic[self.value2]
            end
            if tab == nil then
                self._bMeet = false
            else
                local num = tab[self.value3] or 0
                self._bMeet = num >= self.value4
            end
        end
    end
end

function Lua_SingleStarCondVo:checkCampAndCareerMin(campCareerDic, campDic, careerDic, playerNum)
    if self.value1 == nil or self.value2 == nil or self.value3 == nil then
        return
    end

    if self.value1 == 0 then -- 无阵营要求
        if self.value2 == 0 then -- 无职业要求
            self._bMeet = playerNum <= self.value3
        else
            local num = careerDic[self.value2] or 0
            self._bMeet = num <= self.value3
        end
    else
        if self.value2 == 0 then -- 无职业要求
            local num = campDic[self.value1] or 0
            self._bMeet = num <= self.value3
        else
            local tab = campCareerDic[self.value1]
            if tab == nil then
                self._bMeet = true
            else
                local num = tab[self.value2] or 0
                self._bMeet = num <= self.value3
            end
        end
    end
end

function Lua_SingleStarCondVo:checkRound(value)
    self._bMeet = value <= self.value1
end