Lua_HeroLineupVo = class("Lua_HeroLineupVo")
function Lua_HeroLineupVo:ctor(id, config)
    self._id = id -- 阵容类型
    self._config = config
    self.standNum = 10 -- 站位数量
    self.artifactNum = 3 -- 神器数量
    self.activateConn = {} -- 激活的连携技能
end

function Lua_HeroLineupVo.Get:id()
    return self._id
end

function Lua_HeroLineupVo.Get:config()
    if self._config == nil then
        self._config = getConfig("hero_camp_lineup", self.id)
    end
    return self._config
end

function Lua_HeroLineupVo:initHeroInfo()
    if self.heroNumDic == nil then
        self.heroNumDic = {} -- [lineupIdx] = num
    else
        table.clear(self.heroNumDic)
    end
    if self.heroStandDic == nil then
        self.heroStandDic = {} -- [uid] = standIdx
    else
        table.clear(self.heroStandDic)
    end
    if self.heroTeamDic == nil then
        self.heroTeamDic = {} -- [id] = lineupIdx * 10000 + standIdx * 100
    else
        table.clear(self.heroTeamDic)
    end
end

function Lua_HeroLineupVo:fromServer(msg)
    self:initHeroInfo()
    local obj
    for i = 1, #msg.detachment_list do
        obj = msg.detachment_list[i]
        self.serverStandTypeDic[obj.index] = obj.stand_type
        self:setArtifact(obj.index, self.serverArtifactDic[obj.index], obj.artifact_list)
        self:setHeroStand(obj.index, self.serverLineupDic[obj.index], obj.location_list)
    end
end

function Lua_HeroLineupVo:setArtifact(lineupIdx, arr, list)
    local obj
    for i = 1, #list do
        obj = list[i]
        arr[obj.key] = obj.value
    end
end

-- 设置英雄站位
function Lua_HeroLineupVo:setHeroStand(lineupIdx, arr, list)
    for i = 1, arr:getSize() do
        arr[i] = 0
    end

    local standIdx
    local uniqueId
    local num = 0
    local heroVo
    for i = 1, #list do
        standIdx = tonumber(list[i].key)
        uniqueId = tonumber(list[i].value)

        if self.id == Lua_HeroUtil.LINEUP_EXPEDITION then
            heroVo = DataCenter.expeditionData:getLineupHeroVo(uniqueId)
        elseif self.id == Lua_HeroUtil.LINEUP_INFINITYHALL_TEAM then
            heroVo = DataCenter.infinityHallData.chooseHeroVo
            if heroVo == nil or heroVo.unique_id ~= uniqueId then
                heroVo = DataCenter.heroData:getItemByUniqueId(uniqueId)
            end
        else
            heroVo = DataCenter.heroData:getItemByUniqueId(uniqueId)
        end
        self.heroStandDic[uniqueId] = standIdx

        arr[standIdx] = uniqueId
        num = num + 1
        if heroVo then
            self.heroTeamDic[heroVo.id] = lineupIdx * 10000 + standIdx * 100
        end
    end
    self.heroNumDic[lineupIdx] = num
end

-- [lineupIdx] = 英雄唯一ID数组
function Lua_HeroLineupVo.Get:serverLineupDic()
    if self._serverLineupDic == nil then
        self._serverLineupDic = {}
        local arr
        for i = 1, self.team_number do
            arr = Array()
            for j = 1, self.standNum do
                arr:add(0)
            end
            self._serverLineupDic[i] = arr
        end
    end
    return self._serverLineupDic
end

-- [lineupIdx] = 神器ID数组
function Lua_HeroLineupVo.Get:serverArtifactDic()
    if self._serverArtifactDic == nil then
        self._serverArtifactDic = {}
        local arr
        for i = 1, self.team_number do
            arr = Array()
            for j = 1, self.artifactNum do
                arr:add(0)
            end
            self._serverArtifactDic[i] = arr
        end
    end
    return self._serverArtifactDic
end

-- [lineupIdx] = stand_type 站位类型
function Lua_HeroLineupVo.Get:serverStandTypeDic()
    if self._serverStandTypeDic == nil then
        self._serverStandTypeDic = {}
        for i = 1, self.team_number do
            self._serverStandTypeDic[i] = self.standType
        end
    end
    return self._serverStandTypeDic
end

-------------------- 布阵调用 --------------------

function Lua_HeroLineupVo:initLocalInfo()
    local arr
    if self._localLineupDic == nil then
        self._localLineupDic = {}
        self._localArtifactDic = {}
        self._localStandTypeDic = {}
    end

    for i = 1, self.team_number do
        arr = self.serverLineupDic[i]
        if self._localLineupDic[i] == nil then
            self._localLineupDic[i] = Array()
            for j = 1, arr:getSize() do
                self._localLineupDic[i]:add(arr[j])
            end
        else
            for j = 1, arr:getSize() do
                self._localLineupDic[i][j] = arr[j]
            end
        end

        arr = self.serverArtifactDic[i]
        if self._localArtifactDic[i] == nil then
            self._localArtifactDic[i] = Array()
            for j = 1, arr:getSize() do
                self._localArtifactDic[i]:add(arr[j])
            end
        else
            for j = 1, arr:getSize() do
                self._localArtifactDic[i][j] = arr[j]
            end
        end

        self._localStandTypeDic[i] = self.serverStandTypeDic[i]
    end
end

function Lua_HeroLineupVo.Get:localLineupDic()
    if self._localLineupDic == nil then
        self:initLocalInfo()
    end
    return self._localLineupDic;
end

function Lua_HeroLineupVo.Get:localArtifactDic()
    if self._localArtifactDic == nil then
        self:initLocalInfo()
    end
    return self._localArtifactDic;
end

function Lua_HeroLineupVo.Get:localStandTypeDic()
    if self._localStandTypeDic == nil then
        self:initLocalInfo()
    end
    return self._localStandTypeDic;
end

function Lua_HeroLineupVo:setLocalStandType(lineupIdx, value)
    self._localStandTypeDic[lineupIdx] = value
end

-------------------- 布阵信息 --------------------

function Lua_HeroLineupVo:initLineUpInfo()
    self.bInitLineUp = true
    self:initHeroInfo()

    local dic = self.localLineupDic
    local uniqueId
    local num
    local heroVo
    for i = 1, #dic do
        num = 0
        for j = 1, dic[i]:getSize() do
            uniqueId = dic[i][j]
            if uniqueId > 0 then
                if self.id == Lua_HeroUtil.LINEUP_EXPEDITION then
                    heroVo = DataCenter.expeditionData:getLineupHeroVo(uniqueId)
                elseif self.id == Lua_HeroUtil.LINEUP_INFINITYHALL_TEAM then
                    heroVo = DataCenter.infinityHallData.chooseHeroVo
                    if heroVo == nil or heroVo.unique_id ~= uniqueId then
                        heroVo = DataCenter.heroData:getItemByUniqueId(uniqueId)
                    end
                else
                    heroVo = DataCenter.heroData:getItemByUniqueId(uniqueId)
                end
                if heroVo then
                    num = num + 1
                    self.heroStandDic[heroVo.unique_id] = j
                    self.heroTeamDic[heroVo.id] = i * 10000 + j * 100
                end
            end
        end
        self.heroNumDic[i] = num
    end
end

function Lua_HeroLineupVo:checkHeroConnect(lineupIdx, bPopupWin, nCheckConnect)
    if self.bCheckConnect == true and nCheckConnect ~= true then
        return
    end
    self.bCheckConnect = true

    local arr = self.localLineupDic[lineupIdx]
    local heroDic = {}
    local campDic = {}
    local careerDic = {}
    local heroVo
    local len = arr:getSize()
    for i = 1, len do
        if arr[i] > 0 then
            if self.id == Lua_HeroUtil.LINEUP_EXPEDITION then
                heroVo = DataCenter.expeditionData:getLineupHeroVo(arr[i])
            elseif self.id == Lua_HeroUtil.LINEUP_INFINITYHALL_TEAM then
                heroVo = DataCenter.infinityHallData.chooseHeroVo
                if heroVo == nil or heroVo.unique_id ~= arr[i] then
                    heroVo = DataCenter.heroData:getItemByUniqueId(arr[i])
                end
            else
                heroVo = DataCenter.heroData:getItemByUniqueId(arr[i])
            end

            heroDic[heroVo.id] = heroVo.id
            if campDic[heroVo.campType] == nil then
                campDic[heroVo.campType] = Array()
            end
            campDic[heroVo.campType]:add(heroVo.id)

            if careerDic[heroVo.career] == nil then
                careerDic[heroVo.career] = Array()
            end
            careerDic[heroVo.career]:add(heroVo.id)
        end
    end

    if self.activateConn[lineupIdx] == nil then
        self.activateConn[lineupIdx] = {}
    end
    local tab = self.activateConn[lineupIdx]
    local activateDic = {}
    local connVo
    for i = 1, len do
        if arr[i] > 0 then
            if self.id == Lua_HeroUtil.LINEUP_EXPEDITION then
                heroVo = DataCenter.expeditionData:getLineupHeroVo(arr[i])
            elseif self.id == Lua_HeroUtil.LINEUP_INFINITYHALL_TEAM then
                heroVo = DataCenter.infinityHallData.chooseHeroVo
                if heroVo == nil or heroVo.unique_id ~= arr[i] then
                    heroVo = DataCenter.heroData:getItemByUniqueId(arr[i])
                end
            else
                heroVo = DataCenter.heroData:getItemByUniqueId(arr[i])
            end

            if heroVo and heroVo.connectId > 0 then
                connVo = tab[heroVo.connectId]
                if connVo == nil then
                    connVo = Lua_HeroConnectVo.new(heroVo.connectId)
                else
                    connVo:init()
                end
                connVo:setData(heroDic, campDic, careerDic)
                if connVo.isActivate then
                    activateDic[heroVo.connectId] = connVo
                    if bPopupWin and tab[heroVo.connectId] == nil then
                        DataCenter.heroLineupData:addConnPopupWin(connVo)
                    end
                end
            end
        end
    end

    for k, _ in pairs(tab) do
        if activateDic[k] == nil then
            tab[k] = nil
        end
    end
    for k, v in pairs(activateDic) do
        tab[k] = v
    end
    activateDic = nil
    heroDic = nil
    table.clear(campDic)
    campDic = nil
    table.clear(careerDic)
    careerDic = nil
end

function Lua_HeroLineupVo:checkConnectIsActivate(lineupIdx, connectId)
    self:checkHeroConnect(lineupIdx, false, true)
    return self.activateConn[lineupIdx][connectId]
end

function Lua_HeroLineupVo:getConnectActivateNum(lineupIdx)
    if self.activateConn == nil then
        return 0
    end
    return table.length(self.activateConn[lineupIdx])
end

function Lua_HeroLineupVo:getHeroNumByLineupIdx(lineupIdx)
    if self.heroNumDic == nil then
        return 0
    end
    return self.heroNumDic[lineupIdx] or 0
end

-- 获得英雄所在队伍
function Lua_HeroLineupVo:getHeroLineupIdx(heroId)
    if self.heroTeamDic == nil or self.heroTeamDic[heroId] == nil then
        return nil
    end
    return Mathf.Floor(self.heroTeamDic[heroId] / 10000)
end

-- 获得英雄所在站位
function Lua_HeroLineupVo:getHeroStandIdx(hero_uid)
    if self.heroStandDic == nil then
        return nil
    end
    return self.heroStandDic[hero_uid]
end

-- 获得英雄所在分队索引、站位索引
function Lua_HeroLineupVo:getHeroStandIdxById(heroId)
    if self.heroTeamDic == nil or self.heroTeamDic[heroId] == nil then
        return nil, nil
    end
    local lineupIdx = Mathf.Floor(self.heroTeamDic[heroId] / 10000);
    local standIdx = Mathf.Floor(self.heroTeamDic[heroId] / 100 % 10);
    return lineupIdx, standIdx
end

-- 获得站位索引上的英雄唯一ID
function Lua_HeroLineupVo:getHeroUidByStandIdx(lineupIdx, standIdx)
    if standIdx then
        lineupIdx = lineupIdx or 1
        local arr
        if self._localLineupDic == nil then
            arr = self.serverLineupDic[lineupIdx]
        else
            arr = self.localLineupDic[lineupIdx]
        end
        return arr[standIdx]
    end
    return nil
end

-------------------- 外部调用 --------------------

function Lua_HeroLineupVo.Get:lineupDic()
    if self._localLineupDic == nil then
        return self.serverLineupDic
    end
    return self._localLineupDic
end

function Lua_HeroLineupVo.Get:artifactDic()
    if self._localArtifactDic == nil then
        return self.serverArtifactDic
    end
    return self._localArtifactDic
end

function Lua_HeroLineupVo.Get:standTypeDic()
    if self._localStandTypeDic == nil then
        return self.serverStandTypeDic
    end
    return self._localStandTypeDic
end

-- 获取布阵站位类型
function Lua_HeroLineupVo:getStandType(lineupIdx)
    if self.standTypeDic[lineupIdx] == nil then
        return self.standType
    end
    return self.standTypeDic[lineupIdx]
end

-- 获取上阵神器
function Lua_HeroLineupVo:getArtifactId(lineupIdx, index)
    if self.artifactDic[lineupIdx] == nil then
        return 0
    end
    return self.artifactDic[lineupIdx][index] or 0
end

-- 获取上阵英雄列表
function Lua_HeroLineupVo:getStandArr(lineupIdx)
    if self.lineupDic[lineupIdx] == nil then
        return Array()
    end
    return self.lineupDic[lineupIdx]
end

-------------------- 配置数据 --------------------
function Lua_HeroLineupVo.Get:name()
    return self.config.name
end

-- 是否显示在布阵页签
function Lua_HeroLineupVo.Get:is_show()
    return self.config.is_show
end

function Lua_HeroLineupVo.Get:sort()
    return self.config.sort
end

-- 允许全部下阵
function Lua_HeroLineupVo.Get:down_lineup()
    if self.config.down_lineup == nil then
        return 0
    end
    return self.config.down_lineup;
end

function Lua_HeroLineupVo.Get:team_number()
    if self.config.team_number == nil then
        return 1
    end
    return self.config.team_number
end

function Lua_HeroLineupVo.Get:standType()
    if self.config.stand_type == nil then
        return 1
    end
    return self.config.stand_type
end

function Lua_HeroLineupVo.Get:campposId()
    if self.config.camppos_id == nil then
        return 0
    end
    return self.config.camppos_id
end

function Lua_HeroLineupVo.Get:templateId()
    return self.config.template_id
end