Lua_MailData = class("Lua_MailData");
function Lua_MailData:ctor()
    self:dispose();
end

function Lua_MailData:dispose()
    self.mailDic = {};
    self.isOpenNoticeTag = false;
end

function Lua_MailData:fromServer50104002(msg)
    local info;
    for i = 1, #msg.mail_list do
        info = msg.mail_list[i];
        if tonumber(info.overtime_tick) > DataCenter.roleData.serverTime then
            self.mailDic[info.mail_id] = info;
        end
    end

    if #msg.mail_list > 0 then
        self:showMainIconRedDot();
    end
end

function Lua_MailData:removeMailItem(mailItemId, bRefreshRedDot)
    if self.mailDic[mailItemId] ~= nil then
        self.mailDic[mailItemId] = nil;
        if bRefreshRedDot == nil or bRefreshRedDot == true then
            self:showMainIconRedDot();
        end
    end
end

function Lua_MailData:addOrUpdataMailItem(mailItem, bRefreshRedDot)
    self.mailDic[mailItem.mail_id] = mailItem;
    if bRefreshRedDot == nil or bRefreshRedDot == true then
        self:showMainIconRedDot();
    end
end

function Lua_MailData:getMailListAndSort()
    local arr = Array();
    for k,v in pairs(self.mailDic) do
        if tonumber(v.overtime_tick) > DataCenter.roleData.serverTime then
            arr:add(v);
        end
    end
    if arr:getSize() > 0 then
        arr:sortf(self, self.mailSort);
    end
    return arr;
end

function Lua_MailData:mailSort(a, b)
    if a.has_read > b.has_read then
        return false;
    elseif a.has_attach < b.has_attach and (a.has_read == 1 and b.has_read == 1) then
        return false;
    elseif (a.has_read == 1 and b.has_read == 1) and (a.has_attach == 0 and b.has_attach == 0) then
        if a.send_tick >= b.send_tick then
            return true;
        else
            return false;
        end
    else
        return true;
    end
end

function Lua_MailData:getMailReadAndNoAttachList()
    local list = Array();
    for k,v in pairs(self.mailDic) do
        if v.has_read == 1 and v.has_attach == 0 then
            list:add(v);
        end
    end
    return list;
end

function Lua_MailData:getRedDotCount()
    local count = 0;
    for k,v in pairs(self.mailDic) do
        if v.has_read == 0 or v.has_attach == 1 then
            count = count + 1;
        end
    end
    return count;
end

function Lua_MailData:showMainIconRedDot()
    local count = self:getRedDotCount()
    DataCenter.redPointData:updateRedPointShowNum(Lua_RedPointList.MailTab, count);
end

function Lua_MailData.Get:hasAttach()
    local bl = false;
    for k, v in pairs(self.mailDic) do
        if v.has_attach == 1 then
            bl=true;
            break;
        end
    end
    return bl;
end

function Lua_MailData:mailContent(msg)
    local str = msg.content;
    -- local len = #msg.args;
    -- for i = 1, len do
    --     local sd = msg.args[i];
    --     local substr = nil;

    --     if sd.arg_type == 1 then
    --         substr = self:getInt(sd);
    --     elseif sd.arg_type == 2 then
    --         substr = self:getInt64(sd);
    --     elseif sd.arg_type == 3 then
    --         substr = self:getString(sd);
    --     elseif sd.arg_type == 4 then
    --         substr = self:getRoleName(sd);
    --     elseif sd.arg_type == 5 then
    --         substr = self:getItem(sd, i);
    --     elseif sd.arg_type == 6 then
    --         substr = self:getCelebrityName(sd);
    --     end
    --     if substr == nil then
    --         substr = "";
    --     end
    --     local temp = "{" ..i .. "}";

    --     str = LanUtil.replaceStr(str, temp, substr);
    -- end
    return str;
end

function Lua_MailData:getInt(value)
    return tostring(value.ivalue);
end

function Lua_MailData:getInt64(value)
    return tostring(value.dvalue);
end

function Lua_MailData:getString(value)
    return value.strvalue;
end

function Lua_MailData:getRoleName(value)
    self.playerInfo = value.role_info;
    return "[url=playerClick_" .. value.role_info.role_id .. "_" .. value.role_info.role_name .."][u]" .. value.role_info.role_name .. "[/u][/url]";
end

function Lua_MailData:getItem(value, idx)
    if value.item_info ~= nil and value.item_info.id ~= nil and value.item_info.id > 0 then
        local tvo = Lua_ToolVo.createToolVo(value.item_info.id);
        tvo:fromServer(value.item_info);
        if DataCenter.moneyData:isMoney(value.item_info.id) == true then
            -- 货币
            return tvo.count .. tvo.name;
        else
            local str = value.item_info.id .. idx;
            local nameStr = tvo.broadcastName;
            if tvo.count > 1 then
                return "[url=itemClick_" .. str .. "][u]" .. nameStr .. "[/u][18c828]X" .. tvo.count .. "[-][/url]";
            else
                return "[url=itemClick_" .. str .. "][u]" .. nameStr .. "[/u][/url]";
            end
        end
    end
    return self:getItemName(value);
end

function Lua_MailData:getItemName(value)
    if value.ivalue ~= nil and value.ivalue > 0 then
        local tvo = Lua_ToolVo.new(value.ivalue);
        return "[u]" .. tvo.colorName .. "[/u]";
    end
    return "";
end

function Lua_MailData:getCelebrityName(value)
    if value.ivalue ~= nil and tonumber(value.ivalue) > 0 then
        local celebrityVo = Lua_CelebrityVo.new(value.ivalue);
        if celebrityVo ~= nil then
            return celebrityVo.colorName;
        end
    end
    return "";
end