-- 主城
Lua_MainCityWindow = class("Lua_MainCityWindow", Lua_Window)
function Lua_MainCityWindow:ctor()
    self:superCall("ctor")

    self.layer = MainWinDefine.BASE_LAYER - 500
    self.windowName = "Lua_MainCityWindow"
    self.isFullSizeWin = true
    self.openSound = nil
    self.closeSound = nil
    self.winSceneId = Lua_SceneDefine.MAINCITY
end

function Lua_MainCityWindow:buildShowItem()
    self:superCall("buildShowItem")
    self.redPointId = {}
    
    self:addLuaEvent("SCENE_SHOW", self, self.onSceneShow)
    self:addCSharpEvent("MAP_COLLIDER_CLICK_DOWN", self, self.onMapClickDown)
    self:addCSharpEvent("MAP_COLLIDER_CLICK_UP", self, self.onMapClickUp)
    self:addCSharpEvent("CAM_ANGLE_CHANGE", self, self.onCamAngleChange)
    self:addLuaEvent("RedPoint_Update_State", self, self.onRedPointStateUpdate)

    local config = Lua_SceneConfigData.getSceneConfig(self.winSceneId)
    self.panMinAngle = config.panMinAngle
    self.panMaxAngle = config.panMaxAngle
    
    self.leftIcon = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_zjm_btn_010_n", -322, 550, 0, 0)
    self.leftIcon.anchorValue = 7
    self.leftIcon:setScale(0.75, 0.75, 1)

    self.rightIcon = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_zjm_btn_010_n", 322, 550, 0, 0)
    self.rightIcon.anchorValue = 7
    self.rightIcon:setScale(-0.75, 0.75, 1)

    self.infoArr = Array()
    self:showInfo()
end

function Lua_MainCityWindow:dispose()
    if self.infoArr ~= nil then
        self.infoArr:dispose()
        self.infoArr:clear()
        self.infoArr = nil
    end
    if self.leftEff then
        self.leftEff:dispose()
        self.leftEff = nil
    end
    if self.rightEff then
        self.rightEff:dispose()
        self.rightEff = nil
    end
    self.leftIcon = nil
    self.rightIcon = nil
    self.redPointId = nil;
    self._cameraViewHandle = nil
    self:superCall("dispose")
end

function Lua_MainCityWindow:showWin(param)
    self:superCall("showWin", param)
    Lua_MsgUtil.callFunc("UPDATE_MAIN_MENU", Lua_MenuViewDefine.MEBULIST_ALL)
end

function Lua_MainCityWindow.Get:cameraViewHandle()
    if self._cameraViewHandle == nil then
        self._cameraViewHandle = GlobalVar.cameraViewHandle
    end
    return self._cameraViewHandle
end

function Lua_MainCityWindow:onSceneShow(sceneId)
    if sceneId == self.winSceneId then
        self:addCSharpEvent("CAM_ANGLE_CHANGE", self, self.onCamAngleChange)
        self:onCamAngleChange()
    else
        self:removeCSharpEvent("CAM_ANGLE_CHANGE", self, self.onCamAngleChange)
    end
end

function Lua_MainCityWindow:onMapClickDown(clickStr)

end

function Lua_MainCityWindow:onMapClickUp(clickStr)
    local vo = DataCenter.constructUtil:getVoByModelSrc(clickStr)
    if vo ~= nil and vo.isShield ~= 1 and vo.pathId ~= nil and vo.pathId > 0 then
        DataCenter.funcPathUtil:gotoFunction(vo.pathId)
    end
end

function Lua_MainCityWindow:onCamAngleChange()
    if self.bActive ~= true or DataCenter.sceneData.openSceneId ~= self.winSceneId or self.infoArr == nil then
        return
    end

    local item
    for i = 1, self.infoArr:getSize() do
        item = self.infoArr[i]
        LuaUtil.setUIObjPos(self.content, item.content, 
            item.data.scenePos.x, item.data.scenePos.y, item.data.scenePos.z, 
            item.data.nameOffset[1], item.data.nameOffset[2], item.data.nameOffset[3])
    end
    self:updateEff()
end

function Lua_MainCityWindow:onRedPointStateUpdate(id)
    if self.redPointId[id] == 1 then
        self:updateEff()
    end
end

function Lua_MainCityWindow:showInfo()
    local arr = DataCenter.constructUtil:getStructureArr(Lua_SceneDefine.MAINCITY)
    local vo
    local item
    for i = 1, arr:getSize() do
        vo = DataCenter.constructUtil:getStructureVo(arr[i])
        if vo.isShield == 1 then
            Scene3DManager.instance:setGameObjectCollider(vo.modelSrc, false)
        elseif not strIsNullOrEmpty(vo.name) then
            item = Lua_StructureInfoItem.new(self.content)
            item:setData(vo)
            LuaUtil.setUIObjPos(self.content, item.content, 
                vo.scenePos.x, vo.scenePos.y, vo.scenePos.z, 
                vo.nameOffset[1], vo.nameOffset[2], vo.nameOffset[3])
            if not strIsNullOrEmpty(vo.redPointId) then
                self.redPointId[vo.redPointId] = 1
            end
            self.infoArr:add(item)
        end
    end
end

function Lua_MainCityWindow:updateEff()
    local item
    local vo
    local leftState = false
    local rightState = false
    for i = 1, self.infoArr:getSize() do
        item = self.infoArr[i]
        if item.x - 93.5 <= -self.width / 2 then
            vo = DataCenter.redPointData:getRedPointVoById(item.data.redPointId)
            if vo and vo.showState then
                leftState = true
            end
        elseif item.x + 93.5 >= self.width / 2 then
            vo = DataCenter.redPointData:getRedPointVoById(item.data.redPointId)
            if vo and vo.showState then
                rightState = true
            end
        end
    end

    local bLeft = false
    local bRight = false
    if self.cameraViewHandle.panAngle > self.panMinAngle then
        if leftState then
            bLeft = true
            if self.leftEff == nil then
                self.leftEff = UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_zc_jiantou", "ui_zc_jiantou", -322, 550)
                self.leftEff.anchorValue = 7
                self.leftEff:setScale(0.75, 0.75, 0.75)
            else
                self.leftEff:setEnable(true)
            end
            self.leftIcon:setEnable(false)
        else
            if self.leftEff then
                self.leftEff:setEnable(false)
            end
            self.leftIcon:setEnable(true)
        end
    else
        if self.leftEff then
            self.leftEff:setEnable(false)
        end
        self.leftIcon:setEnable(false)
    end

    if self.cameraViewHandle.panAngle < self.panMaxAngle then
        if rightState then
            bRight = true
            if self.rightEff == nil then
                self.rightEff = UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_zc_jiantou", "ui_zc_jiantou", 322, 550)
                self.rightEff.anchorValue = 7
                self.rightEff:setScale(-0.75, 0.75, 0.75)
            else
                self.rightEff:setEnable(true)
            end
            self.rightIcon:setEnable(false)
        else
            if self.rightEff then
                self.rightEff:setEnable(false)
            end
            self.rightIcon:setEnable(true)
        end
    else
        if self.rightEff then
            self.rightEff:setEnable(false)
        end
        self.rightIcon:setEnable(false)
    end

    -- 保持特效闪烁一致
    if self.leftEff and bLeft and bLeft ~= self.bLeft then
        self.leftEff:setEnable(false)
        self.leftEff:setEnable(true)
        if self.rightEff and bRight then
            self.rightEff:setEnable(false)
            self.rightEff:setEnable(true)
        end
    end
    if self.rightEff and bRight and bRight ~= self.bRight then
        if self.leftEff and bLeft then
            self.leftEff:setEnable(false)
            self.leftEff:setEnable(true)
        end
        self.rightEff:setEnable(false)
        self.rightEff:setEnable(true)
    end
    self.bLeft = bLeft
    self.bRight = bRight
end



















Lua_StructureInfoItem = class("Lua_StructureInfoItem", Lua_DisplayObject)
function Lua_StructureInfoItem:ctor(p)
    self:superCall("ctor", p)
end

function Lua_StructureInfoItem:buildShowItem()
    self:superCall("buildShowItem")
    self:addLuaEvent("FIGHTPLAY_ENTER", self, self.checkIsFighting)
    self:addLuaEvent("FIGHTPLAY_COMPLETE", self, self.checkIsFighting)

    self.nameBg = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_zjm_widget_05", 0, 0, 0, 0)
    self.nameBg:addClickFuntion(self, self.onClick)

    self.nameText = UICreate.createLabel(self.content, nil, 0, 0, 100, 31, 24)
    self.nameText.pivot = Lua_WindowUtil.getPivot(4)
    self.nameText.fontName = Lua_MyUISettings.huakangFontName
    self.nameText.alignmentValue = 4
    self.nameText.horizontalOverflowValue = 1
    self.nameText.effectStyle = 2
    self.nameText.colorStr = Lua_ColorDefine.transColor("FFFFFF")
    self.nameText.effectColorStr = Lua_ColorDefine.transColor("464954")
    self.nameText:setEffectDistance(2, 2)
end

function Lua_StructureInfoItem:dispose()
    if self.redPointIcon ~= nil then
        self.redPointIcon:dispose()
        self.redPointIcon = nil
    end
    self.data = nil
    self:superCall("dispose")
end

function Lua_StructureInfoItem:setData(data)
    self.data = data
    self.content.name = data.id
    self.nameText.text = data.name
    local posX = self.nameBg.width / 2 - 30
    local posY = self.nameBg.height / 2 - 15
    self:setRedPoint(data.redPointId, posX, posY)
    self:checkIsFighting()
end

function Lua_StructureInfoItem:setRedPoint(id, posX, posY)
    if strIsNullOrEmpty(id) then
        if self.redPointIcon ~= nil then
            self.redPointIcon:changeRedPoint(nil)
        end
    else
        if self.redPointIcon == nil then
            self.redPointIcon = Lua_RedPointIcon.new(self.content)
        end
        self.redPointIcon:changeRedPoint(id)
        posX = posX == nil and 0 or posX
        posY = posY == nil and 0 or posY
        self.redPointIcon:setLocation(posX, posY)
    end
end

function Lua_StructureInfoItem:checkIsFighting()
    self.recordId = 0
    if self.data ~= nil and self.data.templateList ~= nil then
        local list = self.data.templateList
        for i = 1, #list do
            self.recordId = DataCenter.fightData:getRecordIdByTemplateId(list[i])
            if self.recordId > 0 then
                break
            end
        end
    end
    self:showFightEff(self.recordId > 0)
end

function Lua_StructureInfoItem:showFightEff(value)
    if value then
        if self.fightEff == nil then
            self.fightEff = UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_ty_zdbs_0101", "ui_ty_zdbs_0101", 0, 80)
        end
        self.fightEff:setEnable(true)
    else
        if self.fightEff then
            self.fightEff:setEnable(false)
        end
    end
end

function Lua_StructureInfoItem:onClick(go)
    DataCenter.funcPathUtil:gotoFunction(self.data.pathId)
end