-- 主界面图标数据
checkRequire("Lua_MenuIconVo");
Lua_MainMenuIconData = class("Lua_MainMenuIconData")
function Lua_MainMenuIconData:ctor()
	self:superCall("ctor");
	self:dispose();
end

function Lua_MainMenuIconData:dispose()
	if self.iconUnIdDic ~= nil then
		for _, vo in pairs(self.iconUnIdDic) do
			vo:resetInfo();
		end
	end

	self.selectMainIconId = nil;
	self.bTopRightShrinkState = true;
	self.bDownRightShrinkState = true;

	if self.topLeftIconArr ~= nil then
		self.topLeftIconArr:clear();
		self.topLeftIconArr = nil;
	end
	if self.topRightIconArr ~= nil then
		self.topRightIconArr:clear();
		self.topRightIconArr = nil;
	end
end

-- 主界面菜单栏显隐状态
function Lua_MainMenuIconData.Get:menuStateDic()
	if self._menuStateDic == nil then
		self._menuStateDic = {}
	end
	return self._menuStateDic
end

function Lua_MainMenuIconData:setMenuState(key, state)
	self.menuStateDic[key] = state
end

-- Lua_MenuViewDefine.INDEX_LIUHAI = 0; -- 刘海遮罩
-- Lua_MenuViewDefine.INDEX_ROLEINFO = 1; -- 玩家信息
-- Lua_MenuViewDefine.INDEX_TOPLEFT = 2; -- 左上方图标
-- Lua_MenuViewDefine.INDEX_TOPRIGHT = 3; -- 右上方图标
-- Lua_MenuViewDefine.INDEX_TOPRIGHT_SHRINK = 4; -- 右上方收缩栏
-- Lua_MenuViewDefine.INDEX_DOWNRIGHT = 5; -- 右下方图标
-- Lua_MenuViewDefine.INDEX_TASK = 6; -- 任务栏
-- Lua_MenuViewDefine.INDEX_CHAT = 7; -- 聊天栏
-- Lua_MenuViewDefine.INDEX_MAINICON = 8; -- 主功能图标
-- return true 显示 false 隐藏
function Lua_MainMenuIconData:getMenuState(key)
	if self._menuStateDic == nil then
		return false
	end
	return self._menuStateDic[key]
end

function Lua_MainMenuIconData:resetNavigationBar(bCloseOther)
	local vo = self:getMenuIconVo(self.selectMainIconId)
	self.selectMainIconId = nil
	self:clickIconEvent(vo, bCloseOther)
end

function Lua_MainMenuIconData:init()
	if self.bInit ~= true then
		self.bInit = true;
		self.menuPosTypeDic = {};
		self.mainIconWinDic = {};

		local vo;
		local config = getConfig("mainMenuIcon");
		for k,v in pairs(config) do
			if v.is_shield ~= 1 and v.icon_type ~= Lua_MenuViewDefine.TYPE_ACTIVITY_ICON then
				vo = self:getMenuIconVo(k, k, v);

				if self.menuPosTypeDic[vo.posType] == nil then
					self.menuPosTypeDic[vo.posType] = Array();
				end
				self.menuPosTypeDic[vo.posType]:add(vo.uniqueId);

				if vo.iconType == Lua_MenuViewDefine.TYPE_MAIN_ICON and not strIsNullOrEmpty(vo.operateParam) then
					self.mainIconWinDic[vo.operateParam] = vo.uniqueId;
				end
			end
		end

		for k, arr in pairs(self.menuPosTypeDic) do
			arr:sortf(self, self.sortMenuIcon);
		end
	end
end

function Lua_MainMenuIconData:sortMenuIcon(a, b)
	local vo_a = self:getMenuIconVo(a);
	local vo_b = self:getMenuIconVo(b);
	if vo_a.sortWeight > vo_b.sortWeight then
		return false;
	end
	return true;
end

function Lua_MainMenuIconData:getMenuIconVo(uniqueId, iconId, config) -- 默认唯一ID和图标ID是同一个，活动子图标才有唯一ID
	if uniqueId == nil then
		return nil
	end
	iconId = iconId or uniqueId

	if self.iconUnIdDic == nil then
		self.iconUnIdDic = {};
	end
	if self.iconUnIdDic[uniqueId] == nil then
		if config == nil then
			config = getConfig("mainMenuIcon", iconId);
		end
		if config == nil then
			return nil;
		end
		self.iconUnIdDic[uniqueId] = Lua_MenuIconVo.new(uniqueId, iconId, config);
	else
		self.iconUnIdDic[uniqueId].iconId = iconId;
	end
	return self.iconUnIdDic[uniqueId];
end

-- 根据posType获取所有图标数据
function Lua_MainMenuIconData:getMenuDataByPosType(posType, result)
	self:init();
	result = result == nil and Array() or result
	local arr = self.menuPosTypeDic[posType];
	if arr ~= nil then
		local iconVo;
		for i = 1, arr:getSize() do
			iconVo = self:getMenuIconVo(arr[i]);
			if iconVo.canShow == true then
				result:add(iconVo);
			end
		end
	end
	return result;
end

function Lua_MainMenuIconData:getIconIdByWinName(winName)
	self:init();
	return self.mainIconWinDic[winName];
end

-- 图标Id，显示状态，剩余时间, 是否计时结束移除图标，显示文本，显示道具Vo
-- showStr显示条件：showStr ~= nil and showStr ~= "" and leftTime ~= nil and leftTime <= 0
-- itemVo显示条件：itemVo ~= nil and leftTime ~= nil and leftTime <= 0
function Lua_MainMenuIconData:setIconShowInfo(iconId, showState, leftTime, bTimeEndRemove, showStr, itemVo)
	if showState == false then
		local actArr = DataCenter.backstageActData:getActArrByMainIconId(iconId);
		if actArr ~= nil then
			local actVo;
			for i = 1, actArr:getSize() do
				actVo = DataCenter.backstageActData:getActVoByIdx(actArr[i]);
				if actVo.bOpened == true then
					showState = true;
				end
			end
		end
	end

	local bChange = false;
	local iconVo = self:getMenuIconVo(iconId);
	iconVo.leftTime = (leftTime == nil) and 0 or leftTime;
	iconVo.bTimeEndRemove = bTimeEndRemove;
	iconVo.showStr = showStr;
	iconVo.itemVo = itemVo;
	if iconVo.showState ~= showState or iconVo.leftTime > 0 then
		iconVo:setShowState(showState);
		bChange = true;
	else
		Lua_MsgUtil.callFunc("MENUICON_SINGLE_UPDATE", iconVo.uniqueId);
	end

	if bChange == true then
		Lua_MsgUtil.callFunc("MENUICON_ALL_UPDATE");
	end
end

function Lua_MainMenuIconData:updateTopIconData()
	if self.topLeftIconArr == nil then
		self.topLeftIconArr = Array();
	else
		self.topLeftIconArr:clear();
	end
	self:getMenuDataByPosType(Lua_MenuViewDefine.POS_TOPLEFT_ICON, self.topLeftIconArr);

	if self.topRightIconArr == nil then
		self.topRightIconArr = Array();
	else
		self.topRightIconArr:clear();
	end
	self:getMenuDataByPosType(Lua_MenuViewDefine.POS_TOPRIGHT_SHRINK_ICON, self.topRightIconArr);

	local arr = DataCenter.backstageActData.actIconArr;
	if arr ~= nil then
		for i = 1, arr:getSize() do
			if arr[i].posType == Lua_MenuViewDefine.POS_TOPLEFT_ICON then
				self.topLeftIconArr:add(arr[i]);
			elseif arr[i].posType == Lua_MenuViewDefine.POS_TOPRIGHT_SHRINK_ICON then
				self.topRightIconArr:add(arr[i]);
			end
		end
	end

	self.topLeftIconArr:sort("sortWeight", false);
	self.topRightIconArr:sort("sortWeight", false);
	Lua_MsgUtil.callFunc("TOP_MENUICON_UPDATE");
end

function Lua_MainMenuIconData:clickIconEvent(vo, bCloseOther)
	if vo == nil then
		return false;
	end

	if vo.bFunOpenState == false then
		if not strIsNullOrEmpty(vo.unOpenAdvanceTips) then
			OperateAlert.show(vo.unOpenAdvanceTips);
		end
		return false;
	end

	if vo.iconType == Lua_MenuViewDefine.TYPE_MAIN_ICON then
		if self.selectMainIconId == vo.iconId and vo.iconId ~= Lua_MenuViewDefine.ICON_MAINCITY then
			return false;
		end
		self.selectMainIconId = vo.iconId;
	end

	local result = true;
	if vo.actIdx ~= nil and vo.actIdx > 0 then -- 运营后台活动子图标
		local actVo = DataCenter.backstageActData:getActVoByIdx(vo.actIdx);
		if actVo.mainIconId and actVo.mainIconId > 0 then -- 打开的是主图标的界面
			local iconVo = self:getMenuIconVo(actVo.mainIconId);
			return self:openWinOrDispatchEvt(iconVo, bCloseOther, actVo)
		elseif strIsNullOrEmpty(actVo.main_win) then
			LogManager.LogError("运营活动 打开界面 配置错误！", false);
		else
			Lua_WindowUtil.openWin(actVo.main_win, Lua_WinParam.ACTION_OPEN, actVo);
		end
	elseif vo.iconType == Lua_MenuViewDefine.TYPE_ACTIVITY_ICON then -- 后台活动图标，则打开后台配置的窗口
		local arr = DataCenter.backstageActData:getActArrByMainIconId(vo.iconId);
		if arr and arr:getSize() > 0 then
			actVo = DataCenter.backstageActData:getActVoByIdx(arr[1]);
			if actVo == nil or strIsNullOrEmpty(actVo.main_win) then
				LogManager.LogError("运营活动 打开界面 配置错误！", false);
			else
				Lua_WindowUtil.openWin(actVo.main_win, Lua_WinParam.ACTION_OPEN, arr);
			end
		end
	else
		result = self:openWinOrDispatchEvt(vo, bCloseOther);
	end
	return result;
end

function Lua_MainMenuIconData:openWinOrDispatchEvt(iconVo, bCloseOther, param)
	if not strIsNullOrEmpty(self.preSignStr) then
		if tonumber(self.preSignStr) == iconVo.iconId then
			return true
		end
		DataCenter.preLoadData:removeBySign(self.preSignStr, self, self.onPreLoaded);
		self.preSignStr = nil;
	end

	if iconVo == nil then
		return false;
	end

	if iconVo.operateType == 1 then -- 打开窗口
		if strIsNullOrEmpty(iconVo.operateParam) then
			return false;
		end

		if iconVo.iconId == Lua_MenuViewDefine.ICON_RACCOON then
			self.preSignStr = tostring(iconVo.iconId)
			local artifactId = DataCenter.artifactUtil:getCurArtifact()
			local vo = DataCenter.artifactUtil:getArtifactVoById(artifactId)
			if Lua_WindowUtil.isWindowShowing(iconVo.operateParam) or Scene3DManager.instance:isShowingScene(vo.uiSceneId, 0) then
				self:onPreLoaded(bCloseOther, nil, true);
				return true;
			end
			DataCenter.preLoadData:preShowModelScene(self.preSignStr, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, bCloseOther, vo)
		else
			if Lua_WindowUtil.isWindowShowing(iconVo.operateParam) then
				if bCloseOther == true then
					self.checkIconVo = iconVo
					self:checkCloseWin(iconVo.operateParam)
				end
			else
				if bCloseOther == true then
					self.checkIconVo = iconVo
				end
				Lua_WindowUtil.openWin(iconVo.operateParam, Lua_WinParam.ACTION_OPEN, param or {});
			end
		end
	elseif iconVo.operateType == 2 then -- 派发事件
		Lua_MsgUtil.callFunc(iconVo.operateParam);
	elseif iconVo.operateType == 3 then -- 打开场景
		local sceneId = iconVo.sceneId;
		if iconVo.iconId == Lua_MenuViewDefine.ICON_MAINLINE then
			-- local recordId = DataCenter.fightData:getRecordIdByTemplateId(DataCenter.mainLineData.curTemplateId);
			-- if recordId > 0 then
			-- 	DataCenter.fightData:showFightToStageByRid(recordId);
			-- 	self.selectMainIconId = nil;
			-- 	self:setMainIconSelected(iconVo.iconId);
			-- 	return true;
			-- end
			sceneId = DataCenter.mainLineData.curSceneId;
		end

		self.preSignStr = tostring(iconVo.iconId);
		if Lua_WindowUtil.isWindowShowing(iconVo.operateParam) and Scene3DManager.instance:isShowingScene(sceneId, 0) then
			self:onPreLoaded(bCloseOther, nil, true);
			return true;
		end
		DataCenter.preLoadData:preShowScene(self.preSignStr, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, bCloseOther, sceneId, 0);
	end

	return true;
end

function Lua_MainMenuIconData:onPreLoaded(param, modelScenePanel, isSuccess)
	if isSuccess == true then
		local iconId = tonumber(self.preSignStr)
		local iconVo = self:getMenuIconVo(iconId)
		if iconVo and not strIsNullOrEmpty(iconVo.operateParam) then
			if Lua_WindowUtil.isWindowShowing(iconVo.operateParam) then
				if param == true then
					self.checkIconVo = iconVo
					self:checkCloseWin(iconVo.operateParam)
				end
			else
				if param == true then
					self.checkIconVo = iconVo
				end
				Lua_WindowUtil.openWin(iconVo.operateParam, Lua_WinParam.ACTION_OPEN, { modelScenePanel = modelScenePanel })
			end
		end
	end

	DataCenter.preLoadData:removeBySign(self.preSignStr, self, self.onPreLoaded)
	self.preSignStr = nil
end

function Lua_MainMenuIconData:setMainIconSelected(id)
	if id ~= nil and id ~= self.selectMainIconId then
		self.selectMainIconId = id;
		Lua_MsgUtil.callFunc("MAIN_ICON_SELECT", id);
	end
end

function Lua_MainMenuIconData:checkCloseWin(winName)
	if self.checkIconVo and self.checkIconVo.operateParam == winName then
		if self.checkIconVo.iconType == Lua_MenuViewDefine.TYPE_MAIN_ICON then
			self.selectMainIconId = nil;
			self:setMainIconSelected(self.checkIconVo.iconId)
		end

		-- 战斗结算窗口不可关闭 const配置忽略隐藏到后台
		WindowMananger.getInstance():closeWinsExceptSkip(self.checkIconVo.operateParam,
			"Lua_AllianceDuelBossResultWindow","Lua_AllianceDuelFailWindow","Lua_AllianceDuelWinWindow",
			"Lua_ArenaFightFailWindow","Lua_ArenaFightWinWindow",
			"Lua_ChaosForestFightWinWindow",
			"Lua_ClimbTowerFightFailWindow","Lua_ClimbTowerFightWinWindow",
			"Lua_EquipCopyFightFailWindow","Lua_EquipCopyFightWinWindow",
			"Lua_FightFailResultWindow","Lua_FightWinResultWindow",
			"Lua_InfinityHallFightFailWindow","Lua_InfinityHallFightWinWindow",
			"Lua_KunXuHolyLandFightFailWindow","Lua_KunXuHolyLandFightWinWindow",
			"Lua_MainLineFightWinWindow",
			"Lua_MonsterTrialFightFailResultWindow","Lua_MonsterTrialFightWinResultWindow",
			"Lua_StarTrialFightFailWindow","Lua_StarTrialFightWinWindow",
			"Lua_TrialFightFailWindow","Lua_TrialFightWinWindow"
			)
		self.checkIconVo = nil
	end

	-- 关闭其他主菜单窗口
	if self.mainIconWinDic and self.mainIconWinDic[winName] then
		for k, _ in pairs(self.mainIconWinDic) do
			if k ~= winName then
				Lua_WindowUtil.openWin(k, Lua_WinParam.ACTION_CLOSE)
			end
		end
	end
end