-- 主界面图标ICON
Lua_MainMenuIcon = class("Lua_MainMenuIcon", Lua_DisplayObject);
function Lua_MainMenuIcon:ctor(p)
	self:superCall("ctor", p);
end

function Lua_MainMenuIcon:buildShowItem()
	self:superCall("buildShowItem");

	self.canClick = true;
	self.bIconEff = true;
	self.iconSprite = "";
	self.bSelected = false;

	self.iconImg = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
	self.iconImg.name = "iconImg";
	self.iconImg:addClickFuntion(self, self.onIconClick);
end

function Lua_MainMenuIcon:dispose()
	self:cleanIconEff();
	self:clear();
	if self._rewardItem ~= nil then
		self._rewardItem:dispose();
		self._rewardItem = nil;
	end
	if self.redPointIcon ~= nil then
		self.redPointIcon:dispose();
		self.redPointIcon = nil;
	end
	self.data = nil;
	self:superCall("dispose");
end

function Lua_MainMenuIcon:clear()
	self:removeAlphaTimer();
	self.alphaOwner = nil;
	self.alphaComplete = nil;
	self:removeCountDown();
end

function Lua_MainMenuIcon:setCanClick(val)
	if self.canClick ~= val then
		self.canClick = val;
		self.iconImg.mouseEnable = val;
	end
end

function Lua_MainMenuIcon.Get:uniqueId()
	if self.data == nil then
		return nil;
	end
	return self.data.uniqueId;
end

function Lua_MainMenuIcon.Get:sortNum()
	if self.data == nil then
		return 0;
	end
	return self.data.sortNum;
end

function Lua_MainMenuIcon.Get:iconBg()
	if self._iconBg == nil then
		self._iconBg = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
		self._iconBg.siblingIndex = 0;
		self._iconBg:setEnable(false);
	end
	return self._iconBg;
end

function Lua_MainMenuIcon.Get:selectImg()
	if self._selectImg == nil then
		self._selectImg = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_common_zgnbutton_07", 0, 0, 0, 0);
		self._selectImg.siblingIndex = self.iconImg.siblingIndex;
		self._selectImg:setEnable(false);
	end
	return self._selectImg;
end

function Lua_MainMenuIcon.Get:textBg()
	if self._textBg == nil then
		self._textBg = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_zjm_widget_04", 0, -60.5, 0, 0);
		self._textBg:setEnable(false);
	end
	return self._textBg;
end

function Lua_MainMenuIcon.Get:showText()
	if self._showText == nil then
		self._showText = UICreate.createLabel(self.content, nil, 0, -60.5, 100, 26, 18);
		self._showText.pivot = Lua_WindowUtil.getPivot(4);
		self._showText.alignmentValue = 4;
		self._showText.colorStr = Lua_ColorDefine.transColor("85E9FF");
		self._showText:setEnable(false);
	end
	return self._showText;
end

function Lua_MainMenuIcon.Get:rewardItem()
	if self._rewardItem == nil then
		self._rewardItem = Lua_ItemIcon.new(self.content);
		self._rewardItem:setCanClick(false);
		self._rewardItem:setScale(0.4, 0.4);
		self._rewardItem:setLocation(0, 5, 0);
		self._rewardItem:setEnable(false);
	end
	return self._rewardItem;
end

function Lua_MainMenuIcon:setData(data)
	self.data = data;
	self:updateData();

	if data ~= nil then
		local redPointId = data.redPointId;
		if redPointId ~= nil and DataCenter.redPointData:hasRedPointVoById(redPointId) then
			if self.redPointIcon == nil then
				self.redPointIcon = Lua_RedPointIcon.new(self.content, redPointId);
			end
			self.redPointIcon:changeRedPoint(redPointId);
			self.redPointIcon:setLocation(data.redPointOffsetX, data.redPointOffsetY);
		else
			if self.redPointIcon ~= nil then
				self.redPointIcon:changeRedPoint(nil);
			end
		end
	end
end

function Lua_MainMenuIcon:updateData()
	if self.data ~= nil then
		if self.data.actIdx ~= nil and self.data.actIdx > 0 then
			self.content.name = self.data.uniqueId.."_"..self.data.iconId;
		else
			self.content.name = self.data.uniqueId;
		end

		if self.data:checkShowIconBg() then
			self.iconBg:setEnable(true);
			self.iconBg:setAltas(self.data.bgAtlas, self.data.bgSprite);
			self.iconBg:resetSize();
		else
			if self._iconBg ~= nil then
				self._iconBg:setEnable(false);
			end
		end

		if self.data.iconType == Lua_MenuViewDefine.TYPE_MAIN_ICON then
			self.iconSprite = string.sub(self.data.iconSprite, 1, string.len(self.data.iconSprite) - 1);
			if self.bSelected == true then
				self.iconImg:setAltas(self.data.iconAtlas, self.iconSprite.."p");
			else
				if self.data.isGray then
					self.iconImg:setAltas(self.data.iconAtlas, self.iconSprite.."d");
				else
					self.iconImg:setAltas(self.data.iconAtlas, self.iconSprite.."n");
				end
			end
		else
			self.iconImg:setAltas(self.data.iconAtlas, self.data.iconSprite);
			self.iconImg:setGray(self.data.isGray);
		end
		self.iconImg:setEnable(true);
		self.iconImg:resetSize();
    	self.iconImg.alpha = self.data.iconAlpha;
    	local pos = self.data.showPos;
    	if pos ~= nil then
	    	self.iconImg:setLocation(pos[1], pos[2], pos[3]);
	    end

    	self.bIconEff = true;
		self:updateEff();
		self:updateText();
		self:updateReward();
	else
		self:clear();
		self.content.name = self.classname;
		if self._iconBg ~= nil then
			self._iconBg:setEnable(false);
		end
		self.iconImg:setEnable(false);
		if self._selectImg ~= nil then
			self.selectImg:setEnable(false);
		end

		self:cleanIconEff();
	end
end

function Lua_MainMenuIcon:onIconClick(go)
	if self.redPointIcon ~= nil then self.redPointIcon:clickHandler(); end
	local bCloseOther = self.data.iconType == Lua_MenuViewDefine.TYPE_MAIN_ICON;
    if DataCenter.mainMenuData:clickIconEvent(self.data, bCloseOther) == true then
        self.data.bClickIcon = true;
        self:updateEff();
    end
end

function Lua_MainMenuIcon:updateEff()
	if self.data == nil then
		return;
	end

	if self.data:checkShowEff() then
		self:createIconEff();
	else
		self:cleanIconEff();
	end
end

function Lua_MainMenuIcon:createIconEff()
	if strIsNullOrEmpty(self.data.effectName) then
		self:cleanIconEff();
	else
		if self.iconEff == nil then
			self.iconEff = UICreate.createUIEffect(self.content, self.data.effectUrl, self.data.effectName, 0, 0);
		else
			self.iconEff:setUrl(self.data.effectUrl, self.data.effectName);
		end
		self.iconEff:setLocation(self.data.effectPos[1], self.data.effectPos[2], self.data.effectPos[3]);
		self.iconEff:setEnable(self.bIconEff);
	end
end

function Lua_MainMenuIcon:cleanIconEff()
	if self.iconEff ~= nil then
		self.iconEff:dispose();
		self.iconEff = nil;
	end
end

function Lua_MainMenuIcon:updateReward()
	if self.data == nil then
		return;
	end
	if self.data.itemVo ~= nil then
		self.rewardItem:setEnable(true);
		self.rewardItem.data = self.data.itemVo;
		self.rewardItem:showEffect(true);
	else
		if self._rewardItem ~= nil then
			self.rewardItem:setEnable(false);
		end
	end
end

function Lua_MainMenuIcon:updateText()
	if self.data == nil then
		return;
	end
	local leftTime = self.data.leftTime;
	if leftTime > 0 then
		self.textBg:setEnable(true);
		self.showText:setEnable(true);

		self:onCountDown();
		if self.bTimer ~= true then
			self.bTimer = true;
			self:addTimer("onCountDown", 0.5, self, self.onCountDown);
		end
	else
		if self._showText ~= nil then
			self.showText:setEnable(false);
		end
		if self._textBg ~= nil then
			self.textBg:setEnable(false);
		end
		self:removeCountDown();
	end
end

function Lua_MainMenuIcon:removeCountDown()
	if self.bTimer == true then
		self.bTimer = false;
		Lua_CommonUtil.removeTimer(self, self.onCountDown);
	end
end

function Lua_MainMenuIcon:onCountDown()
	local leftTime = self.data.leftTime;
	if leftTime > 0 then
		if leftTime > 86400 then
			self.showText.text = Lua_TimeTransform.transSeconds26(leftTime, true, false);
		else
			self.showText.text = Lua_TimeTransform.transSeconds6(leftTime, true, true, true);
		end
		
		if self._rewardItem ~= nil then
			self.rewardItem:showEffect(false);
		end
	else
		if self.data.showStr ~= nil and self.data.showStr ~= "" then
			self.showText.text = self.data.showStr;
		else
			if self._showText ~= nil then
				self.showText:setEnable(false);
			end
			if self._textBg ~= nil then
				self.textBg:setEnable(false);
			end
		end

		self:updateReward();
		self:removeCountDown();

		if self.data.bTimeEndRemove ~= nil and self.data.bTimeEndRemove == true then
			DataCenter.mainMenuData:setIconShowInfo(self.data.uniqueId, false);
		end
	end
end

function Lua_MainMenuIcon:setSelected(bSelected)
	self.bSelected = bSelected;
	if bSelected == true then
		self.iconImg:setAltas(self.data.iconAtlas, self.iconSprite.."p");
	else
		if self.data.isGray then
			self.iconImg:setAltas(self.data.iconAtlas, self.iconSprite.."d");
		else
			self.iconImg:setAltas(self.data.iconAtlas, self.iconSprite.."n");
		end
	end
	self.selectImg:setEnable(bSelected);
end

function Lua_MainMenuIcon:setAlpha(alpha)
	if self._iconBg ~= nil then
		self.iconBg.alpha = alpha;
	end
	if self.data.iconAlpha >= 1 then
		self.iconImg.alpha = alpha;
	end
	if self._selectImg ~= nil then
		self.selectImg.alpha = alpha;
	end
	if self._showText ~= nil then
		self.showText.alpha = alpha;
	end
	if self._textBg ~= nil then
		self.textBg.alpha = alpha;
	end
	if self._rewardItem ~= nil then
		self.rewardItem:setAlpha(alpha);
	end
end

function Lua_MainMenuIcon:setIconEffShow(bShow)
	self.bIconEff = bShow;
	if self.iconEff ~= nil then
		self.iconEff:setEnable(bShow);
	end
	if self.redPointIcon ~= nil then
		self.redPointIcon:setEnable(bShow);
	end
end

function Lua_MainMenuIcon:addAlphaTween(startAlpha, endAlpha, stepNum, owner, callback)
	self.startAlpha = startAlpha;
	self.endAlpha = endAlpha;
	self.stepNum = stepNum == nil and 0.3 or stepNum;
	self.alphaOwner = owner;
	self.alphaComplete = callback;

	self:setAlpha(startAlpha);
	if self.bTweenTimer == nil or self.bTweenTimer == false then
		self.bTweenTimer = true;
		Lua_CommonUtil.addTimer("updateIconAlpha", 0.05, self, self.updateIconAlpha, nil, -1);
	end

	if startAlpha > endAlpha then
		self.iconImg.mouseEnable = false;
	else
		self.iconImg.mouseEnable = true;
	end
end

function Lua_MainMenuIcon:updateIconAlpha()
	self.startAlpha = self.startAlpha + self.stepNum;
	if (self.stepNum > 0 and self.startAlpha >= self.endAlpha) or (self.stepNum < 0 and self.startAlpha <= self.endAlpha) then
		self:setAlpha(self.endAlpha);
		self:removeAlphaTimer();
		if self.endAlpha > 0 then
			self:setIconEffShow(true);
		else
			self:setIconEffShow(false);
		end
		if self.alphaOwner ~= nil and self.alphaComplete ~= nil then
			self.alphaComplete(self.alphaOwner, self);
		end
		return;
	end

	self:setAlpha(self.startAlpha);
end

function Lua_MainMenuIcon:removeAlphaTimer()
	if self.bTweenTimer == true then
		self.bTweenTimer = false;
		Lua_CommonUtil.removeTimer(self, self.updateIconAlpha);
	end
end