Lua_ChatIconItem = class("Lua_ChatIconItem", Lua_DisplayObject);
function Lua_ChatIconItem:ctor(p)
    self:superCall("ctor", p);
end

function Lua_ChatIconItem:buildShowItem()
    self:superCall("buildShowItem");
    self.minX = -325;
    self.maxX = 325;
    self.minY = -555 - self.adaptHeight / 2;
    self.maxY = 620 + self.adaptHeight / 2;

    self.offsetX = 0;
    self.startMouseX = -1;
    self.startMouseY = -1;

    self.parentObj = UICreate.createGameObject(self.content, 720, 0, 0);

    local icon = UICreate.createUIAtlasImage(self.parentObj, "common_altas_1", "ui_liaotian_button_008_n", 0, 0, 0, 0);
    icon:addMousDownFuntion(self, self.onIconClick);
    icon:addMouseUpFuntion(self, self.onIconUpClick);

    self.redPoint = UICreate.createUIAtlasImage(self.parentObj, "common_altas_1", "ui_liaotian_button_009_n", 20, 25, 0, 0);

    self.countTxt = UICreate.createLabel(self.parentObj, nil, 20, 25, 100, 31, 20);
    self.countTxt.fontName = Lua_MyUISettings.defaultFontName;
    self.countTxt.colorStr = Lua_ColorDefine.transColorStr("ebe5d9");
    self.countTxt.pivotValue = 4;
    self.countTxt.alignmentValue = 4;

    self:addLuaEvent("ADD_CHAT_MSG", self, self.updateInfo);
    self:addLuaEvent("CHAT_RED_UPDATE", self, self.updateInfo);
    self:addLuaEvent("CHAT_ICON_POS_UPDETA", self, self.updateIconPos);
    self:updateInfo();
end

function Lua_ChatIconItem:dispose()
    Lua_CommonUtil.removeTimer(self, self.onCheckPos);
    if self.parentObj ~= nil and IsNil(self.parentObj) == false then
        UnityEngine.GameObject.Destroy(self.parentObj);
        self.parentObj = nil;
    end
    self:superCall("dispose");
end

function Lua_ChatIconItem:onScreenResize(adaptHeight)
    self:superCall("onScreenResize", adaptHeight);
    self.minY = -555 - adaptHeight / 2;
    self.maxY = 620 + adaptHeight / 2;
end

function Lua_ChatIconItem:updateIconPos(pos)
    if self.parentObj and pos then
        LuaUtil.setObjPos(self.parentObj, pos[1] + self.offsetX, pos[2], 0);
    end
end

function Lua_ChatIconItem:updateInfo()
    local count = DataCenter.chatData:getChannelNewMsgNum(Lua_ChatData.CHANNEL_FRIEND);--私聊数量
    if count > 0 then
        self.offsetX = -20;
        self.redPoint:SetDimensions(57, 48);
        self.redPoint:setLocation(27, 22, 0);
        self.redPoint:setAltas("chat_atlas","ui_liaotian_button_010");

        self.countTxt:setLocation(21, 22, 0);
        self.countTxt.colorStr = Lua_ColorDefine.transColorStr("6d3513");
        self:updateIconPos(DataCenter.chatData.chatIconPos);
    else
        self.offsetX = 0;
        count = DataCenter.chatData:getAllChannelNewMsgNum();

        self.redPoint:SetDimensions(39, 38);
        self.redPoint:setLocation(20, 25, 0);
        self.redPoint:setAltas("common_altas_1","ui_liaotian_button_009_n");

        self.countTxt:setLocation(20, 25, 0);
        self.countTxt.colorStr = Lua_ColorDefine.transColorStr("ebe5d9");
    end

    local shieldCount = DataCenter.chatData:shieldShowRedDotCount();

    if (count - shieldCount) > 0 and DataCenter.funcOpenData:isOpened(10237) then
        self.redPoint:setEnable(true);
        self.countTxt:setEnable(true);
        self.countTxt.text = (count - shieldCount) > 99 and "99" or (count - shieldCount);
    else
        self.redPoint:setEnable(false);
        self.countTxt:setEnable(false);
    end
end

function Lua_ChatIconItem:onIconClick(go)
    self:onPress(go, true)
end

function Lua_ChatIconItem:onIconUpClick(go)
    self:onPress(go, false)
end

function Lua_ChatIconItem:onPress(go, state)
    if self.content == nil then
        return;
    end

    if state == true then
        self.startMouseX = CommonUtil.mousePositionX;
        self.startMouseY = CommonUtil.mousePositionY;
        if self._isClick ~= true then
            self._isClick = true;
            Lua_CommonUtil.addTimer("Lua_ChatIconItem", 0.001, self, self.onCheckPos);
        end
    else
        local dx = CommonUtil.mousePositionX - self.startMouseX;
        local dy = CommonUtil.mousePositionY - self.startMouseY;
        if Mathf.Abs(dx) + Mathf.Abs(dy) < 4 and DataCenter.funcOpenData:isOpened(10237, true) then
            Lua_WindowUtil.openWin("Lua_ChatWindow", Lua_WinParam.ACTION_OPEN, DataCenter.chatData:getAutoOpenChannel());
        end

        self.startMouseX = -1;
        self.startMouseY = -1;
        if self._isClick == true then
            self._isClick = false;
            Lua_CommonUtil.removeTimer(self, self.onCheckPos);
        end
    end
end

function Lua_ChatIconItem:onCheckPos()
    local dx = CommonUtil.mousePositionX - self.startMouseX;
    local dy = CommonUtil.mousePositionY - self.startMouseY;
    if Mathf.Abs(dx) + Mathf.Abs(dy) < 4 then
        return;
    end

    local winPos = LuaUtil.getTouchPosInverseObj(self.content);
    if winPos.x > self.maxX + self.offsetX then
        winPos.x = self.maxX + self.offsetX;
    elseif winPos.x < self.minX then
        winPos.x = self.minX;
    end
    if winPos.y > self.maxY then
        winPos.y = self.maxY;
    elseif winPos.y < self.minY then
        winPos.y = self.minY;
    end
    DataCenter.chatData.chatIconPos = {winPos.x, winPos.y}
    LuaUtil.setObjPos(self.parentObj, winPos.x, winPos.y, 0);
end