Lua_FuncPathUtil = class("Lua_FuncPathUtil")
function Lua_FuncPathUtil:ctor()
	self:superCall("ctor")
	self.pathDic = {}
end

function Lua_FuncPathUtil:dispose()
	self.fightGotoFuncPathDic = nil;
end

function Lua_FuncPathUtil:getFuncPathVo(funcId)
	if funcId == nil then
		return nil
	end
	if self.pathDic[funcId] == nil then
		local config = getConfig("function_path", funcId)
		if config == nil then
			return nil
		end
		self.pathDic[funcId] = Lua_FuncPathVo.new(funcId, config)
	end
	return self.pathDic[funcId]
end

function Lua_FuncPathUtil:gotoFunction(funcId, bCloseOther, checkIsFighting, unRemoveWin)
	if Lua_PlatformUtil.IsBanShu() and funcId == 1024 then
		return false
	end

	local vo = self:getFuncPathVo(funcId)
	if vo == nil then
		return false
	end

	if vo.openState == false then
		if not strIsNullOrEmpty(vo.promptStr) then
			OperateAlert.show(vo.promptStr)
		end
		return false
	end

	if checkIsFighting ~= false then
		local recordId = self:getRecordId(vo.template_list)
		if recordId > 0 then
			-- self:addUnRemoveWin("Lua_FightMainWindow")
			self:setFightRecordFuncPath(recordId, funcId, bCloseOther, unRemoveWin);
			DataCenter.fightData:showFightToStageByRid(recordId)
			return true -- 策划需求：进入战斗后，结算后显示对应功能界面
		end
	end

	if self.clickTime and (getTime() - self.clickTime) < 1 and self.curGotoVo == vo then
		return false
	end
	self.clickTime = getTime()

	self.bCloseOther = bCloseOther
	self:addUnRemoveWin(unRemoveWin)

	local _vo
	if vo.type == 1 then
		if funcId == 1010 then -- 爬塔入口
			if not DataCenter.climbTowerData:checkCampTowerIsOpen() then
				return self:gotoFunction(1030, bCloseOther, checkIsFighting, unRemoveWin) -- 通天塔
			end
		elseif funcId == 1014 then -- 无穷战殿
			if not DataCenter.infinityHallData.bInfinityHallOpened then
				return self:gotoFunction(1072, bCloseOther, checkIsFighting, unRemoveWin) -- 无尽火域入口
			end
		end

		self.curGotoVo = vo

		-- 提前添加不可移除窗口
		if vo.first_path then
			for i = 1, #vo.first_path do
				_vo = self:getFuncPathVo(vo.first_path[i])
				self:addUnRemoveWin(_vo.winName)
			end
		end

		self:addUnRemoveWin(vo.winName)

		if vo.next_path then
			for i = 1, #vo.next_path do
				_vo = self:getFuncPathVo(vo.next_path[i])
				self:addUnRemoveWin(_vo.winName)
			end
		end

		if vo.isShowing then
			-- if vo.act_id > 0 or vo.act_type > 0 then
			-- 	vo:gotoFunc()
			-- end
			vo:gotoFunc()
			self:checkCloseWin(vo.winName)
		else
			-- 按顺序打开窗口
			if vo.first_path then
				for i = 1, #vo.first_path do
					_vo = self:getFuncPathVo(vo.first_path[i])
					_vo:gotoFunc()
				end
			end
			
			local actVo
			if vo.act_id > 0 then
				actVo = DataCenter.operationalActUtil:setPreLoadById(vo.act_id)
			elseif vo.act_type > 0 then
				actVo = DataCenter.operationalActUtil:setPreLoadByType(vo.act_type)
			end
			if actVo == nil or strIsNullOrEmpty(actVo.winBgUrl) then
				self:onResLoaded()
			else
				LoadManager.getInstance():Load(Lua_LoadFileType.IMG, actVo.winBgUrl, self, self.onResLoaded, self.onResLoaded, nil, 999, 30)
			end
		end
	else
		self.curGotoVo = vo
		if vo.open_path then
			for i = 1, #vo.open_path do
				_vo = self:getFuncPathVo(vo.open_path[i])
				if _vo and _vo.openState then
					return self:gotoFunction(vo.open_path[i], bCloseOther, checkIsFighting, unRemoveWin)
				end
			end
		end
	end

	return true
end

function Lua_FuncPathUtil:onResLoaded()
	if self.curGotoVo == nil then
		return
	end
	local list = self.curGotoVo.next_path
	self.curGotoVo:gotoFunc()
	if list then
		local vo
		for i = 1, #list do
			vo = self:getFuncPathVo(list[i])
			vo:gotoFunc()
		end
	end
end

function Lua_FuncPathUtil:getRecordId(list)
	local recordId = 0
	if list then
		for i = 1, #list do
			recordId = DataCenter.fightData:getRecordIdByTemplateId(list[i])
			if recordId > 0 then
				break
			end
		end
	end
	return recordId
end

function Lua_FuncPathUtil:addUnRemoveWin(winName)
	if self.bCloseOther == true and not strIsNullOrEmpty(winName) then
		if self.unRemoveWins == nil then
			self.unRemoveWins = Array()
		end
		if self.unRemoveWins:indexOf(winName) == -1 then
			self.unRemoveWins:add(winName)
		end
	end
end

function Lua_FuncPathUtil:checkCloseWin(winName)
	if self.curGotoVo and self.curGotoVo.winName == winName then
		-- Lua_MsgUtil.callFunc("FUNC_PATH_SUCCESS", self.curGotoVo.id)
		DataCenter.mainMenuData:setMainIconSelected(self.curGotoVo.mainIconId)
		if self.bCloseOther == true and self.unRemoveWins ~= nil then
			if self.curGotoVo.operate_type == 4 then
				WindowMananger.getInstance():closeWinsExceptSkip(self.unRemoveWins[1], self.unRemoveWins[2], self.unRemoveWins[3], self.unRemoveWins[4], self.unRemoveWins[5])
			else
				WindowMananger.getInstance():closeWinsExceptNever(self.unRemoveWins[1], self.unRemoveWins[2], self.unRemoveWins[3], self.unRemoveWins[4], self.unRemoveWins[5])
			end
			self.bCloseOther = nil
			self.unRemoveWins:clear()
			self.unRemoveWins = nil
			DataCenter.popupWinManager:checkPopupWin()
		end
		self.curGotoVo = nil
		self.clickTime = nil
	end
end

-- 记录战斗跳转
function Lua_FuncPathUtil:setFightRecordFuncPath(recordId, funcId, bCloseOther, unRemoveWin)
	if self.fightGotoFuncPathDic == nil then
		self.fightGotoFuncPathDic = {};
	end
	local obj = self.fightGotoFuncPathDic[recordId];
	if obj == nil then 
		obj = {}; 
		self.fightGotoFuncPathDic[recordId] = obj;
	end
	obj.funcId = funcId;
	obj.bCloseOther = bCloseOther;
	obj.unRemoveWin = unRemoveWin;
end

function Lua_FuncPathUtil:getFightRecordFuncPath(recordId)
	if self.fightGotoFuncPathDic == nil then
		return;
	end
	recordId = tonumber(recordId);
	return self.fightGotoFuncPathDic[recordId];
end

function Lua_FuncPathUtil:clearFightRecordFuncPath(recordId)
	if self.fightGotoFuncPathDic == nil then
		return;
	end
	recordId = tonumber(recordId);
	self.fightGotoFuncPathDic[recordId] = nil;
end

function Lua_FuncPathUtil:checkFightRecordFuncPath(recordId)
	if self.fightGotoFuncPathDic == nil then
		return;
	end

	recordId = tonumber(recordId);
	local obj = self.fightGotoFuncPathDic[recordId];
	if obj == nil then
		return;
	end

	self.fightGotoFuncPathDic[recordId] = nil;
	self:gotoFunction(obj.funcId, obj.bCloseOther, false, obj.unRemoveWin);
end

function Lua_FuncPathUtil:removePreLoad()
	if self.curGotoVo then
		self.curGotoVo:removePreLoad()
		Lua_WindowUtil.clearWinData(self.curGotoVo.winName)
		self.curGotoVo = nil
	end
end


















Lua_FuncPathVo = class("Lua_FuncPathVo")
function Lua_FuncPathVo:ctor(id, config)
	self.id = id
	self._config = config
end

function Lua_FuncPathVo.Get:config()
	if self._config == nil then
		self._config = getConfig("function_path", self.id)
	end
	return self._config
end

-- 开启状态
function Lua_FuncPathVo.Get:openState()
	if self.open_cond > 0 and not DataCenter.funcOpenData:isOpened(self.open_cond) then
		return false
	end
	if self.act_id > 0 and not DataCenter.internalActData:checkActIsOpened(self.act_id) then
		return false
	end
	if self.act_type > 0 and DataCenter.backstageActData:getActIdxArrByType(self.act_type) == nil then
		return false
	end
	if self.id == 1025 then -- 巅峰对决
		return DataCenter.peakDuelData.openState
	end
	if self.id == 1030 or self.id == 1076 or self.id == 1077 or self.id == 1078 or self.id == 1079 then -- 爬塔
		local vo = DataCenter.climbTowerData:getTowerVoById(self.window.main_id)
		if vo and not vo.isOpened then
			return false
		end
	elseif self.id >= 1058 and self.id <= 1063 then -- 公会
		return DataCenter.roleData:isJoinTheLeague();
	end
	return true
end

-- 未开启提示
function Lua_FuncPathVo.Get:promptStr()
	if not strIsNullOrEmpty(self.prompt) then
		return self.prompt
	end

	local vo = DataCenter.funcOpenUtil:getFuncOpenVoById(self.open_cond)
	if self.id == 1030 or self.id == 1076 or self.id == 1077 or self.id == 1078 or self.id == 1079 then
		if not vo.isOpen then
			return vo.advanceMessage
		end

		vo = DataCenter.climbTowerData:getTowerVoById(self.window.main_id)
		if vo and not vo.isOpened then
			return vo.open_str..vo.name
		end
	elseif self.id >= 1058 and self.id <= 1063 then -- 公会
		if vo.isOpen and not DataCenter.roleData:isJoinTheLeague() then
			return LanUtil.getValue(201295);
		end
	end

	if vo then
		return vo.advanceMessage
	end

	return nil
end

-- 功能是否显示
function Lua_FuncPathVo.Get:isShowing()
	if self.act_id > 0 or self.act_type > 0 then
		return false
	end
	if self.operate_type == 1 then
		if self.window then
			return Lua_WindowUtil.isWindowShowing(self.window.winName)
		end
	elseif self.operate_type == 4 then
		if self.window then
			return Lua_WindowUtil.isWindowShowing(self.window.winName)
		end
		local sceneId = self.sceneId
		if self.id == 1005 or self.id == 1066 then -- 主线冒险
			sceneId = DataCenter.mainLineData.curSceneId
		end
		return Scene3DManager.instance:isShowingScene(sceneId, 0)
	end
	return false
end

-- 不提供外部调用
function Lua_FuncPathVo:gotoFunc()
	if self.operate_type == 1 then
		if self.window then
			Lua_WindowUtil.openWin(self.window.winName, Lua_WinParam.ACTION_OPEN, self.window)
		end
	elseif self.operate_type == 2 then
		Lua_MsgUtil.callFunc(self.event_name, self.event_param)
	elseif self.operate_type == 3 then
		local actArr = DataCenter.backstageActData:getActIdxArrByType(self.act_type)
		if actArr then
			local actVo = DataCenter.backstageActData:getActVoByIdx(actArr[1])
			if actVo == nil or strIsNullOrEmpty(actVo.main_win) then
				LogManager.LogError("运营活动 打开界面 配置错误！")
			else
				Lua_WindowUtil.openWin(actVo.main_win, Lua_WinParam.ACTION_OPEN, actVo)
			end
		end
	elseif self.operate_type == 4 then
		self:onPreLoad()
	end
end

function Lua_FuncPathVo:onPreLoad()
	if tostring(self.id) == self.preSignStr then
		return
	end
	self.preSignStr = tostring(self.id)
	if self.id == 1021 or self.id == 1053 then -- 神器
		local artifactId = DataCenter.artifactUtil:getCurArtifact()
		local artifactVo = DataCenter.artifactUtil:getArtifactVoById(artifactId)
		DataCenter.preLoadData:preShowModelScene(self.preSignStr, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, nil, artifactVo)
	elseif self.id == 1036 then -- 幽暗森林
		local config = DataCenter.chaosForestUtil:getLevelConfig(DataCenter.chaosForestData.boss_id)
		config = Lua_MonsterPlayerConfigData.getConfigData(config.monster_id)
		config = Lua_MonsterConfigData.getMonsterConfig(config.fighter_list[1][1][1])
		DataCenter.preLoadData:preShowModelScene(self.preSignStr, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, nil, config.heroVo)
	elseif self.id == 1062 then -- 公会boss
		DataCenter.preLoadData:preShowModelScene(self.preSignStr, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, nil, DataCenter.allianceData.bossStageInfo.monsterVo);
	elseif self.id == 1037 then -- 魔灵岛单人
		local arr = DataCenter.equipCopyUtil:getSingleCapyDicById(DataCenter.equipCopyData.copyOpenId);
		local bossId = DataCenter.equipCopyUtil:getSingleBossIdById(arr[1].id);
		local config = Lua_MonsterConfigData.getMonsterConfig(bossId);
		config.heroVo.uiSceneId = Lua_EquipCopyUtil.moreSceneId;
		DataCenter.preLoadData:preShowModelScene(self.preSignStr, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, nil, config.heroVo)
	elseif self.id == 1117 then -- 魔灵岛组队
		local list = getConfig("equip_copy_more");
		local max_difficulty = DataCenter.equipCopyData.max_difficulty;
		local index = max_difficulty == 0 and 1 or max_difficulty;
		local id = list[index].boss_info[1];
		local boosId = DataCenter.equipCopyUtil:getTheBossId(id);
		local config = Lua_MonsterConfigData.getMonsterConfig(boosId);
		config.heroVo.uiSceneId = Lua_EquipCopyUtil.singleSceneId;
		DataCenter.preLoadData:preShowModelScene(self.preSignStr, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, nil, config.heroVo)
	else
		local sceneId = self.sceneId
		if self.id == 1005 or self.id == 1066 then -- 主线冒险
			sceneId = DataCenter.mainLineData.curSceneId
		end
		if sceneId == nil then
			self:onPreLoaded(nil, nil, true)
		else
			DataCenter.preLoadData:preShowScene(self.preSignStr, Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onPreLoaded, nil, sceneId, 0)
		end
	end
end

function Lua_FuncPathVo:onPreLoaded(param, modelScenePanel, isSuccess)
	self:removePreLoad()
	if isSuccess == true and not strIsNullOrEmpty(self.winName) then
		if self.operate_type == 3 then
			local actArr = DataCenter.backstageActData:getActIdxArrByType(self.act_type)
			local actVo = DataCenter.backstageActData:getActVoByIdx(actArr[1])
			Lua_WindowUtil.openWin(self.winName, Lua_WinParam.ACTION_OPEN, actVo)
		else
			local info = self.window
			if info == nil then
				info = {}
			end
			info.modelScenePanel = modelScenePanel
			Lua_WindowUtil.openWin(self.winName, Lua_WinParam.ACTION_OPEN, info)
		end
	end
end

function Lua_FuncPathVo.Get:winName()
	if self.window then
		return self.window.winName
	end
	local actArr = DataCenter.backstageActData:getActIdxArrByType(self.act_type)
	if actArr ~= nil then
		local actVo
		for i = 1, actArr:getSize() do
			actVo = DataCenter.backstageActData:getActVoByIdx(actArr[i])
			if actVo ~= nil and not strIsNullOrEmpty(actVo.main_win) then
				return actVo.main_win
			end
		end
	end
	return nil
end

function Lua_FuncPathVo:removePreLoad()
	DataCenter.preLoadData:removeBySign(self.preSignStr, self, self.onPreLoaded)
	self.preSignStr = nil
end

-------------------- 配置数据 --------------------

function Lua_FuncPathVo.Get:name()
	return self.config.name
end

function Lua_FuncPathVo.Get:type()
	return self.config.type or 1
end

function Lua_FuncPathVo.Get:icon_atlas()
	return self.config.icon_atlas
end

function Lua_FuncPathVo.Get:icon_sprite()
	return self.config.icon_sprite
end

function Lua_FuncPathVo.Get:description()
	return self.config.description
end

function Lua_FuncPathVo.Get:btn_name()
	return self.config.btn_name
end

function Lua_FuncPathVo.Get:open_path()
	return self.config.open_path
end

function Lua_FuncPathVo.Get:first_path()
	return self.config.first_path
end

function Lua_FuncPathVo.Get:next_path()
	return self.config.next_path
end

function Lua_FuncPathVo.Get:open_cond()
	return self.config.open_cond or 0
end

function Lua_FuncPathVo.Get:prompt()
	return self.config.prompt
end

function Lua_FuncPathVo.Get:operate_type()
	return self.config.operate_type
end

function Lua_FuncPathVo.Get:window()
	return self.config.window
end

function Lua_FuncPathVo.Get:event_name()
	return self.config.event_name
end

function Lua_FuncPathVo.Get:event_param()
	return self.config.event_param
end

function Lua_FuncPathVo.Get:act_id()
	return self.config.act_id or 0
end

function Lua_FuncPathVo.Get:act_type()
	return self.config.act_type or 0
end

function Lua_FuncPathVo.Get:sceneId()
	return self.config.scene_id
end

function Lua_FuncPathVo.Get:template_list()
	return self.config.template_list
end

function Lua_FuncPathVo.Get:red_point_id()
	return self.config.red_point_id
end

function Lua_FuncPathVo.Get:mainIconId()
	return self.config.main_icon_id
end