-- 图标信息
Lua_MenuIconVo = class("Lua_MenuIconVo");
function Lua_MenuIconVo:ctor(uniqueId, iconId, config)
	self.uniqueId = uniqueId;
	self._iconId = iconId;
	self._config = config;
	self:resetInfo();
end

function Lua_MenuIconVo.Set:iconId(value)
	if self._iconId ~= value then
		self._config = nil;
	end
	self._iconId = value
end

function Lua_MenuIconVo.Get:iconId()
	return self._iconId;
end

function Lua_MenuIconVo.Get:config()
	if self._config == nil then
		self._config = getConfig("mainMenuIcon", self.iconId);
	end
	return self._config;
end

-- 重置非配置表数据
function Lua_MenuIconVo:resetInfo()
	self._redPointId = nil;
	self.showState = true;
	self.bFunOpenState = true;
	self.bClickIcon = false; -- 该次登陆已点击
	self.leftTime = nil;
	self.actIdx = 0; -- 活动索引
	self.posX = 0;
	self.posY = 0;
end

function Lua_MenuIconVo:setFunOpenState(state)
	self.bFunOpenState = state;
end

function Lua_MenuIconVo:setShowState(state)
	self.showState = state;
end

function Lua_MenuIconVo.Get:bRedDotState()
	return DataCenter.redPointData:getRedPointStateById(self.redPointId);
end

-- 是否显示
function Lua_MenuIconVo.Get:canShow()
	if self.showState == false then
		return false;
	end

	if self.bFunOpenState == false then
		if self.unOpenShow == 3 then
			return false;
		end
	end

    return true;
end

-- 是否显示特效
function Lua_MenuIconVo:checkShowEff()
	if self.bFunOpenState == false then
		return false;
	end
	
	if self.effectShow == 1 then
		return true;
	elseif self.effectShow == 0 then
		return false;
	end

	if self.bClickIcon == true then
		return false;
	end

	return true;
end

-- 判断图标收缩状态下是否显示
function Lua_MenuIconVo:checkShrinkShow()
	return self.shrinkShow > 0;
end

-- 判断图标是否显示背景底图
function Lua_MenuIconVo:checkShowIconBg()
	return self.bgAtlas ~= nil and self.bgSprite ~= nil;
end

-- 是否置灰
function Lua_MenuIconVo.Get:isGray()
	if self.bFunOpenState == false and self.unOpenShow == 1 then
		return true;
	end

    return false;
end

-- 未开启提示
function Lua_MenuIconVo.Get:unOpenAdvanceTips()
	if self.unOpenShow == 1 or self.unOpenShow == 2 then
		local vo = DataCenter.funcOpenUtil:getFuncOpenVoByIconId(self.iconId);
		if vo ~= nil and vo.isOpen == false then
			return vo.advanceMessage;
		end
	end
    return nil;
end

function Lua_MenuIconVo.Set:leftTime(val)
	self._leftTime = val;
	self.setLeftTime = getTime();
end

function Lua_MenuIconVo.Get:leftTime()
	if self._leftTime == nil then
		return 0;
	end
	
	local num = Mathf.Floor(self._leftTime - (getTime() - self.setLeftTime));
	if num <= 0 then
		return 0;
	end
	return num;
end

function Lua_MenuIconVo.Get:posType()
	if self._posType ~= nil then
		return self._posType;
	end
	
	if self.config.pos_type == nil then
		return 0;
	end
	return self.config.pos_type;
end

function Lua_MenuIconVo.Set:posType(val)
	self._posType = val;
end

-- 放置位置 后台活动可设置
function Lua_MenuIconVo.Get:pos()
	if self._pos ~= nil then
		return self._pos;
	end

	return self.put_pos;
end

function Lua_MenuIconVo.Set:pos(val)
	if val <= 0 then
		val = 3;
	end
	self._pos = val;
end

-- 左上角或右上角图标排序
function Lua_MenuIconVo.Get:sortNum()
	if self._sortNum == nil then
		self._sortNum = self.pos * 1000 + self.sortWeight;
	end
	return self._sortNum;
end

-- 排列权重（越小越靠右）
function Lua_MenuIconVo.Get:sortWeight()
	if self._sortWeight ~= nil then
		return self._sortWeight;
	end

	if self.config.sort_num == nil then
		return 0;
	end
	return self.config.sort_num;
end

function Lua_MenuIconVo.Set:sortWeight(val)
	self._sortWeight = val;
end

function Lua_MenuIconVo.Get:actIdx()
	return self._actId;
end

function Lua_MenuIconVo.Set:actIdx(val)
	self._actId = val;
end

-------------------- 配置数据 --------------------

function Lua_MenuIconVo.Get:name()
	return self.config.name;
end

function Lua_MenuIconVo.Get:type()
	return self.config.type or 0;
end

function Lua_MenuIconVo.Get:iconType()
	return self.config.icon_type or 0;
end

function Lua_MenuIconVo.Get:isFixed()
	return self.config.is_fixed == 1;
end

function Lua_MenuIconVo.Get:put_pos()
	return self.config.put_pos or 1;
end

function Lua_MenuIconVo.Get:put_num()
	return self.config.put_num;
end

function Lua_MenuIconVo.Get:iconAtlas()
	return self.config.icon_atlas;
end

function Lua_MenuIconVo.Get:iconSprite()
	return self.config.icon_sprite;
end

function Lua_MenuIconVo.Get:iconWidth()
	return self.config.icon_width;
end

function Lua_MenuIconVo.Get:iconHeight()
	return self.config.icon_height;
end

function Lua_MenuIconVo.Get:iconAlpha()
	return self.config.icon_alpha or 1;
end

function Lua_MenuIconVo.Get:bgAtlas()
	return self.config.bg_atlas;
end

function Lua_MenuIconVo.Get:bgSprite()
	return self.config.bg_sprite;
end

-- 操作类型
function Lua_MenuIconVo.Get:operateType()
    return self.config.operate_type;
end

-- 操作参数
function Lua_MenuIconVo.Get:operateParam()
	return self.config.operate_param;
end

function Lua_MenuIconVo.Get:sceneId()
	return self.config.scene_id or 0;
end

-- 未开启显示
function Lua_MenuIconVo.Get:unOpenShow()
	return self.config.unopen_show or 3;
end

-- 是否显示特效
function Lua_MenuIconVo.Get:effectShow()
	return self.config.effect_show or 0;
end

function Lua_MenuIconVo.Get:effectUrl()
	if self._effectUrl == nil then
		self._effectUrl = LanUtil.StringProfile("/effects/uiEffects/", self.effectName);
	end
	return self._effectUrl;
end

function Lua_MenuIconVo.Get:effectName()
	return self.config.effect_name;
end

-- 收缩状态下是否显示
function Lua_MenuIconVo.Get:shrinkShow()
	return self.config.shrinkShow or 0;
end

function Lua_MenuIconVo.Get:showPos()
    return self.config.show_pos;
end

function Lua_MenuIconVo.Get:effectPos()
	if self._effectPos == nil then
		if self.config.effect_pos then
			local x = #self.config.effect_pos > 0 and self.config.effect_pos[1] or 0;
			local y = #self.config.effect_pos > 1 and self.config.effect_pos[2] or 0;
			local z = #self.config.effect_pos > 2 and self.config.effect_pos[3] or 0;
			self._effectPos = {x, y, z};
		else
			self._effectPos = {0, 0, 0};
		end
	end
	return self._effectPos;
end

function Lua_MenuIconVo.Set:redPointId(val)
	if self.actIdx ~= nil and self.actIdx > 0 then -- 运营活动
		self._redPointId = val;
	end
end

function Lua_MenuIconVo.Get:redPointId()
	if self.actIdx ~= nil and self.actIdx > 0 then -- 运营活动
		return self._redPointId;
	else
		if self.config == nil or self.config.redPointId == nil then
			return nil;
		end
		return self.config.redPointId;
	end
end

function Lua_MenuIconVo.Get:redPointOffsetX()
	if self.config == nil or self.config.redPointOffset == nil or #self.config.redPointOffset < 1 then
		return 0;
	end
	return self.config.redPointOffset[1];
end

function Lua_MenuIconVo.Get:redPointOffsetY()
	if self.config == nil or self.config.redPointOffset == nil or #self.config.redPointOffset < 2 then
		return 0;
	end
	return self.config.redPointOffset[2];
end