-- 主线场景控制管理器
checkRequire("Lua_SceneWanderPointConfigData");
checkRequire("Lua_MonsterFightGroupConfigData");
Lua_MainLineSceneController = class("Lua_MainLineSceneController")
function Lua_MainLineSceneController:ctor()
	self:superCall("ctor");
	self:dispose();
end

function Lua_MainLineSceneController:dispose()
	self._curShowMainLineSceneId = 0; -- 当前显示主线场景id，用于场景显隐控制，不对外访问
	self._mainLineSceneId = 0; -- 当前主线场景id
	self._bFirstSceneLoad = false; -- 是否初次加载
	self._mainLineFollowId = 0;
    self._stopWanderPId = nil;
    self._bStopWander = false;
    if self._followPlayerIds ~= nil then
        self._followPlayerIds:clear();
        self._followPlayerIds = nil;
    end

	Lua_CommonUtil.removeTimer(self, self.onStartUpdateBackgroundRes);
end

function Lua_MainLineSceneController.Get:mainLineFollowId()
	return self._mainLineFollowId;
end

-- 当前主线场景
function Lua_MainLineSceneController.Get:mainLineSceneId()
	return self._mainLineSceneId;
end

function Lua_MainLineSceneController.Get:bStopWander()
    return self._stopWanderPId ~= nil and self._stopWanderPId > 0;
end

-- 暂停漫游
function Lua_MainLineSceneController:stopWander(pid)
    if pid ~= nil and pid <= 0 then
        pid = nil;
    end

    if pid == nil then
        pid = self._targetPointId;
    end

    if pid == nil then
        local sc = Lua_SceneConfigData.getSceneConfig(self._mainLineSceneId);
        if sc ~= nil and sc.isWanderScene then
            local sd = Lua_SceneWanderPointConfigData.getConfigData(sc.wanderStartPoint);
            if sd ~= nil then
                pid = sd.nextPosId;
            end
        end
    end

    self._stopWanderPId = pid;
    OperateAlert.show("暂停漫游点ID：" .. pid);
end

-- 继续漫游
function Lua_MainLineSceneController:continueWander()
    if self._stopWanderPId ~= nil then
        self._stopWanderPId = nil;
        if self._bStopWander == true then
            self._bStopWander = false;
            self:moveCompleteFun();
        end
    end
end

-- 进入场景
function Lua_MainLineSceneController:enterScene(sceneId)
	if sceneId <= 0 or self._mainLineSceneId == sceneId then
		return;
	end

    local sc = Lua_SceneConfigData.getSceneConfig(sceneId);
    if sc == nil then
        if self._bFirstSceneLoad ~= true then -- 首次
            local tipsData = {
                layer = MainWinDefine.MAX_TOP_LAYER,
                des = LanUtil.formateStr(107470, tostring(sceneId)),
                btnType = 3
            };
            Lua_WindowUtil.openWin("Lua_TipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
        end
        return;
    end

	self._mainLineSceneId = sceneId;
	Scene3DManager.instance:OpenSceneBySceneId(sceneId, 0, not self._bFirstSceneLoad);
end

-- 场景显示完成
function Lua_MainLineSceneController:onSceneShow(sceneId)
    if self._mainLineSceneId ~= sceneId then
        return;
    end

	if self._curShowMainLineSceneId ~= self.mainLineSceneId then
        Scene3DManager.instance:destroyScene(self._curShowMainLineSceneId, 0);
        self._curShowMainLineSceneId = self.mainLineSceneId;
        self:initScenePlayers(self._curShowMainLineSceneId);
        DataCenter.camFollowBackManager:setFollower(self.mainLineSceneId, 0, self.mainLineFollowId);
        DataCenter.camFollowBackManager:checkStartMove(self.mainLineSceneId, 0, self.mainLineFollowId, true);
    else
        DataCenter.camFollowBackManager:setFollower(self.mainLineSceneId, 0, self.mainLineFollowId);
        DataCenter.camFollowBackManager:checkStartMove(self.mainLineSceneId, 0, self.mainLineFollowId, false);
        DataCenter.camFollowBackManager:checkStartMove(self.mainLineSceneId, 0, self.mainLineFollowId, true);
    end
        
    if self._bFirstSceneLoad ~= true then
        self._bFirstSceneLoad = true;
        Lua_WindowUtil.openWin("Lua_MainLineWindow", Lua_WinParam.ACTION_OPEN);
    end
end

function Lua_MainLineSceneController:initScenePlayers(sceneId)
	local sc = Lua_SceneConfigData.getSceneConfig(sceneId);
    if sc ~= nil and sc.isWanderScene then
        self:generatePlayer(sc.wanderGroupId);
        local sd = Lua_SceneWanderPointConfigData.getConfigData(sc.wanderStartPoint);
        if sd ~= nil then
            while (sd ~= nil and sd.isMoveToTarget ~= true) do -- 瞬移
                sd = Lua_SceneWanderPointConfigData.getConfigData(sd.nextPosId);
            end

            if sd == nil then
                return; 
            end

            -- 初始化位置
            if sd.posVec ~= nil then
                local px = sd.posVec.x;
                local py = sd.posVec.y;
                local pz = sd.posVec.z;
                self._targetPointId = sd.nextPosId;
                local nextSd = Lua_SceneWanderPointConfigData.getConfigData(self._targetPointId);
                if nextSd ~= nil and nextSd.posVec ~= nil then
                    local moveDir = MathUtil.getDirectionByXY(px, pz, nextSd.posVec.x, nextSd.posVec.z);
                    if self._followPlayerIds ~= nil then
                        for i = 1, self._followPlayerIds:getSize() do
                            ThingUtil.instance:setThingDir(self.mainLineSceneId, 0, self._followPlayerIds[i], moveDir);
                        end
                    end

                    self.dirVec = Vector3(nextSd.posVec.x - px, py, nextSd.posVec.z - pz);
                    self.dirVec:SetNormalize();
                    local upVec = Vector3(0, 1, 0);
                    self.rightVec = Vector3.Cross(self.dirVec, upVec);
                    local mfgcd = Lua_MonsterFightGroupConfigData.getConfigData(sc.wanderGroupId);
                    local posOffsetxArr = mfgcd.posOffsetxArr;
                    local posOffsetzArr = mfgcd.posOffsetzArr;

                    -- 更新位置
                    if self._followPlayerIds ~= nil then
                        for i = 1, self._followPlayerIds:getSize() do
                            ThingUtil.instance:setThingPos(self.mainLineSceneId, 0, self._followPlayerIds[i], px + posOffsetxArr[i] * self.rightVec.x + posOffsetzArr[i] * self.dirVec.x, py, pz + posOffsetxArr[i] * self.rightVec.z + posOffsetzArr[i] * self.dirVec.z, true);
                        end
                    end
                    
                    if self._stopWanderPId ~= nil and self._stopWanderPId == self._targetPointId then
                        self._bStopWander = true;
                        return;
                    end

                    -- 更新移动目标位置
                    px = nextSd.posVec.x;
                    py = nextSd.posVec.y;
                    pz = nextSd.posVec.z;
                    if self._followPlayerIds ~= nil then
                        for i = 1, self._followPlayerIds:getSize() do
                            ThingUtil.instance:setAnimalMoveTo(self.mainLineSceneId, 0, self._followPlayerIds[i], px + posOffsetxArr[i] * self.rightVec.x + posOffsetzArr[i] * self.dirVec.x, py, pz + posOffsetxArr[i] * self.rightVec.z + posOffsetzArr[i] * self.dirVec.z);
                        end
                    end
                end
            end
        end
    end
end

function Lua_MainLineSceneController:generatePlayer(wanderGroupId)
    local mfgcd = Lua_MonsterFightGroupConfigData.getConfigData(wanderGroupId);
    if mfgcd == nil or mfgcd.formationType ~= Lua_FightDefine.FIGHT_ATTACKER then
        return;
    end

    local list = mfgcd.fighter_list;
    if list == nil or #list <= 0 then
        return;
    end

    for i = 1, #list do
        local objId = list[i];
        if objId == 0 then -- 主角
            local data = UsData();-- 创建主角
            local rid = DataCenter.roleData.roleId * 100 + i;
            data.roleId = rid;
            data.kind = Lua_ThingDefine.PLAYER;
            data.spd = 400;
            data.hp = 100;
            -- data.nickname = "主角";
            data.sceneId = self._curShowMainLineSceneId;
            data.formationType = Lua_FightDefine.FIGHT_ATTACKER;
            data.formationPos = i;
            data.arriveStop = 0;
            data.isShowName = false;
            if self._followPlayerIds == nil then
                self._followPlayerIds = Array();
            end
            self._followPlayerIds:add(rid);
            if i == 1 then 
                self._mainLineFollowId = rid;
                ThingUtil.instance:addUserAndMoveFun(data, self, self.moveCompleteFun);
            else
                ThingUtil.instance:addUser(data);
            end
        else
            local data;
            local cfg = Lua_MonsterConfigData.getMonsterConfig(objId);
            if cfg ~= nil then
                data = MonsterData();
                data.kind = Lua_ThingDefine.MONSTER;
            else
                cfg = Lua_HeroConfigData.getConfig(objId);
                if cfg ~= nil then
                    data = HeroData();
                    data.kind = Lua_ThingDefine.HERO;
                end
            end

            if data ~= nil then
                local rid = objId * 100 + i;
                data.roleId = rid;
                data.objId = objId;
                data.spd = 400;
                data.hp = 100;
                data.sceneId = self._curShowMainLineSceneId;
                data.formationType = Lua_FightDefine.FIGHT_ATTACKER;
                data.formationPos = i;
                data.arriveStop = 0;
                data.isShowName = false;
                if self._followPlayerIds == nil then
                    self._followPlayerIds = Array();
                end
                self._followPlayerIds:add(rid);
                if i == 1 then
                    self._mainLineFollowId = rid;
                    ThingUtil.instance:addAnimalAndMoveFun(data, self, self.moveCompleteFun);
                else
                    ThingUtil.instance:addAnimal(data, true);
                end
            end
        end
    end
end

function Lua_MainLineSceneController:moveCompleteFun()
    if self._stopWanderPId ~= nil and self._stopWanderPId == self._targetPointId then
        self._bStopWander = true;
        return;
    end

    local tp = ThingUtil.instance:getThingById(self._curShowMainLineSceneId, 0, self.mainLineFollowId);
    if tp == nil then
        return;
    end

    -- 确认战斗还是继续漫游到下一个点
    local sd = Lua_SceneWanderPointConfigData.getConfigData(self._targetPointId);
    if sd ~= nil then
        self:generateMonsters(sd.monsterGroupId);   -- 是否触发怪物生成
        local gid = sd.completeCondition;
        if gid > 0 and Lua_MonsterFightGroupConfigData.getConfigData(gid) ~= nil then -- 需战斗, 战斗表现
            for i = 1, self._followPlayerIds:getSize() do
                ThingUtil.instance:setAnimalStopMove(self._curShowMainLineSceneId, 0, self._followPlayerIds[i], true);
            end

            -- 进入战斗
            DataCenter.mainLineSceneFigthController:clear();
            DataCenter.mainLineSceneFigthController:setFightSceneInfo(self._curShowMainLineSceneId, 0);
            DataCenter.mainLineSceneFigthController:setFightInfo(self._followPlayerIds, self._targetPointId, self.dirVec, self.rightVec);
            DataCenter.mainLineSceneFigthController:startFight(self, self.moveNextTargetPosFun);
        else -- 继续漫游
            self:monsterMoveTarget(sd.monsterGroupId);
            if sd.isMoveToTarget ~= true then
                local posVec;
                local vec = tp.currentPoint;
                local offsetx = vec.x - sd.posVec.x;
                local offsetz = vec.z - sd.posVec.z;
                while (sd ~= nil and sd.isMoveToTarget ~= true) do -- 瞬移
                    sd = Lua_SceneWanderPointConfigData.getConfigData(sd.nextPosId);
                    self:monsterMoveTarget(sd.monsterGroupId);
                    posVec = sd.posVec;
                end
                
                if sd == nil then
                    return; 
                end

                -- 瞬移到目的地，然后继续下一目标点
                if posVec ~= nil then
                    local px = posVec.x;
                    local py = posVec.y;
                    local pz = posVec.z;
                    local sc = Lua_SceneConfigData.getSceneConfig(self.mainLineSceneId);
                    local mfgcd = Lua_MonsterFightGroupConfigData.getConfigData(sc.wanderGroupId);
                    local posOffsetxArr = mfgcd.posOffsetxArr;
                    local posOffsetzArr = mfgcd.posOffsetzArr;
                    if self._followPlayerIds ~= nil then
                        for i = 1, self._followPlayerIds:getSize() do
                            ThingUtil.instance:setThingPos(self.mainLineSceneId, 0, self._followPlayerIds[i], px + posOffsetxArr[i] * self.rightVec.x + posOffsetzArr[i] * self.dirVec.x, py, pz + posOffsetxArr[i] * self.rightVec.z + posOffsetzArr[i] * self.dirVec.z, true);
                        end
                    end
                end
            end

            self._targetPointId = sd.nextPosId;
            local nextSd = Lua_SceneWanderPointConfigData.getConfigData(self._targetPointId);
            if nextSd ~= nil and nextSd.posVec ~= nil then
                self:walkToTargetPos(sd.posVec, nextSd.posVec);
            end
        end
    end
end

function Lua_MainLineSceneController:walkToTargetPos(curVec, targetVec)
    if curVec == nil or targetVec == nil then
        return;
    end

    local px = targetVec.x;
    local py = targetVec.y;
    local pz = targetVec.z;
    self.dirVec = Vector3(px - curVec.x, py, pz - curVec.z);
    self.dirVec:SetNormalize();
    local upVec = Vector3(0, 1, 0);
    self.rightVec = Vector3.Cross(self.dirVec, upVec);
    local sc = Lua_SceneConfigData.getSceneConfig(self.mainLineSceneId);
    local mfgcd = Lua_MonsterFightGroupConfigData.getConfigData(sc.wanderGroupId);
    local posOffsetxArr = mfgcd.posOffsetxArr;
    local posOffsetzArr = mfgcd.posOffsetzArr;
    
    -- 更新移动目标位置
    if self._followPlayerIds ~= nil then
        for i = 1, self._followPlayerIds:getSize() do
            ThingUtil.instance:setAnimalMoveTo(self.mainLineSceneId, 0, self._followPlayerIds[i], px + posOffsetxArr[i] * self.rightVec.x + posOffsetzArr[i] * self.dirVec.x, py, pz + posOffsetxArr[i] * self.rightVec.z + posOffsetzArr[i] * self.dirVec.z);
        end
    end
end

function Lua_MainLineSceneController:moveNextTargetPosFun()
    DataCenter.mainLineSceneFigthController:clear();
    local sd = Lua_SceneWanderPointConfigData.getConfigData(self._targetPointId);
    local nextSd = Lua_SceneWanderPointConfigData.getConfigData(sd.nextPosId);
    if sd ~= nil and sd.posVec ~= nil and nextSd ~= nil and nextSd.posVec ~= nil then
        self._targetPointId = sd.nextPosId;
        self:walkToTargetPos(sd.posVec, nextSd.posVec);
    end
end

function Lua_MainLineSceneController:generateMonsters(monsterGroupId)
    if monsterGroupId <= 0 then
        return;
    end

    local mfgcd = Lua_MonsterFightGroupConfigData.getConfigData(monsterGroupId);
    if mfgcd == nil then
        return;
    end

    local list = mfgcd.fighter_list;
    if list == nil or #list <= 0 then
        return;
    end

    self._aniFormationType = mfgcd.formationType;
    for i = 1, #list do
        local monsterId = list[i];
        local mcd = Lua_MonsterConfigData.getMonsterConfig(monsterId)
        if mcd ~= nil then
            local data = MonsterData();
            data.objId = monsterId;
            data.roleId = monsterId;
            data.hp = mcd.health;
            data.spd = mcd.speed;
            data.attack = mcd.attack;
            data.isShowName = false;
            data.sceneId = self.mainLineSceneId;
            if mcd.bornPosVec ~= nil then
                data.px = mcd.bornPosVec.x;
                data.py = mcd.bornPosVec.y;
                data.pz = mcd.bornPosVec.z;
            end
            ThingUtil.instance:addAnimal(data, true);
        end
    end
end

function Lua_MainLineSceneController:monsterMoveTarget(groupId)
    if groupId == nil or groupId <= 0 then
        return;
    end

    local mfgcd = Lua_MonsterFightGroupConfigData.getConfigData(groupId);
    if mfgcd == nil then
        return;
    end

    local list = mfgcd.fighter_list;
    if list == nil or #list <= 0 then
        return;
    end

    for i = 1, #list do
        local monsterId = list[i];
        local mcd = Lua_MonsterConfigData.getMonsterConfig(monsterId)
        if mcd ~= nil and mcd.targetPosVec ~= nil then
            ThingUtil.instance:setAnimalMoveTo(self.mainLineSceneId, 0, monsterId, mcd.targetPosVec.x, mcd.targetPosVec.y, mcd.targetPosVec.z);
        end
    end
end