-- 主线场景战斗控制器
Lua_MainLineSceneFightController = class("Lua_MainLineSceneFightController")
function Lua_MainLineSceneFightController:ctor()
    self:superCall("ctor");
    self:dispose();
end

function Lua_MainLineSceneFightController:dispose()
    self:clear();
end

function Lua_MainLineSceneFightController:clear()
    self.owner = nil;
    self.callFun = nil;
    self.followPlayerIds = nil;
    self.pointId = nil;
    self.dirVec = nil;
    self.rightVec = nil;
    self.sceneId = 0;
    self.fightId = 0;
    self.aniMoveCount = 0;

    if self.aniIds ~= nil then
        self.aniIds:clear();
        self.aniIds = nil;
    end

    if self.attackVo ~= nil then
        self.attackVo:dispose();
        self.attackVo = nil;
    end
end

-- 战斗场景id
function Lua_MainLineSceneFightController:setFightSceneInfo(sid, fid)
    self.sceneId = sid;
    self.fightId = fid;
end

-- 战斗单位数据
function Lua_MainLineSceneFightController:setFightInfo(list, pid, dirVec, rightVec)
    self.followPlayerIds = list;
    self.pointId = pid;
    self.dirVec = dirVec;
    self.rightVec = rightVec;
end

-- 开始战斗
function Lua_MainLineSceneFightController:startFight(owner, completeFun)
    self.owner = owner;
    self.callFun = completeFun;
    if self.followPlayerIds == nil or self.followPlayerIds:getSize() <= 0 then
        if self.callFun ~= nil then
            self.callFun(self.owner);
            self.callFun = nil;
            self.owner = nil;
        end
        return;
    end

    local sd = Lua_SceneWanderPointConfigData.getConfigData(self.pointId);
    if sd == nil then
        if self.callFun ~= nil then
            self.callFun(self.owner);
            self.callFun = nil;
            self.owner = nil;
        end
        return;
    end

    local groupId = sd.completeCondition;
    local mfgcd = Lua_MonsterFightGroupConfigData.getConfigData(groupId);
    if mfgcd == nil or mfgcd.fighter_list == nil or #mfgcd.fighter_list <= 0 then
        if self.callFun ~= nil then
            self.callFun(self.owner);
            self.callFun = nil;
            self.owner = nil;
        end
        return;
    end

    self.aniMoveCount = 0;
    if self.aniIds == nil then
        self.aniIds = Array();
    else
        self.aniIds:clear();
    end

    -- 初始化阵型位置
    local sdPosVec = sd.posVec;
    local formationDis = sd.formationDis; -- 阵型相隔距离
    local aniPosVec = Vector3(sdPosVec.x + formationDis * self.dirVec.x, sdPosVec.y, sdPosVec.z + formationDis * self.dirVec.z);
    local posOffsetxArr = mfgcd.posOffsetxArr;
    local posOffsetzArr = mfgcd.posOffsetzArr;
    local list = mfgcd.fighter_list;
    for i = 1, #list do
        local rid = list[i];
        if ThingUtil.instance:hasThingById(self.sceneId, self.fightId, rid) == true then
            self.aniIds:add(rid);
            local px = aniPosVec.x + posOffsetxArr[i] * self.rightVec.x - posOffsetzArr[i] * self.dirVec.x;
            local pz = aniPosVec.z + posOffsetxArr[i] * self.rightVec.z - posOffsetzArr[i] * self.dirVec.z;
            local bWalk = ThingUtil.instance:setAnimalMoveTo(self.sceneId, self.fightId, rid, px, aniPosVec.y, pz, self, self.onMoveComplete, true);
            if bWalk == true then -- 需要移动
                self.aniMoveCount = self.aniMoveCount + 1;
            end
        end
    end

    if self.aniIds:getSize() <= 0 then
        if self.callFun ~= nil then
            self.callFun(self.owner);
            self.callFun = nil;
            self.owner = nil;
        end
        return;
    end

    self.fightDir = MathUtil.getDirectionByXY(sdPosVec.x, sdPosVec.z, aniPosVec.x, aniPosVec.z);
    if self.aniMoveCount <= 0 then
        self:onMoveComplete();
    end
end

-- 移动完成
function Lua_MainLineSceneFightController:onMoveComplete(rid)
    if self.followPlayerIds == nil then
        return;
    end

    if rid ~= nil then
        rid = tonumber(tostring(rid));
        if self.aniIds ~= nil and self.aniIds:indexOf(rid) > 0 then
            self.aniMoveCount = self.aniMoveCount - 1;
        end
    end

    if self.aniMoveCount <= 0 then -- 移动完成，准备战斗表现
        -- 己方朝向
        for i = 1, self.followPlayerIds:getSize() do
            ThingUtil.instance:setThingDir(self.sceneId, self.fightId, self.followPlayerIds[i], self.fightDir);
        end

        -- 怪物组
        for i = 1, self.aniIds:getSize() do
            if self.fightDir == 0 then
                ThingUtil.instance:setThingDir(self.sceneId, self.fightId, self.aniIds[i], 180);
            else
                ThingUtil.instance:setThingDir(self.sceneId, self.fightId, self.aniIds[i], -self.fightDir);
            end
        end

        -- 攻击
        self.attackIdx = 1;
        Lua_CommonUtil.delayCall(0.5, self, self.attacking, nil);
    end
end

-- 攻击表现
function Lua_MainLineSceneFightController:attacking()
    if self.aniIds:getSize() <= 0 then
        if self.callFun ~= nil then
            self.callFun(self.owner);
            self.callFun = nil;
            self.owner = nil;
        end
        return;
    end

    local attackId = self.followPlayerIds[self.attackIdx];
    self.attackIdx = self.attackIdx + 1;
    if self.attackIdx > self.followPlayerIds:getSize() then
        self.attackIdx = 1;
    end

    -- 攻击
    if attackId ~= nil then
        local skillId = ThingUtil.instance:getThingBaseSkillId(self.sceneId, self.fightId, attackId);
        local scd = Lua_SkillConfigData.getConfigData(skillId);
        if scd ~= nil then
            self.attackVo = AttackVO(self.sceneId, self.fightId);
            self.attackVo:addLuaCallBack(self, self.attackedCallBack, self.attackComplete);
            self.attackVo.attackerId = attackId;
            self.attackVo.skillId = skillId;
            local targetNum = scd.targetAmount;
            if self.aniIds:getSize() < targetNum then
                targetNum = self.aniIds:getSize();
            end
            for i = 1, targetNum do
                self.attackVo:addTargetId(self.aniIds[i]);
            end
            self.attackVo:start();
            return;
        end
    end

    -- 防止无效
    self:attacking();
end

function Lua_MainLineSceneFightController:attackedCallBack(rid, idx)
    rid = tonumber(tostring(rid));
    if self.aniIds ~= nil and self.aniIds:indexOf(rid) > 0 then
        self.aniIds:removeItem(rid);
        ThingUtil.instance:showAniInjury(self.sceneId, self.fightId, rid);
        ThingUtil.instance:updateAniHealth(self.sceneId, self.fightId, rid, 0); -- dead
    end
end

function Lua_MainLineSceneFightController:attackComplete()
    if self.attackVo ~= nil then
        self.attackVo:dispose();
        self.attackVo = nil;
    end

    if self.aniIds:getSize() > 0 then
        self:attacking(); -- 下一
    else
        if self.callFun ~= nil then
            self.callFun(self.owner);
            self.callFun = nil;
            self.owner = nil;
        end
    end
end