checkRequire("Lua_ToolVo");
Lua_PackData = class("Lua_PackData");
function Lua_PackData:ctor()
    self:superCall("ctor");
    self:dispose();
end

function Lua_PackData:dispose()
    self.bInit = false;
    self._uniqueIdDic = nil;
    self._itemIdDic = nil;
    self._packTypeDic = nil;
    self._mainTypeDic = nil;
    self.recordSort = {};
end

function Lua_PackData.Get:uniqueIdDic()
    if self._uniqueIdDic == nil then
        self._uniqueIdDic = {};
    end
    return self._uniqueIdDic;
end

function Lua_PackData.Get:itemIdDic()
    if self._itemIdDic == nil then
        self._itemIdDic = {};
    end
    return self._itemIdDic;
end

function Lua_PackData.Get:packTypeDic()
    if self._packTypeDic == nil then
        self._packTypeDic = {};
    end
    return self._packTypeDic;
end

function Lua_PackData.Get:mainTypeDic()
    if self._mainTypeDic == nil then
        self._mainTypeDic = {};
    end
    return self._mainTypeDic;
end

function Lua_PackData:updatePackInfo(msg)
    for i = 1, #msg.item_list do
        self:addItem(msg.type, msg.item_list[i]);
    end

    -- 只排序一次，后续新增道具根据权重插入对应的索引位
    if self.recordSort[msg.type] == nil then
        self.recordSort[msg.type] = 1;
        if msg.type == Lua_PackDefine.TYPE_CHIP then
            self:packListSort(self.packTypeDic[msg.type], "chipSort", false);
        elseif msg.type == Lua_PackDefine.TYPE_EQUIP then
            self:packListSort(self.packTypeDic[msg.type], "equipSort", false);
        else
            self:packListSort(self.packTypeDic[msg.type], "sortWeight", false);
        end
    end
    if msg.type == Lua_PackDefine.TYPE_CHIP and DataCenter.funcOpenData:isOpened(10008) then
        self:checkChipRedPoint();
    end

    if msg.type == Lua_PackDefine.TYPE_ITEM then
        self.bInit = true;
        DataCenter.funcOpenData:initServerInfo();
    end
end

function Lua_PackData:addItem(pack_type, oneItem)
    local uniqueId = tonumber(oneItem.unique_id)
    local vo = self.uniqueIdDic[uniqueId];
    if vo == nil then
        vo = Lua_ToolVo.createToolVo(oneItem.id, oneItem.amount);
        if vo == nil then
            return;
        end
        
        vo.pack_type = pack_type;
        vo:fromServer(oneItem);
        self.uniqueIdDic[vo.unique_id] = vo;

        if self.itemIdDic[vo.id] == nil then
            self.itemIdDic[vo.id] = Array();
        end
        self.itemIdDic[vo.id]:add(vo.unique_id);

        if self.packTypeDic[pack_type] == nil then
            self.packTypeDic[pack_type] = Array();
        end
        if self.recordSort[pack_type] == nil then
            self.packTypeDic[pack_type]:add(vo.unique_id);
        else
            self:addItemAndSort(vo);
        end

        if self.mainTypeDic[vo.main_type] == nil then
            self.mainTypeDic[vo.main_type] = {};
        end
        if self.mainTypeDic[vo.main_type][vo.sub_type] == nil then
            self.mainTypeDic[vo.main_type][vo.sub_type] = Array();
        end
        self.mainTypeDic[vo.main_type][vo.sub_type]:add(vo.unique_id);
    else
        local haveCount = vo.count;
        vo:fromServer(oneItem);
        if pack_type ~= Lua_PackDefine.TYPE_EQUIP then -- 装备背包数量不叠加
           vo.count = haveCount + oneItem.amount;
        end
        self.packTypeDic[vo.pack_type]:removeItem(vo.unique_id);
        self:addItemAndSort(vo);
    end
end

function Lua_PackData:removeItem(pack_type, oneItem)
    local uniqueId = tonumber(oneItem.unique_id)
    if self.uniqueIdDic[uniqueId] == nil then
        return;
    end

    self.packTypeDic[pack_type]:removeItem(uniqueId);
    local vo = self.uniqueIdDic[uniqueId];
    vo.count = vo.count - oneItem.amount;
    if vo.count > 0 then
        self:addItemAndSort(vo);
    else
        self.uniqueIdDic[uniqueId] = nil;
        self.itemIdDic[vo.id]:removeItem(uniqueId);
        self.mainTypeDic[vo.main_type][vo.sub_type]:removeItem(uniqueId);
    end
end

function Lua_PackData:addItemAndSort(vo)
    local arr = self.packTypeDic[vo.pack_type];
    if arr == nil then
        return;
    end
    
    local len = arr:getSize();
    if len > 0 then
        local name;
        if vo.pack_type == Lua_PackDefine.TYPE_CHIP then
            name = "chipSort";
        elseif vo.pack_type == Lua_PackDefine.TYPE_EQUIP then
            name = "equipSort";
        else
            name = "sortWeight";
        end

        local tempVo = self:getItemVoByUnId(arr[len]);
        if vo[name] > tempVo[name] then
            arr:add(vo.unique_id);
            return;
        end

        for i = 1, len do
            tempVo = self:getItemVoByUnId(arr[i]);
            if tempVo[name] > vo[name] then
                arr:insert(vo.unique_id, i);
                return;
            end
        end
    end
    arr:add(vo.unique_id);
end

-- true降序 false升序
function Lua_PackData:packListSort(arr, name, up)
    if arr == nil then
        return;
    end

    local len = arr:getSize();
    for i = 1, len do
        for j = i + 1, len do
            local a = self:getItemVoByUnId(arr[i])[name];
            local b = self:getItemVoByUnId(arr[j])[name];
            local r = a < b;
            if up == false then
                r = not r;
            end
            if (r) then
                temp = arr[i];
                arr[i] = arr[j];
                arr[j] = temp;
            end
        end
    end
end

-- 获取全部背包数据
function Lua_PackData:getAllPackData()
    local arr = Array();
    for uniqueId, _ in pairs(self.uniqueIdDic) do
        arr:add(uniqueId);
    end
    return arr;
end

-- 根据ID获取uid集合
function Lua_PackData:getItemListById(id)
    if id == nil or self.itemIdDic[id] == nil then
        return nil;
    end
    return self.itemIdDic[id];
end

-- 根据ID获取物品数据
function Lua_PackData:getItemVoById(id)
    if id == nil or self.itemIdDic[id] == nil then
        return nil;
    end

    local arr = self.itemIdDic[id];
    return self:getItemVoByUnId(arr[1]);
end

-- 根据唯一ID获取物品数据
function Lua_PackData:getItemVoByUnId(unId)
    if unId == nil or self.uniqueIdDic[tonumber(unId)] == nil then
        return nil;
    end
    return self.uniqueIdDic[tonumber(unId)];
end

-- 根据物品ID获取数量
function Lua_PackData:getItemCountById(id)
    if id == nil then
        return 0;
    end
    id = tonumber(id);

    if DataCenter.moneyData:isMoney(id) then
        return DataCenter.moneyData:getMoney(id);
    end

    local count = 0;
    local arr = self.itemIdDic[id];
    if arr ~= nil then
        for i = 1, arr:getSize() do
            count = count + self:getItemCountByUnId(arr[i]);
        end
    end
    return count;
end

-- 根据唯一ID获取数量
function Lua_PackData:getItemCountByUnId(unId)
    if unId == nil or self.uniqueIdDic[unId] == nil then
        return 0;
    end
    return self.uniqueIdDic[unId].count;
end

-- 背包标签获取uid集合
function Lua_PackData:getItemArrByPackType(pack_type)
    local arr = Array();
    local data = self.packTypeDic[pack_type];
    if data ~= nil then
        for i = 1, data:getSize() do
            arr:add(data[i]);
        end
    end
    return arr;
end

-- 背包标签获取uid集合和索引列表
function Lua_PackData:getPackArrByPackType(pack_type)
    local arr = Array();
    local dic = {};
    local data = self.packTypeDic[pack_type];
    if data ~= nil then
        local uniqueId;
        for i = 1, data:getSize() do
            uniqueId = data[i];
            arr:add(uniqueId);
            dic[uniqueId] = i;
        end
    end
    return arr, dic;
end

-- 根据main_type, sub_type获取uid集合
function Lua_PackData:getUnIdList(main_type, sub_type)
    local arr = Array();
    local tab = self.mainTypeDic[main_type];
    if tab ~= nil then
        if sub_type == nil then
            for _, unIdList in pairs(tab) do
                for i = 1, unIdList:getSize() do
                    arr:add(unIdList[i]);
                end
            end
        else
            if tab[sub_type] ~= nil then
                local unIdList = tab[sub_type];
                for i = 1, unIdList:getSize() do
                    arr:add(unIdList[i]);
                end
            end
        end
    end
    return arr;
end










-- 检查红点
--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
function Lua_PackData:checkChipRedPoint()
    local state = false;
    local arr = self:getItemArrByPackType(Lua_PackDefine.TYPE_CHIP);
    local tvo;
    for i = 1, arr:getSize() do
        tvo = self:getItemVoByUnId(arr[i]);
        if tvo.isShowChipRed or tvo.isShowHeroChipRed or tvo.isShowGiftBagUseRed then
            state = true;
            break;
        end
    end
    DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Pack_Chip, state);
end










-- 背包界面数据
--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
-- 背包显示页签
function Lua_PackData:getPackTagList()
    if self._packTagList == nil then
        self._packTagList = Array();
        local config = getConfig({ "package_common" });
        for k,v in pairs(config) do
            if v.is_pack_show == 1 then
                self._packTagList:add(v);
            end
        end
        self._packTagList:sort("sort", false);
    end
    return self._packTagList;
end

function Lua_PackData:getFilterByPackType(pack_type)
    return getConfig("package_common", pack_type, "filter");
end

function Lua_PackData:getFilterName(data)
    if data then
        if data[1] == 0 then
            if data[2] == 0 then
                return LanUtil.getValue(309072)
            else
                return LanUtil.formateStr(309074, tostring(data[2])) -- 星级
            end
        else
            if data[2] == 0 then
                return LanUtil.formateStr(309073, LanUtil.getValue(309050 + data[1])) -- 品质
            else
                return LanUtil.formateStr(309075, LanUtil.getValue(309050 + data[1]), tostring(data[2])) -- 品质+星级
            end
        end
    end
    return nil
end

-- 根据筛选条件获取数据Array
function Lua_PackData:getBatchArrByFilter(pack_type, batchType, filterDic)
    local arr = Array();
    local data = self:getItemArrByPackType(pack_type);
    local tvo;
    for i = 1, data:getSize() do
        tvo = self:getItemVoByUnId(data[i]);
        if batchType == 1 then -- 分解
            if tvo.decompose_logo == 1 and self:checkVoIsMeetFilter(filterDic, tvo) then
                arr:add(tvo.unique_id);
            end
        elseif batchType == 2 then -- 出售
            if tvo.sell_logo == 1 and self:checkVoIsMeetFilter(filterDic, tvo) then
                arr:add(tvo.unique_id);
            end
        end
    end
    -- if pack_type == Lua_PackDefine.TYPE_CHIP then
    --     self:packListSort(arr, "chipSort", false);
    -- elseif pack_type == Lua_PackDefine.TYPE_EQUIP then
    --     self:packListSort(arr, "equipSort", false);
    -- else
    --     self:packListSort(arr, "sortWeight", false);
    -- end
    return arr;
end

-- 检查tvo是否满足筛选
function Lua_PackData:checkVoIsMeetFilter(filterDic, tvo)
    local bMeet = true;
    if filterDic ~= nil then
        for key, dic in pairs(filterDic) do
            if key == "color" then
                if dic[tvo.color] == nil then
                    bMeet = false;
                    break;
                end
            elseif key == "camp" then
                if dic[tvo.campType] == nil then
                    bMeet = false;
                    break;
                end
            elseif key == "star" then
                if dic[tvo.star] == nil then
                    bMeet = false;
                    break;
                end
            elseif key == "colorStar" then
                if dic[1] == 0 then
                    if dic[2] > 0 and tvo.star > dic[2] then
                        bMeet = false;
                    end
                else
                    if dic[2] == 0 then
                        if tvo.color > dic[1] then
                            bMeet = false;
                        end
                    elseif dic[2] > 0 then
                        if tvo.color > dic[1] then
                            bMeet = false;
                        elseif tvo.color == dic[1] then
                            if tvo.star > dic[2] then
                                bMeet = false
                            end
                        end

                        -- if tvo.color > dic[1] or tvo.star > dic[2] then
                        --     bMeet = false;
                        -- end
                    end
                end
            end
        end
    end
    return bMeet;
end










-- 外部接口
--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
-- 根据服务端下发list获得奖励文本
function Lua_PackData:getRewardStr(list)
    local str = "";
    if list ~= nil then
        local len = #list;
        local tvo;
        for i = 1, len do
            tvo = Lua_ToolVo.createToolVo(list[i].id, list[i].amount);
            if tvo ~= nil and tvo.count > 0 then
                if tvo.main_type == Lua_PackDefine.HERO then -- 英雄
                    tvo.star = list[i].hero_info.star;
                    tvo.level = list[i].hero_info.level;
                    tvo.color = DataCenter.heroUtil:getHeroStarConfigInfo(tvo.star).color;
                end
                if i == len then
                    str = LanUtil.StringProfile(str, tvo.colorName, "*", tostring(tvo.count));
                else
                    str = LanUtil.StringProfile(str, tvo.colorName, "*", tostring(tvo.count), Lua_SpaceDefine.SPACE_4);
                end
            end
        end
    end
    return str;
end

-- 检查消耗是否足够
function Lua_PackData:checkConsumeEnough(data)
    if data ~= nil then
        if data.classname ~= nil then
            if self:getItemCountById(data.id) < data.count then
                return data.id, data.count;
            end
        else
            function checkConsume(list, len)
                for i = 1, len do
                    if type(list[i]) == type_str_number then
                        if self:getItemCountById(list[1]) < list[2] then
                            return list[1], list[2];
                        end
                    elseif type(list[i]) == type_str_table then
                        local costId, costCount = self:checkConsumeEnough(list[i]);
                        if costId ~= nil and costCount ~= nil then
                            return costId, costCount;
                        end
                    end
                end
                return nil, nil;
            end
            if data.typeof ~= nil and data:typeof() == "Array" then
                return checkConsume(data, data:getSize())
            elseif #data > 0 then
                return checkConsume(data, #data)
            else
                for k, v in pairs(data) do
                    if type(v) == type_str_number then
                        if self:getItemCountById(k) < v then
                            return k, v;
                        end
                    elseif type(v) == type_str_table then
                        local costId, costCount = self:checkConsumeEnough(v);
                        if costId ~= nil and costCount ~= nil then
                            return costId, costCount;
                        end
                    end
                end
            end
        end
    end
    return nil, nil;
end

-- 对相同的道具进行数量叠加
function Lua_PackData:getRewardDic(list, tab)
    tab = tab == nil and {} or tab;
    if list ~= nil then
        local tvo;
        for i = 1, #list do
            tvo = self:getRewardVo(list[i]);
            if tvo.main_type == Lua_PackDefine.RESOURCE then
                if tab[tvo.id] == nil then
                    tab[tvo.id] = tvo;
                else
                    tab[tvo.id].count = tab[tvo.id].count + tvo.count;
                end
            else
                local len;
                local count;
                if tab[tvo.id] == nil then
                    tab[tvo.id] = Array();
                    count = tvo.count;
                    while true do
                        if count > tvo.overlap then
                            count = count - tvo.overlap;
                            local vo = self:getRewardVo(list[i]);
                            vo.count = tvo.overlap;
                            tab[tvo.id]:add(vo);
                        else
                            tvo.count = count;
                            tab[tvo.id]:add(tvo);
                            break;
                        end
                    end
                else
                    len = tab[tvo.id]:getSize();
                    count = tab[tvo.id][len].count + tvo.count;
                    if count > tvo.overlap then
                        tab[tvo.id][len].count = tvo.overlap;
                        tab[tvo.id]:add(tvo);
                        tab[tvo.id][len + 1].count = count - tvo.overlap;
                    else
                        tab[tvo.id][len].count = count;
                    end
                end
            end           
        end
    end
    return tab;
end

function Lua_PackData:getRewardVo(msg)
    local tvo = Lua_ToolVo.createToolVo(msg.id);
    local count = (msg.amount == nil or msg.amount == 0) and 1 or msg.amount;
    if tvo.main_type == Lua_PackDefine.HERO then -- 英雄
        tvo.star = msg.hero_info.star;
        tvo.level = msg.hero_info.level;
        tvo.color = DataCenter.heroUtil:getHeroStarConfigInfo(msg.hero_info.star).color;
        tvo.isNew = msg.hero_info.is_new;
    elseif tvo.main_type == Lua_PackDefine.RUNES then -- 符石
        tvo:fromServer(msg.rune_info);
    else
        tvo:fromServer(msg);
    end
    tvo.count = count;
    return tvo;
end

function Lua_PackData:getRewardList(tab)
    local arr = Array();
    if tab ~= nil then
        for k, v in pairs(tab) do
            if v.typeof ~= nil and v:typeof() == "Array" then
                for i = 1, v:getSize() do
                    arr:add(v[i]);
                end
                v:clear();
            else
                if k ~= Lua_ResourcesDefine.RUNE_SMELT then-- 熔炼值不展示在获取列表
                    arr:add(v);
                end
            end
            tab[k] = nil;
        end
    end
    return arr;
end