Lua_ToolVo = class("Lua_ToolVo");
function Lua_ToolVo.createToolVo(id, count)
    if id == nil or id <= 0 then
        return nil;
    end

    id = tonumber(id);
    local config = getConfig("item", id);
    if config ~= nil then
        if config.main_type == Lua_PackDefine.EQUIP then
            return Lua_EquipVo.new(id, count);
        elseif config.main_type == Lua_PackDefine.RUNES then
            return Lua_RunesVo.new(id, count);
        elseif config.main_type == Lua_PackDefine.COLLECTION then
            return Lua_CollectionVo.new(id, count);
        else
            return Lua_ToolVo.new(id, count, config);
        end
    end

    config = Lua_HeroConfigData.getConfig(id);
    if config ~= nil then
        return Lua_HeroVo.new(id, config);
    end

    LogManager.LogError("no item/hero config id : " .. id, false);
    return nil;
end

function Lua_ToolVo:ctor(id, count, config)
    self.id = nil;
    self:setData(id, count, config);
end

function Lua_ToolVo:setData(id, count, config)
    id = tonumber(id);
    if self.id ~= id then
        self.id = id;

        self.config = config;
        if self.config == nil then
            self.config = getConfig("item", id);
        end

        if self.config == nil then
            self.config = Lua_HeroConfigData.getConfig(id);
        end
        
        if self.config == nil then
            LogManager.LogError("no config item/hero id " .. id, false);
        end
    end
    self.count = count;
end

function Lua_ToolVo:fromServer(val)
    self.index = val.index;
    self._count = val.amount;
    self.unique_id = tonumber(val.unique_id);
    self.bind = val.bind;
    self._expired_tick = tonumber(val.expired_tick); -- 过期时间 0:永久时效
    self:reset();
end

function Lua_ToolVo:reset()
    self._sortWeight = nil;
    self._chipSort = nil;
end

function Lua_ToolVo.Set:count(value)
    self._count = value;
    self:reset();
end

function Lua_ToolVo.Get:count()
    return self._count or 0;
end

function Lua_ToolVo.Set:level(value)
    self._level = value;
end

function Lua_ToolVo.Get:level()
    return self._level == nil and 0 or self._level;
end

function Lua_ToolVo.Get:colorName()
    return Lua_ColorDefine.getColorStr(self.colorStr, self.name);
end

function Lua_ToolVo.Get:colorStr()
    return Lua_ColorDefine.getItemColorStr(self.color);
end

function Lua_ToolVo.Get:broadColorStr()
    return Lua_ColorDefine.getBroadcastColorStr(self.color);
end

function Lua_ToolVo.Get:broadcastName()
    return Lua_ColorDefine.getColorStr(self.broadColorStr, self.name);
end

-- 阵营图标
function Lua_ToolVo.Get:campSprite()
    return "xui_common_attribute_"..29 + self.campType;
end

-- 是否是英雄碎片
function Lua_ToolVo.Get:isHeroChip() 
    if self.main_type == Lua_PackDefine.HERO_CHIP_RANDOM or self.main_type == Lua_PackDefine.HERO_CHIP_FIXED then
        return true;
    end
    return false;
end

-- 是否是英雄完整道具（英雄做成道具展示，例如英雄礼包）
function Lua_ToolVo.Get:isIntactHero() 
    if self.main_type == Lua_PackDefine.HERO then
        return true;
    end
    return false;
end

function Lua_ToolVo.Get:expired_tick()
    return self._expired_tick or 0
end

-- 是否过期
function Lua_ToolVo.Get:isExpired()
    if self.expired_tick == 0 or self.expired_tick > DataCenter.roleData.serverTime then
        return false;
    end
    return true;
end

function Lua_ToolVo.Set:isNew(value)
    self._isNew = value;
end

function Lua_ToolVo.Get:isNew()
    return self._isNew or 0;
end

------------------------------ 配置数据 ------------------------------

function Lua_ToolVo.Get:main_type()
    return self.config.main_type or 0;
end

function Lua_ToolVo.Get:sub_type()
    return self.config.sub_type or 0;
end

function Lua_ToolVo.Get:pack_type()
    if self._pack_type == nil then
        return self.config.type;
    end
    return self._pack_type;
end

function Lua_ToolVo.Set:pack_type(value)
    self._pack_type = value;
end

function Lua_ToolVo.Get:name()
    return self.config.name;
end

function Lua_ToolVo.Get:color()
    if self._color == nil then
        if self.config.color == nil then
            return 1;
        end
        return self.config.color;
    end
    return self._color;
end

function Lua_ToolVo.Set:color(value)
    self._color = value;
end

-- 可叠加数量
function Lua_ToolVo.Get:overlap()
    return self.config.overlap or 1;
end

-- 物品限制次数
function Lua_ToolVo.Get:limit_times()
    return self.config.limit_times;
end

function Lua_ToolVo.Get:atlas()
    return self.config.atlas;
end

function Lua_ToolVo.Get:sprite()
    local sprite = self.config.sprite;
    if strIsNullOrEmpty(sprite) then
        sprite = tostring(self.id);
    end
    return sprite;
end

function Lua_ToolVo.Get:des()
    return self.config.des;
end

-- 出售价格
function Lua_ToolVo.Get:sys_price()
    return self.config.sys_price or 0;
end

-- 出售货币
function Lua_ToolVo.Get:money_id()
    return self.config.money_id or 0;
end

function Lua_ToolVo.Get:use_type()
    return self.config.use_type or 1;
end

-- 使用效果参数
function Lua_ToolVo.Get:effect_param()
    return self.config.effect_parameter;
end

-- 是否批量使用
function Lua_ToolVo.Get:isBatch()
    return self.config.isBatch or 0;
end

-- 作用效果
function Lua_ToolVo.Get:effect_des()
    return self.config.effect_des;
end

function Lua_ToolVo.Get:buttonList()
    return self.config.buttonList;
end

-- 是否显示查看来源
function Lua_ToolVo.Get:showPath()
    return self.config.showPath or 0;
end

-- 合成获得材料(获得道具，消耗数量)
function Lua_ToolVo.Get:compound()
    return self.config.compound;
end

-- 分解获得材料
function Lua_ToolVo.Get:decompose()
    return self.config.decompose;
end

-- 强化石提供经验
function Lua_ToolVo.Get:provideExp()
    return self.config.provideExp;
end

--强化石消耗需要的费用
function Lua_ToolVo.Get:consume_item()
    return self.config.consume_item;
end

-- 出售提示框
function Lua_ToolVo.Get:sell_tips()
    return self.config.sell_tips;
end

-- 获取途径
function Lua_ToolVo.Get:obtain_path()
    if self._obtainPath == nil then
        local obtainList = self.config.obtain_path;
        -- 版署号  屏蔽充值
        if Lua_PlatformUtil.IsBanShu() and obtainList ~= nil then
            self._obtainPath = {};
            for i = 1, #obtainList do
                if obtainList[i][1] ~= 1024 then
                    self._obtainPath[#self._obtainPath + 1] = obtainList[i];
                end
            end
        else
            self._obtainPath = obtainList;
        end
    end
    return self._obtainPath;
end

-- 阵营类型
function Lua_ToolVo.Get:campType()
    return self.config.camp or 0;
end

-- 星
function Lua_ToolVo.Get:star()
    if self._star == nil then
        if self.config.star == nil then
            return 0;
        else
            return self.config.star;
        end
    end
    return self._star;
end

function Lua_ToolVo.Set:star(value)
    self._star = value;
end

-- 展示星级数量
function Lua_ToolVo.Get:showStar()
    local star = self.star;
    if star ~= nil and star > 0 then
        if star > 10 then
            return star - 10;
        elseif star <= 10 and star > 5 then
            return star - 5;
        elseif star <= 5 then
            return star;
        end
    end
    return 0;
end

function Lua_ToolVo.Get:iconEffUrl()
    if self._iconEffUrl == nil then
        self._iconEffUrl = LanUtil.StringProfile("/effects/uiEffects/", self.iconEff);
    end
    return self._iconEffUrl;
end

function Lua_ToolVo.Get:iconEff()
    return self.config.icon_effect;
end

function Lua_ToolVo.Get:buy_money_id()
    return self.config.buy_money_id;
end

function Lua_ToolVo.Get:buy_price()
    return self.config.buy_price;
end

-- 客户端显示物品列表 礼包
function Lua_ToolVo.Get:client_goods_list()
    return self.config.client_goods_list;
end

-- 使用跳转id
function Lua_ToolVo.Get:func_path_id()
    return self.config.func_path_id;
end

-- 装扮Id
function Lua_ToolVo.Set:clothes_id(value)
    self._clothes_id = value;
end

function Lua_ToolVo.Get:clothes_id()
    return self._clothes_id or self.config.clothes_id;
end

-- 战技id
function Lua_ToolVo.Get:talent_id()
    return self.config.talent_id;
end

-- 自选奖励数量
function Lua_ToolVo.Get:option_amount()
    return self.config.option_amount or 1;
end

-- 使用标识
function Lua_ToolVo.Get:use_logo()
    return self.config.use_logo or 0;
end

-- 一键分解标识
function Lua_ToolVo.Get:decompose_logo()
    return self.config.decompose_logo or 0;
end

-- 一键出售标识
function Lua_ToolVo.Get:sell_logo()
    return self.config.sell_logo or 0;
end

-- 排序
function Lua_ToolVo.Get:sort_weight()
    return self.config.sort_weight or 0;
end

-- 英雄id
function Lua_ToolVo.Get:hero_id()
    return self.config.hero_id or 0;
end

-- 使用单位消耗道具数量
function Lua_ToolVo.Get:eachAmount()
    return self.config.each_amount or 0;
end

-- 从道具表中获取英雄圆形图集和图片（非英雄直接使用atlas、sprite）
function Lua_ToolVo:getAvatarFromItemTable()
    if self.main_type == Lua_PackDefine.HERO then
        local config = Lua_HeroConfigData.getConfig(self.hero_id);
        if config then
            return config.avatar_atlas_1, config.avatar_sprite_1;
        end
    end
    return self.atlas, self.sprite;
end

function Lua_ToolVo.Get:showTimeText()
    return self.config.time_text
end

function Lua_ToolVo.Get:agingType()
    return self.config.aging_type
end
















------------------------------ 藏宝柜 ------------------------------
-- 附加的词条列表
function Lua_ToolVo.Get:entry_list()
    return self.config.entry_list;
end

-- 藏品激活奖励
function Lua_ToolVo.Get:active_reward()
    return self.config.active_reward;
end

-- 任务描述
function Lua_ToolVo.Get:task_des()
    return self.config.task_des;
end

-- 藏品获取目标值
function Lua_ToolVo.Get:target_value()
    return self.config.target_value;
end




















------------------------------  ------------------------------
function Lua_ToolVo.Get:iconAtlas()
    if self.config.icon_atlas == nil then
        return self.atlas;
    end
    return self.config.icon_atlas;
end

function Lua_ToolVo.Get:iconSprite()
    if self.config.icon_sprite == nil then
        return self.sprite;
    end
    return self.config.icon_sprite;
end

function Lua_ToolVo.Get:iconWidth()
    if self.config.icon_width == nil then
        return 0;
    end
    return self.config.icon_width;
end

function Lua_ToolVo.Get:iconHeight()
    if self.config.icon_height == nil then
        return 0;
    end
    return self.config.icon_height;
end




















------------------------------ 红点处理 ------------------------------
-- 碎片是否可以合成
function Lua_ToolVo.Get:canCompound()
    if self.compound ~= nil then
        return self.count >= self.compound[2];
    end
    return false;
end

-- 随机碎片合成红点
function Lua_ToolVo.Get:isShowChipRed()
    if self.main_type == Lua_PackDefine.HERO_CHIP_RANDOM and self.canCompound then
        return true;
    end
    return false;
end

-- 指定英雄碎片合成红点
function Lua_ToolVo.Get:isShowHeroChipRed()
    if self.main_type == Lua_PackDefine.HERO_CHIP_FIXED and self.canCompound then
        return true;
    end
    return false;
end

-- 自选碎片礼包可使用红点
function Lua_ToolVo.Get:isShowGiftBagUseRed()
    if self.main_type == Lua_PackDefine.GIFT_BAG and self.sub_type == Lua_PackDefine.GIFT_BAG_SELECT then
        return self.count >= self.eachAmount;
    end
    return false;
end




















------------------------------ 排序处理 ------------------------------
-- 排序权重
function Lua_ToolVo.Get:sortWeight()
    if self._sortWeight == nil then
        local weight = self.sort_weight * 10000 + self.count; -- 数量最大9999
        if self.use_logo == 1 then
            self._sortWeight = weight;
        else
            self._sortWeight = weight + 1000000000;
        end
    end
    return self._sortWeight;
end

-- 合同碎片排序
function Lua_ToolVo.Get:chipSort()
    if self._chipSort == nil then
        self._chipSort = self.sortWeight;
        if not self.isShowHeroChipRed and not self.isShowChipRed then
            self._chipSort = self.sortWeight + 10000000000;
        end
    end
    return self._chipSort;
end