checkRequire("Lua_PackData");
checkRequire("Lua_PackUtil");
checkRequire("Lua_ResourcesData");
checkRequire("Lua_MoneyData");
checkRequire("Lua_PackChipData");
Lua_PackProxy = class("Lua_PackProxy");
function Lua_PackProxy:ctor()
    self:superCall("ctor");
    DataCenter.packData = Lua_PackData.new();
    DataCenter.packUtil = Lua_PackUtil.new();
    DataCenter.moneyData = Lua_MoneyData.new();
    DataCenter.packChipData = Lua_PackChipData.new();

    Lua_MsgUtil.addLuaMsg(80103002, self, self.updatePackInfo); -- 通知更新背包物品信息
    Lua_MsgUtil.addLuaMsg(80103004, self, self.updateMoney); -- 更新货币信息
    Lua_MsgUtil.addLuaMsg(80103005, self, self.handleTips); -- 道具或货币更新提示tips
    Lua_MsgUtil.addLuaMsg(80103006, self, self.addItem); -- 增加背包物品（通知）
    Lua_MsgUtil.addLuaMsg(80103007, self, self.removeItem); -- 删除背包物品（通知）

    Lua_MsgUtil.addLuaMsg(50103006, self, self.rUseItem); -- 使用物品(返回)
    Lua_MsgUtil.addLuaMsg(50103010, self, self.rPurchaseItem); -- 通用道具购买(返回)
    Lua_MsgUtil.addLuaMsg(50103011, self, self.rOneKeySell); -- 一键出售背包道具(返回)
    Lua_MsgUtil.addLuaMsg(50103012, self, self.rOneKeyDecompose); -- 一键分解背包道具(返回)
end





------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- 上行
-- 获取背包信息
function Lua_PackProxy:sPackInfo(pack_type)
    local cmd = ProtoPublic_pb.Proto10103001();
    cmd.type = pack_type;
    sendMsg(10103001, cmd);
end

-- 使用物品
function Lua_PackProxy:sUseItem(pack_type, unique_id, amount, use_argv)
    local cmd = ProtoPublic_pb.Proto10103006();
    cmd.pack_type = pack_type;
    cmd.unique_id = unique_id;
    cmd.amount = amount;
    if use_argv ~= nil then
        for i = 1, use_argv:getSize() do
            cmd.use_argv:append(use_argv[i]);
        end
    end
    sendMsg(10103006, cmd);
end

function Lua_PackProxy:sSellItem(tab)
    if tab == nil then
        return;
    end

    local cmd = ProtoPublic_pb.Proto10103007();
    local struct;
    for k,v in pairs(tab) do
        struct = getProtoMsg("ProtoThreeLong");
        struct.key = tonumber(v.pack_type);
        struct.value = tonumber(v.unique_id);
        struct.sub = tonumber(v.count);
        cmd.sell_list:addToList(struct);
    end
    sendMsg(10103007, cmd);
end

function Lua_PackProxy:sDecomposeItem(decompose_id, amount)
    local cmd = ProtoPublic_pb.Proto10103008();
    cmd.decompose_id = decompose_id;
    cmd.amount = amount;
    sendMsg(10103008, cmd);
end

function Lua_PackProxy:sSyntheticItem(compound_id, amount)
    local cmd = ProtoPublic_pb.Proto10103009();
    cmd.compound_id = compound_id;
    cmd.amount = amount;
    sendMsg(10103009, cmd);
end

function Lua_PackProxy:sPurchaseItem(item_id, amount)
    local cmd = ProtoPublic_pb.Proto10103010();
    cmd.item_id = item_id;
    cmd.amount = amount;
    sendMsg(10103010, cmd);
end

function Lua_PackProxy:sOneKeySell(pack_type, tab)
    if tab == nil then
        return;
    end

    local cmd = ProtoPublic_pb.Proto10103011();
    local struct;
    for k,v in pairs(tab) do
        struct = getProtoMsg("ProtoThreeLong");
        struct.key = tonumber(pack_type);
        struct.value = tonumber(k);
        struct.sub = tonumber(v);
        cmd.sell_list:addToList(struct);
    end
    sendMsg(10103011, cmd);
end

function Lua_PackProxy:sOneKeyDecompose(tab, pack_type)
    if tab == nil then
        return;
    end
    
    local cmd = ProtoPublic_pb.Proto10103012();
    local struct;
    cmd.type = pack_type;
    for uniqueId, count in pairs(tab) do
        struct = getProtoMsg("ProtoPairLong");
        struct.key = tonumber(uniqueId);
        struct.value = tonumber(count);
        cmd.decompose_list:addToList(struct);
    end
    sendMsg(10103012, cmd);
end







------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- 下行
function Lua_PackProxy:updatePackInfo(mp)
    if DataCenter.bConfigLoaded ~= true then
        return;
    end

    if mp.errno == 0 then
        local msg = ProtoPublic_pb.Proto80103002();
        msg:ParseFromString(mp.data);
        DataCenter.packData:updatePackInfo(msg);
        Lua_MsgUtil.callFunc("PACK_UPDATE", msg);
    end
end

function Lua_PackProxy:updateMoney(mp)
    if mp.errno == 0 then
        local msg = ProtoPublic_pb.Proto80103004();
        msg:ParseFromString(mp.data);
        DataCenter.moneyData:fromServer(msg.money_list);
    end
end

function Lua_PackProxy:handleTips(mp)
    if mp.errno == 0 then
        local msg = ProtoPublic_pb.Proto80103005();
        msg:ParseFromString(mp.data);
        if msg.tips_type == 1 then
            LogManager.showLog("恭喜获得弹窗，引导中 id:"..DataCenter.guideCommonData.curGuideId..", 引导步骤:"..DataCenter.guideCommonData.curStep);
            local tab = DataCenter.packData:getRewardDic(msg.item_list);
            local arr = DataCenter.packData:getRewardList(tab); 
            if arr:getSize() > 0 then
                Lua_WindowUtil.openWin("Lua_ObtainItemTipsWindow", Lua_WinParam.ACTION_OPEN, arr);
            end
        elseif msg.tips_type == 2 then
            local str = DataCenter.packData:getRewardStr(msg.item_list);
            if not strIsNullOrEmpty(str) then
                OperateAlert.show(str);
            end
        end
    end
end

function Lua_PackProxy:addItem(mp)
    if mp.errno == 0 then
        local msg = ProtoPublic_pb.Proto80103006();
        msg:ParseFromString(mp.data);
        local obj;
        local changeNumDic = {}; --[id] = num
        for i = 1, #msg.item_list do
            obj = msg.item_list[i];
            changeNumDic[obj.id] = obj.amount;
            DataCenter.packData:addItem(msg.type, obj);
        end

        -- 更新背包界面显示
        if msg.type ~= Lua_PackDefine.TYPE_RESOURCE then -- 货币
            Lua_MsgUtil.callFunc("ADD_PACK_ITEMS", msg); -- 背包界面更新用
            Lua_MsgUtil.callFunc("PACK_ITEMS_UPDATE", changeNumDic); -- 背包道具变化更新数据

            if msg.type == Lua_PackDefine.TYPE_EQUIP then -- 装备
                DataCenter.equipData:updateEquipRedDotState();
            elseif msg.type == Lua_PackDefine.TYPE_CHIP then -- 碎片
                DataCenter.packData:checkChipRedPoint();
            elseif msg.type == Lua_PackDefine.TYPE_TALENT then -- 战技
                DataCenter.talentData:initRedDic();
                Lua_MsgUtil.callFunc("UPDATE_HERO_REDDOT_STATE"); -- 更新战技学习红点状态
            end
        end
    end
end

function Lua_PackProxy:removeItem(mp)
    if mp.errno == 0 then
        local msg = ProtoPublic_pb.Proto80103007();
        msg:ParseFromString(mp.data);
        
        local obj;
        for i = 1, #msg.item_list do
            obj = msg.item_list[i];
            DataCenter.packData:removeItem(msg.type, obj);
            Lua_MsgUtil.callFunc("SINGLE_ITEM_UPDATE", obj.id);
        end

        -- 更新背包界面显示
        if msg.type ~= Lua_PackDefine.TYPE_RESOURCE then
            Lua_MsgUtil.callFunc("REMOVE_PACK_ITEMS", msg); -- 背包界面更新用

            if msg.type == Lua_PackDefine.TYPE_EQUIP then -- 碎片
                DataCenter.equipData:updateEquipRedDotState();
            elseif msg.type == Lua_PackDefine.TYPE_CHIP then -- 碎片
                DataCenter.packData:checkChipRedPoint();
            end
        end
    end
end

function Lua_PackProxy:rUseItem(mp)
    if mp.errno == 0 then
        local msg = ProtoPublic_pb.Proto50103006();
        msg:ParseFromString(mp.data);
        if msg ~= nil and msg.hero_uid_list ~= nil and #msg.hero_uid_list > 0 then
            local arr = Array();
            local uniqueId;
            local heroVo;
            for i = 1, #msg.hero_uid_list do
                uniqueId = msg.hero_uid_list[i];
                heroVo = DataCenter.heroData:getItemByUniqueId(uniqueId);
                if heroVo ~= nil then
                    arr:add(heroVo);
                end
            end
            Lua_WindowUtil.openWin("Lua_ObtainItemTipsWindow", Lua_WinParam.ACTION_OPEN, arr);
        end
    end
end

function Lua_PackProxy:rPurchaseItem(mp)
    if mp.errno == 0 then
        Lua_MsgUtil.callFunc("PURCHASE_ITEM_SUCCESS");
    end
end

function Lua_PackProxy:rOneKeySell(mp)
    if mp.errno == 0 then
        Lua_WindowUtil.openWin("Lua_PackBatchWindow", Lua_WinParam.ACTION_CLOSE);
    end
end

function Lua_PackProxy:rOneKeyDecompose(mp)
    if mp.errno == 0 then
        Lua_WindowUtil.openWin("Lua_PackBatchWindow", Lua_WinParam.ACTION_CLOSE);
    end
end