Lua_RecruitTypeVo = class("Lua_RecruitTypeVo")

function Lua_RecruitTypeVo:ctor(type, config)
	self.type = type;
	self.config = config;
	if self.config == nil then
		self.config = getConfig({"hero_recruit", tonumber(type) });
	end
	if self.config == nil then
		LogManager.LogError("no hero_recruit config="..tostring(type), false)
	end
	self:resetServerInfo();
end

function Lua_RecruitTypeVo:resetServerInfo()
	self.freeTime = 0; -- 是否有免费次数（0：有，非0表示刷新点的时间戳）
	self.matNum = 0--当前招募花费的道具数量
	self.dayDrawTimes = 0;
	if self.type == 3 then
		self.adv_draw_times = 0;
		self.advRounds = 1;
		self.advRemainTimes = 0;
		self.advRoundTimes = 0;
	end
end

function Lua_RecruitTypeVo:updateServerInfo(serverInfo)
	self.freeTime = serverInfo.is_free;--是否有免费次数（0：有，非0表示刷新点的时间戳）
	self.matNum = tonumber(serverInfo.recruit_item[1].value);
	self.dayDrawTimes = serverInfo.day_draw_time;--每日招募次数
	if self.type == 3 then
		self.adv_draw_times = serverInfo.adv_draw_times;--签约次数（高级招募）
		self.advRounds = serverInfo.adv_cur_round ~= 0 and serverInfo.adv_cur_round or 1;--签约次数奖励当前轮数（高级招募）
		self.advRemainTimes = serverInfo.five_star_times;--五星保底剩余次数（高级招募）
		self.advRoundTimes = serverInfo.adv_round_times;--当前奖励轮次签约次数（高级招募）
	end

	if self.freeRedDot ~= nil then
		DataCenter.redPointData:updateRedPointState(self.freeRedDot, self.freeTime == 0);
	end
end


-------------------- 配置数据 --------------------

-- 招募名
function Lua_RecruitTypeVo.Get:name()
	return self.config.name;
end

--招募一次花费的道具数量
function Lua_RecruitTypeVo.Get:oneCost()
	return self.config.item_one_draw_amount;
end

--招募10次花费的道具数量
function Lua_RecruitTypeVo.Get:tenCost()
	return self.config.item_ten_draw_amount;
end

--招募花费的币种
function Lua_RecruitTypeVo.Get:currencyId()
	return Lua_ResourcesDefine.DIAMOND;--钻石
end

--是否花费钻石
function Lua_RecruitTypeVo.Get:isUseGold()
	return self.config.gold_one_draw_amount ~= nil;
end

--招募一次花费的钻石
function Lua_RecruitTypeVo.Get:oneGold()
	return self.config.gold_one_draw_amount;
end

--招募10次花费的钻石
function Lua_RecruitTypeVo.Get:tenGold()
	return self.config.gold_ten_draw_amount;
end

--招募底图
function Lua_RecruitTypeVo.Get:isShowInMain()
	return self.config.isShowInMain;
end

--钻石招募获得道具
function Lua_RecruitTypeVo.Get:goldReward()
	if self.config.gold_recruit_reward == nil then
		return nil;
	end

	if self._goldRewards == nil then
		self._goldRewards = Array();
		for i = 1, #self.config.gold_recruit_reward do
			local vo = Lua_ToolVo.createToolVo(self.config.gold_recruit_reward[i][1], self.config.gold_recruit_reward[i][2]);
			self._goldRewards:add(vo);
		end
	end

	return self._goldRewards;--钻石
end

--签约中心排序索引
function Lua_RecruitTypeVo.Get:mainSort()
	return self.config.main_sort;
end

--是否有免费次数这个功能
function Lua_RecruitTypeVo.Get:isHaveFree()
	local freeTime=self.config.free_time
	if freeTime ~= nil and tonumber(freeTime) >= 0 then
		return true
	end
	return false;
end

--消耗品ID
function Lua_RecruitTypeVo.Get:costId()
	if self.config.recruit_item == nil then
		return 0;
	end
	return self.config.recruit_item;
end

--界面展示道具
function Lua_RecruitTypeVo.Get:exhibitionItem()
	return self.config.exhibition_item;
end

--免费次数红点
function Lua_RecruitTypeVo.Get:freeRedDot()
	return self.config.free_redDot;
end

-- 每日可签约次数
function Lua_RecruitTypeVo.Get:dailyDrawLimitTimes()
	return self.config.daily_draw_times;
end

-- 剩余购买次数
function Lua_RecruitTypeVo:getResidueCount()
	local count = self.dailyDrawLimitTimes == nil and 0 or self.dailyDrawLimitTimes;
	return count - self.dayDrawTimes;
end

-- 是否开启限购
function Lua_RecruitTypeVo.Get:openDailyDrawLimit()
	return self.dailyDrawLimitTimes ~= nil; 
end