--红点系统 红点Data
checkRequire("Lua_RedPointVo");
Lua_RedPointData = class("Lua_RedPointData");
function Lua_RedPointData:ctor()
    self:superCall("ctor");

    self.everyTimeCount = 5; -- 每次更新5个;
    self.waitIdsList = Array(); -- 等待更新的节点
end


--------------------------------------
--------------------------------------
-- 外部调用

--更新红点节点状态，正常只能更改末端节点
-- id 红点配置表id, 
-- state 红点状态 显示true 隐藏false，
-- isClick 是否是点击更新，用于处理点击取消的，正常无需写该参数
function Lua_RedPointData:updateRedPointState(id, state, isClick)
    -- print("updateRedPointState", id, state, isClick);
    self:updateRedPointShowNum(id, state and 1 or 0, isClick);
end

-- 更新红点节点数量，正常只能更改末端节点
-- id 红点配置表id, 
-- num 红点上显示的数量，
-- isClick 是否是点击更新，用于处理点击取消的，正常无需写该参数
function Lua_RedPointData:updateRedPointShowNum(id, num, isClick)
    local vo = self:getRedPointVoById(id);
    if vo == nil then
        return;
    end

    if num == nil then
        num = 0;
    end

    -- 正常情况不是末端会自动进行忽略，排除点击更新的
    if vo.isExtremity ~= true and (isClick ~= true or not vo.clickCancle) then
        -- 如有特殊情况在这加


        print("您设置更新的红点id =", id, "不是末端，请检查");
        return;
    end

    if vo:updateShowNum(num, isClick) and self.waitIdsList:indexOf(id) <= 0 then
        self.waitIdsList:add(id);
    end

    self:addTimer();
end

--获取红点Vo
function Lua_RedPointData:getRedPointVoById(id)
    if self._redPointVoDic == nil then
        if ConfigManager.getInstance().firstConfigLoaded ~= 1 then
            return nil;
        end
        self._redPointVoDic = {};
    end

    if self._redPointVoDic[id] == nil then
        local config = DataCenter.redPointUtil:getRedPointConfig(id);
        if config ~= nil then
            local vo = Lua_RedPointVo.new(id, config);
            self._redPointVoDic[id] = vo;

            local parentVo = vo.parentVo;
            if parentVo ~= nil then
                parentVo.isExtremity = false;
                parentVo:addChildId(id);
            end

            if vo.initState then -- 默认显示红点则初始化的时候刷新一次
                self.waitIdsList:add(id);
            end
        end
    end
    return self._redPointVoDic[id];
end

-- 是否有红点数据
function Lua_RedPointData:hasRedPointVoById(id)
    return self:getRedPointVoById(id) ~= nil;
end

-- 获取红点id的状态
function Lua_RedPointData:getRedPointStateById(id)
    local vo = self:getRedPointVoById(id);
    if vo ~= nil then
        return vo.showState;
    end
    return false;
end

--获取红点显示数量
function Lua_RedPointData:getRedPointShowNumById(id)
    local vo = self:getRedPointVoById(id);
    if vo ~= nil then
        return vo.showNum;
    end
    return 0;
end

--- 外部调用
--------------------------------------
--------------------------------------



----------------------------------- 运营活动红点 -----------------------------------

--更新运营活动红点的父节点(对应的主图标红点)
function Lua_RedPointData:updateMainIconRedId(actIdx, mainIconUId, mainIconId)
    if actIdx ~= nil then
        local id = tostring(actIdx);
        local redPointVo = self:_addOperationalActRedPoint(id);
        local mainIconVo = DataCenter.mainMenuData:getMenuIconVo(mainIconUId, mainIconId);
        if redPointVo ~= nil and mainIconVo ~= nil then
            local parentVo = redPointVo.parentVo;
            local nextParentId = mainIconVo.redPointId;
            if parentVo ~= nil then
                if parentVo.id == nextParentId then
                    return;
                end

                parentVo:removeChildId(id);
                self.waitIdsList:add(parentVo.id);
            end
            redPointVo.mainIconRedId = nil;
            
            if nextParentId ~= nil then
                local nextParentVo = self:getRedPointVoById(nextParentId);
                if nextParentVo ~= nil then
                    nextParentVo.isExtremity = false;
                    nextParentVo:addChildId(id);
                    self.waitIdsList:add(nextParentId);

                    redPointVo.mainIconRedId = nextParentId;
                end
            end
        end
    end
end

function Lua_RedPointData:_addOperationalActRedPoint(actIdx)
    local vo = self:getRedPointVoById(actIdx);
    if vo == nil then
        vo = Lua_RedPointVo.new(actIdx, nil);
        self._redPointVoDic[actIdx] = vo;
    end
    return vo;
end



----------------------------------- 红点逻辑 -----------------------------------

function Lua_RedPointData:dispose()
    self:clearTimer();
    self.waitIdsList:clear();
    if self._redPointVoDic ~= nil then
        for k, v in pairs(self._redPointVoDic) do
            v:reset();
        end
    end
end

function Lua_RedPointData:addTimer()    
    if self.bTimer ~= true and self.waitIdsList:getSize() > 0 then
        self.bTimer = true;
        Lua_CommonUtil.addTimer("RedPoint_Update_ParentNode", 0.5, self, self.onUpdateParentNode);
    end
end

function Lua_RedPointData:clearTimer()
    if self.bTimer == true then
        self.bTimer = false;
        Lua_CommonUtil.removeTimer(self, self.onUpdateParentNode);
    end
end

-- 处理更新父节点
function Lua_RedPointData:onUpdateParentNode()
    local len = self.waitIdsList:getSize();
    if len > self.everyTimeCount then
        len = self.everyTimeCount;
    end

    while (len > 0) do
        len = len - 1;
        local id = self.waitIdsList:shift();
        local vo = self:getRedPointVoById(id);
        self:updateNode(vo.parentId);
    end

    -- 移除刷新
    if self.waitIdsList:getSize() <= 0 then
        self:clearTimer();
    end
end

function Lua_RedPointData:updateNode(id)
    local vo = self:getRedPointVoById(id);
    if vo ~= nil then
        vo:checkChildInfo();
        local parentId = vo.parentId;
        if parentId ~= nil then
            self:updateNode(parentId);
        end
    end
end