--红点系统 红点Vo
Lua_RedPointVo = class("Lua_RedPointVo");
function Lua_RedPointVo:ctor(id, config)
    self.id = id;
    self._config = config;

    -- 初始化配置表数据
    self.bSetState = false;
    self.bShowOnce = false;
    self.bInitState = self.initState;
    self.setShowNum = self.bInitState and 1 or 0;

    self._isExtremity = true; -- 该值只需new的时候初始化一次(除运营活动主图标红点外)
    self._childIdsList = Array(); -- 子节点集合
end

-- 重置
function Lua_RedPointVo:reset()
    self.bSetState = false;
    self.bShowOnce = false;
    self.bInitState = self.initState; 
    self.setShowNum = self.bInitState and 1 or 0;

    self._mainIconRedId = nil;
end

function Lua_RedPointVo.Get:config()
    if self._config == nil then
        self._config = getConfig("redPoint", self.id);
        if self._config == nil then
            self._config = getConfig("redPoint", "common");
        end
    end
    if self._config == nil then
        LogManager.LogError("no redPoint config id : " .. self.id, false);
    end
    return self._config
end

---------------------------配置数据--------------------------

function Lua_RedPointVo.Get:name()
    if self.config == nil then
        return nil;
    end
    return self.config.name;
end

function Lua_RedPointVo.Get:parentId()
    if self.mainIconRedId ~= nil then
        return self.mainIconRedId;
    end

    if self.config == nil or self.config.parentId == nil then
        return nil;
    end
    return self.config.parentId;
end

function Lua_RedPointVo.Get:initState()
    if self.config == nil or self.config.initState == nil then
        return false;
    end
    return self.config.initState == 1;
end

function Lua_RedPointVo.Get:clickCancle()
    if self.config == nil or self.config.clickCancle == nil then
        return false;
    end
    return self.config.clickCancle == 1;
end

function Lua_RedPointVo.Get:showOnce()
    if self.config == nil or self.config.showOnce == nil then
        return false;
    end
    return self.config.showOnce == 1;
end

function Lua_RedPointVo.Get:resetEveryday()
    if self.config == nil or self.config.reset_everyday == nil then
        return false;
    end
    return self.config.reset_everyday == 1;
end

function Lua_RedPointVo.Get:bShowNum()
    if self.config == nil or self.config.bShowNum == nil then
        return false;
    end
    return self.config.bShowNum == 1;
end

function Lua_RedPointVo.Get:atlas()
    if self.config == nil then
        return nil;
    end
    return self.config.atlas;
end

function Lua_RedPointVo.Get:spriteName()
    if self.config == nil then
        return nil;
    end
    return self.config.spriteName;
end

function Lua_RedPointVo.Get:effName()
    if self.config == nil then
        return nil;
    end
    return self.config.effUrl;
end

---------------------------运营活动处理--------------------------

--主图标红点id(运营活动)
function Lua_RedPointVo.Get:mainIconRedId()
    return self._mainIconRedId;
end

function Lua_RedPointVo.Set:mainIconRedId(val)
    self._mainIconRedId = val;
end


---------------------------通用红点数据--------------------------

-- 显示状态
function Lua_RedPointVo.Get:showState()
    if self.serverState then
        return false;
    end

    if self.bShowOnce then
        return false;
    end

    if self.bInitState then
        return true;
    end

    if self.bSetState then
        return true;
    end

    return false;
end

-- 显示数量
function Lua_RedPointVo.Get:showNum()
    if self.serverState then
        return 0;
    end

    if self.bShowOnce then
        return 0;
    end

    if self.setShowNum <= 0 and self.bInitState then
        return 1;
    end

    return self.setShowNum;
end

-- 服务端记录状态
function Lua_RedPointVo.Get:serverState()
    return DataCenter.roleData:getOperationRecord(self.id) == 1;
end

-- 父节点vo
function Lua_RedPointVo.Get:parentVo()
    return DataCenter.redPointData:getRedPointVoById(self.parentId);
end

-- 是否是末端
function Lua_RedPointVo.Get:isExtremity()
    return self._isExtremity;
end

-- 是否是末端
function Lua_RedPointVo.Set:isExtremity(val)
    self._isExtremity = val;
end

-- 增加子节点
function Lua_RedPointVo:addChildId(id)
    if id ~= nil and self._childIdsList:indexOf(id) <= 0 then
        self._childIdsList:add(id);
    end
end

-- 移除子节点
function Lua_RedPointVo:removeChildId(id)
    if id ~= nil then
        self._childIdsList:removeItem(id);
    end
end

-- 更新数量
function Lua_RedPointVo:updateShowNum(val, isClick)
    local bResult = false;
    if isClick then 
        if self.showOnce then-- 只显示一次，点击后都不显示
            self.bShowOnce = true;
        end

        if self.bInitState then
            bResult = true;
            self.bInitState = false; -- 如有变化，则按实时的
        end
    end

    if self.setShowNum ~= val then
        bResult = true;
        self.setShowNum = val;
        self.bSetState = val > 0;
    end

    if bResult then
        if not self.bSetState and self.resetEveryday then
            Lua_ProxyManager.getProxy("Lua_RoleProxy"):sOperate(self.id, 1);
        end
        Lua_MsgUtil.callFunc("RedPoint_Update_State", self.id);
        return true;
    end
    
    return false;
end

-- 根据子节点更新状态
function Lua_RedPointVo:checkChildInfo()
    if self._isExtremity == true then
        return;
    end

    if self._childIdsList == nil then
        return;
    end

    local len = self._childIdsList:getSize();
    if len <= 0 then
        self._isExtremity = true;
    end

    local num = 0;
    for i = 1, len do
        local childId = self._childIdsList[i];
        local vo = DataCenter.redPointData:getRedPointVoById(childId);
        if vo ~= nil then
            num = num + vo.showNum;
        end
    end

    -- 更新红点
    self:updateShowNum(num);
end