-- 六边形地图
checkRequire("Lua_HexagonMapCell");
Lua_HexagonMapManager = class("Lua_HexagonMapManager");
function Lua_HexagonMapManager:ctor()
	self:superCall("ctor");

	self.gridIdsArr = Array();
	self.gridsDic = {};
	self.idleCellsList = Array();
	self.sceneId = 0;
	self.mapRadius = 0;
	self.mapCellSize = 0;
	self.walkPath = nil;
	self.walkIndex = 0;
	self.waitGridId = 0;
	self.startWalk = false;
end

function Lua_HexagonMapManager:createCell()
	local cell;
	local len = self.idleCellsList:getSize();
	if len > 0 then
		cell = self.idleCellsList:remove(len);
	end
	if cell == nil then 
		cell = Lua_HexagonMapCell.new();
	end
	return cell;
end

function Lua_HexagonMapManager:returnCell(cell)
	if cell ~= nil then
		cell:clean();
		self.idleCellsList:add(cell);
	end
end

function Lua_HexagonMapManager:clean()
	self.sceneId = 0;
	self.mapRadius = 0;
	self.mapCellSize = 0;
	self.walkPath = nil;
	self.walkIndex = 0;
	self.waitGridId = 0;
	self.startWalk = false;

	local len = self.gridIdsArr:getSize();
	while (len > 0) do
		local gridId = self.gridIdsArr[len];
		self:returnCell(self.gridsDic[gridId]);
		self.gridsDic[gridId] = nil;
		self.gridIdsArr:remove(len);
		len = len - 1;
	end
end

-- 初始化六边形地图
function Lua_HexagonMapManager:initHexagonMapInfo(sceneData)
	if sceneData == nil then
		return;
	end

	self:clean();
	self.sceneId = sceneData.sceneId;
	self.mapRadius = sceneData.mapRadius;
	self.mapCellSize = sceneData.cellSize;
	local arr = DataCenter.thingData:getThingIds(self.sceneId, 0);
	if arr ~= nil then
		for i = 1, arr:getSize() do
			local playerId = arr[i];
			local cfg = DataCenter.sceneUtil:getGridCfg(playerId);
			if cfg ~= nil and cfg.grid_type == 1 then -- 可走区域
				local cell = self:createCell();
				cell:setSceneInfo(self.sceneId, 0, playerId);
				self.gridsDic[playerId] = cell;
				self.gridIdsArr:add(playerId);
				print("add grid", playerId);
			end
		end
	end

	-- 生成主角在默认观察点上
	self.curGridId = 60101504555;
	local data = UsData();
    data.kind = Lua_ThingDefine.PLAYER;
    data.isShowName = false;
    data.roleId = DataCenter.roleData.roleId;
    data.objId = DataCenter.roleData.roleId;
    data.lvl = DataCenter.roleData.level;
    data.gender = 1;
    data.maxHp = 1;
    data.hp = 1;
    data.sceneId = self.sceneId;
    data.showShadow = false;
    data.bPlanarShadow = false;
    data.spd = 600;
    local position = DataCenter.sceneUtil:getGridPosition(self.curGridId);
    if position ~= nil then
    	data.px = position[1] * 100;
	    data.py = position[2] * 100;
	    data.pz = position[3] * 100;
    else
    	data.px = 0;
	    data.py = 0;
	    data.pz = 0;
    end
    ThingUtil.instance:addAnimal(data, true);
end

function Lua_HexagonMapManager:checkHexagonMapCell(cellKey)
	if self.sceneId <= 0 then 
		return;
	end

	local key = self.sceneId * 1000000 + cellKey;
	local grid = self.gridsDic[key];
	if grid == nil or grid.gridType ~= 1 then -- 不可走
		print("不可走", grid == nil);
		return;
	end

	if self.startWalk == true then -- 寻路中，到达下一个点再重新更改路径
		self.waitGridId = cellKey;
		print("寻路中，到达下一个点再重新更改路径");
		return;
	end

	-- 寻路
	local str = "寻路 ";
	local arr = self:getPath(self.curGridId, key);
	if arr ~= nil then
		for i = 1, arr:getSize() do
			str = str .. arr[i] .. " ";
		end

		self.walkPath = arr;
		self.walkIndex = arr:getSize();
		self:onMoveNext();
	end
	print(str);
end

function Lua_HexagonMapManager:onMoveNext()
	if self.waitGridId > 0 then
		local gid = self.waitGridId;
		self.startWalk = false;
		self.walkIndex = 0;
		self.waitGridId = 0;
		self:checkHexagonMapCell(gid);
		return;
	end

	if self.walkPath == nil or self.walkIndex <= 0 then
		self.startWalk = false;
		self.walkIndex = 0;
		return;
	end

	local gridId = self.walkPath[self.walkIndex];
	if gridId == nil then
		self.startWalk = false;
		self.walkIndex = self.walkIndex - 1;
		self:onMoveNext();
		return;
	end

	local grid = self.gridsDic[gridId];
	if grid == nil then
		self.startWalk = false;
		self.walkIndex = self.walkIndex - 1;
		self:onMoveNext();
		return;
	end

	self.startWalk = true;
	self.curGridId = gridId;
	local position = grid.position;
	local px = position[1] * 100;
	local py = position[2] * 100;
	local pz = position[3] * 100;
	self.walkIndex = self.walkIndex - 1;
	ThingUtil.instance:setAnimalMoveTo(self.sceneId, 0, DataCenter.roleData.roleId, px, py, pz, self, self.onMoveNext, true);
end

function Lua_HexagonMapManager:getPath(startGridId, endGridId)
	if startGridId == endGridId then
		return nil;
	end

	local openSet = Array();
    local closedSet = Array();
    openSet:add(startGridId); 

    local gridData;
    local curGridId;
    local curGridData;
    local comeFrom = {};
    comeFrom[startGridId] = 0;
    while (openSet:getSize() > 0) do
    	local curIdx = 1;
    	curGridId = openSet[curIdx];
    	curGridData = self.gridsDic[curGridId];
    	for i = 2, openSet:getSize() do
            gridData = self.gridsDic[openSet[i]];
            if gridData.fCost <= curGridData.fCost and gridData.hCost < curGridData.hCost then
                curGridId = openSet[i];
                curGridData = gridData;
                curIdx = i;
            end
        end

        openSet:remove(curIdx);
        closedSet:add(curGridId);
        if curGridId == endGridId then
            break;
        end
        
        local neighbors = curGridData.neighbors;
        if neighbors ~= nil and #neighbors > 0 then
        	for i = 1, #neighbors do
        		local neighborId = neighbors[i];
        		local neighborGrid = self.gridsDic[neighborId];
        		if neighborGrid ~= nil and neighborGrid.gridType == 1 and closedSet:indexOf(neighborId) == -1 then
        			local dis = self:getDistanceBetweenTwoVec(curGridData.cube, neighborGrid.cube);
        			local newCost = curGridData.gCost + dis;
        			if newCost < neighborGrid.gCost or openSet:indexOf(neighborId) == -1 then
	                    neighborGrid.hCost = dis;
	                    neighborGrid.gCost = newCost;
	                    comeFrom[neighborId] = curGridId;
	                    if openSet:indexOf(neighborId) == -1 then
	                        openSet:add(neighborId);
	                    end
	                end
        		end
        	end
        end
    end

    openSet:clear();
    curGridId = endGridId;
    openSet:add(curGridId);
    while (curGridId ~= startGridId) do
    	local gid = comeFrom[curGridId];
    	openSet:add(gid);
    	curGridId = gid;
    end
    return openSet;
end

function Lua_HexagonMapManager:getDistanceBetweenTwoVec(cubeA, cubeB)
	return Mathf.Max(Mathf.Abs(cubeA[1] - cubeB[1]), Mathf.Abs(cubeA[2] - cubeB[2]), Mathf.Abs(cubeA[3] - cubeB[3])); 
end