checkRequire("Lua_SceneUtil");
checkRequire("Lua_SceneData");
checkRequire("Lua_ConstructData");
checkRequire("Lua_ConstructUtil");
checkRequire("Lua_SceneConfigData");
checkRequire("Lua_HexagonMapManager");
Lua_SceneProxy = class("Lua_SceneProxy");
function Lua_SceneProxy:ctor()
	self:superCall("ctor");
	DataCenter.sceneUtil = Lua_SceneUtil.new();
	DataCenter.sceneData = Lua_SceneData.new();
	DataCenter.constructData = Lua_ConstructData.new();
	DataCenter.constructUtil = Lua_ConstructUtil.new();
	DataCenter.hexagonMapManager = Lua_HexagonMapManager.new();

	MsgUtil.addEventListener("SCENE_HIDE", self, self.onSceneHide);
	MsgUtil.addEventListener("SCENE_OPEN", self, self.onSceneOpen);
	MsgUtil.addEventListener("SCENE_CHANGE", self, self.onSceneChange);
	MsgUtil.addEventListener("SCENE_LOADED", self, self.onSceneLoaded);
	MsgUtil.addEventListener("SCENE_SHOW", self, self.onSceneShow);
	MsgUtil.addEventListener("LIGHTMAP_DATTIMEORNIGHT_STARTCHANGE", self, self.onChangeDayOrNightStart);
	MsgUtil.addEventListener("LIGHTMAP_DATTIME_OR_NIGHT", self, self.onChangeDayOrNight);
	MsgUtil.addEventListener("HEXAGONMAP_COLLIDER_CLICK", self, self.onHexagonMapClick);

	Lua_MsgUtil.addEventListener("MODEL_SCENE_SHOW", self, self.onModelSceneShow);
	Lua_MsgUtil.addEventListener("MODEL_SCENE_HIDE", self, self.onModelSceneHide);
	Lua_MsgUtil.addEventListener("UPDATE_HUNDRREDCEREMONY_INFO", self, self.updateConstruct);
end





------------------------------------------------------------
------------------------------------------------------------
------------------------------------------------------------
-- 事件监听
-- 场景隐藏
function Lua_SceneProxy:onSceneHide(o)
	local sceneId = tonumber(o);
	-- print("onSceneHide", sceneId);
	if DataCenter.sceneData.openSceneId == sceneId then
		DataCenter.sceneData.curSceneId = 0;
		DataCenter.sceneData.openSceneId = 0;
		Lua_MsgUtil.callFunc("SCENE_HIDE", sceneId);
		self:updateMainMenu();
	end
end

function Lua_SceneProxy:onSceneOpen(o)
	local sceneId = tonumber(o);
	-- print("onSceneOpen", sceneId);
	DataCenter.sceneData.openSceneId = sceneId;
	if sceneId ~= DataCenter.mainLineData.curSceneId then
		DataCenter.mainLineCamActController:stopCamAct(true);-- 移除镜头动画
	end
	Lua_MsgUtil.callFunc("SCENE_OPEN", sceneId);
end

-- 场景变更
function Lua_SceneProxy:onSceneChange(o)
	local sceneId = tonumber(o);
	-- print("onSceneChange", sceneId);
	Lua_MsgUtil.callFunc("SCENE_CHANGE", sceneId);
end

-- 场景完全初始化ok
function Lua_SceneProxy:onSceneLoaded(o)
	local sceneId = tonumber(o);
	-- print("onSceneLoaded", sceneId);
	Lua_MsgUtil.callFunc("SCENE_LOADED", sceneId);
	DataCenter.sceneData:onSceneLoaded();
end

-- 场景显示完成
function Lua_SceneProxy:onSceneShow(o)
	local sceneId = tonumber(o);
	-- print("onSceneShow", sceneId);
	if DataCenter.sceneData.openSceneId == sceneId then
		DataCenter.bCameraDragRange = false;
		if sceneId == DataCenter.mainLineData.curSceneId then
			DataCenter.mainLineCamActController:setDefault();
		end

		local sceneData = Lua_SceneConfigData.getSceneConfig(sceneId);
		if not sceneData.isUIScene then
			-- DataCenter.mainLineSceneController:onSceneShow(sceneId);
			DataCenter.sceneData:showScene(sceneId);
			if sceneData.isHexagonMap then
				DataCenter.hexagonMapManager:initHexagonMapInfo(sceneData);
			else
				DataCenter.constructData:buildStructure(sceneId);
			end
		end

		-- 阴影
		local sceneResId = sceneData.sceneResId;
		if sceneResId ~= nil then
			local bCastShadow = getConfig({"tb_scene_res", sceneResId, "bCastShadow"});
			if bCastShadow ~= nil and bCastShadow == 1 then
				DeviceQuality.instance:setLightShadow(true);
			else
				DeviceQuality.instance:setLightShadow(false);
			end
		end

		-- 场景显示
		Lua_MsgUtil.callFunc("SCENE_SHOWING", sceneId);
		DataCenter.popupWinManager:checkPopupWin();
	end
end

function Lua_SceneProxy:onModelSceneShow(sceneId)
	-- print("onModelSceneShow", sceneId);
	if DataCenter.sceneData.openSceneId == sceneId then
		DataCenter.sceneData:showScene(sceneId);
	end
end

function Lua_SceneProxy:onModelSceneHide(sceneId)
	-- print("onModelSceneHide", sceneId);
end

function Lua_SceneProxy:updateMainMenu()
	local iconId = DataCenter.mainMenuData.selectMainIconId;
	if iconId == nil or iconId == Lua_MenuViewDefine.ICON_MAINCITY then
		Lua_MsgUtil.callFunc("UPDATE_MAIN_MENU", Lua_MenuViewDefine.MEBULIST_ALL);
	else
		Lua_MsgUtil.callFunc("UPDATE_MAIN_MENU", Lua_MenuViewDefine.MEBULIST_MAIN);
	end
end

function Lua_SceneProxy:onChangeDayOrNightStart(idx)
	DataCenter.sceneData.isDayTime = (idx == 1);
	Lua_MsgUtil.callFunc("DAYORNIGHT_UPDATE_START");
end

function Lua_SceneProxy:onChangeDayOrNight(idx)
	Lua_MsgUtil.callFunc("DAY_OR_NIGHT_UPDATE");
end

-- 六边形地图点击
function Lua_SceneProxy:onHexagonMapClick(cellKey)
	DataCenter.hexagonMapManager:checkHexagonMapCell(cellKey);
end

function Lua_SceneProxy:updateConstruct()
	if DataCenter.sceneData.curSceneId == Lua_SceneDefine.MAINCITY then
		if DataCenter.hundredCeremonyData.isActEnd then
			ThingUtil.instance:removeThing(10101, 0, 10101009);
		else
			local thing = DataCenter.thingData:getThingPlayer(10101, 0, 10101009);
			if thing and thing.player then
				if DataCenter.hundredCeremonyData.isCanDraw then
					thing.player:addHangerEffect(Lua_ThingDefine.CONSTRUCTION_EFFECT, "bcsd_ft_0001");
				else
					thing.player:removeEffect(Lua_ThingDefine.CONSTRUCTION_EFFECT, "bcsd_ft_0001");
				end
			end
		end
	end
end