-- 场景配置
Lua_SceneConfigData = class("Lua_SceneConfigData");
function Lua_SceneConfigData.getSceneConfig(sceneId)
    if sceneId == nil then
        return nil;
    end

    if Lua_SceneConfigData.configs == nil then
        Lua_SceneConfigData.configs = {};
    end

    if Lua_SceneConfigData.configs[sceneId] == nil then
        local cfg = getConfig({"tb_scene", sceneId});
        if cfg == nil then return nil; end
        Lua_SceneConfigData.configs[sceneId] = Lua_SceneConfigData.new(sceneId, cfg);
    end
    return Lua_SceneConfigData.configs[sceneId];
end

function Lua_SceneConfigData:ctor(sceneId, config)
    self:superCall("ctor");
    self.sceneId = sceneId;
    self.config = config;
    if self.config == nil then
        self.config = getConfig({"tb_scene", sceneId});
    end
end

function Lua_SceneConfigData.Get:name()
    if self.config == nil or self.config.name == nil then
        return "";
    end
    return self.config.name;
end

function Lua_SceneConfigData.Get:sceneResId()
    if self.config == nil or self.config.sceneResId == nil then
        return 0;
    end
    return self.config.sceneResId;
end

function Lua_SceneConfigData.Get:isUIScene()
    return self.config.uiScene == 1;
end

function Lua_SceneConfigData.Get:isHexagonMap()
    return self.config.uiScene == 2;
end

function Lua_SceneConfigData.Get:minLv()
    if self.config.level ~= nil and #self.config.level > 0 then
        return self.config.level[1];
    end
    return 0;
end

function Lua_SceneConfigData.Get:maxLv()
    if self.config.level ~= nil and #self.config.level > 1 then
        return self.config.level[2];
    end
    return 0;
end

function Lua_SceneConfigData.Get:camType()
    if self.config.camType == nil then
        return 0;
    end
    return self.config.camType;
end

function Lua_SceneConfigData.Get:lookPos()
    if self._defaultLookPos == nil then
        if self.config.lookPos ~= nil then
            if #self.config.lookPos > 2 then
                self._defaultLookPos = Vector3(self.config.lookPos[1] / 100, self.config.lookPos[2] / 100, self.config.lookPos[3] / 100);
            elseif #self.config.lookPos > 1 then
                self._defaultLookPos = Vector3(self.config.lookPos[1] / 100, self.config.lookPos[2] / 100, 0);
            else
                self._defaultLookPos = Vector3(self.config.lookPos[1] / 100, 0, 0);
            end
        else
            self._defaultLookPos = Vector3(0, 0, 0);
        end
    end
    return self._defaultLookPos;
end

function Lua_SceneConfigData.Get:fightStageCenterPos()
    if self._fightStageCenterPos == nil then
        if self.config.fightStageCenter ~= nil then
            if #self.config.fightStageCenter > 2 then
                self._fightStageCenterPos = Vector3(self.config.fightStageCenter[1] / 100, self.config.fightStageCenter[2] / 100, self.config.fightStageCenter[3] / 100);
            elseif #self.config.fightStageCenter > 1 then
                self._fightStageCenterPos = Vector3(self.config.fightStageCenter[1] / 100, self.config.fightStageCenter[2] / 100, 0);
            else
                self._fightStageCenterPos = Vector3(self.config.fightStageCenter[1] / 100, 0, 0);
            end
        else
            self._fightStageCenterPos = self.defaultLookPos;
        end
    end
    return self._fightStageCenterPos;
end

function Lua_SceneConfigData.Get:wanderStartPoint()
    if self.config.wanderStartPoint == nil then
        return 0;
    end
    return self.config.wanderStartPoint;
end

function Lua_SceneConfigData.Get:wanderGroupId()
    if self.config.wanderGroupId == nil then
        return 0;
    end
    return self.config.wanderGroupId;
end

function Lua_SceneConfigData.Get:isWanderScene()
    if self.wanderGroupId <= 0 or Lua_MonsterFightGroupConfigData.getConfigData(self.wanderGroupId) == nil then
        return false;
    end

    if self.wanderStartPoint <= 0 or Lua_SceneWanderPointConfigData.getConfigData(self.wanderStartPoint) == nil then
        return false;
    end

    return true;
end

function Lua_SceneConfigData.Get:orthographicSize()
    if self.config.orthoSize == nil then
        return 15;
    end
    return self.config.orthoSize;
end

function Lua_SceneConfigData.Get:perFieldOfView()
    if self.config.perFieldOfView == nil then
        return 60;
    end
    return self.config.perFieldOfView;
end

function Lua_SceneConfigData.Get:panAngle()
    if self.config == nil then
        return 0;
    end
    return self.config.panAngle or 0;
end

function Lua_SceneConfigData.Get:panMinAngle()
    if self.config == nil then
        return 0;
    end
    return self.config.panMinAngle or 0;
end

function Lua_SceneConfigData.Get:panMaxAngle()
    if self.config == nil then
        return 0;
    end
    return self.config.panMaxAngle or 0;
end

function Lua_SceneConfigData.Get:titleAngle()
    if self.config == nil then
        return 0;
    end
    return self.config.titleAngle or 0;
end

function Lua_SceneConfigData.Get:titleMinAngle()
    if self.config == nil then
        return 0;
    end
    return self.config.titleMinAngle or 0;
end

function Lua_SceneConfigData.Get:titleMaxAngle()
    if self.config == nil then
        return 0;
    end
    return self.config.titleMaxAngle or 0;
end

function Lua_SceneConfigData.Get:menuItem()
    return self.config.menu_item;
end

function Lua_SceneConfigData.Get:openWins()
    return self.config.open_win;
end

function Lua_SceneConfigData.Get:closeWins()
    return self.config.close_win;
end

function Lua_SceneConfigData.Get:closeWinType()
    if self.config == nil or self.config.closeWinType == nil then
        return 0;
    end
    return self.config.closeWinType;
end

function Lua_SceneConfigData.Get:unCloseWins()
    return self.config.unCloseWins;
end

function Lua_SceneConfigData.Get:hideWinType()
    if self.config == nil or self.config.hideWinType == nil then
        return 0;
    end
    return self.config.hideWinType;
end

function Lua_SceneConfigData.Get:bControllDispose()
    if self.config == nil or self.config.controllDispose == nil then
        return false;
    end
    return self.config.controllDispose == 1;
end

function Lua_SceneConfigData.Get:mapRadius()
    if self.config == nil or self.config.mapRadius == nil then
        return 0;
    end
    return self.config.mapRadius;
end

function Lua_SceneConfigData.Get:cellSize()
    if self.config == nil or self.config.cellSize == nil then
        return 0;
    end
    return self.config.cellSize;
end

function Lua_SceneConfigData.Get:uiSceneCam()
    if self.config == nil then
        return nil;
    end
    return self.config.uiSceneCam;
end