checkRequire("Lua_ShopMainTypeVo");
checkRequire("Lua_ShopSubTypeVo");
checkRequire("Lua_ShopTypeVo");
checkRequire("Lua_ShopItemVo");
checkRequire("Lua_ShopLimitData");
Lua_ShopUtil = class("Lua_ShopUtil");

function Lua_ShopUtil:ctor()
	self:dispose();
end

function Lua_ShopUtil:dispose()
	self._mainTypeArr = nil;
	self._shopSubTypeInfo = nil;
	self._shopSubTypeArr = nil;
	self._shopTypeInfo = nil;
	self._shopItemInfo = nil;
end

-------------------------商店信息------------------------

function Lua_ShopUtil.Get:getMainTypeArr()
	if self._mainTypeArr == nil then
		self._mainTypeArr = Array();
		local config = getConfig({"shop_main_type"});
		local vo;
		for k, v in pairs(config) do
			vo = Lua_ShopMainTypeVo.new(k, v);
			self._mainTypeArr:add(vo);
		end
		self._mainTypeArr:sort("sortIdx", false);
	end
	return self._mainTypeArr;
end

function Lua_ShopUtil.Get:getShopSubTypeInfo()
	if self._shopSubTypeInfo == nil then
		self._shopSubTypeInfo = {};
		local config = getConfig({"shop_sub_type"});
		for k, v in pairs(config) do
			self._shopSubTypeInfo[k] = Lua_ShopSubTypeVo.new(k, v);
		end
	end
	return self._shopSubTypeInfo;
end

function Lua_ShopUtil.Get:getShopSubTypeArr()
	if self._shopSubTypeArr == nil then
		self._shopSubTypeArr = Array();
		for k, v in pairs(self.getShopSubTypeInfo) do
			self._shopSubTypeArr:add(v);
		end
		self._shopSubTypeArr:sort("sortIdx", false);
	end
	return self._shopSubTypeArr;
end

function Lua_ShopUtil:getShopSubTypeVo(id)
	if self.getShopSubTypeInfo[id] ~= nil then
		return self.getShopSubTypeInfo[id];
	end
	return nil;
end

function Lua_ShopUtil.Get:getShopTypeInfo()
	if self._shopTypeInfo == nil then
		self._shopTypeInfo = {};
		local config = getConfig({"shopType"});
		local vo;
		for k, v in pairs(config) do
			self._shopTypeInfo[k] = Lua_ShopTypeVo.new(k, v);
		end
	end
	return self._shopTypeInfo;
end

function Lua_ShopUtil:updateShopTypeInfo(shopType, serverInfo)
	if shopType == nil or serverInfo == nil then
		return;
	end
	if self.getShopTypeInfo[shopType] ~= nil then
		self.getShopTypeInfo[shopType]:updateServerInfo(serverInfo);
	end
end

function Lua_ShopUtil:getShopTypeVo(shopType)
	if self.getShopTypeInfo[shopType] ~= nil then
		return self.getShopTypeInfo[shopType];
	end
	return nil;
end

-------------------------商品信息------------------------

function Lua_ShopUtil:getShopItemInfo(shopType, shopId)
	if self._shopItemInfo == nil then
		self._shopItemInfo = {};
		self._config = getConfig("shop");
	end

	if self._shopItemInfo[shopType] == nil then
		self._shopItemInfo[shopType] = {};

		for k, v in pairs(self._config) do
			if v.shop_type == shopType then
				self._shopItemInfo[shopType][k] = Lua_ShopItemVo.new(k, v);
			end
		end
	end

	if shopId ~= nil then
		return self._shopItemInfo[shopType][shopId];
	else
		return self._shopItemInfo[shopType]
	end
	return nil;
end

function Lua_ShopUtil:updateShopItemInfo(shopType, serverInfo)
	if shopType == nil or serverInfo == nil then
		return;
	end

	local shopItemVo = self:getShopItemInfo(shopType, serverInfo.id);
	if shopItemVo ~= nil then
		shopItemVo:resetServerInfo();
		shopItemVo:updateServerInfo(serverInfo);
	end
end

function Lua_ShopUtil:getShopItemList(shopType)
	local list = {};
	local shopItemInfo = self:getShopItemInfo(shopType);
	if shopItemInfo ~= nil then
		for k, v in pairs(shopItemInfo) do
			if v.isOpened then
				list[#list + 1] = v;
			end
		end
	end
	local curLeagueLevel = DataCenter.roleData.league_level;
	if shopType == 5 then
		table.sort( list, function(a, b) return a.leagueSort < b.leagueSort; end);
	else
		table.sort( list, function(a, b) return a.sortIndex < b.sortIndex; end);
	end
	return list;
end

-- 已购买数量重置
function Lua_ShopUtil:resetBuyTime()
	if self._shopItemInfo ~= nil then
		for i, obj in pairs(self._shopItemInfo) do
			for k, vo in pairs(obj) do
				vo:resetBuyTime();
			end
		end
	end
end