Lua_TaskData = class("Lua_TaskData");
Lua_TaskData.dailyTask = 1; -- 每日任务
Lua_TaskData.weeklyTask = 2; -- 周常任务
Lua_TaskData.achievementTaskTask = 3; -- 成就任务
Lua_TaskData.activeWarTask = 4; -- 活跃战令




---总任务类型
Lua_TaskData.AdventureJourney_Task = 1; -- 冒险之旅任务

function Lua_TaskData:ctor()
	self:dispose();
end

function Lua_TaskData:dispose()
	self.taskInfo = nil;
	self.recordDataArr = Array();
	-- 领取奖励间隔
	self.lastReceivedTime = 0; -- 上次领取时间
	self.receiveDeltaTime = 0.5; -- 领取间隔时间
	self.actWarReward = false;

	-- 冒险之旅
	self.curStageId = 1;
	self.curRewardState = 0;
	self.artifactState = 0;
	self.artifactId = nil;
	self.taskRedState = false;
	self.showTaskArr = Array();
	self.openState = false;
	self.selectIdx = 1;
end

-------------------- 通知-任务奖励状态信息 --------------------
function Lua_TaskData:fromServer80178001(msg)
	-- 日常任务功能未开启时，不弹窗
	if not DataCenter.funcOpenData:isOpened(10194) then return; end 

	self:initTaskInfo();
	local obj = msg.task;
	-- self.taskInfo[obj.main_type] = self:dealWithTaskInfo(obj);
	self:updateTaskListInfo(obj.task_list, true);

	-- 单个页面红点显示
	local hasReward = DataCenter.taskUtil:hasRewardByType(obj.main_type);
	self:showRedDot(obj.main_type, hasReward);


	if Lua_WindowUtil.isWindowShowing("Lua_TaskRewardTipWindow") == false then
		self:showCurData();
	end

	-- 弹窗且窗口打开时更新数据
	if Lua_WindowUtil.isWindowShowing("Lua_TaskBaseWindow") then
		Lua_MsgUtil.callFunc("SELECT_DAILYTASK_PANEL");
	end
end

function Lua_TaskData:showCurData()
	if self.showTaskArr:getSize() <= 0 and self.recordDataArr:getSize() > 0 then -- 冒险之旅判断
		Lua_WindowUtil.openWin("Lua_TaskRewardTipWindow", Lua_WinParam.ACTION_OPEN, self.recordDataArr[1]);
	end
end

function Lua_TaskData:removePreData()
	self.recordDataArr:remove(1);
	if self.showTaskArr:getSize() > 0 and Lua_WindowUtil.isWindowShowing("Lua_ExploreTourPopupWindow") == false then -- 冒险之旅判断
		Lua_WindowUtil.openWin("Lua_ExploreTourPopupWindow", Lua_WinParam.ACTION_OPEN);
		return;
	end
	self:showCurData();
end










-------------------- 日常、周常、成就信息 --------------------

function Lua_TaskData:fromServer50178002(msg)
	self:initTaskInfo();

	local obj;
	local vo;
	for i = 1, #msg.task_list do
		obj = msg.task_list[i];
		self.taskInfo[obj.main_type] = self:dealWithTaskInfo(obj);
		self:updateTaskListInfo(obj.task_list);
	end

	-- 更新日常、周常、成就任务红点
	self:updateRedPoint();
end

-- 初始化taskInfo、及读取配置表
function Lua_TaskData:initTaskInfo()
	if self.taskInfo == nil then
		self.taskInfo = {};
		DataCenter.taskUtil:initActivePointRewards();
		DataCenter.taskUtil:initTaskInfoList();
	end
end

-- 根据任务类型获取活跃度、已领取的活跃度奖励id（日常、周常、成就）
function Lua_TaskData:getTaskInfoByType(type)
	if type == nil or self.taskInfo[type] == nil then
		return;
	end

	return self.taskInfo[type];
end

-- 处理服务端传输过来的数据
function Lua_TaskData:dealWithTaskInfo(taskInfo)
	local param = {
		point = taskInfo.point, --point: 活跃度
		-- draw_point_list = taskInfo.draw_point_list; --maxReceivedActiveRewardId: 已领取的活跃度奖励id
	};

	-- 更新活跃度奖励是否已领取
	local type = taskInfo.main_type;
	if type ~= Lua_TaskData.achievementTaskTask then
		local activeRewardList = DataCenter.taskUtil:getActivePointIndexesByType(taskInfo.main_type);
		local count = #activeRewardList;
		local obj;
		local vo;
		for i = 1, count do
			obj = activeRewardList[i];
			vo = DataCenter.taskUtil:getActivePointRewardById(obj);
			if vo ~= nil then
				if self:findActiveIdFromServerData(taskInfo.draw_point_list, obj) then
					vo:updateServer(taskInfo.point, true);
				else
					vo:updateServer(taskInfo.point, false);
				end
			end
		end
	end
	return param;
end

-- id 是否为 已领取的奖励
function Lua_TaskData:findActiveIdFromServerData(list, id)
	local result = false;
	if #list ~= 0 then
		for i = 1, #list do
			if list[i] == id then
				result = true;
				break;
			end
		end
	end
	return result;
end

-- 更新任务列表
function Lua_TaskData:updateTaskListInfo(taskList, isPopUp)
	local vo;
	local obj;
	for i = 1, #taskList do
		obj = taskList[i];
		vo = DataCenter.taskUtil:getTaskInfoById(obj.id);
		if vo ~= nil then
			vo:updateServer(obj.progress, obj.flag, obj.index);

			-- isPopUp:是否弹窗，弹窗需要记录该任务的id 
			-- flag:任务状态  0: 未完成 1: 完成 2: 已领取
			-- is_inform: 是否已弹出过弹窗 0 没有，1 已弹出过
			local isRecord = isPopUp and obj.flag == 1 and obj.is_inform == 0;
			if isRecord then
				self.recordDataArr:add({ id = vo.id, desc = vo.desc });
			end
		end
	end
end

-- 点击主图标按钮时，默认打开界面
function Lua_TaskData:getSelectedPanel()
	local selectedPanel = Lua_TaskData.dailyTask;
	local list = DataCenter.taskUtil:getAllTaskInfoIndexes();
	for k, v in ipairs(list) do
		-- 判断活跃战令是否有奖励
		if k == Lua_TaskData.achievementTaskTask and self.activeWarOpenState and self.activeWarShowState then
			selectedPanel = Lua_TaskData.activeWarTask;
			break;
		end

		local hasReward = DataCenter.taskUtil:hasRewardByType(k);
		if hasReward then
			selectedPanel = k;
			break;
		end
	end
	return selectedPanel;
end

-- 更新日常、周常、成就任务红点
function Lua_TaskData:updateRedPoint()
	local list = DataCenter.taskUtil:getAllTaskInfoIndexes();
	for type, v in ipairs(list) do
		local hasReward = DataCenter.taskUtil:hasRewardByType(type);
		self:showRedDot(type, hasReward);
	end
end

-- 显示日常、周常、成就任务红点
function Lua_TaskData:showRedDot(taskType, redPointState)
	if taskType == Lua_TaskData.dailyTask then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Task_DailyTab, redPointState);
	elseif taskType == Lua_TaskData.weeklyTask then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Task_WeeklyTab, redPointState);
	else
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Task_AchievementTab, redPointState);
	end
end

-- 记录领取奖励前的活跃度和对应的type
function Lua_TaskData:recordActivePointBeforeChanging(type, activePoint)
	if self.preActivePoint ~= nil then
		return;
	end
	self.preActivePoint = activePoint;
	self.preType = type;
end

-- 清空领取奖励前的活跃度和对应的type
function Lua_TaskData:clearPreActivePoint()
	self.preActivePoint = nil;
	self.preType = nil;
end

-- 活跃度进度条是否开始渐变
function Lua_TaskData:enableChangingProcess()
	return self.preActivePoint ~= nil;
end

-- 获取领取奖励前的活跃度
function Lua_TaskData:getPreActivePoint()
	return self.preActivePoint;
end

-- 记录日常、周常当前进度条数值
function Lua_TaskData:setDailyTaskPanelCurProcessValue(value)
	self.curDailyTaskPanelProcessValue = value;
end

-- 获取日常、周常当前进度条数值
function Lua_TaskData:getDailyTaskPanelCurProcessValue()
	return self.curDailyTaskPanelProcessValue == nil and 0 or self.curDailyTaskPanelProcessValue;
end










-------------------- 日常任务-活跃战令 --------------------

Lua_TaskData.primaryWarStyle = 1; -- 初级战令
Lua_TaskData.seniorWarStyle = 2; -- 高级战令
function Lua_TaskData:fromServer50178009(msg)
	self.activeWarRoundId = msg.cur_round;
	self.activeWarPoint = msg.activity;
	self.activeWarRTime = msg.round_time; -- DataCenter.roleData.serverTime;
	self.primaryWarLockState = msg.lock_state; -- 初级奖励解锁状态 0否 1是
	self.seniorWarLockState = msg.stage_lock_state; -- 高级奖励解锁状态 0否 1是
	self.isFirstActWar = msg.first_login == 0; -- 是否首次登录 0否 1是
	self:updateOpenState();

	local obj;
	for i = 1, #msg.reward_state do
		obj = msg.reward_state[i];
		self.actWarReward = false;
		if self.actWarReward == false
			and ((obj.normal_reward_state == 1 and msg.lock_state == 1)
					or (obj.stage_reward_state == 1 and msg.stage_lock_state == 1)) then
			self.actWarReward = true;
			break;
		end
	end

	if self.activeWarRoundId == nil then
		return;
	end
	DataCenter.taskUtil:initAcitveWarInfo(self.activeWarRoundId);
	self:updateAcitveWarConfig(self.activeWarRoundId);
	self:updateAcitveWarServerInfo(msg.reward_state);
	self:updateActiveWarRedPointAndIndex();
end

-- 更新页签的红点显示状态activeWarShowState 及打开活跃战令时dragList列表moveto的索引activeWarShowIndex
function Lua_TaskData:updateActiveWarRedPointAndIndex()
	local rewardList = self.acitveWarList;
	self.activeWarShowState = nil;
	self.activeWarShowIndex = nil; -- 显示阶段索引，默认为1
	if self.primaryWarLockState == 0 and self.seniorWarLockState == 0 then
		self.activeWarShowState = false;
	else
		for k, v in ipairs(rewardList) do
			local AWVo = self:getWarById(v.acitveWarId);
			if (AWVo.primaryRewardState == 1 and self.primaryWarLockState == 1) or
				(AWVo.seniorRewardState == 1 and self.seniorWarLockState == 1)  then
					self.activeWarShowState = true;
					self.activeWarShowIndex = k;
					break;
			end
		end

		if self.activeWarShowIndex == nil then
			for k, v in ipairs(rewardList) do
				local AWVo = self:getWarById(v.acitveWarId);
				if AWVo.primaryRewardState == 0  then
					self.activeWarShowIndex = k;
					break;
				end
			end
		end
	end

	self.activeWarShowIndex = self.activeWarShowIndex == nil and 1 or self.activeWarShowIndex;
	self.activeWarShowState = self.activeWarShowState ~= nil and self.activeWarShowState or false;
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.Task_ActiveWarTab, self.activeWarShowState or self.isFirstActWar);
end

-- 根据轮次选择配置表对应的轮次内容
function Lua_TaskData:updateAcitveWarConfig(roundId)
	if roundId == nil then
		return;
	end

	self.acitveWarList = self:dealwithAcitveWarList();
	self.acitveWarDic = DataCenter.taskUtil:getAcitveWarDic();
	self.lockRewardTVo = DataCenter.taskUtil.lockRewardTVo;
	self.rechargeId = DataCenter.taskUtil.rechargeId;
end

-- 更新各阶段奖励的领取状态
function Lua_TaskData:updateAcitveWarServerInfo(rewardState)
	if rewardState == nil then
		return;
	end

	local vo;
	local obj;
	for i = 1, #rewardState do
		obj = rewardState[i];
		vo = self:getWarById(obj.id);
		if vo ~= nil then
			vo:updateServer(obj);
		end
	end
end

-- 根据id获取阶段的具体信息
function Lua_TaskData:getWarById(id)
	if id == nil or self.acitveWarDic == nil or self.acitveWarDic[id] == nil then
		return;
	end
	return self.acitveWarDic[id];
end

-- 根据id获取self.acitveWarList[id]的具体信息
function Lua_TaskData:getAWLVoById(id)
	if id == nil or self.acitveWarList == nil or self.acitveWarList[id] == nil then
		return;
	end
	return self.acitveWarList[id];
end

-- 处理数据：使得记录战令id携带在acitveWarList中各自的索引
function Lua_TaskData:dealwithAcitveWarList()
	local param = {};
	local acitveWarList = DataCenter.taskUtil:getAcitveWarList();
	for k, v in ipairs(acitveWarList) do
		param[k] = { acitveWarId = v, index = k };
	end
	return param;
end

-- 处理战令状态并返回 （初级or高级奖励Item、战令id、奖励的领取状态、是否已解锁、奖励的类型
-- activeWar_Style为战令的类型：Lua_TaskData.primaryWarStyle为初级战令、 Lua_TaskData.seniorWarStyle为高级战令
function Lua_TaskData:dealwithWarStateData(activeWar_Style, id)
	local vo;
	local rewardList;
	local reward_State;
	local lock_State;
	local param = {};
	-- 获取战令的具体信息，若为nil,返回
	vo = self:getWarById(id);
	if vo == nil then
		return;
	end

	-- 根据战令类型，获取对应奖励列表、领取状态、是否解锁
	if activeWar_Style == Lua_TaskData.primaryWarStyle then
		rewardList = vo.primaryRewardList;
		reward_State = vo.primaryRewardState;
		lock_State = self.primaryWarLockState;
	elseif activeWar_Style == Lua_TaskData.seniorWarStyle then 
		rewardList = vo.seniorRewardList;
		reward_State = vo.seniorRewardState;
		lock_State = self.seniorWarLockState;
	end

	-- 若奖励列表为空，返回
	if rewardList == nil then
		return;
	end

	-- 重构数据：每个奖励均携带战令id、奖励状态、是否解锁、战令类型
	for k, v in ipairs(rewardList) do
		param[k] = {rewardData = v, rewardId = id, rewardState = reward_State, lockState = lock_State, activeWarStyle = activeWar_Style};
	end
	return param;
end

-- 根据战令id，及其所在acitveWarList的索引，计算出当前进度条的curValue,maxValue并返回
function Lua_TaskData:calculateProcessBar(id, data_index)
	local rewardVo = self:getWarById(id);
	if rewardVo == nil then 
		return;
	end

	if rewardVo.process ~= 0 then
		if rewardVo.process <= self.activeWarPoint then
			return 1, 1;
		else
			local AWLVo = self:getAWLVoById(data_index - 1); 
			if AWLVo == nil then
				return;
			end

			local lastId = AWLVo.acitveWarId; -- 获取上一阶段战令的id
			local lastActiveWarVo = self:getWarById(lastId);
			local lastActiveWarPoint = lastActiveWarVo.process; -- 获取上一阶段的活跃度lastActiveWarPoint
			local maxValue = rewardVo.process - lastActiveWarPoint; -- 计算当前进度条的maxValue
			local curValue = self.activeWarPoint - lastActiveWarPoint; -- 计算当前进度条的curValue
			curValue = curValue <= 0 and 0 or curValue; -- 当前进度条最小值为0
			return curValue, maxValue;
		end     
	end
end

function Lua_TaskData:updateOpenState()
	if self.activeWarOpenID == nil then
		self.activeWarOpenID = DataCenter.taskUtil:getConstConfig("activeWar_func_id", 1)
	end
	self.activeWarOpenState = DataCenter.funcOpenData:isOpened(activeWarOpenID, false);
end










-------------------- 冒险之旅 --------------------

-- 请求主线任务信息返回
function Lua_TaskData:fromServer50178005(msg)
	self.curStageId = msg.cur_stage; -- 当前阶段
	self.curRewardState = msg.stage_reward_state; -- 0 不可领 1：可领取 2：已领取
	self.artifactState = msg.is_unlock_artifact; -- 1已解锁 0没有神器或者未解锁
	self.artifactId = DataCenter.taskUtil:getExploreTourStageInfo(self.curStageId, "unlock_artifact"); -- 神器id
	self:updateTaskInfo(msg.stage_task_info);
	self:checOpenState();
	Lua_MsgUtil.callFunc("UPDATE_EXPLORER_TOUR_INFO", self.curStageId);
end

-- 请求领取主线任务奖励返回
function Lua_TaskData:fromServer50178006(msg)
	self.curStageId = msg.stage_id; -- 当前阶段
	self.curRewardState = msg.stage_reward_state; -- 0 不可领 1：可领取 2：已领取
	self:updateTaskInfo(msg.task_info);
	self:checOpenState();
	Lua_MsgUtil.callFunc("UPDATE_EXPLORER_TOUR_INFO", self.curStageId);
end

-- 请求解锁主线神器返回
function Lua_TaskData:fromServer50178007(msg)
	self.curStageId = msg.cur_stage_id; -- 当前阶段
	self.curRewardState = msg.stage_reward_state; -- 0 不可领 1：可领取 2：已领取
	self.artifactState = msg.is_unlock_artifact; -- 1已解锁 0没有神器或者未解锁
	self.artifactId = DataCenter.taskUtil:getExploreTourStageInfo(self.curStageId, "unlock_artifact");--神器id
	self:updateTaskInfo(msg.task_info);
	self:checOpenState();

	local lastStep = self.curStageId;
	if self.artifactState == 0 then
		lastStep = lastStep - 1;
	end

	local lastArtifactId = DataCenter.taskUtil:getExploreTourStageInfo(lastStep, "unlock_artifact");
	if lastArtifactId ~= nil then
		Lua_MsgUtil.callFunc("UNLOCK_EXPLORERTOUR_ARTIFACT", lastArtifactId);
	end
	Lua_MsgUtil.callFunc("UPDATE_EXPLORER_TOUR_INFO", self.curStageId);
end

-- 通知主线任务完成
function Lua_TaskData:fromServer80178008(msg)
	if DataCenter.funcOpenData:isOpened(10028) == false then
		return;
	end

	local vo;
	local isShowTask = false;
	for k, v in ipairs(msg.task_list) do
		vo = DataCenter.taskUtil:getExploreTourTaskVo(v.key);
		if vo ~= nil then
			local progress = tonumber(v.value);
			if vo.task_id == 58 and progress ~= 0 then
				vo.taskProgress = Lua_MainLineCheckpointConfigData.getConfig(progress).number;
			else
				vo.taskProgress = tonumber(progress);--任务进度
			end

			if vo.taskProgress >= vo.process then
				isShowTask = true;
				if vo.rewardState == 0 then
					vo.rewardState = 1;
				end
				self.showTaskArr:add(vo);

				-- self:updateTotalTaskData(Lua_TaskData.AdventureJourney_Task, vo.id, vo.conditionId, vo.rewardState);
			end
		end
	end

	Lua_MsgUtil.callFunc("UPDATE_EXPLORER_TOUR_INFO", self.curStageId);
	if isShowTask == false then	
		return;
	end

	self.taskRedState = true;
	self:checOpenState();

	if not Lua_WindowUtil.isWindowShowing("Lua_TaskRewardTipWindow") and not Lua_WindowUtil.isWindowShowing("Lua_ExploreTourPopupWindow") then
		Lua_WindowUtil.openWin("Lua_ExploreTourPopupWindow", Lua_WinParam.ACTION_OPEN);
	end
end

-- 更新阶段任务信息
function Lua_TaskData:updateTaskInfo(msg)
	local vo;
	self.taskRedState = false;
	for k, v in ipairs(msg) do
		vo = DataCenter.taskUtil:getExploreTourTaskVo(v.key);
		if vo ~= nil then
			vo:fromServer(v);
			if vo.rewardState == 1 then
				self.taskRedState = true;
			end

			-- self:updateTotalTaskData(Lua_TaskData.AdventureJourney_Task, vo.id, vo.conditionId, vo.rewardState);
		end
	end
end

-- 获取任务弹窗第一个数据
function Lua_TaskData.Get:getPopupTaskVo()
	if self.showTaskArr:getSize() > 0 then
		return self.showTaskArr:shift();
	end
	return nil;
end

-- 获取各阶段的进度值
function Lua_TaskData:getStagePrograss(stage)
	if self.curStageId == nil or self.curStageId < stage then
		return 0, 1;
	elseif self.curStageId > stage then
		return 1, 1;
	else
		local curPrograss = 0;
		local totalPrograss = 0;
		local arr = DataCenter.taskUtil:getExploreTourTaskArr(stage);
		for i = 1, arr:getSize() do
			curPrograss = curPrograss + arr[i].taskProgress;
			totalPrograss = totalPrograss + arr[i].process;
		end
		return curPrograss, totalPrograss;
	end
end

-- 检查主界面显示
function Lua_TaskData:checOpenState()
	local state = DataCenter.funcOpenData:isOpened(10028) and self.isExploreFinish == false;
	if state ~= self.openState then
		self.openState = state;
		Lua_MsgUtil.callFunc("UPDATE_TASK_STATE");
	end

	local artifactReward = self.curRewardState == 2 and self.artifactId ~= nil and self.artifactState == 0;
	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.ExploreTour, self.taskRedState or self.curRewardState == 1 or artifactReward);
end

-- 是否完成所有任务且领完奖励
function Lua_TaskData.Get:isExploreFinish()
	if self.curStageId >= DataCenter.taskUtil:getConstConfig("max_stage", 1) and self.curRewardState == 2 then
		return self.artifactId == nil or self.artifactState == 1;
	end
	return false;
end


---------------------------------------- 外部接口 ----------------------------------------

-- 根据任务id获取冒险之旅任务状态 0：不可领 1：可领取 2：已领取
function Lua_TaskData:getExploreTaskState(taskId)
	local vo = DataCenter.taskUtil:getExploreTourTaskVo(taskId);
	if vo ~= nil then
		if vo.stage > self.curStageId then
			return 0;
		elseif vo.stage < self.curStageId then
			return 2;
		else
			return vo.rewardState;
		end
	end
	return 0;
end

-- 获取冒险之旅当前阶段状态 0：阶段奖励不可领 1：阶段奖励可领取 2：可解锁神器 3：阶段已过 4：阶段未到
function Lua_TaskData:getExploreStageState(stageId)
	if stageId < self.curStageId then
		return 3;
	elseif stageId > self.curStageId then
		return 4;
	else
		if self.curRewardState < 2 then
			return self.curRewardState;
		else
			if self.artifactId ~= nil and self.artifactState == 0 then
				return 2;
			end
		end
		return 0;
	end
end













---------暂时存放引导id数据,测试----------------------
Lua_TaskData.UN_ENABLE_RECEIVE = 0; -- 不可领取
Lua_TaskData.ENABLE_RECEIVE_REWARD = 1; -- 可领取
Lua_TaskData.COMPLETE_TASK = 2; -- 已完成

function Lua_TaskData:resetGuideIdData()
	if self.taskIdDir ~= nil then
		self.taskIdDir = nil;
	end
end

function Lua_TaskData:updateTotalTaskData(taskType, curId, conditionId, state)
	-- print("============任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state, "===========");
	if self.taskIdDir == nil then
		self.taskIdDir = {};
	end

	if curId == nil and conditionId == nil then
		return;
	end

	-- 登录或初始化
	if self.taskIdDir[taskType] == nil then
		self.taskIdDir[taskType] = {};
	end

	if self.taskIdDir[taskType][curId] == nil then
		if state == Lua_TaskData.ENABLE_RECEIVE_REWARD then
			-- print("初始化", "任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state)
		elseif state == Lua_TaskData.COMPLETE_TASK then
			-- print("初始化", "任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state)
		end
			
		self.taskIdDir[taskType][curId] = {conditionId = conditionId, state = state};
	-- 任务状态变更
	else
		if self:getBCompleteTask(taskType, curId, conditionId, state) then
			-- print("任务变更，完成", "任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state)
		end

		if self:getEnableReceive(taskType, curId, conditionId, state) then
			-- print("任务变更，可领取", "任务类型", taskType, "配置表id", curId, "总配置表id", conditionId, "任务状态", state)
		end
		self.taskIdDir[taskType][curId].state = state;
		self.taskIdDir[taskType][curId].conditionId = conditionId;
	end
end

-- 任务状态是否转变为<已完成>
function Lua_TaskData:getBCompleteTask(taskType, curId, conditionId, state)
	local info = self.taskIdDir[taskType][curId];
	if info == nil then
		return;
	end
	local enableComplete = info.conditionId ~= conditionId;
	if not enableComplete then
		enableComplete = info.state == Lua_TaskData.ENABLE_RECEIVE_REWARD and state == Lua_TaskData.COMPLETE_TASK;
	end
	return enableComplete;
end

-- 任务状态是否转变为<可领取>
function Lua_TaskData:getEnableReceive(taskType, curId, conditionId, state)
	local info = self.taskIdDir[taskType][curId];
	if info == nil then
		return;
	end
	local enableReceive = info.conditionId ~= conditionId and state == Lua_TaskData.ENABLE_RECEIVE_REWARD;
	if not enableReceive then
		enableReceive = info.state == Lua_TaskData.UN_ENABLE_RECEIVE and state == Lua_TaskData.ENABLE_RECEIVE_REWARD;
	end
	return enableReceive;
end