checkRequire("Lua_TaskActInfoVo");
checkRequire("Lua_TaskInfoVo");
checkRequire("Lua_ExploreTourTaskVo");
checkRequire("Lua_ActiveWarVo");
Lua_TaskUtil = class("Lua_TaskUtil");
function Lua_TaskUtil:ctor()
end

function Lua_TaskUtil:dispose()
	if self.activePointRewards ~= nil then -- 活跃度各阶段可领取的奖励
		for k, v in pairs(self.activePointRewards) do
			v:dispose();
		end
		self.activePointRewards = nil;
	end
	if self.taskInfoList ~= nil then -- 任务的具体信息（日常、周常、成就）
		for k, v in pairs(self.taskInfoList) do
			v:dispose();
		end
		self.taskInfoList = nil;
	end

	self.activePointIndexes = nil; -- 日常、周常各自奖励的id列表（活跃度抵达某阶段可领取的奖励）
	self._targetActivePoint = nil; -- 各类型活跃度各阶段已排序的目标数值List
	self.taskInfoIndexesByType = nil; -- 日常、周常、成就各自的任务列表
	self.taskListCount = nil; -- 日常、周常、成就任务的任务数目
	self.roundId = nil; -- 活跃战令轮次id
end

------------------------- 活跃度抵达某阶段后的奖励item（日常、周常）-------------------------
-- 初始化日常、周常活跃度各阶段可获取的奖励（读取配置表）
function Lua_TaskUtil:initActivePointRewards()
	if self.activePointRewards == nil then
		self.activePointRewards = {};

		local vo;
		local config = getConfig("task_progress");
		for k, v in pairs(config) do
			vo = Lua_TaskActInfoVo.new(v.id, v);
			self.activePointRewards[v.id] = vo;
			self:setActivePointIndexes(vo.mainType, vo.id);
		end

		-- 排序
		local count = #self.activePointIndexes;
		for i = 1, count do
			table.sort(self.activePointIndexes[i],
				function(actId, nextId)
					return self:getActivePointRewardById(actId).active < self:getActivePointRewardById(nextId).active;
				end
			);
		end
	end
end

-- 根据id获取活跃度奖励的具体信息（日常、周常）
function Lua_TaskUtil:getActivePointRewardById(id)
	if self.activePointRewards == nil or id == nil or self.activePointRewards[id] == nil then
		return;
	end
	return self.activePointRewards[id];
end

-- 记录日常、周常各自活跃度奖励的id列表
function Lua_TaskUtil:setActivePointIndexes(taskType, id)
	if self.activePointIndexes == nil then
		self.activePointIndexes = {};
	end
	if self.activePointIndexes[taskType] == nil then
		self.activePointIndexes[taskType] = {};
	end 

	local count = #self.activePointIndexes[taskType];
	self.activePointIndexes[taskType][count + 1] = id;
end

-- 根据类型（日常、周常）获取各自的活跃度id列表
function Lua_TaskUtil:getActivePointIndexesByType(taskType)
	if self.activePointIndexes == nil then
		self:initActivePointRewards();
	end
	return self.activePointIndexes[taskType];
end

-- 根据类型（日常、周常）获取各自的活跃度列表
function Lua_TaskUtil:getTargetActivePointByType(taskType)
	if self._targetActivePoint == nil then
		self._targetActivePoint = {};
		self:initActivePointRewards();

		local vo;
		for _type, idList in pairs(self.activePointIndexes) do
			if self._targetActivePoint[_type] == nil then
				self._targetActivePoint[_type] = {};
			end
			for k, v in ipairs(idList) do
				vo = self:getActivePointRewardById(v);
				self._targetActivePoint[_type][k] = vo.active;
			end
		end
	end
	return self._targetActivePoint[taskType];
end

-- 根据类型（日常、周常）、活跃度，计算出符合显示要求的当前进度条数值
function Lua_TaskUtil:getCurProcessValue(taskType, activePoint)
	local targetActivePoint = self:getTargetActivePointByType(taskType);
	local count = #targetActivePoint;
	local deltaValue = targetActivePoint[count] / count;
	local targetValue = 0;
	local preStageValue = 0;
	local curValue = 0;
	local subValue; -- 活跃度两个阶段之间的差值
	for i = 1, count do
		targetValue = targetActivePoint[i];
		preStageValue = i == 1 and 0 or targetActivePoint[i - 1];
		subValue = i == 1 and targetValue or targetValue - preStageValue;
		if activePoint >= targetValue then
			curValue = curValue + deltaValue / subValue * subValue;
		else
			curValue = curValue + deltaValue / subValue * (activePoint - preStageValue);
			break;
		end
	end
	return curValue;
end










------------------------- 日常、周常、成就任务单 -------------------------
function Lua_TaskUtil:initTaskInfoList()
	if self.taskInfoList == nil then
		self.taskInfoList = {};

		local vo;
		local config = getConfig("task");
		for k, v in pairs(config) do
			vo = Lua_TaskInfoVo.new(v.id, v);
			self.taskInfoList[v.id] = vo;
			self:setTaskInfoIndexes(vo.mainType, vo.id);
		end

		-- 记录各自列表中的任务数目
		for k, v in pairs(self.taskInfoIndexesByType) do
			self:setTaskListCountByType(k, #v);
		end
	end
end

-- 根据id获取任务的具体信息（日常、周常、成就）
function Lua_TaskUtil:getTaskInfoById(id)
	if self.taskInfoList == nil or id == nil or self.taskInfoList[id] == nil then
		return;
	end
	return self.taskInfoList[id];
end

-- 记录日常、周常、成就各自的任务列表
function Lua_TaskUtil:setTaskInfoIndexes(type, id)
	if self.taskInfoIndexesByType == nil then
		self.taskInfoIndexesByType = {};
	end
	if self.taskInfoIndexesByType[type] == nil then
		self.taskInfoIndexesByType[type] = {};
	end 

	local count = #self.taskInfoIndexesByType[type];
	self.taskInfoIndexesByType[type][count + 1] = id;
end

-- 获取全部任务单
function Lua_TaskUtil:getAllTaskInfoIndexes()
	return self.taskInfoIndexesByType;
end

-- 根据类型（日常、周常、成就）获取各自的任务列表
function Lua_TaskUtil:getTaskInfoIndexesByType(type)
	if self.taskInfoIndexesByType == nil then
		self:initTaskInfoList();
	end

	local list = self.taskInfoIndexesByType[type];
	table.sort(list,
		function(a, b)
			return self:getTaskInfoById(a).sortWeight > self:getTaskInfoById(b).sortWeight;
		end
	);
	return self.taskInfoIndexesByType[type];
end

-- 计算日常、周常、成就任务的任务数目
function Lua_TaskUtil:setTaskListCountByType(type, count)
	if self.taskListCount == nil then
		self.taskListCount = {};
	end
	self.taskListCount[type] = count;
end

-- 获取日常、周常、成就任务的任务数目
function Lua_TaskUtil:getTaskListCountByType(type)
	return self.taskListCount[type];
end

-- 根据类型(日常、周常、成就任务)获取该任务界面是否有奖励未领取
function Lua_TaskUtil:hasRewardByType(type)
	local hasReward = false;
	
	-- 活跃度奖励
	if type ~= Lua_TaskData.achievementTaskTask then
		hasReward = self:hasActiveReward(type);
		if hasReward then
			return hasReward;
		end
	end

	-- 任务列表奖励
	local vo;
	local list = self:getTaskInfoIndexesByType(type);
	for i = 1, #list do
		vo = self:getTaskInfoById(list[i]);
		if vo:getReceiveStage() == 1 then
			hasReward = true;
			break;
		end
	end
	return hasReward;
end

-- 根据类型(日常、周常、成就任务)获取界面是否有活跃度奖励可领取
function Lua_TaskUtil:hasActiveReward(type)
	local vo;
	local hasReward = false;
	local activeIdList = self:getActivePointIndexesByType(type); -- 配置表内的活跃度奖励列表
	for i = 1, #activeIdList do
		vo = self:getActivePointRewardById(activeIdList[i]);
		if vo:getRewardState() == 1 then
			hasReward = true;
			break;
		end
	end
	return hasReward;
end










------------------------- 常量配置 -------------------------
function Lua_TaskUtil:getConstConfig(key, type)
	local config = getConfig("task_const", key);
	if config ~= nil then
		if type == 1 then
			return config.value;
		elseif type == 2 then
			return config.arr_value;
		end
	end
	return nil;
end










------------------------- 日常任务-活跃战令 -------------------------
-- 读取active_war_order配置表
function Lua_TaskUtil:initAcitveWarInfo(roundId)
	if roundId == nil then
		return;
	end

	if self.roundId ~= roundId then
		self.roundId = roundId;

		local vo;
		local count;
		local config = getConfig({"active_war_order"});
		self._acitveWarDic = {};
		self._acitveWarList = {};
		for k, v in pairs(config) do
			vo = Lua_ActiveWarVo.new(k, v);
			if type(vo.id) ~= "number" then
				self._constActiveWar = vo;
			elseif vo.round == roundId then		
				count = #self._acitveWarList;
				self._acitveWarList[count + 1] = vo.id;
				self._acitveWarDic[vo.id] = vo;
			end
		end
	end
end

-- 获取记录各战令的id
function Lua_TaskUtil:getAcitveWarList()
	table.sort(self._acitveWarList,
		function(a, b)
			return a < b;
		end
	);
	return self._acitveWarList;
end

-- 获取各战令的信息
function Lua_TaskUtil:getAcitveWarDic()
	return self._acitveWarDic;
end

-- 获取高级战令的充值id
function Lua_TaskUtil.Get:rechargeId()
	if self._constActiveWar == nil then
		return;
	end
	return self._constActiveWar.rechargeId;
end

-- 获取初级战令的解锁道具
function Lua_TaskUtil.Get:lockRewardTVo()
	if self._constActiveWar == nil then
		return;
	end
	return self._constActiveWar.lockRewardTVo;
end










------------------------- 冒险之旅 -------------------------
-- 初始化冒险之旅任务信息
function Lua_TaskUtil:initExploreTourTaskInfo()
	local vo;
	local config = getConfig("mainline_task");
	self._exploreTourTaskDic = {};
	self._exploreTourTaskList = {};
	for k, v in pairs(config) do
		vo = Lua_ExploreTourTaskVo.new(v.id, v);
		self._exploreTourTaskDic[v.id] = vo;

		if self._exploreTourTaskList[v.stage] == nil then
			self._exploreTourTaskList[v.stage] = Array();
		end
		self._exploreTourTaskList[v.stage]:add(vo);
	end
end

-- 获取冒险之旅任务Vo
function Lua_TaskUtil:getExploreTourTaskVo(id)
	if self._exploreTourTaskDic == nil then
		self:initExploreTourTaskInfo();
	end
	return self._exploreTourTaskDic[id];
end

-- 获取冒险之旅阶段任务列表
function Lua_TaskUtil:getExploreTourTaskArr(stage)
	if self._exploreTourTaskList == nil then
		self:initExploreTourTaskInfo();
	end
	return self._exploreTourTaskList[stage];
end

-- 获取冒险之旅阶段配置
function Lua_TaskUtil.Get:exploreTourStageConfig()
	if self._exploreTourStageConfig == nil then
		self._exploreTourStageConfig = getConfig("mainline_task_stage_reward");
	end
	return self._exploreTourStageConfig;
end

-- 获取冒险之旅显示阶段配置
function Lua_TaskUtil.Get:exploreTourStageArr()
	local arr = Array();
	local maxStage = self:getConstConfig("max_stage", 1);
	for k, v in pairs(self.exploreTourStageConfig) do
		if k <= maxStage then
			arr:add(v);
		end
	end
	return arr;
end

-- 获取冒险之旅阶段信息字段
function Lua_TaskUtil:getExploreTourStageInfo(stage, key)
	local config = self.exploreTourStageConfig[stage];
	if config ~= nil then
		return config[key];
	end
	return nil;
end