-- 图腾（圣物）
Lua_TotemVo = class("Lua_TotemVo")
function Lua_TotemVo:ctor(id, config)
    self.id = id;
    self.config = config;
    if self.config == nil then
        self.config = getConfig( { "totem", id });
    end
    self:initialize();
end

function Lua_TotemVo:initialize()
    self.level = 0;
    self.order = 0;
    self.cur_progress = 0;
    self.level_prop = nil;
    self.total_prop = nil;
    self.is_activate = 0;
    self._orderHeroVo = nil;
end

function Lua_TotemVo:fromServerInfo(msg)
    self.level = msg.level;-- 等级
    self.order = msg.order;-- 阶级
    self.cur_progress = msg.progress;-- 当前等级进度
    self.level_prop = msg.level_prop;-- 注灵属性
    self.total_prop = msg.total_prop;-- 总属性
    self.is_activate = msg.is_activate;-- 激活状态 0: 未激活 1: 激活
    self.prev_level_prop = msg.prev_level_prop;-- 上等级注灵属性
    self._orderHeroVo = nil;
end



---------------------------------- 注灵数据 -------------------------------------------

-- 注灵配置
function Lua_TotemVo.Get:levelCofing()
    --图腾系统阵营 * 1000 + 等级
    local lvl = self.level;
    if self.is_activate == 0 then
        lvl = 1;
    end
    local id = self.id * 1000 + lvl;
    return DataCenter.totemUtil:getTotemInfoByLevel(id);
end

-- 当前进度最大值
function Lua_TotemVo.Get:total_progress()
    return self.levelCofing.total_progress;
end

-- 注灵消耗
function Lua_TotemVo.Get:level_up_cost()
    return self.levelCofing.level_up_cost;
end

--服务端下发属性数据
function Lua_TotemVo.Get:curPropInfo()
    return DataCenter.attributesUtil:getAttrArrByServer(self.level_prop);
end

--服务端下发总属性数据
function Lua_TotemVo.Get:allPropInfo()
    return DataCenter.attributesUtil:getAttrArrByServer(self.total_prop);
end

-- 未激活属性
function Lua_TotemVo.Get:lock_prop()
    if self._lockPropInfo == nil then
        self._lockPropInfo = {};
        local id = self.id * 1000 + 1;
        local extra_attr = self.levelCofing.prop_list;
        for i=1,#extra_attr do
            local value = getConfig({ "totem_level",id, extra_attr[i]});
            -- print(extra_attr[i],"  未激活属性  = =",value)
            self._lockPropInfo[extra_attr[i]] = value; 
        end
    end
    return DataCenter.attributesUtil:getAttrArrByConfig(self._lockPropInfo);
end

-- 注灵成功属性
function Lua_TotemVo.Get:showUpgradePropInfo()
    local list
    if #self.prev_level_prop > #self.level_prop then
        list = self.prev_level_prop;
    else
        list = self.level_prop;
    end

    local arr = Array();
    for i=1,#list do
        local vo = DataCenter.attributesUtil:createAttrVo(list[i].name);
        local showValue = 0;
        if self.prev_level_prop[i] ~= nil then
            showValue = self.prev_level_prop[i].value;
        end

        local showNextValue = 0;
        if self.level_prop[i] ~= nil then
            showNextValue = self.level_prop[i].value;
        end

        arr:add({name = vo.show_name, showValue = showValue, showNextValue = showNextValue,sort = vo.sort});
    end
    arr:sort("sort",false);
    return arr;
end

-- 解锁消耗英雄
function Lua_TotemVo.Get:unlockHeroVo()
    if self._unlockHeroVo == nil then
        self._unlockHeroVo = Lua_HeroVo.new(self.activate_cost[1][4]);
        self._unlockHeroVo.star = self.activate_cost[1][1];
        self._unlockHeroVo.campType = self.activate_cost[1][2];
        self._unlockHeroVo.need_num = self.activate_cost[1][3];
    end
    return self._unlockHeroVo;
end

-- 是否最大注灵等级
function Lua_TotemVo.Get:isMaxLevel()
    if self.level >= self.level_limit then
        return true;
    end
    return false;
end

-- 是否可以注灵
function Lua_TotemVo.Get:isUpgrade()
    local list = self.level_up_cost;
    for i=1,#list do
    	local have_num = DataCenter.packData:getItemCountById(list[i][1]); 
		local need_num = list[i][2];
		if have_num < need_num then
			return list[i][1];
		end
    end

    return true;
end


-- 解锁材料是否足够
function Lua_TotemVo.Get:isUnlockCost()
    

    return false;
end


-- 解锁红点
function Lua_TotemVo.Get:totemUnlockRedDot()
    return false;
end


-- 注灵红点
function Lua_TotemVo.Get:totemUpgradeRedDot()
    return false;
end






---------------------------------- 升阶数据 -------------------------------------------

-- 升阶配置id
function Lua_TotemVo.Get:order_id()
    --图腾系统阵营 * 1000 + 阶级
    return self.id * 1000 + self.order;
end

-- 下阶id
function Lua_TotemVo.Get:next_id()
    if self.isMaxOrder == true then
        return self.order_id;
    end
    return self.totemOredrCofing.next_id;
end

-- 升阶配置
function Lua_TotemVo.Get:totemOredrCofing()
    return Lua_TotemOrderCofingData.getConfigData(self.order_id);
end

-- 下阶级配置
function Lua_TotemVo.Get:nextTotemOredrCofing()
    return Lua_TotemOrderCofingData.getConfigData(self.next_id);
end

-- 升阶条件
function Lua_TotemVo.Get:level_cond()
    return self.totemOredrCofing.level_cond;
end

-- 升阶英雄消耗
function Lua_TotemVo.Get:oderHeroCostInfo()
    return self.totemOredrCofing.order_up_hero_cost;
end

-- 升阶道具消耗
function Lua_TotemVo.Get:orderCostInfo()
    return self.totemOredrCofing.order_up_item_cost;
end

-- 技能图标
function Lua_TotemVo.Get:skill_sprite()
    return self.totemOredrCofing.skill_sprite;
end

-- 技能名字
function Lua_TotemVo.Get:skill_name()
    return self.totemOredrCofing.skill_name;
end

-- 升阶消耗英雄
function Lua_TotemVo.Get:orderHeroVo()
    if self._orderHeroVo == nil then
        self._orderHeroVo = Lua_HeroVo.new(self.oderHeroCostInfo[1][4]);
        self._orderHeroVo.star = self.oderHeroCostInfo[1][1];
        self._orderHeroVo.campType = self.oderHeroCostInfo[1][2];
        self._orderHeroVo.need_num = self.oderHeroCostInfo[1][3];
    end
    return self._orderHeroVo;
end

-- 技能描述
function Lua_TotemVo.Get:skillDesInfo()
    local arr = Array();
    local list = self.totemOredrCofing.prop_list;
    if list == nil then
        return Array();
    end
    for i=1,#list do
        local str = list[i];
        local num = 126650;
        if str ~= "speed" then
            num = 126649;
            str = str.."_extra";
        end

        local value = getConfig({ "totem_order",self.order_id, str});
        if value ~= nil and value > 0 then
            local propVo = DataCenter.attributesUtil:createAttrVo(list[i]);
            if list[i] ~= "speed" then
                value = (value * 100).."%";
            end

            local str = self.name..LanUtil.formateStr(num, propVo.show_name,tostring(value));
            arr:add(str);
        end
    end
    return arr;
end


-- 下级技能描述
function Lua_TotemVo.Get:nextSkillDesInfo()
    local arr = Array();
    local list = self.nextTotemOredrCofing.prop_list;
    if list == nil then
        return Array();
    end
    for i=1,#list do
        local str = list[i];
        local num = 126650;
        if str ~= "speed" then
            num = 126649;
            str = str.."_extra";
        end

        local value = getConfig({ "totem_order",self.next_id, str});
        if value ~= nil and value > 0 then
            local propVo = DataCenter.attributesUtil:createAttrVo(list[i]);
            if list[i] ~= "speed" then
                value = (value * 100).."%";
            end

            local str = self.name..LanUtil.formateStr(num, propVo.show_name,tostring(value));
            arr:add(str);
        end
    end
    return arr;
end

-- 是否最大阶级
function Lua_TotemVo.Get:isMaxOrder()
    if self.order >= self.order_limit then
        return true;
    end
    return false;
end

-- 升阶 等级限制是否满足
function Lua_TotemVo.Get:isLevelCond()
    if self.isMaxLevel == true then
        return true;
    end
	return self.level >= self.level_cond;
end

-- 升阶 消耗英雄是否足够
function Lua_TotemVo.Get:isCostHero()

end

-- 升阶 消耗材料是否足够
function Lua_TotemVo.Get:isCostOredr()
    local list = self.orderCostInfo;
    for i=1,#list do
    	local have_num = DataCenter.packData:getItemCountById(list[i][1]); 
		local need_num = list[i][2];
		if have_num < need_num then
			return list[i][1];
		end
    end

    return true;
end










------------------------------------ 红点 ---------------------------------------------
-- 红点检测
function Lua_TotemVo.Get:checkTotemRedDot()
    return false;
end

-- 图腾升阶
function Lua_TotemVo.Get:checkTotemOrderRed()
    return false;
end















---------------------------------- 配置数据 -------------------------------------------

-- 名字
function Lua_TotemVo.Get:name()
    return self.config.name;
end

-- 系别加成UI
function Lua_TotemVo.Get:camp_spr()
    return self.config.camp_spr;
end

-- 图腾images
function Lua_TotemVo.Get:images()
    return self.config.images;
end

-- 特效
function Lua_TotemVo.Get:effect_name()
    return self.config.effect_name;
end

-- 激活消耗
function Lua_TotemVo.Get:activate_cost()
    return self.config.activate_cost;
end

-- 等级上限
function Lua_TotemVo.Get:level_limit()
    return self.config.level_limit;
end

-- 阶级上限
function Lua_TotemVo.Get:order_limit()
    return self.config.order_limit;
end

-- 描述
function Lua_TotemVo.Get:des()
    return self.config.des;
end