Lua_TrialTypeInfoVo = class("Lua_TrialTypeInfoVo")
function Lua_TrialTypeInfoVo:ctor(id, config)
	self.id = id;
	self._config = config;
	self:dispose();
end

function Lua_TrialTypeInfoVo:dispose()
	self.script_arr = Array(); -- 副本信息
	self.type = 0; -- 副本类型
	self.state = 0; -- 开启状态：0未开启 1已开启
	self.free_times = 0; -- 今日免费次数
	self.challenge_times = 0; -- 今日剩余购买挑战/扫荡次数
end

function Lua_TrialTypeInfoVo:updateTrialData(info)
	self.type = info.type;
	self.state = info.state;
	self.free_times = info.free_times;
	self.challenge_times = info.challenge_times;
	self.buy_times = info.buy_times; -- 今天已购买次数

	self.script_arr:clear();
	local vo;
	local isUnlock = false; -- 存在可解锁关卡
	for i = 1, #info.script_info do
		vo = DataCenter.TrialData:getTrialEachById(info.script_info[i].index);
		vo:updateEachInfo(info.script_info[i]);
		self.script_arr:add(vo);

		if vo.state == 1 then
			isUnlock = true;
		end
	end
	self.script_arr:sort("id", false)

	DataCenter.redPointData:updateRedPointState(self.redPointId, self.free_times > 0 or isUnlock);
end





function Lua_TrialTypeInfoVo.Get:config()
	if self._config == nil then
		self._config = getConfig("trial_type", self.id);
	end
	return self._config;
end

-- 副本名字
function Lua_TrialTypeInfoVo.Get:name()
	return self.config.name;
end

-- 功能开启id
function Lua_TrialTypeInfoVo.Get:open_id()
	return self.config.open_id;
end

-- 功能开启状态
function Lua_TrialTypeInfoVo.Get:open_state()
	return DataCenter.funcOpenData:isOpened(self.open_id);
end

-- 开启等级
function Lua_TrialTypeInfoVo.Get:open_level()
	local vo = DataCenter.funcOpenUtil:getFuncOpenVoById(self.open_id);
	if vo == nil or vo.role_level == nil or vo.role_level <= 0 then
		return 1;
	else
		return vo.role_level;
	end
end

-- 主线关卡id
function Lua_TrialTypeInfoVo.Get:game_level_id() 
	local vo = DataCenter.funcOpenUtil:getFuncOpenVoById(self.open_id);
	if vo ~= nil then
		return vo.game_level_id;
	end
	return nil;
end

-- 扫荡消耗道具id
function Lua_TrialTypeInfoVo.Get:sweep_item()
	return self.config.sweep_item;
end

-- 扫荡消耗道具数量
function Lua_TrialTypeInfoVo.Get:sweep_cost()
	local list = self.config.sweep_cost;
	local sweep_cost_times = self.buy_times + 1;
	if sweep_cost_times > #list then
		return list[#list]
	else
		return list[sweep_cost_times];
	end
end

-- 红点id
function Lua_TrialTypeInfoVo.Get:redPointId()
	return self.config.redPointId;
end

-- 战斗模板ID
function Lua_TrialTypeInfoVo.Get:fight_id()
	if self.config.fight_id == nil then
		return Lua_FightDefine.TEMPLATE_ID_TRIAL;
	end
	return self.config.fight_id;
end

-- 按钮图片
function Lua_TrialTypeInfoVo.Get:trialIcon()
	return self.config.trialIcon;
end

-- 最大购买次数的vip等级
function Lua_TrialTypeInfoVo.Get:buy_max_vip()
	return self.config.buy_max_vip;
end