checkRequire("Lua_HeroConfigData");
checkRequire("Lua_MonsterConfigData");
checkRequire("Lua_HeroHeadInfoPanel");
checkRequire("Lua_FightPlayerHeadInfoPanel");
checkRequire("Lua_ThingDefine");
Lua_ThingPlayer = class("Lua_ThingPlayer");
function Lua_ThingPlayer:setPlayer(player)
    self.isActive = true;
    self.bCompleted = false;
    self.player = player;
end

function Lua_ThingPlayer.Get:player()
    return self._player;
end

function Lua_ThingPlayer.Set:player(value)
    if value == nil then
        print("Lua_ThingPlayer player == nil");
    end
    self._player = value;
end

function Lua_ThingPlayer:initData()
    if self.player.IsUI == true then
        print("self.player.IsUI == true", self.player.data.roleId);
        return;
    end

    self.maxHp = 1;
    self.hp = 0;
    self.level = 1;
    self.camp = 0;

    self.data = self.player.data;
    self.sceneId = self.data.sceneId; -- 场景
    self.formationType = self.data.formationType; -- 攻守方
    self.formationPos = self.data.formationPos; -- 阵型位置
    self.playerId = tonumber(tostring(self.data.roleId)); -- 玩家唯一id
    self.objId = tonumber(tostring(self.data.objId)); -- objId
    self.fightRecordId = tonumber(tostring(self.data.fightRecordId)); -- 战斗id

    self.kind = self.data.kind;
    if self.kind ~= Lua_ThingDefine.SCENE_EFFECT and self.kind ~= Lua_ThingDefine.HEXAGONMAP_GRID then
        self.unitType = self.data.unitType;
        self.level = self.data.lvl;
        self.maxHp = tonumber(tostring(self.data.maxHp));
        self.hp = tonumber(tostring(self.data.hp));
    end

    if self.kind == Lua_ThingDefine.MONSTER then
        self.configData = Lua_MonsterConfigData.getMonsterConfig(self.objId);
    elseif self.kind == Lua_ThingDefine.HERO then
        self.configData = Lua_HeroConfigData.getConfig(self.objId);
    end
    if self.configData then
        self.camp = self.configData.camp;
    end
    DataCenter.thingData:addThingPlayer(self, self.playerId, self.objId, self.sceneId, self.fightRecordId, self.formationType, self.formationPos);
end

function Lua_ThingPlayer:getLuaData()
    return DataCenter.thingData:getThingData(self.playerId);
end

function Lua_ThingPlayer:loadBodyComplete()
    self.bCompleted = true;
    if self.player.viewState == true and self.data.isShowName == true then
        -- if self.fightRecordId > 0 then -- 战斗UI
        --     self:updateThingHeadPos();
        --     Lua_CommonUtil.addTimer("THINGPLAYER_UPDATE_HEADINFO", 0.02, self, self.updateThingHeadPos);
        -- else
        --     self:getHeadShowContainer();
        --     self:showName();
        -- end
        self.player.bRefreshHeadSprPos = true;
        self:updateThingHeadPos();
        Lua_CommonUtil.addTimer("THINGPLAYER_UPDATE_HEADINFO", 0.02, self, self.updateThingHeadPos);
    end
    Lua_MsgUtil.callFunc("thingPlayerLoadComplete", {sceneId = self.sceneId, playerId = self.playerId});
end

-- 更新头顶挂件偏移数据
function Lua_ThingPlayer.Get:headContainerPosY()
    if self._headContainerPosY == nil then
        self._headContainerPosY = 230;
        if self.configData ~= nil then
            self._headContainerPosY = self.configData.nameHeight;
            self._scaleValue = self.configData.nameScale;
        end
    end
    return self._headContainerPosY;
end

-- 更新头顶挂件偏移数据
function Lua_ThingPlayer.Get:scaleValue()
    if self._scaleValue == nil then
        self._scaleValue = 1;
        if self.configData ~= nil then
            self._headContainerPosY = self.configData.nameHeight;
            self._scaleValue = self.configData.nameScale;
        end
    end
    return self._scaleValue;
end

function Lua_ThingPlayer:getHeadShowContainer()
    if self.data == nil or self.data.isShowName == false or self.fightRecordId > 0 then 
        return;
    end

    if self.uiContainer == nil then
        self.uiContainer = UICreate.createGameObject(self.player.root);
        self.uiContainer.name = "headContainer";
        self.uiContainer:AddComponent(typeof(UnityEngine.Canvas));
        local rtf = self.uiContainer:GetComponent(typeof(UnityEngine.RectTransform));
        LuaUtil.setRectSize(rtf, 0, 0);

        self.player.bRefreshHeadSprPos = true;
        self.headContainer = UICreate.createGameObject(self.uiContainer);
        self.headContainer.name = "name";
        LuaUtil.setObjPos(self.headContainer, 0, self.headContainerPosY, 0);

        local scale = self.scaleValue * 0.01;
        LuaUtil.setObjScale(self.uiContainer, scale, scale, scale);
    end

    self.player:faceToCam();
end

function Lua_ThingPlayer:showName()
    if self.data == nil or self.data.isShowName == false or self.headContainer == nil or self.fightRecordId > 0 then
        return;
    end

    if self.kind == Lua_ThingDefine.PLAYER then
        local name = self.data.nickname;
        if strIsNullOrEmpty(name) then
            return;
        end

        if self.nameText == nil then
            self.nameText = UICreate.createLabel(self.headContainer, nil, 0, 0, 100, 31, 22);
            self.nameText.pivotValue = 4;
            self.nameText.alignmentValue = 4;
            self.nameText.horizontalOverflowValue = 1;
            self.nameText.effectStyle = 2;
            self.nameText.effectColorStr = Lua_ColorDefine.BLACK;
            self.nameText:setEffectDistance(1, 1);
        end
        self.nameText.text = name;
    end
end

--------------------------------------------------
--------------------------------------------------
--------------------------------------------------
-- 战斗UI
function Lua_ThingPlayer:updateThingHeadPos()
    if self.player == nil or self.player.IsUI == true then
        return;
    end

    local isView = self.player.viewState;
    if isView == true then -- 视野范围内
        if self.uiContainer == nil then
            self.uiContainer = UICreate.createGameObject(GlobalVar.uiHeadContainer);
            self.uiContainer.name = self.playerId.."_headContainer";
            -- self.uiContainer:AddComponent(typeof(UnityEngine.Canvas));
            LuaUtil.setObjScale(self.uiContainer, self.scaleValue, self.scaleValue, self.scaleValue);
        end

        if self.headContainer == nil then
            self.player.bRefreshHeadSprPos = true;
            self.headContainer = UICreate.createGameObject(self.uiContainer);
            self.headContainer.name = "headSpr";
            LuaUtil.setObjRot(self.headContainer, 0, 0, 0);
            LuaUtil.setObjPos(self.headContainer, 0, 0, 0);
            self:showFightHeadInfo();
        end 

        if self.fightFlyContainer == nil then
            self.fightFlyContainer = UICreate.createGameObject(self.uiContainer);
            self.fightFlyContainer.name = "flyNumSpr";
            LuaUtil.setObjRot(self.fightFlyContainer, 0, 0, 0);
            LuaUtil.setObjPos(self.fightFlyContainer, 0, -self.headContainerPosY, 0);
        end

        self:refreshHeadPos();
        self:refreshActive(self.player.bActive);
    else
        if self.uiContainer ~= nil then
            self.uiContainer:SetActive(false);
        end
    end
end

function Lua_ThingPlayer:showFightHeadInfo()
    if self.headContainer == nil or self.data.isShowName == false then
        return;
    end

    if self.fightRecordId > 0 then  -- 战斗播放
        if self.headInfoPanel ~= nil then
            self.headInfoPanel:dispose();
            self.headInfoPanel = nil;
        end
        if self.data.isShowHpBar then
            if self.formationType == Lua_FightDefine.FIGHT_DEFENDER and self.configData and self.configData.hpBarType == 1 then
                Lua_MsgUtil.callFunc("FIGHT_BOSS_INFO_UPDATE", self.data);
            else
                if self.fightHeadInfoPanel == nil then
                    self.fightHeadInfoPanel = Lua_FightPlayerHeadInfoPanel.new(self.headContainer, self.formationType);
                end
                self.hp = tonumber(tostring(self.data.hp));
                self.fightHeadInfoPanel:updateCamp(self.camp);
                self.fightHeadInfoPanel:updateHealth(self.data.aniSpdRate, self.maxHp, self.hp);
                self.fightHeadInfoPanel:updateBuffIcon(self.data);
                self:updateHudun(self.maxHp, self.data.hudun);
            end
        end
    else
        if self.fightHeadInfoPanel ~= nil then 
            self.fightHeadInfoPanel:dispose();
            self.fightHeadInfoPanel = nil;
        end
        if self.headInfoPanel == nil then
            self.headInfoPanel = Lua_HeroHeadInfoPanel.new(self.headContainer);
        end
        self.headInfoPanel:setData(self.camp, self.level);
    end
end

function Lua_ThingPlayer:refreshHeadPos()
    if self.uiContainer ~= nil then
        if self.player.bRefreshHeadSprPos == true then
            if self.fightRecordId > 0 and self.bDelaySetHeadSprPos ~= true then -- 战斗UI需要延迟刷新
                self.bDelaySetHeadSprPos = true;
                Lua_CommonUtil.delayCall(0.5, self, self.delaySetHeadSprPos, nil);
            else
                self.rootPos = self:getTfWorldPos(self.player.root.transform, self.rootPos, true);
                self.rootPos.y = self.rootPos.y + self.headContainerPosY * self.scaleValue * 0.01;
                LuaUtil.setUIHeadObjPos(self.uiContainer, self.rootPos.x, self.rootPos.y, self.rootPos.z);
            end
        end
    end
end

function Lua_ThingPlayer:delaySetHeadSprPos()
    Lua_CommonUtil.removeTimer(self, self.delaySetHeadSprPos);
    self.player.bRefreshHeadSprPos = false;
    self.bDelaySetHeadSprPos = false;
end

function Lua_ThingPlayer:getTfWorldPos(tf, vec, reset)
    if tf == nil then
        return vec;
    end

    if vec == nil then
        vec = Vector3(0, 0, 0);
    elseif reset == true then
        vec:Set(0, 0, 0);
    end

    local pos = tf.localPosition;
    vec.x = vec.x + pos.x;
    vec.y = vec.y + pos.y;
    vec.z = vec.z + pos.z;
    return self:getTfWorldPos(tf.parent, vec, false);
end

function Lua_ThingPlayer:refreshActive(val)
    if self.isActive == val or not self.bCompleted then
        return;
    end
    self.isActive = val;
    -- if self.uiContainer ~= nil then
    --     self.uiContainer:SetActive(val);
    -- end

    if self.fightRecordId > 0 then
        if self.headContainer ~= nil then
            self.headContainer:SetActive(val);
        end
    else
        if self.uiContainer ~= nil then
            self.uiContainer:SetActive(val);
        end
    end

    if val == true then
        if self.formationType == Lua_FightDefine.FIGHT_DEFENDER and self.configData and self.configData.hpBarType == 1 then
            Lua_MsgUtil.callFunc("FIGHT_BOSS_INFO_UPDATE", self.data);
        end
    end
end

-- 更新buff图标
function Lua_ThingPlayer:updateBuffIcon()
    if self.data == nil then
        return;
    end
    if self.formationType == Lua_FightDefine.FIGHT_DEFENDER and self.configData and self.configData.hpBarType == 1 then
        Lua_MsgUtil.callFunc("FIGHT_BOSS_INFO_UPDATE", self.data);
    elseif self.fightHeadInfoPanel ~= nil then
        self.fightHeadInfoPanel:updateBuffIcon(self.data);
    end
end

-- 更新头顶护盾
function Lua_ThingPlayer:updateHudun(maxHudun, hudun)
    if self.fightHeadInfoPanel ~= nil then 
        maxHudun = tonumber(tostring(maxHudun));
        hudun = tonumber(tostring(hudun));
        self.fightHeadInfoPanel:updateShield(self.data.aniSpdRate, maxHudun, hudun);
    end
end

-- 更新头顶血条
function Lua_ThingPlayer:updateHpBar()
    if self.data == nil then
        return;
    end
    if self.formationType == Lua_FightDefine.FIGHT_DEFENDER and self.configData and self.configData.hpBarType == 1 then
        Lua_MsgUtil.callFunc("FIGHT_BOSS_INFO_UPDATE", self.data);
    elseif self.fightHeadInfoPanel ~= nil then
        self.hp = tonumber(tostring(self.data.hp));
        self.fightHeadInfoPanel:updateHealth(self.data.aniSpdRate, self.maxHp, self.hp);
    end
end

function Lua_ThingPlayer:clearRes()
    if self.headInfoPanel ~= nil then
        self.headInfoPanel:dispose();
        self.headInfoPanel = nil;
    end

    if self.fightHeadInfoPanel ~= nil then 
        self.fightHeadInfoPanel:dispose();
        self.fightHeadInfoPanel = nil;
    end

    if self.nameText ~= nil then
        self.nameText:dispose();
        self.nameText = nil;
    end

    if self.headContainer ~= nil then
        UnityEngine.GameObject.Destroy(self.headContainer);
        self.headContainer = nil;
    end

    if self.fightFlyContainer ~= nil then
        UnityEngine.GameObject.Destroy(self.fightFlyContainer);
        self.fightFlyContainer = nil;
    end

    if self.uiContainer ~= nil then
        UnityEngine.GameObject.Destroy(self.uiContainer);
        self.uiContainer = nil;
    end

    self.rootPos = nil;
end

function Lua_ThingPlayer:dispose()
    if self.isDispose == true then
        return;
    end

    self.isDispose = true;
    Lua_CommonUtil.removeTimer(self, self.updateThingHeadPos);
    Lua_CommonUtil.removeTimer(self, self.delaySetHeadSprPos);
    if not self.player.IsUI then
        DataCenter.thingData:removeThingPlayer(self, self.playerId, self.objId, self.sceneId, self.fightRecordId, self.formationType, self.formationPos);
    end
    self:clearRes();
    self._player = nil;
    self.data = nil;
    self.configData = nil;
end

function Lua_ThingPlayer:showClickInfo(clickStr)
    Lua_MsgUtil.callFunc("MapClick_Select_Thing", self);
end